/*
 * JNpp - Java bindings for NPP, to be used with JCuda
 *
 * Copyright (c) 2010-2014 Marco Hutter - http://www.jcuda.org
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#include "JNpp_common.hpp"

// TODO: Many of these methods have been auto-generated and could 
// be replaced by more generic ones 

//=== Primitive arrays =======================================================

/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jbyteArray input, Npp8s output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jbyte* arrayContents = (jbyte*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp8s)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp8s output[], jbyteArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jbyte* arrayContents = (jbyte*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jbyte)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jbyteArray input, Npp8u output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jbyte* arrayContents = (jbyte*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp8u)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp8u output[], jbyteArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jbyte* arrayContents = (jbyte*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jbyte)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jshortArray input, Npp16s output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jshort* arrayContents = (jshort*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp16s)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp16s output[], jshortArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jshort* arrayContents = (jshort*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jshort)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jshortArray input, Npp16u output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jshort* arrayContents = (jshort*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp16u)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp16u output[], jshortArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jshort* arrayContents = (jshort*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jshort)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jintArray input, Npp32s output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jint* arrayContents = (jint*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp32s)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp32s output[], jintArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jint* arrayContents = (jint*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jint)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jintArray input, Npp32u output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jint* arrayContents = (jint*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp32u)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp32u output[], jintArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jint* arrayContents = (jint*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jint)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jlongArray input, Npp64s output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jlong* arrayContents = (jlong*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp64s)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp64s output[], jlongArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jlong* arrayContents = (jlong*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jlong)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jlongArray input, Npp64u output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jlong* arrayContents = (jlong*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp64u)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp64u output[], jlongArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jlong* arrayContents = (jlong*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jlong)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jfloatArray input, Npp32f output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jfloat* arrayContents = (jfloat*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp32f)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp32f output[], jfloatArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jfloat* arrayContents = (jfloat*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jfloat)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jdoubleArray input, Npp64f output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jdouble* arrayContents = (jdouble*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp64f)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp64f output[], jdoubleArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jdouble* arrayContents = (jdouble*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jdouble)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}
