/*
 * JNpp - Java bindings for NPP, to be used with JCuda
 *
 * Copyright (c) 2010-2014 Marco Hutter - http://www.jcuda.org
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#include "JNpp_common.hpp"

// TODO: Many of these methods have been auto-generated and could 
// be replaced by more generic ones 

//=== Complex number struct init =============================================

/**
 * Initializes the given native output object with the values from the
 * given java input object.
 */
bool initNative(JNIEnv *env, jobject input, Npp16sc &output)
{
    output.re = (Npp16s)env->GetShortField(input, Npp16sc_re);
    output.im = (Npp16s)env->GetShortField(input, Npp16sc_im);
    return true;
}

/**
 * Initializes the given native output object with the values from the
 * given java input object.
 */
bool initNative(JNIEnv *env, jobject input, Npp32sc &output)
{
    output.re = (Npp32s)env->GetIntField(input, Npp32sc_re);
    output.im = (Npp32s)env->GetIntField(input, Npp32sc_im);
    return true;
}

/**
 * Initializes the given native output object with the values from the
 * given java input object.
 */
bool initNative(JNIEnv *env, jobject input, Npp64sc &output)
{
    output.re = (Npp64s)env->GetLongField(input, Npp64sc_re);
    output.im = (Npp64s)env->GetLongField(input, Npp64sc_im);
    return true;
}

/**
 * Initializes the given native output object with the values from the
 * given java input object.
 */
bool initNative(JNIEnv *env, jobject input, Npp32fc &output)
{
    output.re = (Npp32f)env->GetFloatField(input, Npp32fc_re);
    output.im = (Npp32f)env->GetFloatField(input, Npp32fc_im);
    return true;
}

/**
 * Initializes the given native output object with the values from the
 * given java input object.
 */
bool initNative(JNIEnv *env, jobject input, Npp64fc &output)
{
    output.re = (Npp64f)env->GetDoubleField(input, Npp64fc_re);
    output.im = (Npp64f)env->GetDoubleField(input, Npp64fc_im);
    return true;
}



//=== Complex number struct release ==========================================

/**
 * Write the values from the given native output object back to
 * the java input object
 */
bool releaseNative(JNIEnv *env, Npp16sc &output, jobject input)
{
    env->SetShortField(input, Npp16sc_re, (jshort)output.re);
    env->SetShortField(input, Npp16sc_im, (jshort)output.im);
    return true;
}

/**
 * Write the values from the given native output object back to
 * the java input object
 */
bool releaseNative(JNIEnv *env, Npp32sc &output, jobject input)
{
    env->SetIntField(input, Npp32sc_re, (jint)output.re);
    env->SetIntField(input, Npp32sc_im, (jint)output.im);
    return true;
}

/**
 * Write the values from the given native output object back to
 * the java input object
 */
bool releaseNative(JNIEnv *env, Npp64sc &output, jobject input)
{
    env->SetLongField(input, Npp64sc_re, (jlong)output.re);
    env->SetLongField(input, Npp64sc_im, (jlong)output.im);
    return true;
}

/**
 * Write the values from the given native output object back to
 * the java input object
 */
bool releaseNative(JNIEnv *env, Npp32fc &output, jobject input)
{
    env->SetFloatField(input, Npp32fc_re, (jfloat)output.re);
    env->SetFloatField(input, Npp32fc_im, (jfloat)output.im);
    return true;
}

/**
 * Write the values from the given native output object back to
 * the java input object
 */
bool releaseNative(JNIEnv *env, Npp64fc &output, jobject input)
{
    env->SetDoubleField(input, Npp64fc_re, (jdouble)output.re);
    env->SetDoubleField(input, Npp64fc_im, (jdouble)output.im);
    return true;
}
