/*
 * JNpp - Java bindings for NPP, to be used with JCuda
 *
 * Copyright (c) 2010-2014 Marco Hutter - http://www.jcuda.org
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */
package jcuda.jnpp;

public class NppiInterpolationMode
{
    public static final int NPPI_INTER_UNDEFINED = 0;
    /**  Nearest neighbor filtering. */
    public static final int NPPI_INTER_NN = 1;
    /**  Linear interpolation. */
    public static final int NPPI_INTER_LINEAR = 2;
    /**  Cubic interpolation. */
    public static final int NPPI_INTER_CUBIC = 4;
    /**  Two-parameter cubic filter (B=1, C=0) */
    public static final int NPPI_INTER_CUBIC2P_BSPLINE = 5;
    /**  Two-parameter cubic filter (B=0, C=1/2) */
    public static final int NPPI_INTER_CUBIC2P_CATMULLROM = 6;
    /**  Two-parameter cubic filter (B=1/2, C=3/10) */
    public static final int NPPI_INTER_CUBIC2P_B05C03 = 7;
    /**  Super sampling. */
    public static final int NPPI_INTER_SUPER = 8;
    /**  Lanczos filtering. */
    public static final int NPPI_INTER_LANCZOS = 16;
    /**  Smooth edge filtering. */
    public static final int NPPI_SMOOTH_EDGE = (1 << 31);

    /**
     * Private constructor to prevent instantiation
     */
    private NppiInterpolationMode(){}

    /**
     * Returns a string representation of the given constant
     *
     * @return A string representation of the given constant
     */
    public static String stringFor(int n)
    {
        switch (n)
        {
            case NPPI_INTER_UNDEFINED: return "NPPI_INTER_UNDEFINED";
            case NPPI_INTER_NN: return "NPPI_INTER_NN";
            case NPPI_INTER_LINEAR: return "NPPI_INTER_LINEAR";
            case NPPI_INTER_CUBIC: return "NPPI_INTER_CUBIC";
            case NPPI_INTER_CUBIC2P_BSPLINE: return "NPPI_INTER_CUBIC2P_BSPLINE";
            case NPPI_INTER_CUBIC2P_CATMULLROM: return "NPPI_INTER_CUBIC2P_CATMULLROM";
            case NPPI_INTER_CUBIC2P_B05C03: return "NPPI_INTER_CUBIC2P_B05C03";
            case NPPI_INTER_SUPER: return "NPPI_INTER_SUPER";
            case NPPI_INTER_LANCZOS: return "NPPI_INTER_LANCZOS";
            case NPPI_SMOOTH_EDGE: return "NPPI_SMOOTH_EDGE";
        }
        return "INVALID NppiInterpolationMode: "+n;
    }
}

