/*
 * JNpp - Java bindings for NPP, to be used with JCuda
 *
 * Copyright (c) 2010-2014 Marco Hutter - http://www.jcuda.org
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */
package jcuda.jnpp;

import jcuda.*;
import jcuda.jnpp.types.*;

/**
 * Java bindings for the image functions of NPP. Most of the documentation
 * is extracted from the NPPI header files.
 */
public class JNppi
{
    /**
     * Private constructor to prevent instantiation
     */
    private JNppi()
    {
    }

    static
    {
        JNpp.initialize();
    }

    //=== Auto-generated part ================================================

    /**
     * <pre>
     * 8-bit unsigned image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp8u> nppiMalloc_8u_C1(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_8u_C1Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp8u> nppiMalloc_8u_C1Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 2 channel 8-bit unsigned image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp8u> nppiMalloc_8u_C2(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_8u_C2Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp8u> nppiMalloc_8u_C2Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 3 channel 8-bit unsigned image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp8u> nppiMalloc_8u_C3(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_8u_C3Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp8u> nppiMalloc_8u_C3Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 4 channel 8-bit unsigned image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp8u> nppiMalloc_8u_C4(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_8u_C4Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp8u> nppiMalloc_8u_C4Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 16-bit unsigned image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp16u> nppiMalloc_16u_C1(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_16u_C1Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp16u> nppiMalloc_16u_C1Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 2 channel 16-bit unsigned image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp16u> nppiMalloc_16u_C2(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_16u_C2Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp16u> nppiMalloc_16u_C2Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 3 channel 16-bit unsigned image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp16u> nppiMalloc_16u_C3(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_16u_C3Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp16u> nppiMalloc_16u_C3Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 4 channel 16-bit unsigned image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp16u> nppiMalloc_16u_C4(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_16u_C4Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp16u> nppiMalloc_16u_C4Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 16-bit signed image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp16s> nppiMalloc_16s_C1(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_16s_C1Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp16s> nppiMalloc_16s_C1Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 2 channel 16-bit signed image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp16s> nppiMalloc_16s_C2(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_16s_C2Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp16s> nppiMalloc_16s_C2Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 4 channel 16-bit signed image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp16s> nppiMalloc_16s_C4(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_16s_C4Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp16s> nppiMalloc_16s_C4Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 1 channel 16-bit signed complex image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp16s> nppiMalloc_16sc_C1(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_16sc_C1Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp16s> nppiMalloc_16sc_C1Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 2 channel 16-bit signed complex image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp16s> nppiMalloc_16sc_C2(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_16sc_C2Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp16s> nppiMalloc_16sc_C2Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 3 channel 16-bit signed complex image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp16s> nppiMalloc_16sc_C3(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_16sc_C3Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp16s> nppiMalloc_16sc_C3Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 4 channel 16-bit signed complex image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp16s> nppiMalloc_16sc_C4(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_16sc_C4Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp16s> nppiMalloc_16sc_C4Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 32-bit signed image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp32s> nppiMalloc_32s_C1(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_32s_C1Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp32s> nppiMalloc_32s_C1Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 3 channel 32-bit signed image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp32s> nppiMalloc_32s_C3(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_32s_C3Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp32s> nppiMalloc_32s_C3Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 4 channel 32-bit signed image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp32s> nppiMalloc_32s_C4(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_32s_C4Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp32s> nppiMalloc_32s_C4Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    public static TypedPointer<Npp32s> nppiMalloc_32sc_C1(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_32sc_C1Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp32s> nppiMalloc_32sc_C1Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    public static TypedPointer<Npp32s> nppiMalloc_32sc_C2(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_32sc_C2Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp32s> nppiMalloc_32sc_C2Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 3 channel 32-bit integer complex image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp32s> nppiMalloc_32sc_C3(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_32sc_C3Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp32s> nppiMalloc_32sc_C3Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 4 channel 32-bit integer complex image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp32s> nppiMalloc_32sc_C4(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_32sc_C4Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp32s> nppiMalloc_32sc_C4Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 32-bit floating point image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp32f> nppiMalloc_32f_C1(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_32f_C1Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp32f> nppiMalloc_32f_C1Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 2 channel 32-bit floating point image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp32f> nppiMalloc_32f_C2(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_32f_C2Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp32f> nppiMalloc_32f_C2Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 3 channel 32-bit floating point image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp32f> nppiMalloc_32f_C3(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_32f_C3Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp32f> nppiMalloc_32f_C3Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 4 channel 32-bit floating point image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp32f> nppiMalloc_32f_C4(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_32f_C4Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp32f> nppiMalloc_32f_C4Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    public static TypedPointer<Npp32f> nppiMalloc_32fc_C1(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_32fc_C1Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp32f> nppiMalloc_32fc_C1Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 2 channel 32-bit float complex image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp32f> nppiMalloc_32fc_C2(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_32fc_C2Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp32f> nppiMalloc_32fc_C2Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 3 channel 32-bit float complex image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp32f> nppiMalloc_32fc_C3(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_32fc_C3Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp32f> nppiMalloc_32fc_C3Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * 4 channel 32-bit float complex image memory allocator.
     * @param nWidthPixels Image width.
     * @param nHeightPixels Image height.
     * @param pStepBytes \ref line_step.
     * @return
     *  Pointer to new image data.
     * </pre>
     */
    public static TypedPointer<Npp32f> nppiMalloc_32fc_C4(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes)
    {
        return JNpp.checkResult(nppiMalloc_32fc_C4Native(nWidthPixels, nHeightPixels, pStepBytes));
    }
    private static native TypedPointer<Npp32f> nppiMalloc_32fc_C4Native(
        int nWidthPixels,
        int nHeightPixels,
        int[] pStepBytes);


    /**
     * <pre>
     * Free method for any 2D allocated memory.
     * This method should be used to free memory allocated with any of the nppiMalloc_<modifier> methods.
     * @param pData A pointer to memory allocated using nppiMalloc_<modifier>.
     * </pre>
     */
    public static void nppiFree(
        Pointer pData)
    {
        nppiFreeNative(pData);
    }
    private static native void nppiFreeNative(
        Pointer pData);


    /**
     * <pre>
     * 8-bit image set.
     * @param nValue The pixel value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_8s_C1R(
        byte nValue,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_8s_C1RNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_8s_C1RNative(
        byte nValue,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 8-bit two-channel image set.
     * @param aValue The pixel value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_8s_C2R(
        byte[] aValue,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_8s_C2RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_8s_C2RNative(
        byte[] aValue,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiSet_8s_C3R(
        byte[] aValue,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_8s_C3RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_8s_C3RNative(
        byte[] aValue,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiSet_8s_C4R(
        byte[] aValue,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_8s_C4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_8s_C4RNative(
        byte[] aValue,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 8-bit four-channel image set ignoring alpha channel.
     * @param aValue The pixel value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_8s_AC4R(
        byte[] aValue,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_8s_AC4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_8s_AC4RNative(
        byte[] aValue,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiSet_8u_C1R(
        byte nValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_8u_C1RNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_8u_C1RNative(
        byte nValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned image set.
     * @param aValue The pixel value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_8u_C2R(
        byte[] aValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_8u_C2RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_8u_C2RNative(
        byte[] aValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned image set.
     * @param aValue The pixel value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_8u_C3R(
        byte[] aValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_8u_C3RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_8u_C3RNative(
        byte[] aValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiSet_8u_C4R(
        byte[] aValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_8u_C4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_8u_C4RNative(
        byte[] aValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned image set method, not affecting Alpha channel.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_8u_AC4R(
        byte[] aValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_8u_AC4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_8u_AC4RNative(
        byte[] aValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiSet_16u_C1R(
        short nValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16u_C1RNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16u_C1RNative(
        short nValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 16-bit unsigned image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16u_C2R(
        short[] aValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16u_C2RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16u_C2RNative(
        short[] aValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 16-bit unsigned image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16u_C3R(
        short[] aValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16u_C3RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16u_C3RNative(
        short[] aValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 16-bit unsigned image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16u_C4R(
        short[] aValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16u_C4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16u_C4RNative(
        short[] aValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 16-bit unsigned image set method, not affecting Alpha channel.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16u_AC4R(
        short[] aValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16u_AC4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16u_AC4RNative(
        short[] aValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiSet_16s_C1R(
        short nValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16s_C1RNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16s_C1RNative(
        short nValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 16-bit image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16s_C2R(
        short[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16s_C2RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16s_C2RNative(
        short[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 16-bit image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16s_C3R(
        short[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16s_C3RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16s_C3RNative(
        short[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 16-bit image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16s_C4R(
        short[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16s_C4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16s_C4RNative(
        short[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 16-bit image set method, not affecting Alpha channel.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16s_AC4R(
        short[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16s_AC4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16s_AC4RNative(
        short[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 16-bit complex integer image set.
     * @param oValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16sc_C1R(
        Npp16sc oValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16sc_C1RNative(oValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16sc_C1RNative(
        Npp16sc oValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 16-bit complex integer two-channel image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16sc_C2R(
        Npp16sc[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16sc_C2RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16sc_C2RNative(
        Npp16sc[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 16-bit complex integer three-channel image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16sc_C3R(
        Npp16sc[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16sc_C3RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16sc_C3RNative(
        Npp16sc[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 16-bit complex integer four-channel image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16sc_C4R(
        Npp16sc[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16sc_C4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16sc_C4RNative(
        Npp16sc[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 16-bit complex integer four-channel image set ignoring alpha.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16sc_AC4R(
        Npp16sc[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16sc_AC4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16sc_AC4RNative(
        Npp16sc[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 32-bit image set.
     * @param nValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32s_C1R(
        int nValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32s_C1RNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32s_C1RNative(
        int nValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 32-bit image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32s_C2R(
        int[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32s_C2RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32s_C2RNative(
        int[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 32-bit image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32s_C3R(
        int[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32s_C3RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32s_C3RNative(
        int[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 32-bit image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32s_C4R(
        int[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32s_C4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32s_C4RNative(
        int[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 32-bit image set method, not affecting Alpha channel.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32s_AC4R(
        int[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32s_AC4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32s_AC4RNative(
        int[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiSet_32u_C1R(
        int nValue,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32u_C1RNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32u_C1RNative(
        int nValue,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 32-bit unsigned image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32u_C2R(
        int[] aValue,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32u_C2RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32u_C2RNative(
        int[] aValue,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 32-bit unsigned image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32u_C3R(
        int[] aValue,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32u_C3RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32u_C3RNative(
        int[] aValue,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiSet_32u_C4R(
        int[] aValue,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32u_C4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32u_C4RNative(
        int[] aValue,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 32-bit unsigned image set method, not affecting Alpha channel.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32u_AC4R(
        int[] aValue,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32u_AC4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32u_AC4RNative(
        int[] aValue,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 32-bit complex integer image set.
     * @param oValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32sc_C1R(
        Npp32sc oValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32sc_C1RNative(oValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32sc_C1RNative(
        Npp32sc oValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Two channel 32-bit complex integer image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32sc_C2R(
        Npp32sc[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32sc_C2RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32sc_C2RNative(
        Npp32sc[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 32-bit complex integer image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32sc_C3R(
        Npp32sc[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32sc_C3RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32sc_C3RNative(
        Npp32sc[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit complex integer image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32sc_C4R(
        Npp32sc[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32sc_C4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32sc_C4RNative(
        Npp32sc[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiSet_32sc_AC4R(
        Npp32sc[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32sc_AC4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32sc_AC4RNative(
        Npp32sc[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiSet_32f_C1R(
        float nValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32f_C1RNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32f_C1RNative(
        float nValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiSet_32f_C2R(
        float[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32f_C2RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32f_C2RNative(
        float[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 32-bit floating point image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32f_C3R(
        float[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32f_C3RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32f_C3RNative(
        float[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 32-bit floating point image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32f_C4R(
        float[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32f_C4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32f_C4RNative(
        float[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 32-bit floating point image set method, not affecting Alpha channel.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32f_AC4R(
        float[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32f_AC4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32f_AC4RNative(
        float[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 32-bit complex image set.
     * @param oValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32fc_C1R(
        Npp32fc oValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32fc_C1RNative(oValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32fc_C1RNative(
        Npp32fc oValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiSet_32fc_C2R(
        Npp32fc[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32fc_C2RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32fc_C2RNative(
        Npp32fc[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 32-bit complex image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32fc_C3R(
        Npp32fc[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32fc_C3RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32fc_C3RNative(
        Npp32fc[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit complex image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32fc_C4R(
        Npp32fc[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32fc_C4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32fc_C4RNative(
        Npp32fc[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 32-bit complex four-channel image set ignoring alpha.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32fc_AC4R(
        Npp32fc[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32fc_AC4RNative(aValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32fc_AC4RNative(
        Npp32fc[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Masked 8-bit unsigned image set.
     * @param nValue The pixel value to be set.
     * @param pDst Pointer \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_8u_C1MR(
        byte nValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_8u_C1MRNative(nValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_8u_C1MRNative(
        byte nValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Masked 3 channel 8-bit unsigned image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_8u_C3MR(
        byte[] aValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_8u_C3MRNative(aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_8u_C3MRNative(
        byte[] aValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Masked 4 channel 8-bit unsigned image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_8u_C4MR(
        byte[] aValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_8u_C4MRNative(aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_8u_C4MRNative(
        byte[] aValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Masked 4 channel 8-bit unsigned image set method, not affecting Alpha channel.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_8u_AC4MR(
        byte[] aValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_8u_AC4MRNative(aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_8u_AC4MRNative(
        byte[] aValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    public static int nppiSet_16u_C1MR(
        short nValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_16u_C1MRNative(nValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_16u_C1MRNative(
        short nValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    public static int nppiSet_16u_C3MR(
        short[] aValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_16u_C3MRNative(aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_16u_C3MRNative(
        short[] aValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Masked 4 channel 16-bit unsigned image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16u_C4MR(
        short[] aValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_16u_C4MRNative(aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_16u_C4MRNative(
        short[] aValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Masked 4 channel 16-bit unsigned image set method, not affecting Alpha channel.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16u_AC4MR(
        short[] aValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_16u_AC4MRNative(aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_16u_AC4MRNative(
        short[] aValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Masked 16-bit image set.
     * @param nValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16s_C1MR(
        short nValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_16s_C1MRNative(nValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_16s_C1MRNative(
        short nValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Masked 3 channel 16-bit image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16s_C3MR(
        short[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_16s_C3MRNative(aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_16s_C3MRNative(
        short[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    public static int nppiSet_16s_C4MR(
        short[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_16s_C4MRNative(aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_16s_C4MRNative(
        short[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Masked 4 channel 16-bit image set method, not affecting Alpha channel.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16s_AC4MR(
        short[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_16s_AC4MRNative(aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_16s_AC4MRNative(
        short[] aValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Masked 32-bit image set.
     * @param nValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32s_C1MR(
        int nValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_32s_C1MRNative(nValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_32s_C1MRNative(
        int nValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Masked 3 channel 32-bit image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32s_C3MR(
        int[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_32s_C3MRNative(aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_32s_C3MRNative(
        int[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Masked 4 channel 32-bit image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32s_C4MR(
        int[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_32s_C4MRNative(aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_32s_C4MRNative(
        int[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Masked 4 channel 16-bit image set method, not affecting Alpha channel.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32s_AC4MR(
        int[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_32s_AC4MRNative(aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_32s_AC4MRNative(
        int[] aValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Masked 32-bit floating point image set.
     * @param nValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32f_C1MR(
        float nValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_32f_C1MRNative(nValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_32f_C1MRNative(
        float nValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Masked 3 channel 32-bit floating point image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32f_C3MR(
        float[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_32f_C3MRNative(aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_32f_C3MRNative(
        float[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Masked 4 channel 32-bit floating point image set.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32f_C4MR(
        float[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_32f_C4MRNative(aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_32f_C4MRNative(
        float[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Masked 4 channel 32-bit floating point image set method, not affecting Alpha channel.
     * @param aValue The pixel-value to be set.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32f_AC4MR(
        float[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiSet_32f_AC4MRNative(aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiSet_32f_AC4MRNative(
        float[] aValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * 3 channel 8-bit unsigned image set affecting only single channel.
     * @param nValue The pixel-value to be set.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_8u_C3CR(
        byte nValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_8u_C3CRNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_8u_C3CRNative(
        byte nValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned image set affecting only single channel.
     * @param nValue The pixel-value to be set.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_8u_C4CR(
        byte nValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_8u_C4CRNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_8u_C4CRNative(
        byte nValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 16-bit unsigned image set affecting only single channel.
     * @param nValue The pixel-value to be set.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16u_C3CR(
        short nValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16u_C3CRNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16u_C3CRNative(
        short nValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 16-bit unsigned image set affecting only single channel.
     * @param nValue The pixel-value to be set.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16u_C4CR(
        short nValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16u_C4CRNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16u_C4CRNative(
        short nValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 16-bit signed image set affecting only single channel.
     * @param nValue The pixel-value to be set.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16s_C3CR(
        short nValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16s_C3CRNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16s_C3CRNative(
        short nValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 16-bit signed image set affecting only single channel.
     * @param nValue The pixel-value to be set.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_16s_C4CR(
        short nValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_16s_C4CRNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_16s_C4CRNative(
        short nValue,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiSet_32s_C3CR(
        int nValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32s_C3CRNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32s_C3CRNative(
        int nValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 32-bit unsigned image set affecting only single channel.
     * @param nValue The pixel-value to be set.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32s_C4CR(
        int nValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32s_C4CRNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32s_C4CRNative(
        int nValue,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 32-bit floating point image set affecting only single channel.
     * @param nValue The pixel-value to be set.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32f_C3CR(
        float nValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32f_C3CRNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32f_C3CRNative(
        float nValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 32-bit floating point image set affecting only single channel.
     * @param nValue The pixel-value to be set.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSet_32f_C4CR(
        float nValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSet_32f_C4CRNative(nValue, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSet_32f_C4CRNative(
        float nValue,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 8-bit image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Two-channel 8-bit image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8s_C2R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8s_C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8s_C2RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 8-bit image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8s_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8s_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 8-bit image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8s_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8s_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 8-bit image copy, ignoring alpha channel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8s_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8s_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 8-bit unsigned image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiCopy_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned image copy, not affecting Alpha channel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 16-bit unsigned image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 16-bit unsigned image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 16-bit unsigned image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 16-bit unsigned image copy, not affecting Alpha channel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 16-bit image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiCopy_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 16-bit image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 16-bit image copy, not affecting Alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 16-bit complex image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16sc_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16sc_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16sc_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Two-channel 16-bit complex image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16sc_C2R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16sc_C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16sc_C2RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 16-bit complex image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16sc_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16sc_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16sc_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 16-bit complex image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16sc_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16sc_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16sc_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 16-bit complex image copy, ignoring alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16sc_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16sc_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16sc_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiCopy_32s_C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 32-bit image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32s_C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 32-bit image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32s_C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 32-bit image copy, not affecting Alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32s_AC4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 32-bit complex image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32sc_C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32sc_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32sc_C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiCopy_32sc_C2R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32sc_C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32sc_C2RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 32-bit complex image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32sc_C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32sc_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32sc_C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 32-bit complex image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32sc_C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32sc_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32sc_C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiCopy_32sc_AC4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32sc_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32sc_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 32-bit floating point image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 32-bit floating point image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 32-bit floating point image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 32-bit floating point image copy, not affecting Alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 32-bit floating-point complex image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32fc_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32fc_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32fc_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Two-channel 32-bit floating-point complex image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32fc_C2R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32fc_C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32fc_C2RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiCopy_32fc_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32fc_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32fc_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 32-bit floating-point complex image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32fc_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32fc_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32fc_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 32-bit floating-point complex image copy, ignoring alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32fc_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32fc_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32fc_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * \ref masked_operation 8-bit unsigned image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8u_C1MR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_8u_C1MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_8u_C1MRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    public static int nppiCopy_8u_C3MR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_8u_C3MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_8u_C3MRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation four channel 8-bit unsigned image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8u_C4MR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_8u_C4MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_8u_C4MRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation four channel 8-bit unsigned image copy, ignoring alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8u_AC4MR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_8u_AC4MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_8u_AC4MRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation 16-bit unsigned image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_C1MR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_16u_C1MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_16u_C1MRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation three channel 16-bit unsigned image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_C3MR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_16u_C3MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_16u_C3MRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation four channel 16-bit unsigned image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_C4MR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_16u_C4MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_16u_C4MRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation four channel 16-bit unsigned image copy, ignoring alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_AC4MR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_16u_AC4MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_16u_AC4MRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation 16-bit signed image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16s_C1MR(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_16s_C1MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_16s_C1MRNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation three channel 16-bit signed image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16s_C3MR(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_16s_C3MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_16s_C3MRNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation four channel 16-bit signed image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16s_C4MR(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_16s_C4MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_16s_C4MRNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation four channel 16-bit signed image copy, ignoring alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16s_AC4MR(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_16s_AC4MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_16s_AC4MRNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation 32-bit signed image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32s_C1MR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_32s_C1MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_32s_C1MRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation three channel 32-bit signed image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32s_C3MR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_32s_C3MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_32s_C3MRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation four channel 32-bit signed image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32s_C4MR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_32s_C4MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_32s_C4MRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation four channel 32-bit signed image copy, ignoring alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32s_AC4MR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_32s_AC4MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_32s_AC4MRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation 32-bit float image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_C1MR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_32f_C1MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_32f_C1MRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation three channel 32-bit float image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_C3MR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_32f_C3MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_32f_C3MRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation four channel 32-bit float image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_C4MR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_32f_C4MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_32f_C4MRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * \ref masked_operation four channel 32-bit float image copy, ignoring alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_AC4MR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep)
    {
        return JNpp.checkResult(nppiCopy_32f_AC4MRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep));
    }
    private static native int nppiCopy_32f_AC4MRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        int nMaskStep);


    /**
     * <pre>
     * Select-channel 8-bit unsigned image copy for three-channel images.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8u_C3CR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8u_C3CRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8u_C3CRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Select-channel 8-bit unsigned image copy for four-channel images.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8u_C4CR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8u_C4CRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8u_C4CRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Select-channel 16-bit signed image copy for three-channel images.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16s_C3CR(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16s_C3CRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16s_C3CRNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Select-channel 16-bit signed image copy for four-channel images.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16s_C4CR(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16s_C4CRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16s_C4CRNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Select-channel 16-bit unsigned image copy for three-channel images.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_C3CR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16u_C3CRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16u_C3CRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Select-channel 16-bit unsigned image copy for four-channel images.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_C4CR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16u_C4CRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16u_C4CRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Select-channel 32-bit signed image copy for three-channel images.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32s_C3CR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32s_C3CRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32s_C3CRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Select-channel 32-bit signed image copy for four-channel images.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32s_C4CR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32s_C4CRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32s_C4CRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Select-channel 32-bit float image copy for three-channel images.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_C3CR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32f_C3CRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32f_C3CRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Select-channel 32-bit float image copy for four-channel images.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_C4CR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32f_C4CRNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32f_C4CRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel to single-channel 8-bit unsigned image copy.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8u_C3C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8u_C3C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8u_C3C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel to single-channel 8-bit unsigned image copy.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8u_C4C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8u_C4C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8u_C4C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel to single-channel 16-bit signed image copy.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16s_C3C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16s_C3C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16s_C3C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel to single-channel 16-bit signed image copy.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16s_C4C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16s_C4C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16s_C4C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel to single-channel 16-bit unsigned image copy.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_C3C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16u_C3C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16u_C3C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel to single-channel 16-bit unsigned image copy.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_C4C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16u_C4C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16u_C4C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel to single-channel 32-bit signed image copy.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32s_C3C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32s_C3C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32s_C3C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel to single-channel 32-bit signed image copy.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32s_C4C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32s_C4C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32s_C4C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel to single-channel 32-bit float image copy.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_C3C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32f_C3C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32f_C3C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel to single-channel 32-bit float image copy.
     * @param pSrc \ref select_source_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_C4C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32f_C4C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32f_C4C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiCopy_8u_C1C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8u_C1C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8u_C1C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single-channel to four-channel 8-bit unsigned image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8u_C1C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8u_C1C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8u_C1C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single-channel to three-channel 16-bit signed image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16s_C1C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16s_C1C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16s_C1C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single-channel to four-channel 16-bit signed image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16s_C1C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16s_C1C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16s_C1C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiCopy_16u_C1C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16u_C1C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16u_C1C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single-channel to four-channel 16-bit unsigned image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_C1C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16u_C1C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16u_C1C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single-channel to three-channel 32-bit signed image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32s_C1C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32s_C1C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32s_C1C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single-channel to four-channel 32-bit signed image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32s_C1C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32s_C1C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32s_C1C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single-channel to three-channel 32-bit float image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_C1C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32f_C1C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32f_C1C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single-channel to four-channel 32-bit float image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref select_destination_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_C1C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32f_C1C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32f_C1C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 8-bit unsigned packed to planar image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param aDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] aDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8u_C3P3RNative(pSrc, nSrcStep, aDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] aDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 8-bit unsigned packed to planar image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param aDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8u_C4P4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] aDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8u_C4P4RNative(pSrc, nSrcStep, aDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8u_C4P4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] aDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 16-bit signed packed to planar image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param aDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16s_C3P3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s>[] aDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16s_C3P3RNative(pSrc, nSrcStep, aDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16s_C3P3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s>[] aDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiCopy_16s_C4P4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s>[] aDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16s_C4P4RNative(pSrc, nSrcStep, aDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16s_C4P4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s>[] aDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 16-bit unsigned packed to planar image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param aDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_C3P3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u>[] aDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16u_C3P3RNative(pSrc, nSrcStep, aDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16u_C3P3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u>[] aDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 16-bit unsigned packed to planar image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param aDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_C4P4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u>[] aDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16u_C4P4RNative(pSrc, nSrcStep, aDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16u_C4P4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u>[] aDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 32-bit signed packed to planar image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param aDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32s_C3P3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s>[] aDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32s_C3P3RNative(pSrc, nSrcStep, aDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32s_C3P3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s>[] aDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 32-bit signed packed to planar image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param aDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32s_C4P4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s>[] aDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32s_C4P4RNative(pSrc, nSrcStep, aDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32s_C4P4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s>[] aDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiCopy_32f_C3P3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f>[] aDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32f_C3P3RNative(pSrc, nSrcStep, aDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32f_C3P3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f>[] aDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 32-bit float packed to planar image copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param aDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_C4P4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f>[] aDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32f_C4P4RNative(pSrc, nSrcStep, aDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32f_C4P4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f>[] aDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 8-bit unsigned planar to packed image copy.
     * @param aSrc Planar \ref source_image_pointer.
     * @param nSrcStep \ref source_planar_image_pointer_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8u_P3C3R(
        TypedPointer<Npp8u>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8u_P3C3RNative(aSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8u_P3C3RNative(
        TypedPointer<Npp8u>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 8-bit unsigned planar to packed image copy.
     * @param aSrc Planar \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_8u_P4C4R(
        TypedPointer<Npp8u>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_8u_P4C4RNative(aSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_8u_P4C4RNative(
        TypedPointer<Npp8u>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 16-bit unsigned planar to packed image copy.
     * @param aSrc Planar \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_P3C3R(
        TypedPointer<Npp16u>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16u_P3C3RNative(aSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16u_P3C3RNative(
        TypedPointer<Npp16u>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 16-bit unsigned planar to packed image copy.
     * @param aSrc Planar \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16u_P4C4R(
        TypedPointer<Npp16u>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16u_P4C4RNative(aSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16u_P4C4RNative(
        TypedPointer<Npp16u>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 16-bit signed planar to packed image copy.
     * @param aSrc Planar \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16s_P3C3R(
        TypedPointer<Npp16s>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16s_P3C3RNative(aSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16s_P3C3RNative(
        TypedPointer<Npp16s>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 16-bit signed planar to packed image copy.
     * @param aSrc Planar \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_16s_P4C4R(
        TypedPointer<Npp16s>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_16s_P4C4RNative(aSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_16s_P4C4RNative(
        TypedPointer<Npp16s>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 32-bit signed planar to packed image copy.
     * @param aSrc Planar \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32s_P3C3R(
        TypedPointer<Npp32s>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32s_P3C3RNative(aSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32s_P3C3RNative(
        TypedPointer<Npp32s>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiCopy_32s_P4C4R(
        TypedPointer<Npp32s>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32s_P4C4RNative(aSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32s_P4C4RNative(
        TypedPointer<Npp32s>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 32-bit float planar to packed image copy.
     * @param aSrc Planar \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_P3C3R(
        TypedPointer<Npp32f>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32f_P3C3RNative(aSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32f_P3C3RNative(
        TypedPointer<Npp32f>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 32-bit float planar to packed image copy.
     * @param aSrc Planar \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopy_32f_P4C4R(
        TypedPointer<Npp32f>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCopy_32f_P4C4RNative(aSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCopy_32f_P4C4RNative(
        TypedPointer<Npp32f>[] aSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 16-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8u16u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8u16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8u16u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 8-bit unsigned to 16-bit unsigned  conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8u16u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8u16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8u16u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned to 16-bit unsigned  conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8u16u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8u16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8u16u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned to 16-bit unsigned conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8u16u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8u16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8u16u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 16-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8u16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 8-bit unsigned to 16-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8u16s_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8u16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8u16s_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned to 16-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8u16s_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8u16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8u16s_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned to 16-bit signed conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8u16s_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8u16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8u16s_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 32-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8u32s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8u32s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8u32s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 8-bit unsigned to 32-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8u32s_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8u32s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8u32s_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiConvert_8u32s_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8u32s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8u32s_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned to 32-bit signed conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8u32s_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8u32s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8u32s_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 32-bit floating-point conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8u32f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8u32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8u32f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 8-bit unsigned to 32-bit floating-point conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8u32f_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8u32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8u32f_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned to 32-bit floating-point conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8u32f_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8u32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8u32f_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned to 32-bit floating-point conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8u32f_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8u32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8u32f_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit signed to 32-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8s32s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8s32s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8s32s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 8-bit signed to 32-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8s32s_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8s32s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8s32s_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit signed to 32-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8s32s_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8s32s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8s32s_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit signed to 32-bit signed conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8s32s_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8s32s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8s32s_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit signed to 32-bit floating-point conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8s32f_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8s32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8s32f_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 8-bit signed to 32-bit floating-point conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8s32f_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8s32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8s32f_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit signed to 32-bit floating-point conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8s32f_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8s32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8s32f_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit signed to 32-bit floating-point conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8s32f_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8s32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8s32f_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 16-bit unsigned to 32-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16u32s_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16u32s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16u32s_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 16-bit unsigned to 32-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16u32s_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16u32s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16u32s_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit unsigned to 32-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16u32s_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16u32s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16u32s_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit unsigned to 32-bit signed conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16u32s_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16u32s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16u32s_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 16-bit unsigned to 32-bit floating-point conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16u32f_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16u32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16u32f_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 16-bit unsigned to 32-bit floating-point conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16u32f_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16u32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16u32f_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit unsigned to 32-bit floating-point conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16u32f_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16u32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16u32f_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiConvert_16u32f_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16u32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16u32f_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 16-bit signed to 32-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16s32s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16s32s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16s32s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiConvert_16s32s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16s32s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16s32s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed to 32-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16s32s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16s32s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16s32s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed to 32-bit signed conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16s32s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16s32s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16s32s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 16-bit signed to 32-bit floating-point conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16s32f_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16s32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16s32f_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 16-bit signed to 32-bit floating-point conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16s32f_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16s32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16s32f_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed to 32-bit floating-point conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16s32f_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16s32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16s32f_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed to 32-bit floating-point conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16s32f_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16s32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16s32f_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit signed to 8-bit unsigned conversion with saturation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8s8u_C1Rs(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8s8u_C1RsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8s8u_C1RsNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit signed to 16-bit unsigned conversion with saturation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8s16u_C1Rs(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8s16u_C1RsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8s16u_C1RsNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit signed to 16-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8s16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit signed to 32-bit unsigned conversion with saturation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_8s32u_C1Rs(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_8s32u_C1RsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_8s32u_C1RsNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 16-bit signed to 16-bit unsigned conversion with saturation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16s16u_C1Rs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16s16u_C1RsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16s16u_C1RsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 16-bit signed to 32-bit unsigned conversion with saturation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16s32u_C1Rs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16s32u_C1RsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16s32u_C1RsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 16-bit unsigned to 32-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16u32u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16u32u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16u32u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 32-bit signed to 32-bit unsigned conversion with saturation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32s32u_C1Rs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_32s32u_C1RsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_32s32u_C1RsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 32-bit signed to 32-bit floating-point conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32s32f_C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_32s32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_32s32f_C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiConvert_32u32f_C1R(
        TypedPointer<Npp32u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_32u32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_32u32f_C1RNative(
        TypedPointer<Npp32u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiConvert_16u8u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16u8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16u8u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 16-bit unsigned to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16u8u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16u8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16u8u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit unsigned to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16u8u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16u8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16u8u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit unsigned to 8-bit unsigned conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16u8u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16u8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16u8u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 16-bit signed to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16s8u_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16s8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16s8u_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 16-bit signed to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16s8u_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16s8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16s8u_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16s8u_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16s8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16s8u_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed to 8-bit unsigned conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16s8u_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_16s8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_16s8u_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 32-bit signed to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32s8u_C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_32s8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_32s8u_C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 32-bit signed to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32s8u_C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_32s8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_32s8u_C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit signed to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32s8u_C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_32s8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_32s8u_C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit signed to 8-bit unsigned conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32s8u_AC4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_32s8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_32s8u_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 32-bit signed to 8-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32s8s_C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_32s8s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_32s8s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 32-bit signed to 8-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32s8s_C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_32s8s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_32s8s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiConvert_32s8s_C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_32s8s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_32s8s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit signed to 8-bit signed conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32s8s_AC4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiConvert_32s8s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiConvert_32s8s_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiConvert_8u8s_C1RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_8u8s_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_8u8s_C1RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    /**
     * <pre>
     * Single channel 16-bit unsigned to 8-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode \ref rounding_mode_parameter.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16u8s_C1RSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_16u8s_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_16u8s_C1RSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    /**
     * <pre>
     * Single channel 16-bit signed to 8-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode \ref rounding_mode_parameter.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_16s8s_C1RSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_16s8s_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_16s8s_C1RSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    public static int nppiConvert_16u16s_C1RSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_16u16s_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_16u16s_C1RSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    /**
     * <pre>
     * Single channel 32-bit unsigned to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode \ref rounding_mode_parameter.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32u8u_C1RSfs(
        TypedPointer<Npp32u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_32u8u_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_32u8u_C1RSfsNative(
        TypedPointer<Npp32u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    public static int nppiConvert_32u8s_C1RSfs(
        TypedPointer<Npp32u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_32u8s_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_32u8s_C1RSfsNative(
        TypedPointer<Npp32u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    /**
     * <pre>
     * Single channel 32-bit unsigned to 16-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode \ref rounding_mode_parameter.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32u16u_C1RSfs(
        TypedPointer<Npp32u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_32u16u_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_32u16u_C1RSfsNative(
        TypedPointer<Npp32u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    /**
     * <pre>
     * Single channel 32-bit unsigned to 16-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode \ref rounding_mode_parameter.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32u16s_C1RSfs(
        TypedPointer<Npp32u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_32u16s_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_32u16s_C1RSfsNative(
        TypedPointer<Npp32u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    /**
     * <pre>
     * Single channel 32-bit unsigned to 32-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode \ref rounding_mode_parameter.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32u32s_C1RSfs(
        TypedPointer<Npp32u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_32u32s_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_32u32s_C1RSfsNative(
        TypedPointer<Npp32u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    /**
     * <pre>
     * Single channel 32-bit unsigned to 16-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode \ref rounding_mode_parameter.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32s16u_C1RSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_32s16u_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_32s16u_C1RSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    /**
     * <pre>
     * Single channel 32-bit unsigned to 16-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode \ref rounding_mode_parameter.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32s16s_C1RSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_32s16s_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_32s16s_C1RSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    /**
     * <pre>
     * Single channel 32-bit floating point to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f8u_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode)
    {
        return JNpp.checkResult(nppiConvert_32f8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode));
    }
    private static native int nppiConvert_32f8u_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode);


    /**
     * <pre>
     * Three channel 32-bit floating point to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f8u_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode)
    {
        return JNpp.checkResult(nppiConvert_32f8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode));
    }
    private static native int nppiConvert_32f8u_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode);


    /**
     * <pre>
     * Four channel 32-bit floating point to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f8u_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode)
    {
        return JNpp.checkResult(nppiConvert_32f8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode));
    }
    private static native int nppiConvert_32f8u_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode);


    /**
     * <pre>
     * Four channel 32-bit floating point to 8-bit unsigned conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f8u_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode)
    {
        return JNpp.checkResult(nppiConvert_32f8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode));
    }
    private static native int nppiConvert_32f8u_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode);


    public static int nppiConvert_32f8s_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode)
    {
        return JNpp.checkResult(nppiConvert_32f8s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode));
    }
    private static native int nppiConvert_32f8s_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode);


    /**
     * <pre>
     * Three channel 32-bit floating point to 8-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f8s_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode)
    {
        return JNpp.checkResult(nppiConvert_32f8s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode));
    }
    private static native int nppiConvert_32f8s_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode);


    /**
     * <pre>
     * Four channel 32-bit floating point to 8-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f8s_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode)
    {
        return JNpp.checkResult(nppiConvert_32f8s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode));
    }
    private static native int nppiConvert_32f8s_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode);


    /**
     * <pre>
     * Four channel 32-bit floating point to 8-bit signed conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f8s_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode)
    {
        return JNpp.checkResult(nppiConvert_32f8s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode));
    }
    private static native int nppiConvert_32f8s_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode);


    /**
     * <pre>
     * Single channel 32-bit floating point to 16-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f16u_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode)
    {
        return JNpp.checkResult(nppiConvert_32f16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode));
    }
    private static native int nppiConvert_32f16u_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode);


    /**
     * <pre>
     * Three channel 32-bit floating point to 16-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f16u_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode)
    {
        return JNpp.checkResult(nppiConvert_32f16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode));
    }
    private static native int nppiConvert_32f16u_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode);


    /**
     * <pre>
     * Four channel 32-bit floating point to 16-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f16u_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode)
    {
        return JNpp.checkResult(nppiConvert_32f16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode));
    }
    private static native int nppiConvert_32f16u_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode);


    /**
     * <pre>
     * Four channel 32-bit floating point to 16-bit unsigned conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f16u_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode)
    {
        return JNpp.checkResult(nppiConvert_32f16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode));
    }
    private static native int nppiConvert_32f16u_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode);


    /**
     * <pre>
     * Single channel 32-bit floating point to 16-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f16s_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode)
    {
        return JNpp.checkResult(nppiConvert_32f16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode));
    }
    private static native int nppiConvert_32f16s_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode);


    /**
     * <pre>
     * Three channel 32-bit floating point to 16-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f16s_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode)
    {
        return JNpp.checkResult(nppiConvert_32f16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode));
    }
    private static native int nppiConvert_32f16s_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode);


    /**
     * <pre>
     * Four channel 32-bit floating point to 16-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f16s_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode)
    {
        return JNpp.checkResult(nppiConvert_32f16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode));
    }
    private static native int nppiConvert_32f16s_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode);


    /**
     * <pre>
     * Four channel 32-bit floating point to 16-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f16s_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode)
    {
        return JNpp.checkResult(nppiConvert_32f16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode));
    }
    private static native int nppiConvert_32f16s_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode);


    /**
     * <pre>
     * Single channel 32-bit floating point to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f8u_C1RSfs(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_32f8u_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_32f8u_C1RSfsNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    /**
     * <pre>
     * Single channel 32-bit floating point to 8-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f8s_C1RSfs(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_32f8s_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_32f8s_C1RSfsNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    /**
     * <pre>
     * Single channel 32-bit floating point to 16-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f16u_C1RSfs(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_32f16u_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_32f16u_C1RSfsNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    /**
     * <pre>
     * Single channel 32-bit floating point to 16-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f16s_C1RSfs(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_32f16s_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_32f16s_C1RSfsNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    public static int nppiConvert_32f32u_C1RSfs(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_32f32u_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_32f32u_C1RSfsNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    /**
     * <pre>
     * Single channel 32-bit floating point to 32-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiConvert_32f32s_C1RSfs(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiConvert_32f32s_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor));
    }
    private static native int nppiConvert_32f32s_C1RSfsNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eRoundMode,
        int nScaleFactor);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 16-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_8u16u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiScale_8u16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiScale_8u16u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 8-bit unsigned to 16-bit unsigned  conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_8u16u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiScale_8u16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiScale_8u16u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiScale_8u16u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiScale_8u16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiScale_8u16u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned to 16-bit unsigned conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_8u16u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiScale_8u16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiScale_8u16u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 16-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiScale_8u16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiScale_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiScale_8u16s_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiScale_8u16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiScale_8u16s_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned to 16-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_8u16s_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiScale_8u16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiScale_8u16s_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned to 16-bit signed conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_8u16s_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiScale_8u16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiScale_8u16s_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 32-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_8u32s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiScale_8u32s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiScale_8u32s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 8-bit unsigned to 32-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_8u32s_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiScale_8u32s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiScale_8u32s_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned to 32-bit signed conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_8u32s_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiScale_8u32s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiScale_8u32s_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiScale_8u32s_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiScale_8u32s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiScale_8u32s_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 32-bit floating-point conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nMin specifies the minimum saturation value to which every output value will be clamped.
     * @param nMax specifies the maximum saturation value to which every output value will be clamped.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, ::NPP_SCALE_RANGE_ERROR indicates an error condition if nMax <= nMin.
     * </pre>
     */
    public static int nppiScale_8u32f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nMin,
        float nMax)
    {
        return JNpp.checkResult(nppiScale_8u32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nMin, nMax));
    }
    private static native int nppiScale_8u32f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nMin,
        float nMax);


    /**
     * <pre>
     * Three channel 8-bit unsigned to 32-bit floating-point conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nMin specifies the minimum saturation value to which every output value will be clamped.
     * @param nMax specifies the maximum saturation value to which every output value will be clamped.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, ::NPP_SCALE_RANGE_ERROR indicates an error condition if nMax <= nMin.
     * </pre>
     */
    public static int nppiScale_8u32f_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nMin,
        float nMax)
    {
        return JNpp.checkResult(nppiScale_8u32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nMin, nMax));
    }
    private static native int nppiScale_8u32f_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nMin,
        float nMax);


    /**
     * <pre>
     * Four channel 8-bit unsigned to 32-bit floating-point conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nMin specifies the minimum saturation value to which every output value will be clamped.
     * @param nMax specifies the maximum saturation value to which every output value will be clamped.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, ::NPP_SCALE_RANGE_ERROR indicates an error condition if nMax <= nMin.
     * </pre>
     */
    public static int nppiScale_8u32f_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nMin,
        float nMax)
    {
        return JNpp.checkResult(nppiScale_8u32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nMin, nMax));
    }
    private static native int nppiScale_8u32f_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nMin,
        float nMax);


    /**
     * <pre>
     * Four channel 8-bit unsigned to 32-bit floating-point conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nMin specifies the minimum saturation value to which every output value will be clamped.
     * @param nMax specifies the maximum saturation value to which every output value will be clamped.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, ::NPP_SCALE_RANGE_ERROR indicates an error condition if nMax <= nMin.
     * </pre>
     */
    public static int nppiScale_8u32f_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nMin,
        float nMax)
    {
        return JNpp.checkResult(nppiScale_8u32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nMin, nMax));
    }
    private static native int nppiScale_8u32f_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nMin,
        float nMax);


    /**
     * <pre>
     * Single channel 16-bit unsigned to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param hint algorithm performance or accuracy selector, currently ignored
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_16u8u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint)
    {
        return JNpp.checkResult(nppiScale_16u8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint));
    }
    private static native int nppiScale_16u8u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint);


    /**
     * <pre>
     * Three channel 16-bit unsigned to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param hint algorithm performance or accuracy selector, currently ignored
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_16u8u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint)
    {
        return JNpp.checkResult(nppiScale_16u8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint));
    }
    private static native int nppiScale_16u8u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint);


    /**
     * <pre>
     * Four channel 16-bit unsigned to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param hint algorithm performance or accuracy selector, currently ignored
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_16u8u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint)
    {
        return JNpp.checkResult(nppiScale_16u8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint));
    }
    private static native int nppiScale_16u8u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint);


    /**
     * <pre>
     * Four channel 16-bit unsigned to 8-bit unsigned conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param hint algorithm performance or accuracy selector, currently ignored
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_16u8u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint)
    {
        return JNpp.checkResult(nppiScale_16u8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint));
    }
    private static native int nppiScale_16u8u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint);


    /**
     * <pre>
     * Single channel 16-bit signed to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param hint algorithm performance or accuracy selector, currently ignored
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_16s8u_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint)
    {
        return JNpp.checkResult(nppiScale_16s8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint));
    }
    private static native int nppiScale_16s8u_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint);


    /**
     * <pre>
     * Three channel 16-bit signed to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param hint algorithm performance or accuracy selector, currently ignored
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_16s8u_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint)
    {
        return JNpp.checkResult(nppiScale_16s8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint));
    }
    private static native int nppiScale_16s8u_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint);


    /**
     * <pre>
     * Four channel 16-bit signed to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param hint algorithm performance or accuracy selector, currently ignored
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_16s8u_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint)
    {
        return JNpp.checkResult(nppiScale_16s8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint));
    }
    private static native int nppiScale_16s8u_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint);


    /**
     * <pre>
     * Four channel 16-bit signed to 8-bit unsigned conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param hint algorithm performance or accuracy selector, currently ignored
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_16s8u_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint)
    {
        return JNpp.checkResult(nppiScale_16s8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint));
    }
    private static native int nppiScale_16s8u_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint);


    /**
     * <pre>
     * Single channel 32-bit signed to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param hint algorithm performance or accuracy selector, currently ignored
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_32s8u_C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint)
    {
        return JNpp.checkResult(nppiScale_32s8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint));
    }
    private static native int nppiScale_32s8u_C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint);


    /**
     * <pre>
     * Three channel 32-bit signed to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param hint algorithm performance or accuracy selector, currently ignored
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_32s8u_C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint)
    {
        return JNpp.checkResult(nppiScale_32s8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint));
    }
    private static native int nppiScale_32s8u_C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint);


    /**
     * <pre>
     * Four channel 32-bit signed to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param hint algorithm performance or accuracy selector, currently ignored
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_32s8u_C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint)
    {
        return JNpp.checkResult(nppiScale_32s8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint));
    }
    private static native int nppiScale_32s8u_C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint);


    /**
     * <pre>
     * Four channel 32-bit signed to 8-bit unsigned conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param hint algorithm performance or accuracy selector, currently ignored
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiScale_32s8u_AC4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint)
    {
        return JNpp.checkResult(nppiScale_32s8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint));
    }
    private static native int nppiScale_32s8u_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int hint);


    public static int nppiScale_32f8u_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nMin,
        float nMax)
    {
        return JNpp.checkResult(nppiScale_32f8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nMin, nMax));
    }
    private static native int nppiScale_32f8u_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nMin,
        float nMax);


    /**
     * <pre>
     * Three channel 32-bit floating point to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nMin specifies the minimum saturation value to which every output value will be clamped.
     * @param nMax specifies the maximum saturation value to which every output value will be clamped.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, ::NPP_SCALE_RANGE_ERROR indicates an error condition if nMax <= nMin.
     * </pre>
     */
    public static int nppiScale_32f8u_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nMin,
        float nMax)
    {
        return JNpp.checkResult(nppiScale_32f8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nMin, nMax));
    }
    private static native int nppiScale_32f8u_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nMin,
        float nMax);


    /**
     * <pre>
     * Four channel 32-bit floating point to 8-bit unsigned conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nMin specifies the minimum saturation value to which every output value will be clamped.
     * @param nMax specifies the maximum saturation value to which every output value will be clamped.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, ::NPP_SCALE_RANGE_ERROR indicates an error condition if nMax <= nMin.
     * </pre>
     */
    public static int nppiScale_32f8u_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nMin,
        float nMax)
    {
        return JNpp.checkResult(nppiScale_32f8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nMin, nMax));
    }
    private static native int nppiScale_32f8u_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nMin,
        float nMax);


    /**
     * <pre>
     * Four channel 32-bit floating point to 8-bit unsigned conversion, not affecting Alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nMin specifies the minimum saturation value to which every output value will be clamped.
     * @param nMax specifies the maximum saturation value to which every output value will be clamped.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, ::NPP_SCALE_RANGE_ERROR indicates an error condition if nMax <= nMin.
     * </pre>
     */
    public static int nppiScale_32f8u_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nMin,
        float nMax)
    {
        return JNpp.checkResult(nppiScale_32f8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nMin, nMax));
    }
    private static native int nppiScale_32f8u_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nMin,
        float nMax);


    /**
     * <pre>
     * 1 channel 8-bit unsigned integer image copy with constant border color.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region of pixels.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image (inner part) and constant border color (outer part).
     * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
     *      destination ROI that will be filled with the constant border color.
     *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
     * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
     *      destination ROI is implicitly defined by the size of the source ROI:
     *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
     * @param nValue The pixel value to be set for border pixels.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        byte nValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_8u_C1RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, nValue));
    }
    private static native int nppiCopyConstBorder_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        byte nValue);


    /**
     * <pre>
     * 3 channel 8-bit unsigned integer image copy with constant border color.
     * See nppiCopyConstBorder_8u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @param aValue Vector of the RGBA values of the border pixels to be set.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        byte[] aValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_8u_C3RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue));
    }
    private static native int nppiCopyConstBorder_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        byte[] aValue);


    /**
     * <pre>
     * 4 channel 8-bit unsigned integer image copy with constant border color.
     * See nppiCopyConstBorder_8u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @param aValue Vector of the RGBA values of the border pixels to be set.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        byte[] aValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_8u_C4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue));
    }
    private static native int nppiCopyConstBorder_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        byte[] aValue);


    /**
     * <pre>
     * 4 channel 8-bit unsigned integer image copy with constant border color with alpha channel unaffected.
     * See nppiCopyConstBorder_8u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @param aValue Vector of the RGB values of the border pixels. Because this method does not
     *      affect the destination image's alpha channel, only three components of the border color
     *      are needed.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        byte[] aValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_8u_AC4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue));
    }
    private static native int nppiCopyConstBorder_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        byte[] aValue);


    /**
     * <pre>
     * 1 channel 16-bit unsigned integer image copy with constant border color.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region of pixels.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image (inner part) and constant border color (outer part).
     * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
     *      destination ROI that will be filled with the constant border color.
     *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
     * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
     *      destination ROI is implicitly defined by the size of the source ROI:
     *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
     * @param nValue The pixel value to be set for border pixels.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        short nValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_16u_C1RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, nValue));
    }
    private static native int nppiCopyConstBorder_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        short nValue);


    /**
     * <pre>
     * 3 channel 16-bit unsigned integer image copy with constant border color.
     * See nppiCopyConstBorder_16u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @param aValue Vector of the RGBA values of the border pixels to be set.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        short[] aValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_16u_C3RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue));
    }
    private static native int nppiCopyConstBorder_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        short[] aValue);


    /**
     * <pre>
     * 4 channel 16-bit unsigned integer image copy with constant border color.
     * See nppiCopyConstBorder_16u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @param aValue Vector of the RGBA values of the border pixels to be set.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        short[] aValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_16u_C4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue));
    }
    private static native int nppiCopyConstBorder_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        short[] aValue);


    /**
     * <pre>
     * 4 channel 16-bit unsigned integer image copy with constant border color with alpha channel unaffected.
     * See nppiCopyConstBorder_16u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @param aValue Vector of the RGB values of the border pixels. Because this method does not
     *      affect the destination image's alpha channel, only three components of the border color
     *      are needed.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        short[] aValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_16u_AC4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue));
    }
    private static native int nppiCopyConstBorder_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        short[] aValue);


    /**
     * <pre>
     * 1 channel 16-bit signed integer image copy with constant border color.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region of pixels.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image (inner part) and constant border color (outer part).
     * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
     *      destination ROI that will be filled with the constant border color.
     *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
     * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
     *      destination ROI is implicitly defined by the size of the source ROI:
     *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
     * @param nValue The pixel value to be set for border pixels.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        short nValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_16s_C1RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, nValue));
    }
    private static native int nppiCopyConstBorder_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        short nValue);


    /**
     * <pre>
     * 3 channel 16-bit signed integer image copy with constant border color.
     * See nppiCopyConstBorder_16s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @param aValue Vector of the RGBA values of the border pixels to be set.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        short[] aValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_16s_C3RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue));
    }
    private static native int nppiCopyConstBorder_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        short[] aValue);


    /**
     * <pre>
     * 4 channel 16-bit signed integer image copy with constant border color.
     * See nppiCopyConstBorder_16s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @param aValue Vector of the RGBA values of the border pixels to be set.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        short[] aValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_16s_C4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue));
    }
    private static native int nppiCopyConstBorder_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        short[] aValue);


    /**
     * <pre>
     * 4 channel 16-bit signed integer image copy with constant border color with alpha channel unaffected.
     * See nppiCopyConstBorder_16s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @param aValue Vector of the RGB values of the border pixels. Because this method does not
     *      affect the destination image's alpha channel, only three components of the border color
     *      are needed.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        short[] aValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_16s_AC4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue));
    }
    private static native int nppiCopyConstBorder_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        short[] aValue);


    /**
     * <pre>
     * 1 channel 32-bit signed integer image copy with constant border color.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region of pixels.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image (inner part) and constant border color (outer part).
     * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
     *      destination ROI that will be filled with the constant border color.
     *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
     * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
     *      destination ROI is implicitly defined by the size of the source ROI:
     *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
     * @param nValue The pixel value to be set for border pixels.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_32s_C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        int nValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_32s_C1RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, nValue));
    }
    private static native int nppiCopyConstBorder_32s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        int nValue);


    /**
     * <pre>
     * 3 channel 32-bit signed integer image copy with constant border color.
     * See nppiCopyConstBorder_32s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @param aValue Vector of the RGBA values of the border pixels to be set.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_32s_C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        int[] aValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_32s_C3RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue));
    }
    private static native int nppiCopyConstBorder_32s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        int[] aValue);


    /**
     * <pre>
     * 4 channel 32-bit signed integer image copy with constant border color.
     * See nppiCopyConstBorder_32s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @param aValue Vector of the RGBA values of the border pixels to be set.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_32s_C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        int[] aValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_32s_C4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue));
    }
    private static native int nppiCopyConstBorder_32s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        int[] aValue);


    /**
     * <pre>
     * 4 channel 32-bit signed integer image copy with constant border color with alpha channel unaffected.
     * See nppiCopyConstBorder_32s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @param aValue Vector of the RGB values of the border pixels. Because this method does not
     *      affect the destination image's alpha channel, only three components of the border color
     *      are needed.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_32s_AC4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        int[] aValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_32s_AC4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue));
    }
    private static native int nppiCopyConstBorder_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        int[] aValue);


    /**
     * <pre>
     * 1 channel 32-bit floating point image copy with constant border color.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region of pixels.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image (inner part) and constant border color (outer part).
     * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
     *      destination ROI that will be filled with the constant border color.
     *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
     * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
     *      destination ROI is implicitly defined by the size of the source ROI:
     *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
     * @param nValue The pixel value to be set for border pixels.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        float nValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, nValue));
    }
    private static native int nppiCopyConstBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        float nValue);


    public static int nppiCopyConstBorder_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        float[] aValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_32f_C3RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue));
    }
    private static native int nppiCopyConstBorder_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        float[] aValue);


    public static int nppiCopyConstBorder_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        float[] aValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_32f_C4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue));
    }
    private static native int nppiCopyConstBorder_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        float[] aValue);


    /**
     * <pre>
     * 4 channel 32-bit floating point image copy with constant border color with alpha channel unaffected.
     * See nppiCopyConstBorder_32f_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @param aValue Vector of the RGB values of the border pixels. Because this method does not
     *      affect the destination image's alpha channel, only three components of the border color
     *      are needed.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyConstBorder_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        float[] aValue)
    {
        return JNpp.checkResult(nppiCopyConstBorder_32f_AC4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue));
    }
    private static native int nppiCopyConstBorder_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth,
        float[] aValue);


    /**
     * <pre>
     * 1 channel 8-bit unsigned integer image copy with nearest source image pixel color.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region of pixels.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image (inner part) and nearest source image pixel color (outer part).
     * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
     *      destination ROI that will be filled with the nearest source image pixel color.
     *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
     * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
     *      destination ROI is implicitly defined by the size of the source ROI:
     *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_8u_C1RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 3 channel 8-bit unsigned integer image copy with nearest source image pixel color.
     * See nppiCopyReplicateBorder_8u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_8u_C3RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 8-bit unsigned integer image copy with nearest source image pixel color.
     * See nppiCopyReplicateBorder_8u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_8u_C4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 8-bit unsigned integer image copy with nearest source image pixel color with alpha channel unaffected.
     * See nppiCopyReplicateBorder_8u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_8u_AC4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    public static int nppiCopyReplicateBorder_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_16u_C1RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 3 channel 16-bit unsigned integer image copy with nearest source image pixel color.
     * See nppiCopyReplicateBorder_16u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_16u_C3RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 16-bit unsigned integer image copy with nearest source image pixel color.
     * See nppiCopyReplicateBorder_16u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_16u_C4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 16-bit unsigned image copy with nearest source image pixel color with alpha channel unaffected.
     * See nppiCopyReplicateBorder_16u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_16u_AC4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 1 channel 16-bit signed integer image copy with nearest source image pixel color.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region of pixels.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image (inner part) and nearest source image pixel color (outer part).
     * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
     *      destination ROI that will be filled with the nearest source image pixel color.
     *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
     * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
     *      destination ROI is implicitly defined by the size of the source ROI:
     *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_16s_C1RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 3 channel 16-bit signed integer image copy with nearest source image pixel color.
     * See nppiCopyReplicateBorder_16s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_16s_C3RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 16-bit signed integer image copy with nearest source image pixel color.
     * See nppiCopyReplicateBorder_16s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_16s_C4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 16-bit signed integer image copy with nearest source image pixel color with alpha channel unaffected.
     * See nppiCopyReplicateBorder_16s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_16s_AC4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 1 channel 32-bit signed integer image copy with nearest source image pixel color.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region of pixels.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image (inner part) and nearest source image pixel color (outer part).
     * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
     *      destination ROI that will be filled with the nearest source image pixel color.
     *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
     * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
     *      destination ROI is implicitly defined by the size of the source ROI:
     *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_32s_C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_32s_C1RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_32s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 3 channel 32-bit signed image copy with nearest source image pixel color.
     * See nppiCopyReplicateBorder_32s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_32s_C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_32s_C3RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_32s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 32-bit signed integer image copy with nearest source image pixel color.
     * See nppiCopyReplicateBorder_32s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_32s_C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_32s_C4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_32s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 32-bit signed integer image copy with nearest source image pixel color with alpha channel unaffected.
     * See nppiCopyReplicateBorder_32s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_32s_AC4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_32s_AC4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 1 channel 32-bit floating point image copy with nearest source image pixel color.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region of pixels.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image (inner part) and nearest source image pixel color (outer part).
     * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
     *      destination ROI that will be filled with the nearest source image pixel color.
     *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
     * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
     *      destination ROI is implicitly defined by the size of the source ROI:
     *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 3 channel 32-bit floating point image copy with nearest source image pixel color.
     * See nppiCopyReplicateBorder_32f_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_32f_C3RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 32-bit floating point image copy with nearest source image pixel color.
     * See nppiCopyReplicateBorder_32f_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_32f_C4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 32-bit floating point image copy with nearest source image pixel color with alpha channel unaffected.
     * See nppiCopyReplicateBorder_32f_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyReplicateBorder_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyReplicateBorder_32f_AC4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyReplicateBorder_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 1 channel 8-bit unsigned integer image copy with the borders wrapped by replication of source image pixel colors.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region of pixels.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image (inner part) and a border consisting of wrapped replication of the source image pixel colors (outer part).
     * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
     *      destination ROI that will be filled with the wrapped replication of the corresponding column of source image pixels colors.
     *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
     * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
     *      destination ROI is implicitly defined by the size of the source ROI:
     *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyWrapBorder_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_8u_C1RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    public static int nppiCopyWrapBorder_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_8u_C3RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 8-bit unsigned integer image copy with the borders wrapped by replication of source image pixel colors.
     * See nppiCopyWrapBorder_8u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyWrapBorder_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_8u_C4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 8-bit unsigned integer image copy with the borders wrapped by replication of source image pixel colors with alpha channel unaffected.
     * See nppiCopyWrapBorder_8u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyWrapBorder_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_8u_AC4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 1 channel 16-bit unsigned integer image copy with the borders wrapped by replication of source image pixel colors.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region of pixels.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image (inner part) and a border consisting of wrapped replication of the source image pixel colors (outer part).
     * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
     *      destination ROI that will be filled with the wrapped replication of the corresponding column of source image pixels colors.
     *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
     * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
     *      destination ROI is implicitly defined by the size of the source ROI:
     *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyWrapBorder_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_16u_C1RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 3 channel 16-bit unsigned integer image copy with the borders wrapped by replication of source image pixel colors.
     * See nppiCopyWrapBorder_16u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyWrapBorder_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_16u_C3RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 16-bit unsigned integer image copy with the borders wrapped by replication of source image pixel colors.
     * See nppiCopyWrapBorder_16u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyWrapBorder_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_16u_C4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 16-bit unsigned integer image copy with the borders wrapped by replication of source image pixel colors with alpha channel unaffected.
     * See nppiCopyWrapBorder_16u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyWrapBorder_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_16u_AC4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 1 channel 16-bit signed integer image copy with the borders wrapped by replication of source image pixel colors.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region of pixels.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image (inner part) and a border consisting of wrapped replication of the source image pixel colors (outer part).
     * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
     *      destination ROI that will be filled with the wrapped replication of the corresponding column of source image pixels colors.
     *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
     * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
     *      destination ROI is implicitly defined by the size of the source ROI:
     *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyWrapBorder_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_16s_C1RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 3 channel 16-bit signed integer image copy with the borders wrapped by replication of source image pixel colors.
     * See nppiCopyWrapBorder_16s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyWrapBorder_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_16s_C3RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    public static int nppiCopyWrapBorder_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_16s_C4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 16-bit signed integer image copy with the borders wrapped by replication of source image pixel colors with alpha channel unaffected.
     * See nppiCopyWrapBorder_16s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyWrapBorder_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_16s_AC4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    public static int nppiCopyWrapBorder_32s_C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_32s_C1RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_32s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 3 channel 32-bit signed integer image copy with the borders wrapped by replication of source image pixel colors.
     * See nppiCopyWrapBorder_32s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyWrapBorder_32s_C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_32s_C3RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_32s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 32-bit signed integer image copy with the borders wrapped by replication of source image pixel colors.
     * See nppiCopyWrapBorder_32s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyWrapBorder_32s_C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_32s_C4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_32s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    public static int nppiCopyWrapBorder_32s_AC4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_32s_AC4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 1 channel 32-bit floating point image copy with the borders wrapped by replication of source image pixel colors.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region of pixels.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image (inner part) and a border consisting of wrapped replication of the source image pixel colors (outer part).
     * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
     *      destination ROI that will be filled with the wrapped replication of the corresponding column of source image pixels colors.
     *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
     * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
     *      destination ROI is implicitly defined by the size of the source ROI:
     *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyWrapBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 3 channel 32-bit floating point image copy with the borders wrapped by replication of source image pixel colors.
     * See nppiCopyWrapBorder_32f_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyWrapBorder_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_32f_C3RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 4 channel 32-bit floating point image copy with the borders wrapped by replication of source image pixel colors.
     * See nppiCopyWrapBorder_32f_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyWrapBorder_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_32f_C4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 1 channel 32-bit floating point image copy with the borders wrapped by replication of source image pixel colors with alpha channel unaffected.
     * See nppiCopyWrapBorder_32f_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSizeROI Size of the source region-of-interest.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nTopBorderHeight Height of top border.
     * @param nLeftBorderWidth Width of left border.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopyWrapBorder_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth)
    {
        return JNpp.checkResult(nppiCopyWrapBorder_32f_AC4RNative(pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth));
    }
    private static native int nppiCopyWrapBorder_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSizeROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int nTopBorderHeight,
        int nLeftBorderWidth);


    /**
     * <pre>
     * 1 channel 8-bit unsigned integer linearly interpolated source image subpixel coordinate color copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image, source image ROI is assumed to be same as destination image ROI.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 3 channel 8-bit unsigned integer linearly interpolated source image subpixel coordinate color copy.
     * See nppiCopySubpix_8u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 4 channel 8-bit unsigned integer linearly interpolated source image subpixel coordinate color copy.
     * See nppiCopySubpix_8u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 4 channel 8-bit unsigned integer linearly interpolated source image subpixel coordinate color copy with alpha channel unaffected.
     * See nppiCopySubpix_8u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 1 channel 16-bit unsigned integer linearly interpolated source image subpixel coordinate color copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image, source image ROI is assumed to be same as destination image ROI.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 3 channel 16-bit unsigned integer linearly interpolated source image subpixel coordinate color copy.
     * See nppiCopySubpix_16u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    public static int nppiCopySubpix_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 4 channel 16-bit unsigned linearly interpolated source image subpixel coordinate color copy with alpha channel unaffected.
     * See nppiCopySubpix_16u_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 1 channel 16-bit signed integer linearly interpolated source image subpixel coordinate color copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image, source image ROI is assumed to be same as destination image ROI.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 3 channel 16-bit signed integer linearly interpolated source image subpixel coordinate color copy.
     * See nppiCopySubpix_16s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 4 channel 16-bit signed integer linearly interpolated source image subpixel coordinate color copy.
     * See nppiCopySubpix_16s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 4 channel 16-bit signed integer linearly interpolated source image subpixel coordinate color copy with alpha channel unaffected.
     * See nppiCopySubpix_16s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 1 channel 32-bit signed integer linearly interpolated source image subpixel coordinate color copy.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image, source image ROI is assumed to be same as destination image ROI.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_32s_C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_32s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_32s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 3 channel 32-bit signed linearly interpolated source image subpixel coordinate color copy.
     * See nppiCopySubpix_32s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_32s_C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_32s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_32s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 4 channel 32-bit signed integer linearly interpolated source image subpixel coordinate color copy.
     * See nppiCopySubpix_32s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_32s_C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_32s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_32s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 4 channel 32-bit signed integer linearly interpolated source image subpixel coordinate color copy with alpha channel unaffected.
     * See nppiCopySubpix_32s_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_32s_AC4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_32s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    public static int nppiCopySubpix_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 3 channel 32-bit floating point linearly interpolated source image subpixel coordinate color copy.
     * See nppiCopySubpix_32f_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 4 channel 32-bit floating point linearly interpolated source image subpixel coordinate color copy.
     * See nppiCopySubpix_32f_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 4 channel 32-bit floating point linearly interpolated source image subpixel coordinate color copy with alpha channel unaffected.
     * See nppiCopySubpix_32f_C1R() for detailed documentation.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @param nDx Fractional part of source image X coordinate.
     * @param nDy Fractional part of source image Y coordinate.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCopySubpix_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy)
    {
        return JNpp.checkResult(nppiCopySubpix_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy));
    }
    private static native int nppiCopySubpix_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        float nDx,
        float nDy);


    /**
     * <pre>
     * 1 channel 8-bit unsigned integer source image duplicated in all 3 channels of destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image, source image ROI is assumed to be same as destination image ROI.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDup_8u_C1C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI)
    {
        return JNpp.checkResult(nppiDup_8u_C1C3RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI));
    }
    private static native int nppiDup_8u_C1C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI);


    /**
     * <pre>
     * 1 channel 8-bit unsigned integer source image duplicated in all 4 channels of destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDup_8u_C1C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI)
    {
        return JNpp.checkResult(nppiDup_8u_C1C4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI));
    }
    private static native int nppiDup_8u_C1C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI);


    /**
     * <pre>
     * 1 channel 8-bit unsigned integer source image duplicated in 3 channels of 4 channel destination image with alpha channel unaffected.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDup_8u_C1AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI)
    {
        return JNpp.checkResult(nppiDup_8u_C1AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI));
    }
    private static native int nppiDup_8u_C1AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI);


    /**
     * <pre>
     * 1 channel 16-bit unsigned integer source image duplicated in all 3 channels of destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image, source image ROI is assumed to be same as destination image ROI.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDup_16u_C1C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI)
    {
        return JNpp.checkResult(nppiDup_16u_C1C3RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI));
    }
    private static native int nppiDup_16u_C1C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI);


    /**
     * <pre>
     * 1 channel 16-bit unsigned integer source image duplicated in all 4 channels of destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDup_16u_C1C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI)
    {
        return JNpp.checkResult(nppiDup_16u_C1C4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI));
    }
    private static native int nppiDup_16u_C1C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI);


    /**
     * <pre>
     * 1 channel 16-bit unsigned integer source image duplicated in 3 channels of 4 channel destination image with alpha channel unaffected.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDup_16u_C1AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI)
    {
        return JNpp.checkResult(nppiDup_16u_C1AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI));
    }
    private static native int nppiDup_16u_C1AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI);


    /**
     * <pre>
     * 1 channel 16-bit signed integer source image duplicated in all 3 channels of destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image, source image ROI is assumed to be same as destination image ROI.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDup_16s_C1C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI)
    {
        return JNpp.checkResult(nppiDup_16s_C1C3RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI));
    }
    private static native int nppiDup_16s_C1C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI);


    /**
     * <pre>
     * 1 channel 16-bit signed integer source image duplicated in all 4 channels of destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDup_16s_C1C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI)
    {
        return JNpp.checkResult(nppiDup_16s_C1C4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI));
    }
    private static native int nppiDup_16s_C1C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI);


    /**
     * <pre>
     * 1 channel 16-bit signed integer source image duplicated in 3 channels of 4 channel destination image with alpha channel unaffected.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDup_16s_C1AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI)
    {
        return JNpp.checkResult(nppiDup_16s_C1AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI));
    }
    private static native int nppiDup_16s_C1AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI);


    /**
     * <pre>
     * 1 channel 32-bit signed integer source image duplicated in all 3 channels of destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image, source image ROI is assumed to be same as destination image ROI.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDup_32s_C1C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI)
    {
        return JNpp.checkResult(nppiDup_32s_C1C3RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI));
    }
    private static native int nppiDup_32s_C1C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI);


    /**
     * <pre>
     * 1 channel 32-bit signed integer source image duplicated in all 4 channels of destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDup_32s_C1C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI)
    {
        return JNpp.checkResult(nppiDup_32s_C1C4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI));
    }
    private static native int nppiDup_32s_C1C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI);


    /**
     * <pre>
     * 1 channel 32-bit signed integer source image duplicated in 3 channels of 4 channel destination image with alpha channel unaffected.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDup_32s_C1AC4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI)
    {
        return JNpp.checkResult(nppiDup_32s_C1AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI));
    }
    private static native int nppiDup_32s_C1AC4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI);


    /**
     * <pre>
     * 1 channel 32-bit floating point source image duplicated in all 3 channels of destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
     *      data from the source image, source image ROI is assumed to be same as destination image ROI.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDup_32f_C1C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI)
    {
        return JNpp.checkResult(nppiDup_32f_C1C3RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI));
    }
    private static native int nppiDup_32f_C1C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI);


    /**
     * <pre>
     * 1 channel 32-bit floating point source image duplicated in all 4 channels of destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDup_32f_C1C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI)
    {
        return JNpp.checkResult(nppiDup_32f_C1C4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI));
    }
    private static native int nppiDup_32f_C1C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI);


    /**
     * <pre>
     * 1 channel 32-bit floating point source image duplicated in 3 channels of 4 channel destination image with alpha channel unaffected.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Size of the destination region-of-interest.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDup_32f_C1AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI)
    {
        return JNpp.checkResult(nppiDup_32f_C1AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI));
    }
    private static native int nppiDup_32f_C1AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI);


    /**
     * <pre>
     * 1 channel 8-bit unsigned int image transpose.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst Pointer to the destination ROI.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     *
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiTranspose_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSrcROI)
    {
        return JNpp.checkResult(nppiTranspose_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSrcROI));
    }
    private static native int nppiTranspose_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSrcROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned int image transpose.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst Pointer to the destination ROI.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     *
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiTranspose_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSrcROI)
    {
        return JNpp.checkResult(nppiTranspose_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSrcROI));
    }
    private static native int nppiTranspose_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSrcROI);


    public static int nppiTranspose_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSrcROI)
    {
        return JNpp.checkResult(nppiTranspose_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSrcROI));
    }
    private static native int nppiTranspose_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSrcROI);


    /**
     * <pre>
     * 1 channel 16-bit unsigned int image transpose.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst Pointer to the destination ROI.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     *
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiTranspose_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSrcROI)
    {
        return JNpp.checkResult(nppiTranspose_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSrcROI));
    }
    private static native int nppiTranspose_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSrcROI);


    /**
     * <pre>
     * 3 channel 16-bit unsigned int image transpose.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst Pointer to the destination ROI.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     *
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiTranspose_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSrcROI)
    {
        return JNpp.checkResult(nppiTranspose_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSrcROI));
    }
    private static native int nppiTranspose_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSrcROI);


    /**
     * <pre>
     * 4 channel 16-bit unsigned int image transpose.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst Pointer to the destination ROI.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     *
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiTranspose_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSrcROI)
    {
        return JNpp.checkResult(nppiTranspose_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSrcROI));
    }
    private static native int nppiTranspose_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSrcROI);


    /**
     * <pre>
     * 1 channel 16-bit signed int image transpose.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst Pointer to the destination ROI.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     *
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiTranspose_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSrcROI)
    {
        return JNpp.checkResult(nppiTranspose_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSrcROI));
    }
    private static native int nppiTranspose_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSrcROI);


    /**
     * <pre>
     * 3 channel 16-bit signed int image transpose.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst Pointer to the destination ROI.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     *
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiTranspose_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSrcROI)
    {
        return JNpp.checkResult(nppiTranspose_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSrcROI));
    }
    private static native int nppiTranspose_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSrcROI);


    /**
     * <pre>
     * 4 channel 16-bit signed int image transpose.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst Pointer to the destination ROI.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     *
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiTranspose_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSrcROI)
    {
        return JNpp.checkResult(nppiTranspose_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSrcROI));
    }
    private static native int nppiTranspose_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSrcROI);


    /**
     * <pre>
     * 1 channel 32-bit signed int image transpose.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst Pointer to the destination ROI.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     *
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiTranspose_32s_C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSrcROI)
    {
        return JNpp.checkResult(nppiTranspose_32s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSrcROI));
    }
    private static native int nppiTranspose_32s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSrcROI);


    /**
     * <pre>
     * 3 channel 32-bit signed int image transpose.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst Pointer to the destination ROI.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     *
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiTranspose_32s_C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSrcROI)
    {
        return JNpp.checkResult(nppiTranspose_32s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSrcROI));
    }
    private static native int nppiTranspose_32s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSrcROI);


    /**
     * <pre>
     * 4 channel 32-bit signed int image transpose.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst Pointer to the destination ROI.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     *
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiTranspose_32s_C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSrcROI)
    {
        return JNpp.checkResult(nppiTranspose_32s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSrcROI));
    }
    private static native int nppiTranspose_32s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSrcROI);


    /**
     * <pre>
     * 1 channel 32-bit floating point image transpose.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst Pointer to the destination ROI.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     *
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiTranspose_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSrcROI)
    {
        return JNpp.checkResult(nppiTranspose_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSrcROI));
    }
    private static native int nppiTranspose_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSrcROI);


    /**
     * <pre>
     * 3 channel 32-bit floating point image transpose.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst Pointer to the destination ROI.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     *
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiTranspose_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSrcROI)
    {
        return JNpp.checkResult(nppiTranspose_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSrcROI));
    }
    private static native int nppiTranspose_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSrcROI);


    /**
     * <pre>
     * 4 channel 32-bit floating point image transpose.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst Pointer to the destination ROI.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     *
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiTranspose_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSrcROI)
    {
        return JNpp.checkResult(nppiTranspose_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSrcROI));
    }
    private static native int nppiTranspose_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSrcROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned integer source image to 3 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 3 channel 8-bit unsigned integer in place image.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_8u_C3IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_8u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_8u_C3IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 4 channel 8-bit unsigned integer source image to 3 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to a 3 channel BGR
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_8u_C4C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_8u_C4C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_8u_C4C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 4 channel 8-bit unsigned integer source image to 4 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 4 channel 8-bit unsigned integer in place image.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_8u_C4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_8u_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_8u_C4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 3 channel 8-bit unsigned integer source image to 4 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGB image, aDstOrder = [3,2,1,0] converts this to VBGR
     *      channel order.
     * @param nValue (V) Single channel constant value that can be replicated in one or more of the 4 destination channels.
     *      nValue is either written or not written to a particular channel depending on the aDstOrder entry for that destination
     *      channel. An aDstOrder value of 3 will output nValue to that channel, an aDstOrder value greater than 3 will leave that
     *      particular destination channel value unmodified.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_8u_C3C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder,
        byte nValue)
    {
        return JNpp.checkResult(nppiSwapChannels_8u_C3C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder, nValue));
    }
    private static native int nppiSwapChannels_8u_C3C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder,
        byte nValue);


    /**
     * <pre>
     * 4 channel 8-bit unsigned integer source image to 4 channel destination image with destination alpha channel unaffected.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGB image, aDstOrder = [3,2,1,0] converts this to VBGR
     *      channel order.
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to BGRA
     *      channel order. In the AC4R case, the alpha channel is always assumed to be channel 3.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 3 channel 16-bit unsigned integer source image to 3 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    public static int nppiSwapChannels_16u_C3IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_16u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_16u_C3IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 4 channel 16-bit unsigned integer source image to 3 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to a 3 channel BGR
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_16u_C4C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_16u_C4C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_16u_C4C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 4 channel 16-bit unsigned integer source image to 4 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 4 channel 16-bit unsigned integer in place image.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_16u_C4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_16u_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_16u_C4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 3 channel 16-bit unsigned integer source image to 4 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGB image, aDstOrder = [3,2,1,0] converts this to VBGR
     *      channel order.
     * @param nValue (V) Single channel constant value that can be replicated in one or more of the 4 destination channels.
     *      nValue is either written or not written to a particular channel depending on the aDstOrder entry for that destination
     *      channel. An aDstOrder value of 3 will output nValue to that channel, an aDstOrder value greater than 3 will leave that
     *      particular destination channel value unmodified.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_16u_C3C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder,
        short nValue)
    {
        return JNpp.checkResult(nppiSwapChannels_16u_C3C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder, nValue));
    }
    private static native int nppiSwapChannels_16u_C3C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder,
        short nValue);


    /**
     * <pre>
     * 4 channel 16-bit unsigned integer source image to 4 channel destination image with destination alpha channel unaffected.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to BGRA
     *      channel order. In the AC4R case, the alpha channel is always assumed to be channel 3.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 3 channel 16-bit signed integer source image to 3 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 3 channel 16-bit signed integer in place image.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_16s_C3IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_16s_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_16s_C3IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    public static int nppiSwapChannels_16s_C4C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_16s_C4C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_16s_C4C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 4 channel 16-bit signed integer source image to 4 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    public static int nppiSwapChannels_16s_C4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_16s_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_16s_C4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 3 channel 16-bit signed integer source image to 4 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGB image, aDstOrder = [3,2,1,0] converts this to VBGR
     *      channel order.
     * @param nValue (V) Single channel constant value that can be replicated in one or more of the 4 destination channels.
     *      nValue is either written or not written to a particular channel depending on the aDstOrder entry for that destination
     *      channel. An aDstOrder value of 3 will output nValue to that channel, an aDstOrder value greater than 3 will leave that
     *      particular destination channel value unmodified.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_16s_C3C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder,
        short nValue)
    {
        return JNpp.checkResult(nppiSwapChannels_16s_C3C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder, nValue));
    }
    private static native int nppiSwapChannels_16s_C3C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder,
        short nValue);


    /**
     * <pre>
     * 4 channel 16-bit signed integer source image to 4 channel destination image with destination alpha channel unaffected.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to BGRA
     *      channel order. In the AC4R case, the alpha channel is always assumed to be channel 3.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 3 channel 32-bit signed integer source image to 3 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_32s_C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_32s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_32s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 3 channel 32-bit signed integer in place image.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_32s_C3IR(
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_32s_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_32s_C3IRNative(
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 4 channel 32-bit signed integer source image to 3 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to a 3 channel BGR
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_32s_C4C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_32s_C4C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_32s_C4C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 4 channel 32-bit signed integer source image to 4 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_32s_C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_32s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_32s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 4 channel 32-bit signed integer in place image.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_32s_C4IR(
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_32s_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_32s_C4IRNative(
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 3 channel 32-bit signed integer source image to 4 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGB image, aDstOrder = [3,2,1,0] converts this to VBGR
     *      channel order.
     * @param nValue (V) Single channel constant value that can be replicated in one or more of the 4 destination channels.
     *      nValue is either written or not written to a particular channel depending on the aDstOrder entry for that destination
     *      channel. An aDstOrder value of 3 will output nValue to that channel, an aDstOrder value greater than 3 will leave that
     *      particular destination channel value unmodified.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_32s_C3C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder,
        int nValue)
    {
        return JNpp.checkResult(nppiSwapChannels_32s_C3C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder, nValue));
    }
    private static native int nppiSwapChannels_32s_C3C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder,
        int nValue);


    /**
     * <pre>
     * 4 channel 32-bit signed integer source image to 4 channel destination image with destination alpha channel unaffected.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to BGRA
     *      channel order. In the AC4R case, the alpha channel is always assumed to be channel 3.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_32s_AC4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_32s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 3 channel 32-bit floating point source image to 3 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 3 channel 32-bit floating point in place image.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 4 channel 32-bit floating point source image to 3 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to a 3 channel BGR
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_32f_C4C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_32f_C4C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_32f_C4C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 4 channel 32-bit floating point source image to 4 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 4 channel 32-bit floating point in place image.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
     *      channel order.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_32f_C4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_32f_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_32f_C4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    /**
     * <pre>
     * 3 channel 32-bit floating point source image to 4 channel destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGB image, aDstOrder = [3,2,1,0] converts this to VBGR
     *      channel order.
     * @param nValue (V) Single channel constant value that can be replicated in one or more of the 4 destination channels.
     *      nValue is either written or not written to a particular channel depending on the aDstOrder entry for that destination
     *      channel. An aDstOrder value of 3 will output nValue to that channel, an aDstOrder value greater than 3 will leave that
     *      particular destination channel value unmodified.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_32f_C3C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder,
        float nValue)
    {
        return JNpp.checkResult(nppiSwapChannels_32f_C3C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder, nValue));
    }
    private static native int nppiSwapChannels_32f_C3C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder,
        float nValue);


    /**
     * <pre>
     * 4 channel 32-bit floating point source image to 4 channel destination image with destination alpha channel unaffected.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
     *      of the array contains the number of the channel that is stored in the n-th channel of
     *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to BGRA
     *      channel order. In the AC4R case, the alpha channel is always assumed to be channel 3.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSwapChannels_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder)
    {
        return JNpp.checkResult(nppiSwapChannels_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder));
    }
    private static native int nppiSwapChannels_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int[] aDstOrder);


    public static int nppiAddC_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_8u_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAddC_8u_C1IRSfs(
        byte nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_8u_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_8u_C1IRSfsNative(
        byte nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image add constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel..
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_8u_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel 8-bit unsigned char in place image add constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel..
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_8u_C3IRSfs(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_8u_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_8u_C3IRSfsNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha image add constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel..
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_8u_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha in place image add constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel..
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_8u_AC4IRSfs(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_8u_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_8u_AC4IRSfsNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAddC_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_8u_C4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image add constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_8u_C4IRSfs(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_8u_C4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_8u_C4IRSfsNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel image add constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_16u_C1RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16u_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16u_C1RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAddC_16u_C1IRSfs(
        short nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16u_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16u_C1IRSfsNative(
        short nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image add constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_16u_C3RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16u_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16u_C3RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAddC_16u_C3IRSfs(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16u_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16u_C3IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAddC_16u_AC4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16u_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16u_AC4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAddC_16u_AC4IRSfs(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16u_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16u_AC4IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAddC_16u_C4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16u_C4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16u_C4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image add constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_16u_C4IRSfs(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16u_C4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16u_C4IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAddC_16s_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16s_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16s_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image add constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_16s_C1IRSfs(
        short nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16s_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16s_C1IRSfsNative(
        short nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel image add constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_16s_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16s_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16s_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel in place image add constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_16s_C3IRSfs(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16s_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16s_C3IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel with unmodified alpha image add constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_16s_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16s_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16s_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel with unmodified alpha in place image add constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_16s_AC4IRSfs(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16s_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16s_AC4IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel image add constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_16s_C4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16s_C4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16s_C4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel in place image add constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_16s_C4IRSfs(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16s_C4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16s_C4IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image add constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_16sc_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16sc_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16sc_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image add constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_16sc_C1IRSfs(
        Npp16sc nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16sc_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16sc_C1IRSfsNative(
        Npp16sc nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAddC_16sc_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16sc_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16sc_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image add constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_16sc_C3IRSfs(
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16sc_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16sc_C3IRSfsNative(
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image add constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_16sc_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16sc_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16sc_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAddC_16sc_AC4IRSfs(
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_16sc_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_16sc_AC4IRSfsNative(
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer channel image add constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32s_C1RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_32s_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_32s_C1RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image add constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32s_C1IRSfs(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_32s_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_32s_C1IRSfsNative(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed integer channel image add constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32s_C3RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_32s_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_32s_C3RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed integer channel in place image add constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32s_C3IRSfs(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_32s_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_32s_C3IRSfsNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image add constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32sc_C1RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_32sc_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_32sc_C1RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image add constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32sc_C1IRSfs(
        Npp32sc nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_32sc_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_32sc_C1IRSfsNative(
        Npp32sc nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image add constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32sc_C3RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_32sc_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_32sc_C3RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAddC_32sc_C3IRSfs(
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_32sc_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_32sc_C3IRSfsNative(
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAddC_32sc_AC4RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_32sc_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_32sc_AC4RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image add constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32sc_AC4IRSfs(
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAddC_32sc_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAddC_32sc_AC4IRSfsNative(
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit floating point channel image add constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float nConstant,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddC_32f_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAddC_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float nConstant,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel in place image add constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32f_C1IR(
        float nConstant,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddC_32f_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddC_32f_C1IRNative(
        float nConstant,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiAddC_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddC_32f_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAddC_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel in place image add constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32f_C3IR(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddC_32f_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddC_32f_C3IRNative(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel with unmodified alpha image add constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddC_32f_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAddC_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel with unmodified alpha in place image add constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32f_AC4IR(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddC_32f_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddC_32f_AC4IRNative(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiAddC_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddC_32f_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAddC_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel in place image add constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32f_C4IR(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddC_32f_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddC_32f_C4IRNative(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image add constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32fc_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc nConstant,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddC_32fc_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAddC_32fc_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc nConstant,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image add constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32fc_C1IR(
        Npp32fc nConstant,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddC_32fc_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddC_32fc_C1IRNative(
        Npp32fc nConstant,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image add constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32fc_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddC_32fc_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAddC_32fc_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiAddC_32fc_C3IR(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddC_32fc_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddC_32fc_C3IRNative(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha image add constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32fc_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddC_32fc_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAddC_32fc_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha in place image add constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32fc_AC4IR(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddC_32fc_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddC_32fc_AC4IRNative(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image add constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32fc_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddC_32fc_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAddC_32fc_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image add constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddC_32fc_C4IR(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddC_32fc_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddC_32fc_C4IRNative(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_8u_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image multiply by constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_8u_C1IRSfs(
        byte nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_8u_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_8u_C1IRSfsNative(
        byte nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_8u_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel 8-bit unsigned char in place image multiply by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_8u_C3IRSfs(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_8u_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_8u_C3IRSfsNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_8u_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha in place image multiply by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_8u_AC4IRSfs(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_8u_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_8u_AC4IRSfsNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_8u_C4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image multiply by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_8u_C4IRSfs(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_8u_C4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_8u_C4IRSfsNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16u_C1RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16u_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16u_C1RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image multiply by constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16u_C1IRSfs(
        short nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16u_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16u_C1IRSfsNative(
        short nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16u_C3RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16u_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16u_C3RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiMulC_16u_C3IRSfs(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16u_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16u_C3IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiMulC_16u_AC4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16u_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16u_AC4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel with unmodified alpha in place image multiply by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16u_AC4IRSfs(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16u_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16u_AC4IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16u_C4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16u_C4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16u_C4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image multiply by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16u_C4IRSfs(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16u_C4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16u_C4IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16s_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16s_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16s_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image multiply by constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16s_C1IRSfs(
        short nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16s_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16s_C1IRSfsNative(
        short nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16s_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16s_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16s_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel in place image multiply by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16s_C3IRSfs(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16s_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16s_C3IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel with unmodified alpha image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16s_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16s_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16s_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel with unmodified alpha in place image multiply by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16s_AC4IRSfs(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16s_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16s_AC4IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16s_C4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16s_C4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16s_C4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel in place image multiply by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16s_C4IRSfs(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16s_C4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16s_C4IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16sc_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16sc_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16sc_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image multiply by constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16sc_C1IRSfs(
        Npp16sc nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16sc_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16sc_C1IRSfsNative(
        Npp16sc nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16sc_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16sc_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16sc_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiMulC_16sc_C3IRSfs(
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16sc_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16sc_C3IRSfsNative(
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16sc_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16sc_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16sc_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image multiply by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_16sc_AC4IRSfs(
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_16sc_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_16sc_AC4IRSfsNative(
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer channel image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32s_C1RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_32s_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_32s_C1RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image multiply by constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32s_C1IRSfs(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_32s_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_32s_C1IRSfsNative(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed integer channel image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32s_C3RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_32s_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_32s_C3RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed integer channel in place image multiply by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32s_C3IRSfs(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_32s_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_32s_C3IRSfsNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32sc_C1RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_32sc_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_32sc_C1RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image multiply by constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32sc_C1IRSfs(
        Npp32sc nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_32sc_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_32sc_C1IRSfsNative(
        Npp32sc nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32sc_C3RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_32sc_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_32sc_C3RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image multiply by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32sc_C3IRSfs(
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_32sc_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_32sc_C3IRSfsNative(
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha image multiply by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32sc_AC4RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_32sc_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_32sc_AC4RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image multiply by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32sc_AC4IRSfs(
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMulC_32sc_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMulC_32sc_AC4IRSfsNative(
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit floating point channel image multiply by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float nConstant,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulC_32f_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulC_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float nConstant,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel in place image multiply by constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32f_C1IR(
        float nConstant,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulC_32f_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulC_32f_C1IRNative(
        float nConstant,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel image multiply by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulC_32f_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulC_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel in place image multiply by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32f_C3IR(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulC_32f_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulC_32f_C3IRNative(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel with unmodified alpha image multiply by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulC_32f_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulC_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel with unmodified alpha in place image multiply by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32f_AC4IR(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulC_32f_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulC_32f_AC4IRNative(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiMulC_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulC_32f_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulC_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel in place image multiply by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32f_C4IR(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulC_32f_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulC_32f_C4IRNative(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image multiply by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32fc_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc nConstant,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulC_32fc_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulC_32fc_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc nConstant,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiMulC_32fc_C1IR(
        Npp32fc nConstant,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulC_32fc_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulC_32fc_C1IRNative(
        Npp32fc nConstant,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image multiply by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32fc_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulC_32fc_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulC_32fc_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image multiply by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32fc_C3IR(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulC_32fc_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulC_32fc_C3IRNative(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha image multiply by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32fc_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulC_32fc_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulC_32fc_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiMulC_32fc_AC4IR(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulC_32fc_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulC_32fc_AC4IRNative(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image multiply by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32fc_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulC_32fc_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulC_32fc_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image multiply by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulC_32fc_C4IR(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulC_32fc_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulC_32fc_C4IRNative(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiMulCScale_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulCScale_8u_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulCScale_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image multiply by constant and scale by max bit width value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulCScale_8u_C1IR(
        byte nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulCScale_8u_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulCScale_8u_C1IRNative(
        byte nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image multiply by constant and scale by max bit width value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulCScale_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulCScale_8u_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulCScale_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel 8-bit unsigned char in place image multiply by constant and scale by max bit width value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulCScale_8u_C3IR(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulCScale_8u_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulCScale_8u_C3IRNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiMulCScale_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulCScale_8u_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulCScale_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha in place image multiply by constant, scale and scale by max bit width value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulCScale_8u_AC4IR(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulCScale_8u_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulCScale_8u_AC4IRNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image multiply by constant and scale by max bit width value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulCScale_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulCScale_8u_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulCScale_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image multiply by constant and scale by max bit width value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulCScale_8u_C4IR(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulCScale_8u_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulCScale_8u_C4IRNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel image multiply by constant and scale by max bit width value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulCScale_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulCScale_16u_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulCScale_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiMulCScale_16u_C1IR(
        short nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulCScale_16u_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulCScale_16u_C1IRNative(
        short nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image multiply by constant and scale by max bit width value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulCScale_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulCScale_16u_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulCScale_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image multiply by constant and scale by max bit width value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulCScale_16u_C3IR(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulCScale_16u_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulCScale_16u_C3IRNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiMulCScale_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulCScale_16u_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulCScale_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel with unmodified alpha in place image multiply by constant and scale by max bit width value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulCScale_16u_AC4IR(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulCScale_16u_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulCScale_16u_AC4IRNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image multiply by constant and scale by max bit width value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulCScale_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulCScale_16u_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulCScale_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image multiply by constant and scale by max bit width value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulCScale_16u_C4IR(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulCScale_16u_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulCScale_16u_C4IRNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiSubC_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_8u_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image subtract constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_8u_C1IRSfs(
        byte nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_8u_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_8u_C1IRSfsNative(
        byte nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image subtract constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_8u_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel 8-bit unsigned char in place image subtract constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_8u_C3IRSfs(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_8u_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_8u_C3IRSfsNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiSubC_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_8u_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha in place image subtract constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_8u_AC4IRSfs(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_8u_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_8u_AC4IRSfsNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image subtract constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_8u_C4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiSubC_8u_C4IRSfs(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_8u_C4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_8u_C4IRSfsNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel image subtract constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16u_C1RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16u_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16u_C1RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image subtract constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16u_C1IRSfs(
        short nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16u_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16u_C1IRSfsNative(
        short nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image subtract constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16u_C3RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16u_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16u_C3RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image subtract constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16u_C3IRSfs(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16u_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16u_C3IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel with unmodified alpha image subtract constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16u_AC4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16u_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16u_AC4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel with unmodified alpha in place image subtract constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16u_AC4IRSfs(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16u_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16u_AC4IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiSubC_16u_C4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16u_C4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16u_C4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image subtract constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16u_C4IRSfs(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16u_C4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16u_C4IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel image subtract constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16s_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16s_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16s_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image subtract constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16s_C1IRSfs(
        short nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16s_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16s_C1IRSfsNative(
        short nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel image subtract constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16s_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16s_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16s_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel in place image subtract constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16s_C3IRSfs(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16s_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16s_C3IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel with unmodified alpha image subtract constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16s_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16s_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16s_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel with unmodified alpha in place image subtract constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16s_AC4IRSfs(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16s_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16s_AC4IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel image subtract constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16s_C4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16s_C4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16s_C4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel in place image subtract constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16s_C4IRSfs(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16s_C4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16s_C4IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image subtract constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16sc_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16sc_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16sc_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image subtract constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16sc_C1IRSfs(
        Npp16sc nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16sc_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16sc_C1IRSfsNative(
        Npp16sc nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image subtract constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16sc_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16sc_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16sc_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image subtract constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16sc_C3IRSfs(
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16sc_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16sc_C3IRSfsNative(
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image subtract constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16sc_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16sc_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16sc_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image subtract constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_16sc_AC4IRSfs(
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_16sc_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_16sc_AC4IRSfsNative(
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer channel image subtract constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32s_C1RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_32s_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_32s_C1RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image subtract constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32s_C1IRSfs(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_32s_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_32s_C1IRSfsNative(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed integer channel image subtract constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32s_C3RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_32s_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_32s_C3RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed integer channel in place image subtract constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32s_C3IRSfs(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_32s_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_32s_C3IRSfsNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiSubC_32sc_C1RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_32sc_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_32sc_C1RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image subtract constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32sc_C1IRSfs(
        Npp32sc nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_32sc_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_32sc_C1IRSfsNative(
        Npp32sc nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image subtract constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32sc_C3RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_32sc_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_32sc_C3RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image subtract constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32sc_C3IRSfs(
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_32sc_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_32sc_C3IRSfsNative(
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha image subtract constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32sc_AC4RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_32sc_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_32sc_AC4RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image subtract constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32sc_AC4IRSfs(
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSubC_32sc_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSubC_32sc_AC4IRSfsNative(
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit floating point channel image subtract constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float nConstant,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSubC_32f_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSubC_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float nConstant,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel in place image subtract constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32f_C1IR(
        float nConstant,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSubC_32f_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSubC_32f_C1IRNative(
        float nConstant,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel image subtract constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSubC_32f_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSubC_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel in place image subtract constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32f_C3IR(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSubC_32f_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSubC_32f_C3IRNative(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel with unmodified alpha image subtract constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSubC_32f_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSubC_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel with unmodified alpha in place image subtract constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32f_AC4IR(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSubC_32f_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSubC_32f_AC4IRNative(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel image subtract constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSubC_32f_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSubC_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel in place image subtract constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32f_C4IR(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSubC_32f_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSubC_32f_C4IRNative(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image subtract constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32fc_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc nConstant,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSubC_32fc_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSubC_32fc_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc nConstant,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image subtract constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32fc_C1IR(
        Npp32fc nConstant,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSubC_32fc_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSubC_32fc_C1IRNative(
        Npp32fc nConstant,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image subtract constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32fc_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSubC_32fc_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSubC_32fc_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image subtract constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32fc_C3IR(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSubC_32fc_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSubC_32fc_C3IRNative(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha image subtract constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32fc_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSubC_32fc_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSubC_32fc_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiSubC_32fc_AC4IR(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSubC_32fc_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSubC_32fc_AC4IRNative(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image subtract constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32fc_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSubC_32fc_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSubC_32fc_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image subtract constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSubC_32fc_C4IR(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSubC_32fc_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSubC_32fc_C4IRNative(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiDivC_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_8u_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiDivC_8u_C1IRSfs(
        byte nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_8u_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_8u_C1IRSfsNative(
        byte nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_8u_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel 8-bit unsigned char in place image divided by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_8u_C3IRSfs(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_8u_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_8u_C3IRSfsNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_8u_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha in place image divided by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_8u_AC4IRSfs(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_8u_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_8u_AC4IRSfsNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiDivC_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_8u_C4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image divided by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_8u_C4IRSfs(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_8u_C4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_8u_C4IRSfsNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16u_C1RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16u_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16u_C1RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image divided by constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16u_C1IRSfs(
        short nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16u_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16u_C1IRSfsNative(
        short nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16u_C3RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16u_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16u_C3RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image divided by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16u_C3IRSfs(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16u_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16u_C3IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel with unmodified alpha image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16u_AC4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16u_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16u_AC4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel with unmodified alpha in place image divided by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16u_AC4IRSfs(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16u_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16u_AC4IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16u_C4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16u_C4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16u_C4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image divided by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16u_C4IRSfs(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16u_C4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16u_C4IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16s_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16s_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16s_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image divided by constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16s_C1IRSfs(
        short nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16s_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16s_C1IRSfsNative(
        short nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16s_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16s_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16s_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel in place image divided by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16s_C3IRSfs(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16s_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16s_C3IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiDivC_16s_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16s_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16s_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel with unmodified alpha in place image divided by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16s_AC4IRSfs(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16s_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16s_AC4IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16s_C4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16s_C4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16s_C4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel in place image divided by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16s_C4IRSfs(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16s_C4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16s_C4IRSfsNative(
        short[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16sc_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16sc_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16sc_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image divided by constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16sc_C1IRSfs(
        Npp16sc nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16sc_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16sc_C1IRSfsNative(
        Npp16sc nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16sc_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16sc_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16sc_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image divided by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16sc_C3IRSfs(
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16sc_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16sc_C3IRSfsNative(
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16sc_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16sc_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16sc_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image divided by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_16sc_AC4IRSfs(
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_16sc_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_16sc_AC4IRSfsNative(
        Npp16sc[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer channel image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32s_C1RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_32s_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_32s_C1RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image divided by constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32s_C1IRSfs(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_32s_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_32s_C1IRSfsNative(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed integer channel image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32s_C3RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_32s_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_32s_C3RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed integer channel in place image divided by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32s_C3IRSfs(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_32s_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_32s_C3IRSfsNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32sc_C1RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_32sc_C1RSfsNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_32sc_C1RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image divided by constant, scale, then clamp to saturated value.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32sc_C1IRSfs(
        Npp32sc nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_32sc_C1IRSfsNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_32sc_C1IRSfsNative(
        Npp32sc nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32sc_C3RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_32sc_C3RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_32sc_C3RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image divided by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32sc_C3IRSfs(
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_32sc_C3IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_32sc_C3IRSfsNative(
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha image divided by constant, scale, then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32sc_AC4RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_32sc_AC4RSfsNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_32sc_AC4RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image divided by constant, scale, then clamp to saturated value.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32sc_AC4IRSfs(
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDivC_32sc_AC4IRSfsNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDivC_32sc_AC4IRSfsNative(
        Npp32sc[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit floating point channel image divided by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float nConstant,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDivC_32f_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDivC_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float nConstant,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiDivC_32f_C1IR(
        float nConstant,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDivC_32f_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiDivC_32f_C1IRNative(
        float nConstant,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiDivC_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDivC_32f_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDivC_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel in place image divided by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32f_C3IR(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDivC_32f_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiDivC_32f_C3IRNative(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel with unmodified alpha image divided by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDivC_32f_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDivC_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel with unmodified alpha in place image divided by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32f_AC4IR(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDivC_32f_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiDivC_32f_AC4IRNative(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiDivC_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDivC_32f_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDivC_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiDivC_32f_C4IR(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDivC_32f_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiDivC_32f_C4IRNative(
        float[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image divided by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32fc_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc nConstant,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDivC_32fc_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDivC_32fc_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc nConstant,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image divided by constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32fc_C1IR(
        Npp32fc nConstant,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDivC_32fc_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiDivC_32fc_C1IRNative(
        Npp32fc nConstant,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image divided by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32fc_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDivC_32fc_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDivC_32fc_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image divided by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32fc_C3IR(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDivC_32fc_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiDivC_32fc_C3IRNative(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha image divided by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32fc_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDivC_32fc_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDivC_32fc_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha in place image divided by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32fc_AC4IR(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDivC_32fc_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiDivC_32fc_AC4IRNative(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image divided by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDivC_32fc_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDivC_32fc_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDivC_32fc_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiDivC_32fc_C4IR(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDivC_32fc_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiDivC_32fc_C4IRNative(
        Npp32fc[] aConstants,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image absolute difference with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbsDiffC_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nConstant)
    {
        return JNpp.checkResult(nppiAbsDiffC_8u_C1RNative(pSrc1, nSrc1Step, pDst, nDstStep, oSizeROI, nConstant));
    }
    private static native int nppiAbsDiffC_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nConstant);


    /**
     * <pre>
     * One 16-bit unsigned short channel image absolute difference with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbsDiffC_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nConstant)
    {
        return JNpp.checkResult(nppiAbsDiffC_16u_C1RNative(pSrc1, nSrc1Step, pDst, nDstStep, oSizeROI, nConstant));
    }
    private static native int nppiAbsDiffC_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nConstant);


    public static int nppiAbsDiffC_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nConstant)
    {
        return JNpp.checkResult(nppiAbsDiffC_32f_C1RNative(pSrc1, nSrc1Step, pDst, nDstStep, oSizeROI, nConstant));
    }
    private static native int nppiAbsDiffC_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nConstant);


    /**
     * <pre>
     * One 8-bit unsigned char channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_8u_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAdd_8u_C1IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_8u_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_8u_C1IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_8u_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_8u_C3IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_8u_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_8u_C3IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_8u_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_8u_AC4IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_8u_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_8u_AC4IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_8u_C4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_8u_C4IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_8u_C4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_8u_C4IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16u_C1RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16u_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16u_C1RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16u_C1IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16u_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16u_C1IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16u_C3RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16u_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16u_C3RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16u_C3IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16u_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16u_C3IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel with unmodified alpha image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16u_AC4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16u_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16u_AC4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel with unmodified alpha in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16u_AC4IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16u_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16u_AC4IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16u_C4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16u_C4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16u_C4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16u_C4IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16u_C4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16u_C4IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16s_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16s_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16s_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16s_C1IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16s_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16s_C1IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16s_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16s_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16s_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16s_C3IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16s_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16s_C3IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel with unmodified alpha image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16s_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16s_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16s_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAdd_16s_AC4IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16s_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16s_AC4IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16s_C4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16s_C4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16s_C4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16s_C4IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16s_C4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16s_C4IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16sc_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16sc_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16sc_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16sc_C1IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16sc_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16sc_C1IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16sc_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16sc_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16sc_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16sc_C3IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16sc_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16sc_C3IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16sc_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16sc_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16sc_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_16sc_AC4IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_16sc_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_16sc_AC4IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32s_C1RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_32s_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_32s_C1RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Note: This function is to be deprecated in future NPP releases, use the function above with a scale factor of 0 instead.
     * 32-bit image add.
     * Add the pixel values of corresponding pixels in the ROI and write them to the output image.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAdd_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32s_C1IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_32s_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_32s_C1IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed integer channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32s_C3RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_32s_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_32s_C3RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAdd_32s_C3IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_32s_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_32s_C3IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32sc_C1RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_32sc_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_32sc_C1RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32sc_C1IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_32sc_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_32sc_C1IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32sc_C3RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_32sc_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_32sc_C3RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32sc_C3IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_32sc_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_32sc_C3IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAdd_32sc_AC4RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_32sc_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_32sc_AC4RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32sc_AC4IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiAdd_32sc_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiAdd_32sc_AC4IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiAdd_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAdd_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiAdd_32f_C1IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32f_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAdd_32f_C1IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel image addition.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAdd_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel in place image addition.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32f_C3IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32f_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAdd_32f_C3IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel with unmodified alpha image addition.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAdd_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel with unmodified alpha in place image addition.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32f_AC4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32f_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAdd_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel image addition.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAdd_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel in place image addition.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32f_C4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32f_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAdd_32f_C4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image addition.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32fc_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32fc_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAdd_32fc_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image addition.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32fc_C1IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32fc_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAdd_32fc_C1IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image addition.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32fc_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32fc_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAdd_32fc_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image addition.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32fc_C3IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32fc_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAdd_32fc_C3IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image addition.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32fc_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32fc_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAdd_32fc_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image addition.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32fc_AC4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32fc_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAdd_32fc_AC4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image addition.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32fc_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32fc_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAdd_32fc_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image addition.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAdd_32fc_C4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAdd_32fc_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAdd_32fc_C4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image squared then added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddSquare_8u32f_C1IMR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddSquare_8u32f_C1IMRNative(pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddSquare_8u32f_C1IMRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image squared then added to in place floating point destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddSquare_8u32f_C1IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddSquare_8u32f_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddSquare_8u32f_C1IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiAddSquare_16u32f_C1IMR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddSquare_16u32f_C1IMRNative(pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddSquare_16u32f_C1IMRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel image squared then added to in place floating point destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddSquare_16u32f_C1IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddSquare_16u32f_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddSquare_16u32f_C1IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel image squared then added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddSquare_32f_C1IMR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddSquare_32f_C1IMRNative(pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddSquare_32f_C1IMRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel image squared then added to in place floating point destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddSquare_32f_C1IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddSquare_32f_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddSquare_32f_C1IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiAddProduct_8u32f_C1IMR(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddProduct_8u32f_C1IMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddProduct_8u32f_C1IMRNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image product added to in place floating point destination image.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddProduct_8u32f_C1IR(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddProduct_8u32f_C1IRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddProduct_8u32f_C1IRNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiAddProduct_16u32f_C1IMR(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddProduct_16u32f_C1IMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddProduct_16u32f_C1IMRNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel image product added to in place floating point destination image.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddProduct_16u32f_C1IR(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddProduct_16u32f_C1IRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddProduct_16u32f_C1IRNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel image product added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddProduct_32f_C1IMR(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddProduct_32f_C1IMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddProduct_32f_C1IMRNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel image product added to in place floating point destination image.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddProduct_32f_C1IR(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAddProduct_32f_C1IRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAddProduct_32f_C1IRNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel alpha weighted image added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddWeighted_8u32f_C1IMR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nAlpha)
    {
        return JNpp.checkResult(nppiAddWeighted_8u32f_C1IMRNative(pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha));
    }
    private static native int nppiAddWeighted_8u32f_C1IMRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nAlpha);


    /**
     * <pre>
     * One 8-bit unsigned char channel alpha weighted image added to in place floating point destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddWeighted_8u32f_C1IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nAlpha)
    {
        return JNpp.checkResult(nppiAddWeighted_8u32f_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha));
    }
    private static native int nppiAddWeighted_8u32f_C1IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nAlpha);


    /**
     * <pre>
     * One 16-bit unsigned short channel alpha weighted image added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddWeighted_16u32f_C1IMR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nAlpha)
    {
        return JNpp.checkResult(nppiAddWeighted_16u32f_C1IMRNative(pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha));
    }
    private static native int nppiAddWeighted_16u32f_C1IMRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nAlpha);


    /**
     * <pre>
     * One 16-bit unsigned short channel alpha weighted image added to in place floating point destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddWeighted_16u32f_C1IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nAlpha)
    {
        return JNpp.checkResult(nppiAddWeighted_16u32f_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha));
    }
    private static native int nppiAddWeighted_16u32f_C1IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nAlpha);


    public static int nppiAddWeighted_32f_C1IMR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nAlpha)
    {
        return JNpp.checkResult(nppiAddWeighted_32f_C1IMRNative(pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha));
    }
    private static native int nppiAddWeighted_32f_C1IMRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nAlpha);


    /**
     * <pre>
     * One 32-bit floating point channel alpha weighted image added to in place floating point destination image.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAddWeighted_32f_C1IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nAlpha)
    {
        return JNpp.checkResult(nppiAddWeighted_32f_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha));
    }
    private static native int nppiAddWeighted_32f_C1IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nAlpha);


    public static int nppiMul_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_8u_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiMul_8u_C1IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_8u_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_8u_C1IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_8u_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiMul_8u_C3IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_8u_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_8u_C3IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_8u_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_8u_AC4IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_8u_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_8u_AC4IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiMul_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_8u_C4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_8u_C4IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_8u_C4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_8u_C4IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16u_C1RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16u_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16u_C1RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16u_C1IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16u_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16u_C1IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16u_C3RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16u_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16u_C3RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16u_C3IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16u_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16u_C3IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel with unmodified alpha image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16u_AC4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16u_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16u_AC4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel with unmodified alpha in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16u_AC4IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16u_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16u_AC4IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16u_C4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16u_C4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16u_C4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16u_C4IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16u_C4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16u_C4IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16s_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16s_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16s_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16s_C1IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16s_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16s_C1IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16s_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16s_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16s_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16s_C3IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16s_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16s_C3IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiMul_16s_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16s_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16s_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel with unmodified alpha in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16s_AC4IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16s_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16s_AC4IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiMul_16s_C4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16s_C4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16s_C4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiMul_16s_C4IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16s_C4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16s_C4IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16sc_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16sc_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16sc_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16sc_C1IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16sc_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16sc_C1IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiMul_16sc_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16sc_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16sc_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16sc_C3IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16sc_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16sc_C3IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16sc_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16sc_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16sc_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_16sc_AC4IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_16sc_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_16sc_AC4IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32s_C1RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_32s_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_32s_C1RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Note: This function is to be deprecated in future NPP releases, use the function above with a scale factor of 0 instead.
     * 1 channel 32-bit image multiplication.
     * Multiply corresponding pixels in ROI.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMul_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32s_C1IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_32s_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_32s_C1IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed integer channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32s_C3RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_32s_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_32s_C3RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiMul_32s_C3IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_32s_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_32s_C3IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32sc_C1RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_32sc_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_32sc_C1RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32sc_C1IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_32sc_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_32sc_C1IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32sc_C3RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_32sc_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_32sc_C3RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32sc_C3IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_32sc_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_32sc_C3IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32sc_AC4RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_32sc_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_32sc_AC4RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32sc_AC4IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiMul_32sc_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiMul_32sc_AC4IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit floating point channel image multiplication.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMul_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel in place image multiplication.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32f_C1IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32f_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMul_32f_C1IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel image multiplication.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMul_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiMul_32f_C3IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32f_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMul_32f_C3IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel with unmodified alpha image multiplication.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMul_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel with unmodified alpha in place image multiplication.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32f_AC4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32f_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMul_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel image multiplication.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMul_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel in place image multiplication.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32f_C4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32f_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMul_32f_C4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image multiplication.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32fc_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32fc_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMul_32fc_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image multiplication.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32fc_C1IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32fc_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMul_32fc_C1IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image multiplication.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32fc_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32fc_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMul_32fc_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image multiplication.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32fc_C3IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32fc_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMul_32fc_C3IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image multiplication.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32fc_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32fc_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMul_32fc_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image multiplication.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32fc_AC4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32fc_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMul_32fc_AC4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiMul_32fc_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32fc_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMul_32fc_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image multiplication.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMul_32fc_C4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMul_32fc_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMul_32fc_C4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image multiplication then scale by maximum value for pixel bit width.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulScale_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulScale_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulScale_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image multiplication then scale by maximum value for pixel bit width.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulScale_8u_C1IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulScale_8u_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulScale_8u_C1IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image multiplication then scale by maximum value for pixel bit width.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulScale_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulScale_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulScale_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image multiplication then scale by maximum value for pixel bit width.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulScale_8u_C3IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulScale_8u_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulScale_8u_C3IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha image multiplication then scale by maximum value for pixel bit width.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulScale_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulScale_8u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulScale_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha in place image multiplication then scale by maximum value for pixel bit width.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulScale_8u_AC4IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulScale_8u_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulScale_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image multiplication then scale by maximum value for pixel bit width.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulScale_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulScale_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulScale_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image multiplication then scale by maximum value for pixel bit width.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulScale_8u_C4IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulScale_8u_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulScale_8u_C4IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel image multiplication then scale by maximum value for pixel bit width.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulScale_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulScale_16u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulScale_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image multiplication then scale by maximum value for pixel bit width.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulScale_16u_C1IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulScale_16u_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulScale_16u_C1IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image multiplication then scale by maximum value for pixel bit width.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulScale_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulScale_16u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulScale_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image multiplication then scale by maximum value for pixel bit width.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulScale_16u_C3IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulScale_16u_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulScale_16u_C3IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel with unmodified alpha image multiplication then scale by maximum value for pixel bit width.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulScale_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulScale_16u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulScale_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiMulScale_16u_AC4IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulScale_16u_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulScale_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image multiplication then scale by maximum value for pixel bit width.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulScale_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulScale_16u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMulScale_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image multiplication then scale by maximum value for pixel bit width.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMulScale_16u_C4IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMulScale_16u_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMulScale_16u_C4IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_8u_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_8u_C1IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_8u_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_8u_C1IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_8u_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_8u_C3IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_8u_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_8u_C3IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiSub_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_8u_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_8u_AC4IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_8u_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_8u_AC4IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_8u_C4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_8u_C4IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_8u_C4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_8u_C4IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiSub_16u_C1RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16u_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16u_C1RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16u_C1IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16u_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16u_C1IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiSub_16u_C3RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16u_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16u_C3RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16u_C3IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16u_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16u_C3IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel with unmodified alpha image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16u_AC4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16u_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16u_AC4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel with unmodified alpha in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16u_AC4IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16u_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16u_AC4IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiSub_16u_C4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16u_C4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16u_C4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16u_C4IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16u_C4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16u_C4IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16s_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16s_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16s_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16s_C1IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16s_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16s_C1IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16s_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16s_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16s_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16s_C3IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16s_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16s_C3IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel with unmodified alpha image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16s_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16s_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16s_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel with unmodified alpha in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16s_AC4IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16s_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16s_AC4IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiSub_16s_C4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16s_C4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16s_C4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16s_C4IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16s_C4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16s_C4IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16sc_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16sc_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16sc_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16sc_C1IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16sc_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16sc_C1IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16sc_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16sc_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16sc_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16sc_C3IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16sc_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16sc_C3IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16sc_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16sc_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16sc_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_16sc_AC4IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_16sc_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_16sc_AC4IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32s_C1RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_32s_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_32s_C1RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Note: This function is to be deprecated in future NPP releases, use the function above with a scale factor of 0 instead.
     * 32-bit image subtraction.
     * Subtract pSrc1's pixels from corresponding pixels in pSrc2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSub_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32s_C1IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_32s_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_32s_C1IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiSub_32s_C3RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_32s_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_32s_C3RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32s_C3IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_32s_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_32s_C3IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 32-bit signed integer channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32s_C4RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_32s_C4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_32s_C4RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32s_C4IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_32s_C4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_32s_C4IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32sc_C1RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_32sc_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_32sc_C1RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32sc_C1IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_32sc_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_32sc_C1IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32sc_C3RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_32sc_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_32sc_C3RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32sc_C3IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_32sc_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_32sc_C3IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32sc_AC4RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_32sc_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_32sc_AC4RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiSub_32sc_AC4IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSub_32sc_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSub_32sc_AC4IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit floating point channel image subtraction.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSub_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiSub_32f_C1IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32f_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSub_32f_C1IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel image subtraction.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSub_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel in place image subtraction.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32f_C3IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32f_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSub_32f_C3IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel with unmodified alpha image subtraction.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSub_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel with unmodified alpha in place image subtraction.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32f_AC4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32f_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSub_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel image subtraction.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSub_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel in place image subtraction.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32f_C4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32f_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSub_32f_C4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image subtraction.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32fc_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32fc_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSub_32fc_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image subtraction.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32fc_C1IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32fc_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSub_32fc_C1IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image subtraction.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32fc_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32fc_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSub_32fc_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image subtraction.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32fc_C3IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32fc_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSub_32fc_C3IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiSub_32fc_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32fc_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSub_32fc_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image subtraction.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32fc_AC4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32fc_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSub_32fc_AC4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image subtraction.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32fc_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32fc_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSub_32fc_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image subtraction.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSub_32fc_C4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSub_32fc_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSub_32fc_C4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_8u_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_8u_C1IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_8u_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_8u_C1IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_8u_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiDiv_8u_C3IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_8u_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_8u_C3IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_8u_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel with unmodified alpha in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_8u_AC4IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_8u_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_8u_AC4IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiDiv_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_8u_C4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_8u_C4IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_8u_C4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_8u_C4IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16u_C1RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16u_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16u_C1RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16u_C1IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16u_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16u_C1IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16u_C3RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16u_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16u_C3RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    public static int nppiDiv_16u_C3IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16u_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16u_C3IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel with unmodified alpha image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16u_AC4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16u_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16u_AC4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel with unmodified alpha in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16u_AC4IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16u_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16u_AC4IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16u_C4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16u_C4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16u_C4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16u_C4IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16u_C4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16u_C4IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16s_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16s_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16s_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16s_C1IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16s_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16s_C1IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16s_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16s_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16s_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16s_C3IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16s_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16s_C3IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel with unmodified alpha image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16s_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16s_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16s_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel with unmodified alpha in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16s_AC4IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16s_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16s_AC4IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16s_C4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16s_C4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16s_C4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16s_C4IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16s_C4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16s_C4IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16sc_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16sc_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16sc_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16sc_C1IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16sc_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16sc_C1IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16sc_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16sc_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16sc_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16sc_C3IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16sc_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16sc_C3IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16sc_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16sc_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16sc_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_16sc_AC4IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_16sc_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_16sc_AC4IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32s_C1RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_32s_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_32s_C1RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Note: This function is to be deprecated in future NPP releases, use the function above with a scale factor of 0 instead.
     * 32-bit image division.
     * Divide pixels in pSrc2 by pSrc1's pixels.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDiv_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32s_C1IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_32s_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_32s_C1IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed integer channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32s_C3RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_32s_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_32s_C3RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32s_C3IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_32s_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_32s_C3IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32sc_C1RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_32sc_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_32sc_C1RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32sc_C1IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_32sc_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_32sc_C1IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32sc_C3RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_32sc_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_32sc_C3RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32sc_C3IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_32sc_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_32sc_C3IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32sc_AC4RSfs(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_32sc_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_32sc_AC4RSfsNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32sc_AC4IRSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_32sc_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiDiv_32sc_AC4IRSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit floating point channel image division.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDiv_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel in place image division.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32f_C1IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32f_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiDiv_32f_C1IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel image division.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDiv_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel in place image division.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32f_C3IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32f_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiDiv_32f_C3IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel with unmodified alpha image division.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDiv_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel with unmodified alpha in place image division.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32f_AC4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32f_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiDiv_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiDiv_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDiv_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiDiv_32f_C4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32f_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiDiv_32f_C4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image division.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32fc_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32fc_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDiv_32fc_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image division.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32fc_C1IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32fc_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiDiv_32fc_C1IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image division.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32fc_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32fc_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDiv_32fc_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image division.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32fc_C3IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32fc_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiDiv_32fc_C3IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image division.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32fc_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32fc_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDiv_32fc_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image division.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32fc_AC4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32fc_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiDiv_32fc_AC4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image division.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32fc_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32fc_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDiv_32fc_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image division.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_32fc_C4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDiv_32fc_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiDiv_32fc_C4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_8u_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_8u_C1IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_8u_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_8u_C1IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_8u_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_8u_C3IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_8u_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_8u_C3IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_8u_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_8u_AC4IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_8u_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_8u_AC4IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_8u_C4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_8u_C4IRSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_8u_C4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_8u_C4IRSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_16u_C1RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_16u_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_16u_C1RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_16u_C1IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_16u_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_16u_C1IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_16u_C3RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_16u_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_16u_C3RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_16u_C3IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_16u_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_16u_C3IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_16u_AC4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_16u_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_16u_AC4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_16u_AC4IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_16u_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_16u_AC4IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_16u_C4RSfs(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_16u_C4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_16u_C4RSfsNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_16u_C4IRSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_16u_C4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_16u_C4IRSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_16s_C1RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_16s_C1RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_16s_C1RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_16s_C1IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_16s_C1IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_16s_C1IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_16s_C3RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_16s_C3RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_16s_C3RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_16s_C3IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_16s_C3IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_16s_C3IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_16s_AC4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_16s_AC4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_16s_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel in place image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_16s_AC4IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_16s_AC4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_16s_AC4IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_16s_C4RSfs(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_16s_C4RSfsNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_16s_C4RSfsNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDiv_Round_16s_C4IRSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiDiv_Round_16s_C4IRSfsNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor));
    }
    private static native int nppiDiv_Round_16s_C4IRSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int rndMode,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel image absolute value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbs_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbs_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAbs_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit signed short channel in place image absolute value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbs_16s_C1IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbs_16s_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAbs_16s_C1IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit signed short channel image absolute value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbs_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbs_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAbs_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit signed short channel in place image absolute value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbs_16s_C3IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbs_16s_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAbs_16s_C3IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit signed short channel image absolute value with unmodified alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbs_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbs_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAbs_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit signed short channel in place image absolute value with unmodified alpha.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbs_16s_AC4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbs_16s_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAbs_16s_AC4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit signed short channel image absolute value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbs_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbs_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAbs_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit signed short channel in place image absolute value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbs_16s_C4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbs_16s_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAbs_16s_C4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel image absolute value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbs_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbs_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAbs_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel in place image absolute value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbs_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbs_32f_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAbs_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel image absolute value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbs_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbs_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAbs_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel in place image absolute value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbs_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbs_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAbs_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel image absolute value with unmodified alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbs_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbs_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAbs_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel in place image absolute value with unmodified alpha.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbs_32f_AC4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbs_32f_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAbs_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel image absolute value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbs_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbs_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAbs_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel in place image absolute value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbs_32f_C4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbs_32f_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAbs_32f_C4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel absolute difference of image1 minus image2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbsDiff_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbsDiff_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAbsDiff_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channels absolute difference of image1 minus image2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbsDiff_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbsDiff_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAbsDiff_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channels absolute difference of image1 minus image2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbsDiff_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbsDiff_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAbsDiff_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel absolute difference of image1 minus image2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbsDiff_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbsDiff_16u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAbsDiff_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel absolute difference of image1 minus image2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAbsDiff_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAbsDiff_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAbsDiff_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_8u_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_8u_C1IRSfs(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_8u_C1IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_8u_C1IRSfsNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_8u_C3RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_8u_C3IRSfs(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_8u_C3IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_8u_C3IRSfsNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_8u_AC4RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_8u_AC4IRSfs(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_8u_AC4IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_8u_AC4IRSfsNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_8u_C4RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_8u_C4IRSfs(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_8u_C4IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_8u_C4IRSfsNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_16u_C1RSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_16u_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_16u_C1RSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_16u_C1IRSfs(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_16u_C1IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_16u_C1IRSfsNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_16u_C3RSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_16u_C3RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_16u_C3RSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_16u_C3IRSfs(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_16u_C3IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_16u_C3IRSfsNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_16u_AC4RSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_16u_AC4RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_16u_AC4RSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_16u_AC4IRSfs(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_16u_AC4IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_16u_AC4IRSfsNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_16u_C4RSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_16u_C4RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_16u_C4RSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_16u_C4IRSfs(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_16u_C4IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_16u_C4IRSfsNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_16s_C1RSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_16s_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_16s_C1RSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_16s_C1IRSfs(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_16s_C1IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_16s_C1IRSfsNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_16s_C3RSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_16s_C3RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_16s_C3RSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_16s_C3IRSfs(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_16s_C3IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_16s_C3IRSfsNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_16s_AC4RSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_16s_AC4RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_16s_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel in place image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_16s_AC4IRSfs(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_16s_AC4IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_16s_AC4IRSfsNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_16s_C4RSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_16s_C4RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_16s_C4RSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_16s_C4IRSfs(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqr_16s_C4IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqr_16s_C4IRSfsNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit floating point channel image squared.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSqr_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSqr_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiSqr_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSqr_32f_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSqr_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel image squared.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSqr_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSqr_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel in place image squared.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSqr_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSqr_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel image squared with unmodified alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSqr_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSqr_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel in place image squared with unmodified alpha.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_32f_AC4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSqr_32f_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSqr_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel image squared.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSqr_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSqr_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel in place image squared.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqr_32f_C4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSqr_32f_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSqr_32f_C4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_8u_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_8u_C1IRSfs(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_8u_C1IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_8u_C1IRSfsNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_8u_C3RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_8u_C3IRSfs(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_8u_C3IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_8u_C3IRSfsNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_8u_AC4RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_8u_AC4IRSfs(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_8u_AC4IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_8u_AC4IRSfsNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_16u_C1RSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_16u_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_16u_C1RSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_16u_C1IRSfs(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_16u_C1IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_16u_C1IRSfsNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_16u_C3RSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_16u_C3RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_16u_C3RSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_16u_C3IRSfs(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_16u_C3IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_16u_C3IRSfsNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_16u_AC4RSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_16u_AC4RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_16u_AC4RSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_16u_AC4IRSfs(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_16u_AC4IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_16u_AC4IRSfsNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_16s_C1RSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_16s_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_16s_C1RSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_16s_C1IRSfs(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_16s_C1IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_16s_C1IRSfsNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_16s_C3RSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_16s_C3RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_16s_C3RSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_16s_C3IRSfs(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_16s_C3IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_16s_C3IRSfsNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_16s_AC4RSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_16s_AC4RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_16s_AC4RSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Four 16-bit signed short channel in place image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_16s_AC4IRSfs(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrt_16s_AC4IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiSqrt_16s_AC4IRSfsNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit floating point channel image square root.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSqrt_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSqrt_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel in place image square root.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSqrt_32f_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSqrt_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel image square root.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSqrt_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSqrt_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel in place image square root.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSqrt_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSqrt_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel image square root with unmodified alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSqrt_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSqrt_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel in place image square root with unmodified alpha.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_32f_AC4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSqrt_32f_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSqrt_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel image square root.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSqrt_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiSqrt_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit floating point channel in place image square root.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrt_32f_C4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiSqrt_32f_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiSqrt_32f_C4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLn_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiLn_8u_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiLn_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLn_8u_C1IRSfs(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiLn_8u_C1IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiLn_8u_C1IRSfsNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLn_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiLn_8u_C3RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiLn_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLn_8u_C3IRSfs(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiLn_8u_C3IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiLn_8u_C3IRSfsNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLn_16u_C1RSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiLn_16u_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiLn_16u_C1RSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLn_16u_C1IRSfs(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiLn_16u_C1IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiLn_16u_C1IRSfsNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLn_16u_C3RSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiLn_16u_C3RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiLn_16u_C3RSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLn_16u_C3IRSfs(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiLn_16u_C3IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiLn_16u_C3IRSfsNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLn_16s_C1RSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiLn_16s_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiLn_16s_C1RSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLn_16s_C1IRSfs(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiLn_16s_C1IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiLn_16s_C1IRSfsNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLn_16s_C3RSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiLn_16s_C3RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiLn_16s_C3RSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLn_16s_C3IRSfs(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiLn_16s_C3IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiLn_16s_C3IRSfsNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit floating point channel image natural logarithm.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLn_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLn_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLn_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel in place image natural logarithm.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLn_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLn_32f_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiLn_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel image natural logarithm.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLn_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLn_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLn_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel in place image natural logarithm.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLn_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLn_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiLn_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiExp_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiExp_8u_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiExp_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiExp_8u_C1IRSfs(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiExp_8u_C1IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiExp_8u_C1IRSfsNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiExp_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiExp_8u_C3RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiExp_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 8-bit unsigned char channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiExp_8u_C3IRSfs(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiExp_8u_C3IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiExp_8u_C3IRSfsNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiExp_16u_C1RSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiExp_16u_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiExp_16u_C1RSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiExp_16u_C1IRSfs(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiExp_16u_C1IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiExp_16u_C1IRSfsNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiExp_16u_C3RSfs(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiExp_16u_C3RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiExp_16u_C3RSfsNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiExp_16u_C3IRSfs(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiExp_16u_C3IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiExp_16u_C3IRSfsNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiExp_16s_C1RSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiExp_16s_C1RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiExp_16s_C1RSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 16-bit signed short channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiExp_16s_C1IRSfs(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiExp_16s_C1IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiExp_16s_C1IRSfsNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiExp_16s_C3RSfs(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiExp_16s_C3RSfsNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiExp_16s_C3RSfsNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * Three 16-bit signed short channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiExp_16s_C3IRSfs(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiExp_16s_C3IRSfsNative(pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor));
    }
    private static native int nppiExp_16s_C3IRSfsNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        int nScaleFactor);


    /**
     * <pre>
     * One 32-bit floating point channel image exponential.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiExp_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiExp_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiExp_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit floating point channel in place image exponential.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiExp_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiExp_32f_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiExp_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel image exponential.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiExp_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiExp_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiExp_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit floating point channel in place image exponential.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiExp_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiExp_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiExp_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image logical and with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_8u_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAndC_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image logical and with constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_8u_C1IR(
        byte nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_8u_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAndC_8u_C1IRNative(
        byte nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image logical and with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_8u_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAndC_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel in place image logical and with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_8u_C3IR(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_8u_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAndC_8u_C3IRNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image logical and with constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_8u_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAndC_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image logical and with constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_8u_AC4IR(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_8u_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAndC_8u_AC4IRNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image logical and with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_8u_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAndC_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image logical and with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_8u_C4IR(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_8u_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAndC_8u_C4IRNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel image logical and with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_16u_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAndC_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image logical and with constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_16u_C1IR(
        short nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_16u_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAndC_16u_C1IRNative(
        short nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image logical and with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_16u_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAndC_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image logical and with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_16u_C3IR(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_16u_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAndC_16u_C3IRNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image logical and with constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_16u_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAndC_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image logical and with constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_16u_AC4IR(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_16u_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAndC_16u_AC4IRNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image logical and with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_16u_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAndC_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image logical and with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_16u_C4IR(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_16u_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAndC_16u_C4IRNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel image logical and with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_32s_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAndC_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image logical and with constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_32s_C1IR(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_32s_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAndC_32s_C1IRNative(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit signed integer channel image logical and with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_32s_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_32s_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAndC_32s_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit signed integer channel in place image logical and with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_32s_C3IR(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_32s_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAndC_32s_C3IRNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel image logical and with constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_32s_AC4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_32s_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAndC_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image logical and with constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_32s_AC4IR(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_32s_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAndC_32s_AC4IRNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel image logical and with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_32s_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_32s_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAndC_32s_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image logical and with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAndC_32s_C4IR(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAndC_32s_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAndC_32s_C4IRNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image logical or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_8u_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOrC_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiOrC_8u_C1IR(
        byte nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_8u_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOrC_8u_C1IRNative(
        byte nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image logical or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_8u_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOrC_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel in place image logical or with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_8u_C3IR(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_8u_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOrC_8u_C3IRNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image logical or with constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_8u_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOrC_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image logical or with constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_8u_AC4IR(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_8u_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOrC_8u_AC4IRNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image logical or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_8u_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOrC_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image logical or with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_8u_C4IR(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_8u_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOrC_8u_C4IRNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel image logical or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_16u_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOrC_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image logical or with constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_16u_C1IR(
        short nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_16u_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOrC_16u_C1IRNative(
        short nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image logical or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_16u_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOrC_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image logical or with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_16u_C3IR(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_16u_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOrC_16u_C3IRNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image logical or with constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_16u_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOrC_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image logical or with constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_16u_AC4IR(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_16u_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOrC_16u_AC4IRNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image logical or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_16u_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOrC_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image logical or with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_16u_C4IR(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_16u_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOrC_16u_C4IRNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel image logical or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_32s_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOrC_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image logical or with constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_32s_C1IR(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_32s_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOrC_32s_C1IRNative(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiOrC_32s_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_32s_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOrC_32s_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit signed integer channel in place image logical or with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_32s_C3IR(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_32s_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOrC_32s_C3IRNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel image logical or with constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_32s_AC4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_32s_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOrC_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image logical or with constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_32s_AC4IR(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_32s_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOrC_32s_AC4IRNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel image logical or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_32s_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_32s_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOrC_32s_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image logical or with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOrC_32s_C4IR(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOrC_32s_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOrC_32s_C4IRNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image logical exclusive or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_8u_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXorC_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image logical exclusive or with constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_8u_C1IR(
        byte nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_8u_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXorC_8u_C1IRNative(
        byte nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image logical exclusive or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_8u_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXorC_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel in place image logical exclusive or with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_8u_C3IR(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_8u_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXorC_8u_C3IRNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image logical exclusive or with constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_8u_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXorC_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image logical exclusive or with constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_8u_AC4IR(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_8u_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXorC_8u_AC4IRNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image logical exclusive or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_8u_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXorC_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image logical exclusive or with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_8u_C4IR(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_8u_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXorC_8u_C4IRNative(
        byte[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel image logical exclusive or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_16u_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXorC_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image logical exclusive or with constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_16u_C1IR(
        short nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_16u_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXorC_16u_C1IRNative(
        short nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image logical exclusive or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_16u_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXorC_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image logical exclusive or with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_16u_C3IR(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_16u_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXorC_16u_C3IRNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image logical exclusive or with constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_16u_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXorC_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image logical exclusive or with constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_16u_AC4IR(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_16u_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXorC_16u_AC4IRNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image logical exclusive or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_16u_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXorC_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image logical exclusive or with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_16u_C4IR(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_16u_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXorC_16u_C4IRNative(
        short[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel image logical exclusive or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_32s_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXorC_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image logical exclusive or with constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_32s_C1IR(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_32s_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXorC_32s_C1IRNative(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit signed integer channel image logical exclusive or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_32s_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_32s_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXorC_32s_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit signed integer channel in place image logical exclusive or with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_32s_C3IR(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_32s_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXorC_32s_C3IRNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel image logical exclusive or with constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_32s_AC4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_32s_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXorC_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image logical exclusive or with constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_32s_AC4IR(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_32s_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXorC_32s_AC4IRNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel image logical exclusive or with constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_32s_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_32s_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXorC_32s_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image logical exclusive or with constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXorC_32s_C4IR(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXorC_32s_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXorC_32s_C4IRNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image right shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_8u_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image right shift by constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_8u_C1IR(
        int nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_8u_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_8u_C1IRNative(
        int nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image right shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_8u_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel in place image right shift by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_8u_C3IR(
        int[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_8u_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_8u_C3IRNative(
        int[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image right shift by constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_8u_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image right shift by constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_8u_AC4IR(
        int[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_8u_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_8u_AC4IRNative(
        int[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image right shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_8u_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image right shift by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_8u_C4IR(
        int[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_8u_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_8u_C4IRNative(
        int[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit signed char channel image right shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_8s_C1R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_8s_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_8s_C1RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit signed char channel in place image right shift by constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_8s_C1IR(
        int nConstant,
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_8s_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_8s_C1IRNative(
        int nConstant,
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit signed char channel image right shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_8s_C3R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_8s_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_8s_C3RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit signed char channel in place image right shift by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_8s_C3IR(
        int[] aConstants,
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_8s_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_8s_C3IRNative(
        int[] aConstants,
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit signed char channel image right shift by constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_8s_AC4R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_8s_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_8s_AC4RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit signed char channel in place image right shift by constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_8s_AC4IR(
        int[] aConstants,
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_8s_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_8s_AC4IRNative(
        int[] aConstants,
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit signed char channel image right shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_8s_C4R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_8s_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_8s_C4RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit signed char channel in place image right shift by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_8s_C4IR(
        int[] aConstants,
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_8s_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_8s_C4IRNative(
        int[] aConstants,
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel image right shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_16u_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image right shift by constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_16u_C1IR(
        int nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_16u_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_16u_C1IRNative(
        int nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image right shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_16u_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image right shift by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_16u_C3IR(
        int[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_16u_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_16u_C3IRNative(
        int[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image right shift by constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_16u_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image right shift by constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_16u_AC4IR(
        int[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_16u_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_16u_AC4IRNative(
        int[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image right shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_16u_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image right shift by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_16u_C4IR(
        int[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_16u_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_16u_C4IRNative(
        int[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit signed short channel image right shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_16s_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_16s_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_16s_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit signed short channel in place image right shift by constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_16s_C1IR(
        int nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_16s_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_16s_C1IRNative(
        int nConstant,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit signed short channel image right shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_16s_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_16s_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_16s_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit signed short channel in place image right shift by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_16s_C3IR(
        int[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_16s_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_16s_C3IRNative(
        int[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit signed short channel image right shift by constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_16s_AC4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_16s_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit signed short channel in place image right shift by constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_16s_AC4IR(
        int[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_16s_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_16s_AC4IRNative(
        int[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit signed short channel image right shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_16s_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_16s_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_16s_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit signed short channel in place image right shift by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_16s_C4IR(
        int[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_16s_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_16s_C4IRNative(
        int[] aConstants,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel image right shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_32s_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image right shift by constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_32s_C1IR(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_32s_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_32s_C1IRNative(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit signed integer channel image right shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_32s_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_32s_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_32s_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit signed integer channel in place image right shift by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_32s_C3IR(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_32s_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_32s_C3IRNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel image right shift by constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_32s_AC4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_32s_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image right shift by constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_32s_AC4IR(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_32s_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_32s_AC4IRNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel image right shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_32s_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_32s_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_32s_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image right shift by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRShiftC_32s_C4IR(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRShiftC_32s_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiRShiftC_32s_C4IRNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /** @} image_rshiftc */
    /**
     * <pre>
     * @defgroup image_lshiftc LShiftC
     *
     * Pixel by pixel left shift of an image by a constant value.
     *
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One 8-bit unsigned char channel image left shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_8u_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image left shift by constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_8u_C1IR(
        int nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_8u_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_8u_C1IRNative(
        int nConstant,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image left shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_8u_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel in place image left shift by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_8u_C3IR(
        int[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_8u_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_8u_C3IRNative(
        int[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image left shift by constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_8u_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image left shift by constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_8u_AC4IR(
        int[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_8u_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_8u_AC4IRNative(
        int[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image left shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_8u_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image left shift by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_8u_C4IR(
        int[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_8u_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_8u_C4IRNative(
        int[] aConstants,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiLShiftC_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_16u_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image left shift by constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_16u_C1IR(
        int nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_16u_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_16u_C1IRNative(
        int nConstant,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image left shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_16u_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image left shift by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_16u_C3IR(
        int[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_16u_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_16u_C3IRNative(
        int[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image left shift by constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_16u_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image left shift by constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_16u_AC4IR(
        int[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_16u_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_16u_AC4IRNative(
        int[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image left shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_16u_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image left shift by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_16u_C4IR(
        int[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_16u_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_16u_C4IRNative(
        int[] aConstants,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel image left shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nConstant Constant.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_32s_C1RNative(pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nConstant,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image left shift by constant.
     * @param nConstant Constant.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_32s_C1IR(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_32s_C1IRNative(nConstant, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_32s_C1IRNative(
        int nConstant,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit signed integer channel image left shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_32s_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_32s_C3RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_32s_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit signed integer channel in place image left shift by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_32s_C3IR(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_32s_C3IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_32s_C3IRNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel image left shift by constant with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_32s_AC4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_32s_AC4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image left shift by constant with unmodified alpha.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_32s_AC4IR(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_32s_AC4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_32s_AC4IRNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel image left shift by constant.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_32s_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_32s_C4RNative(pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_32s_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int[] aConstants,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image left shift by constant.
     * @param aConstants fixed size array of constant values, one per channel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLShiftC_32s_C4IR(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLShiftC_32s_C4IRNative(aConstants, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiLShiftC_32s_C4IRNative(
        int[] aConstants,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image logical and.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAnd_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image logical and.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_8u_C1IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_8u_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAnd_8u_C1IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image logical and.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAnd_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel in place image logical and.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_8u_C3IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_8u_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAnd_8u_C3IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image logical and with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_8u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAnd_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image logical and with unmodified alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_8u_AC4IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_8u_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAnd_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image logical and.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAnd_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image logical and.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_8u_C4IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_8u_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAnd_8u_C4IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel image logical and.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_16u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAnd_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image logical and.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_16u_C1IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_16u_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAnd_16u_C1IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image logical and.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_16u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAnd_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image logical and.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_16u_C3IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_16u_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAnd_16u_C3IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image logical and with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_16u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAnd_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image logical and with unmodified alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_16u_AC4IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_16u_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAnd_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image logical and.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_16u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAnd_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image logical and.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_16u_C4IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_16u_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAnd_16u_C4IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel image logical and.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_32s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAnd_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image logical and.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_32s_C1IR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_32s_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAnd_32s_C1IRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit signed integer channel image logical and.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_32s_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_32s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAnd_32s_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit signed integer channel in place image logical and.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_32s_C3IR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_32s_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAnd_32s_C3IRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel image logical and with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_32s_AC4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_32s_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAnd_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image logical and with unmodified alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_32s_AC4IR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_32s_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAnd_32s_AC4IRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel image logical and.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_32s_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_32s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAnd_32s_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image logical and.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAnd_32s_C4IR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAnd_32s_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAnd_32s_C4IRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image logical or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOr_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image logical or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_8u_C1IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_8u_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOr_8u_C1IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image logical or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOr_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel in place image logical or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_8u_C3IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_8u_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOr_8u_C3IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image logical or with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_8u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOr_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image logical or with unmodified alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_8u_AC4IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_8u_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOr_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image logical or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOr_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image logical or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_8u_C4IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_8u_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOr_8u_C4IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel image logical or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_16u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOr_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image logical or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_16u_C1IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_16u_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOr_16u_C1IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image logical or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_16u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOr_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image logical or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_16u_C3IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_16u_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOr_16u_C3IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image logical or with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_16u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOr_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image logical or with unmodified alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_16u_AC4IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_16u_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOr_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image logical or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_16u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOr_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image logical or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_16u_C4IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_16u_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOr_16u_C4IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel image logical or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_32s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOr_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image logical or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_32s_C1IR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_32s_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOr_32s_C1IRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit signed integer channel image logical or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_32s_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_32s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOr_32s_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit signed integer channel in place image logical or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_32s_C3IR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_32s_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOr_32s_C3IRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel image logical or with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_32s_AC4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_32s_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOr_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image logical or with unmodified alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_32s_AC4IR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_32s_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOr_32s_AC4IRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel image logical or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_32s_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_32s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiOr_32s_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image logical or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiOr_32s_C4IR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiOr_32s_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiOr_32s_C4IRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image logical exclusive or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXor_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image logical exclusive or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_8u_C1IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_8u_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXor_8u_C1IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image logical exclusive or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXor_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel in place image logical exclusive or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_8u_C3IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_8u_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXor_8u_C3IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image logical exclusive or with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_8u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXor_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image logical exclusive or with unmodified alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_8u_AC4IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_8u_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXor_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image logical exclusive or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXor_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image logical exclusive or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_8u_C4IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_8u_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXor_8u_C4IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel image logical exclusive or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_16u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXor_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image logical exclusive or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_16u_C1IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_16u_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXor_16u_C1IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image logical exclusive or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_16u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXor_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image logical exclusive or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_16u_C3IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_16u_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXor_16u_C3IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image logical exclusive or with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_16u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXor_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image logical exclusive or with unmodified alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_16u_AC4IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_16u_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXor_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image logical exclusive or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_16u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXor_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image logical exclusive or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_16u_C4IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_16u_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXor_16u_C4IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel image logical exclusive or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_32s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXor_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 32-bit signed integer channel in place image logical exclusive or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_32s_C1IR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_32s_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXor_32s_C1IRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit signed integer channel image logical exclusive or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_32s_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_32s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXor_32s_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 32-bit signed integer channel in place image logical exclusive or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_32s_C3IR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_32s_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXor_32s_C3IRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel image logical exclusive or with unmodified alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_32s_AC4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_32s_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXor_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image logical exclusive or with unmodified alpha.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_32s_AC4IR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_32s_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXor_32s_AC4IRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel image logical exclusive or.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_32s_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_32s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXor_32s_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 32-bit signed integer channel in place image logical exclusive or.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXor_32s_C4IR(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXor_32s_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiXor_32s_C4IRNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image logical not.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiNot_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiNot_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiNot_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image logical not.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiNot_8u_C1IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiNot_8u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiNot_8u_C1IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image logical not.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiNot_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiNot_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiNot_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel in place image logical not.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiNot_8u_C3IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiNot_8u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiNot_8u_C3IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiNot_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiNot_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiNot_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image logical not with unmodified alpha.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiNot_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiNot_8u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiNot_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image logical not.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiNot_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiNot_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiNot_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image logical not.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiNot_8u_C4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiNot_8u_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiNot_8u_C4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image composition using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaCompC_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        byte nAlpha2,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaCompC_8u_C1RNative(pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaCompC_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        byte nAlpha2,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    public static int nppiAlphaCompC_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        byte nAlpha2,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaCompC_8u_C3RNative(pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaCompC_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        byte nAlpha2,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image composition using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaCompC_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        byte nAlpha2,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaCompC_8u_C4RNative(pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaCompC_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        byte nAlpha2,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image composition with alpha using constant source alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaCompC_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        byte nAlpha2,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaCompC_8u_AC4RNative(pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaCompC_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        byte nAlpha2,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * One 8-bit signed char channel image composition using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaCompC_8s_C1R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        byte nAlpha2,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaCompC_8s_C1RNative(pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaCompC_8s_C1RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        byte nAlpha2,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * One 16-bit unsigned short channel image composition using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaCompC_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        short nAlpha2,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaCompC_16u_C1RNative(pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaCompC_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        short nAlpha2,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image composition using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaCompC_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        short nAlpha2,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaCompC_16u_C3RNative(pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaCompC_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        short nAlpha2,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image composition using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaCompC_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        short nAlpha2,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaCompC_16u_C4RNative(pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaCompC_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        short nAlpha2,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image composition with alpha using constant source alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaCompC_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        short nAlpha2,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaCompC_16u_AC4RNative(pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaCompC_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        short nAlpha2,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    public static int nppiAlphaCompC_16s_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        short nAlpha2,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaCompC_16s_C1RNative(pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaCompC_16s_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        short nAlpha2,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * One 32-bit unsigned integer channel image composition using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaCompC_32u_C1R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        int nAlpha1,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        int nAlpha2,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaCompC_32u_C1RNative(pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaCompC_32u_C1RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        int nAlpha1,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        int nAlpha2,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * One 32-bit signed integer channel image composition using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaCompC_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nAlpha1,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        int nAlpha2,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaCompC_32s_C1RNative(pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaCompC_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        int nAlpha1,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        int nAlpha2,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * One 32-bit floating point channel image composition using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0.0 - 1.0).
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param nAlpha2 Image alpha opacity (0.0 - 1.0).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaCompC_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float nAlpha1,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        float nAlpha2,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaCompC_32f_C1RNative(pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaCompC_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        float nAlpha1,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        float nAlpha2,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * One 8-bit unsigned char channel image premultiplication using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremulC_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremulC_8u_C1RNative(pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremulC_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel in place image premultiplication using constant alpha.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremulC_8u_C1IR(
        byte nAlpha1,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremulC_8u_C1IRNative(nAlpha1, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremulC_8u_C1IRNative(
        byte nAlpha1,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel image premultiplication using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremulC_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremulC_8u_C3RNative(pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremulC_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 8-bit unsigned char channel in place image premultiplication using constant alpha.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremulC_8u_C3IR(
        byte nAlpha1,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremulC_8u_C3IRNative(nAlpha1, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremulC_8u_C3IRNative(
        byte nAlpha1,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image premultiplication using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremulC_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremulC_8u_C4RNative(pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremulC_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image premultiplication using constant alpha.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremulC_8u_C4IR(
        byte nAlpha1,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremulC_8u_C4IRNative(nAlpha1, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremulC_8u_C4IRNative(
        byte nAlpha1,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image premultiplication with alpha using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremulC_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremulC_8u_AC4RNative(pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremulC_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image premultiplication with alpha using constant alpha.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremulC_8u_AC4IR(
        byte nAlpha1,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremulC_8u_AC4IRNative(nAlpha1, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremulC_8u_AC4IRNative(
        byte nAlpha1,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel image premultiplication using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremulC_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremulC_16u_C1RNative(pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremulC_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 16-bit unsigned short channel in place image premultiplication using constant alpha.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremulC_16u_C1IR(
        short nAlpha1,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremulC_16u_C1IRNative(nAlpha1, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremulC_16u_C1IRNative(
        short nAlpha1,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel image premultiplication using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremulC_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremulC_16u_C3RNative(pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremulC_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three 16-bit unsigned short channel in place image premultiplication using constant alpha.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremulC_16u_C3IR(
        short nAlpha1,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremulC_16u_C3IRNative(nAlpha1, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremulC_16u_C3IRNative(
        short nAlpha1,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image premultiplication using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremulC_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremulC_16u_C4RNative(pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremulC_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image premultiplication using constant alpha.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremulC_16u_C4IR(
        short nAlpha1,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremulC_16u_C4IRNative(nAlpha1, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremulC_16u_C4IRNative(
        short nAlpha1,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image premultiplication with alpha using constant alpha.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremulC_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremulC_16u_AC4RNative(pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremulC_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        short nAlpha1,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image premultiplication with alpha using constant alpha.
     * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremulC_16u_AC4IR(
        short nAlpha1,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremulC_16u_AC4IRNative(nAlpha1, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremulC_16u_AC4IRNative(
        short nAlpha1,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One 8-bit unsigned char channel image composition using image alpha values (0 - max channel pixel value).
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaComp_8u_AC1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaComp_8u_AC1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaComp_8u_AC1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    public static int nppiAlphaComp_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaComp_8u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaComp_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * One 8-bit signed char channel image composition using image alpha values (0 - max channel pixel value).
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaComp_8s_AC1R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaComp_8s_AC1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaComp_8s_AC1RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * One 16-bit unsigned short channel image composition using image alpha values (0 - max channel pixel value).
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaComp_16u_AC1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaComp_16u_AC1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaComp_16u_AC1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * Four 16-bit unsigned short channel image composition using image alpha values (0 - max channel pixel value).
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaComp_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaComp_16u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaComp_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * One 16-bit signed short channel image composition using image alpha values (0 - max channel pixel value).
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaComp_16s_AC1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaComp_16s_AC1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaComp_16s_AC1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * One 32-bit unsigned integer channel image composition using image alpha values (0 - max channel pixel value).
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaComp_32u_AC1R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaComp_32u_AC1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaComp_32u_AC1RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * Four 32-bit unsigned integer channel image composition using image alpha values (0 - max channel pixel value).
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaComp_32u_AC4R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaComp_32u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaComp_32u_AC4RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * One 32-bit signed integer channel image composition using image alpha values (0 - max channel pixel value).
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaComp_32s_AC1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaComp_32s_AC1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaComp_32s_AC1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * Four 32-bit signed integer channel image composition using image alpha values (0 - max channel pixel value).
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaComp_32s_AC4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaComp_32s_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaComp_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * One 32-bit floating point channel image composition using image alpha values (0.0 - 1.0).
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaComp_32f_AC1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaComp_32f_AC1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaComp_32f_AC1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * Four 32-bit floating point channel image composition using image alpha values (0.0 - 1.0).
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eAlphaOp alpha-blending operation..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaComp_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaComp_32f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp));
    }
    private static native int nppiAlphaComp_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eAlphaOp);


    /**
     * <pre>
     * Four 8-bit unsigned char channel image premultiplication with pixel alpha (0 - max channel pixel value).
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremul_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremul_8u_AC4RNative(pSrc1, nSrc1Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremul_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 8-bit unsigned char channel in place image premultiplication with pixel alpha (0 - max channel pixel value).
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremul_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremul_8u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremul_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiAlphaPremul_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremul_16u_AC4RNative(pSrc1, nSrc1Step, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremul_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four 16-bit unsigned short channel in place image premultiplication with pixel alpha (0 - max channel pixel value).
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaPremul_16u_AC4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiAlphaPremul_16u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiAlphaPremul_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed RGB to 3 channel 8-bit unsigned packed YUV color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYUV_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYUV_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYUV_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed RGB with alpha to 4 channel 8-bit unsigned packed YUV color conversion with alpha, not affecting alpha.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYUV_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYUV_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYUV_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar RGB to 3 channel 8-bit unsigned planar YUV color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYUV_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYUV_8u_P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYUV_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed RGB to 3 channel 8-bit unsigned planar YUV color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYUV_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYUV_8u_C3P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYUV_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed RGB with alpha to 4 channel 8-bit unsigned planar YUV color conversion with alpha.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYUV_8u_AC4P4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYUV_8u_AC4P4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYUV_8u_AC4P4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiBGRToYUV_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYUV_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToYUV_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiBGRToYUV_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYUV_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToYUV_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar BGR to 3 channel 8-bit unsigned planar YUV color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYUV_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYUV_8u_P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToYUV_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiBGRToYUV_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYUV_8u_C3P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToYUV_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiBGRToYUV_8u_AC4P4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYUV_8u_AC4P4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToYUV_8u_AC4P4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed YUV to 3 channel 8-bit unsigned packed RGB color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYUVToRGB_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUVToRGB_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUVToRGB_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiYUVToRGB_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUVToRGB_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUVToRGB_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YUV to 3 channel 8-bit unsigned planar RGB color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYUVToRGB_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUVToRGB_8u_P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUVToRGB_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YUV to 3 channel 8-bit unsigned packed RGB color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYUVToRGB_8u_P3C3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUVToRGB_8u_P3C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUVToRGB_8u_P3C3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed YUV to 3 channel 8-bit unsigned packed BGR color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYUVToBGR_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUVToBGR_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUVToBGR_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiYUVToBGR_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUVToBGR_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUVToBGR_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YUV to 3 channel 8-bit unsigned planar BGR color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYUVToBGR_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUVToBGR_8u_P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUVToBGR_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YUV to 3 channel 8-bit unsigned packed BGR color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYUVToBGR_8u_P3C3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUVToBGR_8u_P3C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUVToBGR_8u_P3C3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed RGB to 2 channel 8-bit unsigned packed YUV422 color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYUV422_8u_C3C2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYUV422_8u_C3C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYUV422_8u_C3C2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar RGB to 3 channel 8-bit unsigned planar YUV422 color conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYUV422_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYUV422_8u_P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiRGBToYUV422_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed RGB to 3 channel 8-bit unsigned planar YUV422 color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYUV422_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYUV422_8u_C3P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiRGBToYUV422_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YUV422 to 3 channel 8-bit unsigned packed RGB color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYUV422ToRGB_8u_C2C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUV422ToRGB_8u_C2C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUV422ToRGB_8u_C2C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YUV422 to 3 channel 8-bit unsigned planar RGB color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_planar_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYUV422ToRGB_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUV422ToRGB_8u_P3RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUV422ToRGB_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YUV422 to 3 channel 8-bit unsigned packed RGB color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYUV422ToRGB_8u_P3C3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUV422ToRGB_8u_P3C3RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUV422ToRGB_8u_P3C3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YUV422 to 4 channel 8-bit unsigned packed RGB color conversion with alpha.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYUV422ToRGB_8u_P3AC4R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUV422ToRGB_8u_P3AC4RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUV422ToRGB_8u_P3AC4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiRGBToYUV420_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYUV420_8u_P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiRGBToYUV420_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    public static int nppiRGBToYUV420_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYUV420_8u_C3P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiRGBToYUV420_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YUV420 to 3 channel 8-bit unsigned planar RGB color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_planar_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYUV420ToRGB_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUV420ToRGB_8u_P3RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUV420ToRGB_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YUV420 to 3 channel 8-bit unsigned packed RGB color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYUV420ToRGB_8u_P3C3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUV420ToRGB_8u_P3C3RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUV420ToRGB_8u_P3C3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YUV420 to 4 channel 8-bit unsigned packed RGB color conversion with constant alpha (0xFF).
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYUV420ToRGB_8u_P3C4R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUV420ToRGB_8u_P3C4RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUV420ToRGB_8u_P3C4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YUV420 to 4 channel 8-bit unsigned packed RGB color conversion with alpha.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYUV420ToRGB_8u_P3AC4R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUV420ToRGB_8u_P3AC4RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUV420ToRGB_8u_P3AC4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     *  2 channel 8-bit unsigned planar NV21 to 4 channel 8-bit unsigned packed ARGB color conversion with constant alpha (0xFF).
     *
     * @param pSrc \ref source_planar_image_pointer_array (one for Y plane, one for VU plane).
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiNV21ToRGB_8u_P2C4R(
        TypedPointer<Npp8u>[] pSrc,
        int rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiNV21ToRGB_8u_P2C4RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiNV21ToRGB_8u_P2C4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiBGRToYUV420_8u_AC4P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYUV420_8u_AC4P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiBGRToYUV420_8u_AC4P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    public static int nppiYUV420ToBGR_8u_P3C3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUV420ToBGR_8u_P3C3RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUV420ToBGR_8u_P3C3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YUV420 to 4 channel 8-bit unsigned packed BGR color conversion with constant alpha (0xFF).
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYUV420ToBGR_8u_P3C4R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYUV420ToBGR_8u_P3C4RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYUV420ToBGR_8u_P3C4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     *  2 channel 8-bit unsigned planar NV21 to 4 channel 8-bit unsigned packed BGRA color conversion with constant alpha (0xFF).
     *
     * @param pSrc \ref source_planar_image_pointer_array (one for Y plane, one for VU plane).
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiNV21ToBGR_8u_P2C4R(
        TypedPointer<Npp8u>[] pSrc,
        int rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiNV21ToBGR_8u_P2C4RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiNV21ToBGR_8u_P2C4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiRGBToYCbCr_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYCbCr_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYCbCr_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed RGB with alpha to 4 channel unsigned 8-bit packed YCbCr with alpha color conversion, not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYCbCr_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYCbCr_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYCbCr_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiRGBToYCbCr_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYCbCr_8u_P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYCbCr_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed RGB to 3 channel unsigned 8-bit planar YCbCr color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYCbCr_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYCbCr_8u_C3P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYCbCr_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiRGBToYCbCr_8u_AC4P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYCbCr_8u_AC4P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYCbCr_8u_AC4P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiYCbCrToRGB_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCrToRGB_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCrToRGB_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed YCbCr with alpha to 4 channel 8-bit unsigned packed RGB with alpha color conversion, not affecting alpha.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCrToRGB_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCrToRGB_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCrToRGB_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr to 3 channel 8-bit unsigned planar RGB color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCrToRGB_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCrToRGB_8u_P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCrToRGB_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr to 3 channel 8-bit unsigned packed RGB color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCrToRGB_8u_P3C3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCrToRGB_8u_P3C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCrToRGB_8u_P3C3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr to 4 channel 8-bit unsigned packed RGB color conversion with constant alpha.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nAval 8-bit unsigned alpha constant.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCrToRGB_8u_P3C4R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval)
    {
        return JNpp.checkResult(nppiYCbCrToRGB_8u_P3C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nAval));
    }
    private static native int nppiYCbCrToRGB_8u_P3C4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr to 3 channel 8-bit unsigned packed BGR color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCrToBGR_8u_P3C3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCrToBGR_8u_P3C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCrToBGR_8u_P3C3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiYCbCrToBGR_8u_P3C4R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval)
    {
        return JNpp.checkResult(nppiYCbCrToBGR_8u_P3C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nAval));
    }
    private static native int nppiYCbCrToBGR_8u_P3C4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr to 3 channel 8-bit unsigned packed BGR_709CSC color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCrToBGR_709CSC_8u_P3C3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCrToBGR_709CSC_8u_P3C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCrToBGR_709CSC_8u_P3C3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiYCbCrToBGR_709CSC_8u_P3C4R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval)
    {
        return JNpp.checkResult(nppiYCbCrToBGR_709CSC_8u_P3C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nAval));
    }
    private static native int nppiYCbCrToBGR_709CSC_8u_P3C4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed RGB to 2 channel 8-bit unsigned packed YCbCr422 color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYCbCr422_8u_C3C2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYCbCr422_8u_C3C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYCbCr422_8u_C3C2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed RGB to 3 channel 8-bit unsigned planar YCbCr422 color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYCbCr422_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYCbCr422_8u_C3P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiRGBToYCbCr422_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar RGB to 2 channel 8-bit unsigned packed YCbCr422 color conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYCbCr422_8u_P3C2R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYCbCr422_8u_P3C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYCbCr422_8u_P3C2RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YCbCr422 to 3 channel 8-bit unsigned packed RGB color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422ToRGB_8u_C2C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToRGB_8u_C2C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToRGB_8u_C2C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YCbCr422 to 3 channel 8-bit unsigned planar RGB color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422ToRGB_8u_C2P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToRGB_8u_C2P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToRGB_8u_C2P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiYCbCr422ToRGB_8u_P3C3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToRGB_8u_P3C3RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToRGB_8u_P3C3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed RGB to 2 channel 8-bit unsigned packed YCrCb422 color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYCrCb422_8u_C3C2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYCrCb422_8u_C3C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYCrCb422_8u_C3C2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiRGBToYCrCb422_8u_P3C2R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYCrCb422_8u_P3C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYCrCb422_8u_P3C2RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YCrCb422 to 3 channel 8-bit unsigned packed RGB color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCrCb422ToRGB_8u_C2C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCrCb422ToRGB_8u_C2C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCrCb422ToRGB_8u_C2C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YCrCb422 to 3 channel 8-bit unsigned planar RGB color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCrCb422ToRGB_8u_C2P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCrCb422ToRGB_8u_C2P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCrCb422ToRGB_8u_C2P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed BGR to 2 channel 8-bit unsigned packed YCrCb422 color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYCbCr422_8u_C3C2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCbCr422_8u_C3C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCbCr422_8u_C3C2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiBGRToYCbCr422_8u_AC4C2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCbCr422_8u_AC4C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCbCr422_8u_AC4C2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed BGR to 3 channel 8-bit unsigned planar YCbCr422 color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYCbCr422_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCbCr422_8u_C3P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCbCr422_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed BGR with alpha to 3 channel 8-bit unsigned planar YCbCr422 color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYCbCr422_8u_AC4P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCbCr422_8u_AC4P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCbCr422_8u_AC4P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YCrCb422 to 3 channel 8-bit unsigned packed BGR color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422ToBGR_8u_C2C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToBGR_8u_C2C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToBGR_8u_C2C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YCrCb422 to 4 channel 8-bit unsigned packed BGR color conversion with constant alpha.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nAval 8-bit unsigned alpha constant.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422ToBGR_8u_C2C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval)
    {
        return JNpp.checkResult(nppiYCbCr422ToBGR_8u_C2C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nAval));
    }
    private static native int nppiYCbCr422ToBGR_8u_C2C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr422 to 3 channel 8-bit unsigned packed BGR color conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422ToBGR_8u_P3C3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToBGR_8u_P3C3RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToBGR_8u_P3C3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed RGB to 2 channel 8-bit unsigned packed CbYCr422 color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToCbYCr422_8u_C3C2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToCbYCr422_8u_C3C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToCbYCr422_8u_C3C2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed RGB first gets forward gamma corrected then converted to 2 channel 8-bit unsigned packed CbYCr422 color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToCbYCr422Gamma_8u_C3C2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToCbYCr422Gamma_8u_C3C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToCbYCr422Gamma_8u_C3C2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed CbYCrC22 to 3 channel 8-bit unsigned packed RGB color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCbYCr422ToRGB_8u_C2C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCbYCr422ToRGB_8u_C2C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCbYCr422ToRGB_8u_C2C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed BGR with alpha to 2 channel 8-bit unsigned packed CbYCr422 color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToCbYCr422_8u_AC4C2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToCbYCr422_8u_AC4C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToCbYCr422_8u_AC4C2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed BGR to 2 channel 8-bit unsigned packed CbYCr422_709HDTV color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToCbYCr422_709HDTV_8u_C3C2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToCbYCr422_709HDTV_8u_C3C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToCbYCr422_709HDTV_8u_C3C2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed BGR with alpha to 2 channel 8-bit unsigned packed CbYCr422_709HDTV color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToCbYCr422_709HDTV_8u_AC4C2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToCbYCr422_709HDTV_8u_AC4C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToCbYCr422_709HDTV_8u_AC4C2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiCbYCr422ToBGR_8u_C2C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval)
    {
        return JNpp.checkResult(nppiCbYCr422ToBGR_8u_C2C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nAval));
    }
    private static native int nppiCbYCr422ToBGR_8u_C2C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval);


    public static int nppiCbYCr422ToBGR_709HDTV_8u_C2C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCbYCr422ToBGR_709HDTV_8u_C2C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCbYCr422ToBGR_709HDTV_8u_C2C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed CbYCr422 to 4 channel 8-bit unsigned packed BGR_709HDTV color conversion with constant alpha.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nAval 8-bit unsigned alpha constant.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCbYCr422ToBGR_709HDTV_8u_C2C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval)
    {
        return JNpp.checkResult(nppiCbYCr422ToBGR_709HDTV_8u_C2C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nAval));
    }
    private static native int nppiCbYCr422ToBGR_709HDTV_8u_C2C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed RGB to 3 channel 8-bit unsigned planar YCbCr420 color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYCbCr420_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYCbCr420_8u_C3P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiRGBToYCbCr420_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    public static int nppiYCbCr420ToRGB_8u_P3C3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr420ToRGB_8u_P3C3RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr420ToRGB_8u_P3C3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed RGB with alpha to 3 channel 8-bit unsigned planar YCrCb420 color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYCrCb420_8u_AC4P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYCrCb420_8u_AC4P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiRGBToYCrCb420_8u_AC4P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCrCb420 to 4 channel 8-bit unsigned packed RGB color conversion with constant alpha.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nAval 8-bit unsigned alpha constant.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCrCb420ToRGB_8u_P3C4R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval)
    {
        return JNpp.checkResult(nppiYCrCb420ToRGB_8u_P3C4RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI, nAval));
    }
    private static native int nppiYCrCb420ToRGB_8u_P3C4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed BGR to 3 channel 8-bit unsigned planar YCbCr420 color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYCbCr420_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCbCr420_8u_C3P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCbCr420_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed BGR with alpha to 3 channel 8-bit unsigned planar YCbCr420 color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYCbCr420_8u_AC4P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCbCr420_8u_AC4P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCbCr420_8u_AC4P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed BGR to 3 channel 8-bit unsigned planar YCbCr420_709CSC color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYCbCr420_709CSC_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCbCr420_709CSC_8u_C3P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCbCr420_709CSC_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed BGR with alpha to 3 channel 8-bit unsigned planar YCbCr420_709CSC color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYCbCr420_709CSC_8u_AC4P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCbCr420_709CSC_8u_AC4P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCbCr420_709CSC_8u_AC4P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed BGR with alpha to 3 channel 8-bit unsigned planar YCbCr420_709HDTV color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYCbCr420_709HDTV_8u_AC4P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCbCr420_709HDTV_8u_AC4P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCbCr420_709HDTV_8u_AC4P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed BGR to 3 channel 8-bit unsigned planar YCrCb420_709CSC color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYCrCb420_709CSC_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCrCb420_709CSC_8u_C3P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCrCb420_709CSC_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed BGR with alpha to 3 channel 8-bit unsigned planar YCrCb420_709CSC color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYCrCb420_709CSC_8u_AC4P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCrCb420_709CSC_8u_AC4P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCrCb420_709CSC_8u_AC4P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    public static int nppiYCbCr420ToBGR_8u_P3C3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr420ToBGR_8u_P3C3RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr420ToBGR_8u_P3C3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiYCbCr420ToBGR_8u_P3C4R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval)
    {
        return JNpp.checkResult(nppiYCbCr420ToBGR_8u_P3C4RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI, nAval));
    }
    private static native int nppiYCbCr420ToBGR_8u_P3C4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr420 to 3 channel 8-bit unsigned packed BGR_709CSC color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr420ToBGR_709CSC_8u_P3C3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr420ToBGR_709CSC_8u_P3C3RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr420ToBGR_709CSC_8u_P3C3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr420 to 4 channel 8-bit unsigned packed BGR_709HDTV color conversion with constant alpha.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nAval 8-bit unsigned alpha constant.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr420ToBGR_709HDTV_8u_P3C4R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval)
    {
        return JNpp.checkResult(nppiYCbCr420ToBGR_709HDTV_8u_P3C4RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI, nAval));
    }
    private static native int nppiYCbCr420ToBGR_709HDTV_8u_P3C4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed BGR to 3 channel 8-bit unsigned planar YCrCb420 color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYCrCb420_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCrCb420_8u_C3P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCrCb420_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed BGR with alpha to 3 channel 8-bit unsigned planar YCrCb420 color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYCrCb420_8u_AC4P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCrCb420_8u_AC4P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCrCb420_8u_AC4P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed BGR to 3 channel 8-bit unsigned planar YCbCr411 color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYCbCr411_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCbCr411_8u_C3P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCbCr411_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    public static int nppiBGRToYCbCr411_8u_AC4P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCbCr411_8u_AC4P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCbCr411_8u_AC4P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed BGR to 3 channel 8-bit unsigned planar YCbCr color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYCbCr_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCbCr_8u_C3P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCbCr_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed BGR with alpha to 3 channel 8-bit unsigned planar YCbCr color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYCbCr_8u_AC4P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCbCr_8u_AC4P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCbCr_8u_AC4P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed BGR with alpha to 4 channel 8-bit unsigned planar YCbCr color conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToYCbCr_8u_AC4P4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToYCbCr_8u_AC4P4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToYCbCr_8u_AC4P4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiYCbCr411ToBGR_8u_P3C3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr411ToBGR_8u_P3C3RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr411ToBGR_8u_P3C3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiYCbCr411ToBGR_8u_P3C4R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval)
    {
        return JNpp.checkResult(nppiYCbCr411ToBGR_8u_P3C4RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI, nAval));
    }
    private static native int nppiYCbCr411ToBGR_8u_P3C4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nAval);


    public static int nppiRGBToXYZ_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToXYZ_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToXYZ_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed RGB with alpha to 4 channel 8-bit unsigned packed XYZ with alpha color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToXYZ_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToXYZ_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToXYZ_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed XYZ to 3 channel 8-bit unsigned packed RGB color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXYZToRGB_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXYZToRGB_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXYZToRGB_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed XYZ with alpha to 4 channel 8-bit unsigned packed RGB with alpha color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiXYZToRGB_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiXYZToRGB_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiXYZToRGB_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed RGB to 3 channel 8-bit unsigned packed LUV color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToLUV_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToLUV_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToLUV_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed RGB with alpha to 4 channel 8-bit unsigned packed LUV with alpha color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToLUV_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToLUV_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToLUV_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed LUV to 3 channel 8-bit unsigned packed RGB color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLUVToRGB_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLUVToRGB_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLUVToRGB_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed LUV with alpha to 4 channel 8-bit unsigned packed RGB with alpha color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLUVToRGB_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLUVToRGB_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLUVToRGB_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiBGRToLab_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToLab_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToLab_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed Lab to 3 channel 8-bit unsigned packed BGR color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiLabToBGR_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiLabToBGR_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiLabToBGR_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed RGB to 3 channel 8-bit unsigned packed YCC color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYCC_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYCC_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYCC_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed RGB with alpha to 4 channel 8-bit unsigned packed YCC with alpha color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToYCC_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToYCC_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToYCC_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiYCCToRGB_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCCToRGB_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCCToRGB_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed YCC with alpha to 4 channel 8-bit unsigned packed RGB with alpha color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCCToRGB_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCCToRGB_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCCToRGB_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed RGB to 3 channel 8-bit unsigned packed HLS color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToHLS_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToHLS_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToHLS_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed RGB with alpha to 4 channel 8-bit unsigned packed HLS with alpha color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToHLS_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToHLS_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToHLS_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed HLS to 3 channel 8-bit unsigned packed RGB color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHLSToRGB_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiHLSToRGB_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiHLSToRGB_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed HLS with alpha to 4 channel 8-bit unsigned packed RGB with alpha color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHLSToRGB_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiHLSToRGB_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiHLSToRGB_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed BGR with alpha to 4 channel 8-bit unsigned packed HLS with alpha color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToHLS_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToHLS_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToHLS_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed BGR to 3 channel 8-bit unsigned planar HLS color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToHLS_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToHLS_8u_C3P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToHLS_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed BGR with alpha to 4 channel 8-bit unsigned planar HLS with alpha color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToHLS_8u_AC4P4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToHLS_8u_AC4P4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToHLS_8u_AC4P4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiBGRToHLS_8u_P3C3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToHLS_8u_P3C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToHLS_8u_P3C3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned planar BGR with alpha to 4 channel 8-bit unsigned packed HLS with alpha color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToHLS_8u_AP4C4R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToHLS_8u_AP4C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToHLS_8u_AP4C4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar BGR to 3 channel 8-bit unsigned planar HLS color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToHLS_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToHLS_8u_P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToHLS_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned planar BGR with alpha to 4 channel 8-bit unsigned planar HLS with alpha color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiBGRToHLS_8u_AP4R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiBGRToHLS_8u_AP4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiBGRToHLS_8u_AP4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed HLS to 3 channel 8-bit unsigned planar BGR color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHLSToBGR_8u_C3P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiHLSToBGR_8u_C3P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiHLSToBGR_8u_C3P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed HLS with alpha to 4 channel 8-bit unsigned planar BGR with alpha color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHLSToBGR_8u_AC4P4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiHLSToBGR_8u_AC4P4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiHLSToBGR_8u_AC4P4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiHLSToBGR_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiHLSToBGR_8u_P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiHLSToBGR_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiHLSToBGR_8u_AP4R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiHLSToBGR_8u_AP4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiHLSToBGR_8u_AP4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed HLS with alpha to 4 channel 8-bit unsigned packed BGR with alpha color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHLSToBGR_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiHLSToBGR_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiHLSToBGR_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar HLS to 3 channel 8-bit unsigned packed BGR color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHLSToBGR_8u_P3C3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiHLSToBGR_8u_P3C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiHLSToBGR_8u_P3C3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned planar HLS with alpha to 4 channel 8-bit unsigned packed BGR with alpha color conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHLSToBGR_8u_AP4C4R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiHLSToBGR_8u_AP4C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiHLSToBGR_8u_AP4C4RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiRGBToHSV_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToHSV_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToHSV_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed RGB with alpha to 4 channel 8-bit unsigned packed HSV with alpha color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToHSV_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToHSV_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToHSV_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiHSVToRGB_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiHSVToRGB_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiHSVToRGB_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed HSV with alpha to 4 channel 8-bit unsigned packed RGB with alpha color conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHSVToRGB_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiHSVToRGB_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiHSVToRGB_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed RGB to 1 channel 8-bit unsigned packed Gray conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToGray_8u_C3C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToGray_8u_C3C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToGray_8u_C3C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed RGB with alpha to 1 channel 8-bit unsigned packed Gray conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToGray_8u_AC4C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToGray_8u_AC4C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToGray_8u_AC4C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 16-bit unsigned packed RGB to 1 channel 16-bit unsigned packed Gray conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToGray_16u_C3C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToGray_16u_C3C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToGray_16u_C3C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 16-bit unsigned packed RGB with alpha to 1 channel 16-bit unsigned packed Gray conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToGray_16u_AC4C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToGray_16u_AC4C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToGray_16u_AC4C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 16-bit signed packed RGB to 1 channel 16-bit signed packed Gray conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToGray_16s_C3C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToGray_16s_C3C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToGray_16s_C3C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 16-bit signed packed RGB with alpha to 1 channel 16-bit signed packed Gray conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToGray_16s_AC4C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToGray_16s_AC4C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToGray_16s_AC4C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiRGBToGray_32f_C3C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToGray_32f_C3C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToGray_32f_C3C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 32-bit floating point packed RGB with alpha to 1 channel 32-bit floating point packed Gray conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRGBToGray_32f_AC4C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiRGBToGray_32f_AC4C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiRGBToGray_32f_AC4C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiColorToGray_8u_C3C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] aCoeffs)
    {
        return JNpp.checkResult(nppiColorToGray_8u_C3C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aCoeffs));
    }
    private static native int nppiColorToGray_8u_C3C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] aCoeffs);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed RGB with alpha to 1 channel 8-bit unsigned packed Gray conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aCoeffs fixed size array of constant floating point conversion coefficient values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorToGray_8u_AC4C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] aCoeffs)
    {
        return JNpp.checkResult(nppiColorToGray_8u_AC4C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aCoeffs));
    }
    private static native int nppiColorToGray_8u_AC4C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] aCoeffs);


    /**
     * <pre>
     * 3 channel 16-bit unsigned packed RGB to 1 channel 16-bit unsigned packed Gray conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aCoeffs fixed size array of constant floating point conversion coefficient values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorToGray_16u_C3C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] aCoeffs)
    {
        return JNpp.checkResult(nppiColorToGray_16u_C3C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aCoeffs));
    }
    private static native int nppiColorToGray_16u_C3C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] aCoeffs);


    /**
     * <pre>
     * 4 channel 16-bit unsigned packed RGB with alpha to 1 channel 16-bit unsigned packed Gray conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aCoeffs fixed size array of constant floating point conversion coefficient values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorToGray_16u_AC4C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] aCoeffs)
    {
        return JNpp.checkResult(nppiColorToGray_16u_AC4C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aCoeffs));
    }
    private static native int nppiColorToGray_16u_AC4C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] aCoeffs);


    /**
     * <pre>
     * 3 channel 16-bit signed packed RGB to 1 channel 16-bit signed packed Gray conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aCoeffs fixed size array of constant floating point conversion coefficient values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorToGray_16s_C3C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] aCoeffs)
    {
        return JNpp.checkResult(nppiColorToGray_16s_C3C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aCoeffs));
    }
    private static native int nppiColorToGray_16s_C3C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] aCoeffs);


    /**
     * <pre>
     * 4 channel 16-bit signed packed RGB with alpha to 1 channel 16-bit signed packed Gray conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aCoeffs fixed size array of constant floating point conversion coefficient values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorToGray_16s_AC4C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] aCoeffs)
    {
        return JNpp.checkResult(nppiColorToGray_16s_AC4C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aCoeffs));
    }
    private static native int nppiColorToGray_16s_AC4C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] aCoeffs);


    /**
     * <pre>
     * 3 channel 32-bit floating point packed RGB to 1 channel 32-bit floating point packed Gray conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aCoeffs fixed size array of constant floating point conversion coefficient values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorToGray_32f_C3C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] aCoeffs)
    {
        return JNpp.checkResult(nppiColorToGray_32f_C3C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aCoeffs));
    }
    private static native int nppiColorToGray_32f_C3C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] aCoeffs);


    /**
     * <pre>
     * 4 channel 32-bit floating point packed RGB with alpha to 1 channel 32-bit floating point packed Gray conversion.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aCoeffs fixed size array of constant floating point conversion coefficient values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorToGray_32f_AC4C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] aCoeffs)
    {
        return JNpp.checkResult(nppiColorToGray_32f_AC4C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aCoeffs));
    }
    private static native int nppiColorToGray_32f_AC4C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] aCoeffs);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr420 to 2 channel 8-bit unsigned planar YCbCr411 sampling format conversion.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDstY \ref destination_planar_image_pointer.
     * @param nDstYStep \ref destination_planar_image_line_step.
     * @param pDstCbCr \ref destination_planar_image_pointer.
     * @param nDstCbCrStep \ref destination_planar_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr420ToYCbCr411_8u_P3P2R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr420ToYCbCr411_8u_P3P2RNative(pSrc, rSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI));
    }
    private static native int nppiYCbCr420ToYCbCr411_8u_P3P2RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned planar YCbCr420 to 3 channel 8-bit unsigned planar YCbCr411 sampling format conversion.
     *
     * @param pSrcY \ref source_planar_image_pointer.
     * @param nSrcYStep \ref source_planar_image_line_step.
     * @param pSrcCbCr \ref source_planar_image_pointer.
     * @param nSrcCbCrStep \ref source_planar_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr420ToYCbCr411_8u_P2P3R(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr420ToYCbCr411_8u_P2P3RNative(pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCbCr420ToYCbCr411_8u_P2P3RNative(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YCbCr422 to 3 channel 8-bit unsigned planar YCbCr422 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422_8u_C2P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422_8u_C2P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCbCr422_8u_C2P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr422 to 2 channel 8-bit unsigned packed YCbCr422 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422_8u_P3C2R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422_8u_P3C2RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr422_8u_P3C2RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YCbCr422 to 2 channel 8-bit unsigned packed YCrCb422 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422ToYCrCb422_8u_C2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToYCrCb422_8u_C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToYCrCb422_8u_C2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr422 to 2 channel 8-bit unsigned packed YCrCb422 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422ToYCrCb422_8u_P3C2R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToYCrCb422_8u_P3C2RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToYCrCb422_8u_P3C2RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YCbCr422 to 2 channel 8-bit unsigned packed CbYCr422 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422ToCbYCr422_8u_C2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToCbYCr422_8u_C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToCbYCr422_8u_C2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiCbYCr422ToYCbCr411_8u_C2P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCbYCr422ToYCbCr411_8u_C2P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiCbYCr422ToYCbCr411_8u_C2P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr422 to 3 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422ToYCbCr420_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToYCbCr420_8u_P3RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToYCbCr420_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr422 to 2 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDstY \ref destination_planar_image_pointer.
     * @param nDstYStep \ref destination_planar_image_line_step.
     * @param pDstCbCr \ref destination_planar_image_pointer.
     * @param nDstCbCrStep \ref destination_planar_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422ToYCbCr420_8u_P3P2R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToYCbCr420_8u_P3P2RNative(pSrc, rSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToYCbCr420_8u_P3P2RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YCbCr422 to 3 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422ToYCbCr420_8u_C2P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToYCbCr420_8u_C2P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToYCbCr420_8u_C2P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YCbCr422 to 2 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDstY \ref destination_planar_image_pointer.
     * @param nDstYStep \ref destination_planar_image_line_step.
     * @param pDstCbCr \ref destination_planar_image_pointer.
     * @param nDstCbCrStep \ref destination_planar_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422ToYCbCr420_8u_C2P2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToYCbCr420_8u_C2P2RNative(pSrc, nSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToYCbCr420_8u_C2P2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCrCb420 to 3 channel 8-bit unsigned planar YCbCr422 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCrCb420ToYCbCr422_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCrCb420ToYCbCr422_8u_P3RNative(pSrc, rSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCrCb420ToYCbCr422_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCrCb420 to 2 channel 8-bit unsigned packed YCbCr422 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCrCb420ToYCbCr422_8u_P3C2R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCrCb420ToYCbCr422_8u_P3C2RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCrCb420ToYCbCr422_8u_P3C2RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YCbCr422 to 3 channel 8-bit unsigned planar YCrCb420 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422ToYCrCb420_8u_C2P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToYCrCb420_8u_C2P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToYCrCb420_8u_C2P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    public static int nppiYCbCr422ToYCbCr411_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToYCbCr411_8u_P3RNative(pSrc, rSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToYCbCr411_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr422 to 2 channel 8-bit unsigned planar YCbCr411 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDstY \ref destination_planar_image_pointer.
     * @param nDstYStep \ref destination_planar_image_line_step.
     * @param pDstCbCr \ref destination_planar_image_pointer.
     * @param nDstCbCrStep \ref destination_planar_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422ToYCbCr411_8u_P3P2R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToYCbCr411_8u_P3P2RNative(pSrc, rSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToYCbCr411_8u_P3P2RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YCbCr422 to 3 channel 8-bit unsigned planar YCbCr411 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422ToYCbCr411_8u_C2P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToYCbCr411_8u_C2P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToYCbCr411_8u_C2P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YCbCr422 to 2 channel 8-bit unsigned planar YCbCr411 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDstY \ref destination_planar_image_pointer.
     * @param nDstYStep \ref destination_planar_image_line_step.
     * @param pDstCbCr \ref destination_planar_image_pointer.
     * @param nDstCbCrStep \ref destination_planar_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr422ToYCbCr411_8u_C2P2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr422ToYCbCr411_8u_C2P2RNative(pSrc, nSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI));
    }
    private static native int nppiYCbCr422ToYCbCr411_8u_C2P2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YCrCb422 to 3 channel 8-bit unsigned planar YCbCr422 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCrCb422ToYCbCr422_8u_C2P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCrCb422ToYCbCr422_8u_C2P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCrCb422ToYCbCr422_8u_C2P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed YCrCb422 to 3 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCrCb422ToYCbCr420_8u_C2P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCrCb422ToYCbCr420_8u_C2P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCrCb422ToYCbCr420_8u_C2P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    public static int nppiYCrCb422ToYCbCr411_8u_C2P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCrCb422ToYCbCr411_8u_C2P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCrCb422ToYCbCr411_8u_C2P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed CbYCr422 to 2 channel 8-bit unsigned packed YCbCr422 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCbYCr422ToYCbCr422_8u_C2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCbYCr422ToYCbCr422_8u_C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiCbYCr422ToYCbCr422_8u_C2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed CbYCr422 to 3 channel 8-bit unsigned planar YCbCr422 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCbYCr422ToYCbCr422_8u_C2P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCbYCr422ToYCbCr422_8u_C2P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiCbYCr422ToYCbCr422_8u_C2P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed CbYCr422 to 3 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCbYCr422ToYCbCr420_8u_C2P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCbYCr422ToYCbCr420_8u_C2P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiCbYCr422ToYCbCr420_8u_C2P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed CbYCr422 to 2 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDstY \ref destination_planar_image_pointer.
     * @param nDstYStep \ref destination_planar_image_line_step.
     * @param pDstCbCr \ref destination_planar_image_pointer.
     * @param nDstCbCrStep \ref destination_planar_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCbYCr422ToYCbCr420_8u_C2P2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCbYCr422ToYCbCr420_8u_C2P2RNative(pSrc, nSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI));
    }
    private static native int nppiCbYCr422ToYCbCr420_8u_C2P2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned packed CbYCr422 to 3 channel 8-bit unsigned planar YCrCb420 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCbYCr422ToYCrCb420_8u_C2P3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiCbYCr422ToYCrCb420_8u_C2P3RNative(pSrc, nSrcStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiCbYCr422ToYCrCb420_8u_C2P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr420 to 2 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDstY \ref destination_planar_image_pointer.
     * @param nDstYStep \ref destination_planar_image_line_step.
     * @param pDstCbCr \ref destination_planar_image_pointer.
     * @param nDstCbCrStep \ref destination_planar_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr420_8u_P3P2R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr420_8u_P3P2RNative(pSrc, rSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI));
    }
    private static native int nppiYCbCr420_8u_P3P2RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI);


    public static int nppiYCbCr420_8u_P2P3R(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr420_8u_P2P3RNative(pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCbCr420_8u_P2P3RNative(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr420 to 3 channel 8-bit unsigned planar YCbCr422 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr420ToYCbCr422_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr420ToYCbCr422_8u_P3RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr420ToYCbCr422_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned planar YCbCr420 to 3 channel 8-bit unsigned planar YCbCr422 sampling format conversion.
     *
     * @param pSrcY \ref source_planar_image_pointer.
     * @param nSrcYStep \ref source_planar_image_line_step.
     * @param pSrcCbCr \ref source_planar_image_pointer.
     * @param nSrcCbCrStep \ref source_planar_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr420ToYCbCr422_8u_P2P3R(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr420ToYCbCr422_8u_P2P3RNative(pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCbCr420ToYCbCr422_8u_P2P3RNative(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned planar YCbCr420 to 2 channel 8-bit unsigned packed YCbCr422 sampling format conversion.
     *
     * @param pSrcY \ref source_planar_image_pointer.
     * @param nSrcYStep \ref source_planar_image_line_step.
     * @param pSrcCbCr \ref source_planar_image_pointer.
     * @param nSrcCbCrStep \ref source_planar_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr420ToYCbCr422_8u_P2C2R(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr420ToYCbCr422_8u_P2C2RNative(pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr420ToYCbCr422_8u_P2C2RNative(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned planar YCbCr420 to 2 channel 8-bit unsigned packed CbYCr422 sampling format conversion.
     *
     * @param pSrcY \ref source_planar_image_pointer.
     * @param nSrcYStep \ref source_planar_image_line_step.
     * @param pSrcCbCr \ref source_planar_image_pointer.
     * @param nSrcCbCrStep \ref source_planar_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr420ToCbYCr422_8u_P2C2R(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr420ToCbYCr422_8u_P2C2RNative(pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr420ToCbYCr422_8u_P2C2RNative(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiYCbCr420ToYCrCb420_8u_P2P3R(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr420ToYCrCb420_8u_P2P3RNative(pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCbCr420ToYCrCb420_8u_P2P3RNative(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCrCb420 to 2 channel 8-bit unsigned packed CbYCr422 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCrCb420ToCbYCr422_8u_P3C2R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCrCb420ToCbYCr422_8u_P3C2RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCrCb420ToCbYCr422_8u_P3C2RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCrCb420 to 2 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDstY \ref destination_planar_image_pointer.
     * @param nDstYStep \ref destination_planar_image_line_step.
     * @param pDstCbCr \ref destination_planar_image_pointer.
     * @param nDstCbCrStep \ref destination_planar_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCrCb420ToYCbCr420_8u_P3P2R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCrCb420ToYCbCr420_8u_P3P2RNative(pSrc, rSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI));
    }
    private static native int nppiYCrCb420ToYCbCr420_8u_P3P2RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCrCb420 to 2 channel 8-bit unsigned planar YCbCr411 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDstY \ref destination_planar_image_pointer.
     * @param nDstYStep \ref destination_planar_image_line_step.
     * @param pDstCbCr \ref destination_planar_image_pointer.
     * @param nDstCbCrStep \ref destination_planar_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCrCb420ToYCbCr411_8u_P3P2R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCrCb420ToYCbCr411_8u_P3P2RNative(pSrc, rSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI));
    }
    private static native int nppiYCrCb420ToYCbCr411_8u_P3P2RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr411 to 2 channel 8-bit unsigned planar YCbCr411 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDstY \ref destination_planar_image_pointer.
     * @param nDstYStep \ref destination_planar_image_line_step.
     * @param pDstCbCr \ref destination_planar_image_pointer.
     * @param nDstCbCrStep \ref destination_planar_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr411_8u_P3P2R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr411_8u_P3P2RNative(pSrc, rSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI));
    }
    private static native int nppiYCbCr411_8u_P3P2RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI);


    public static int nppiYCbCr411_8u_P2P3R(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr411_8u_P2P3RNative(pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCbCr411_8u_P2P3RNative(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr411 to 3 channel 8-bit unsigned planar YCbCr422 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr411ToYCbCr422_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr411ToYCbCr422_8u_P3RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr411ToYCbCr422_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] nDstStep,
        NppiSize oSizeROI);


    public static int nppiYCbCr411ToYCbCr422_8u_P3C2R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr411ToYCbCr422_8u_P3C2RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr411ToYCbCr422_8u_P3C2RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned planar YCbCr411 to 3 channel 8-bit unsigned planar YCbCr422 sampling format conversion.
     *
     * @param pSrcY \ref source_planar_image_pointer.
     * @param nSrcYStep \ref source_planar_image_line_step.
     * @param pSrcCbCr \ref source_planar_image_pointer.
     * @param nSrcCbCrStep \ref source_planar_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr411ToYCbCr422_8u_P2P3R(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr411ToYCbCr422_8u_P2P3RNative(pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCbCr411ToYCbCr422_8u_P2P3RNative(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned planar YCbCr411 to 2 channel 8-bit unsigned packed YCbCr422 sampling format conversion.
     *
     * @param pSrcY \ref source_planar_image_pointer.
     * @param nSrcYStep \ref source_planar_image_line_step.
     * @param pSrcCbCr \ref source_planar_image_pointer.
     * @param nSrcCbCrStep \ref source_planar_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr411ToYCbCr422_8u_P2C2R(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr411ToYCbCr422_8u_P2C2RNative(pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr411ToYCbCr422_8u_P2C2RNative(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr411 to 3 channel 8-bit unsigned planar YCrCb422 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr411ToYCrCb422_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr411ToYCrCb422_8u_P3RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr411ToYCrCb422_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr411 to 2 channel 8-bit unsigned packed YCrCb422 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr411ToYCrCb422_8u_P3C2R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr411ToYCrCb422_8u_P3C2RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr411ToYCrCb422_8u_P3C2RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiYCbCr411ToYCbCr420_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr411ToYCbCr420_8u_P3RNative(pSrc, rSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiYCbCr411ToYCbCr420_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int[] nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar YCbCr411 to 2 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
     * images.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param rSrcStep \ref source_planar_image_line_step_array.
     * @param pDstY \ref destination_planar_image_pointer.
     * @param nDstYStep \ref destination_planar_image_line_step.
     * @param pDstCbCr \ref destination_planar_image_pointer.
     * @param nDstCbCrStep \ref destination_planar_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr411ToYCbCr420_8u_P3P2R(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr411ToYCbCr420_8u_P3P2RNative(pSrc, rSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI));
    }
    private static native int nppiYCbCr411ToYCbCr420_8u_P3P2RNative(
        TypedPointer<Npp8u>[] pSrc,
        int[] rSrcStep,
        TypedPointer<Npp8u> pDstY,
        int nDstYStep,
        TypedPointer<Npp8u> pDstCbCr,
        int nDstCbCrStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned planar YCbCr411 to 3 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
     *
     * @param pSrcY \ref source_planar_image_pointer.
     * @param nSrcYStep \ref source_planar_image_line_step.
     * @param pSrcCbCr \ref source_planar_image_pointer.
     * @param nSrcCbCrStep \ref source_planar_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr411ToYCbCr420_8u_P2P3R(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr411ToYCbCr420_8u_P2P3RNative(pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCbCr411ToYCbCr420_8u_P2P3RNative(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 2 channel 8-bit unsigned planar YCbCr411 to 3 channel 8-bit unsigned planar YCrCb420 sampling format conversion.
     *
     * @param pSrcY \ref source_planar_image_pointer.
     * @param nSrcYStep \ref source_planar_image_line_step.
     * @param pSrcCbCr \ref source_planar_image_pointer.
     * @param nSrcCbCrStep \ref source_planar_image_line_step.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param rDstStep \ref destination_planar_image_line_step_array.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiYCbCr411ToYCrCb420_8u_P2P3R(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiYCbCr411ToYCrCb420_8u_P2P3RNative(pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, rDstStep, oSizeROI));
    }
    private static native int nppiYCbCr411ToYCrCb420_8u_P2P3RNative(
        TypedPointer<Npp8u> pSrcY,
        int nSrcYStep,
        TypedPointer<Npp8u> pSrcCbCr,
        int nSrcCbCrStep,
        TypedPointer<Npp8u>[] pDst,
        int[] rDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed color not in place forward gamma correction.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiGammaFwd_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiGammaFwd_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiGammaFwd_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed color in place forward gamma correction.
     *
     * @param pSrcDst in place packed pixel image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiGammaFwd_8u_C3IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiGammaFwd_8u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiGammaFwd_8u_C3IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed color with alpha not in place forward gamma correction.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiGammaFwd_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiGammaFwd_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiGammaFwd_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiGammaFwd_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiGammaFwd_8u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiGammaFwd_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar color not in place forward gamma correction.
     *
     * @param pSrc source planar pixel format image pointer array.
     * @param nSrcStep source planar pixel format image line step.
     * @param pDst destination planar pixel format image pointer array.
     * @param nDstStep destination planar pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiGammaFwd_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiGammaFwd_8u_P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiGammaFwd_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar color in place forward gamma correction.
     *
     * @param pSrcDst in place planar pixel format image pointer array.
     * @param nSrcDstStep in place planar pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiGammaFwd_8u_IP3R(
        TypedPointer<Npp8u>[] pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiGammaFwd_8u_IP3RNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiGammaFwd_8u_IP3RNative(
        TypedPointer<Npp8u>[] pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiGammaInv_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiGammaInv_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiGammaInv_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed color in place inverse gamma correction.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiGammaInv_8u_C3IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiGammaInv_8u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiGammaInv_8u_C3IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiGammaInv_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiGammaInv_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiGammaInv_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed color with alpha in place inverse gamma correction.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiGammaInv_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiGammaInv_8u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiGammaInv_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar color not in place inverse gamma correction.
     *
     * @param pSrc source planar pixel format image pointer array.
     * @param nSrcStep source planar pixel format image line step.
     * @param pDst destination planar pixel format image pointer array.
     * @param nDstStep destination planar pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiGammaInv_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiGammaInv_8u_P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiGammaInv_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar color in place inverse gamma correction.
     *
     * @param pSrcDst in place planar pixel format image pointer array.
     * @param nSrcDstStep in place planar pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiGammaInv_8u_IP3R(
        TypedPointer<Npp8u>[] pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiGammaInv_8u_IP3RNative(pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiGammaInv_8u_IP3RNative(
        TypedPointer<Npp8u>[] pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 1 channel 8-bit unsigned packed color complement color key replacement of source image 1 by source image 2.
     *
     * @param pSrc1 source1 packed pixel format image pointer.
     * @param nSrc1Step source1 packed pixel format image line step.
     * @param pSrc2 source2 packed pixel format image pointer.
     * @param nSrc2Step source2 packed pixel format image line step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nColorKeyConst color key constant
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompColorKey_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nColorKeyConst)
    {
        return JNpp.checkResult(nppiCompColorKey_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nColorKeyConst));
    }
    private static native int nppiCompColorKey_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nColorKeyConst);


    /**
     * <pre>
     * 3 channel 8-bit unsigned packed color complement color key replacement of source image 1 by source image 2.
     *
     * @param pSrc1 source1 packed pixel format image pointer.
     * @param nSrc1Step source1 packed pixel format image line step.
     * @param pSrc2 source2 packed pixel format image pointer.
     * @param nSrc2Step source2 packed pixel format image line step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nColorKeyConst color key constant array
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompColorKey_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] nColorKeyConst)
    {
        return JNpp.checkResult(nppiCompColorKey_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nColorKeyConst));
    }
    private static native int nppiCompColorKey_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] nColorKeyConst);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed color complement color key replacement of source image 1 by source image 2.
     *
     * @param pSrc1 source1 packed pixel format image pointer.
     * @param nSrc1Step source1 packed pixel format image line step.
     * @param pSrc2 source2 packed pixel format image pointer.
     * @param nSrc2Step source2 packed pixel format image line step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nColorKeyConst color key constant array
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompColorKey_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] nColorKeyConst)
    {
        return JNpp.checkResult(nppiCompColorKey_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nColorKeyConst));
    }
    private static native int nppiCompColorKey_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] nColorKeyConst);


    /**
     * <pre>
     * 4 channel 8-bit unsigned packed color complement color key replacement of source image 1 by source image 2 with alpha blending.
     *
     * @param pSrc1 source1 packed pixel format image pointer.
     * @param nSrc1Step source1 packed pixel format image line step.
     * @param nAlpha1 source1 image alpha opacity (0 - max channel pixel value).
     * @param pSrc2 source2 packed pixel format image pointer.
     * @param nSrc2Step source2 packed pixel format image line step.
     * @param nAlpha2 source2 image alpha opacity (0 - max channel pixel value).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nColorKeyConst color key constant array
     * @param nppAlphaOp NppiAlphaOp alpha compositing operation selector  (excluding premul ops).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiAlphaCompColorKey_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        byte nAlpha2,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] nColorKeyConst,
        int nppAlphaOp)
    {
        return JNpp.checkResult(nppiAlphaCompColorKey_8u_AC4RNative(pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, nColorKeyConst, nppAlphaOp));
    }
    private static native int nppiAlphaCompColorKey_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        byte nAlpha1,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        byte nAlpha2,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] nColorKeyConst,
        int nppAlphaOp);


    /**
     * <pre>
     * 1 channel 8-bit unsigned color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 1 channel 8-bit unsigned in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8u_C1IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8u_C1IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 2 channel 8-bit unsigned color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8u_C2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8u_C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8u_C2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 2 channel 8-bit unsigned in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8u_C2IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8u_C2IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8u_C2IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    public static int nppiColorTwist32f_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 3 channel 8-bit unsigned in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8u_C3IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8u_C3IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 8-bit unsigned color twist, with alpha copy.
     *
     * An input color twist matrix with floating-point coefficient values is applied with
     * in ROI.
     * Alpha channel is the last channel and is copied unmodified from the source pixel to the destination pixel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 8-bit unsigned in place color twist, not affecting Alpha.
     *
     * An input color twist matrix with floating-point coefficient values is applied with
     * in ROI.
     * Alpha channel is the last channel and is unmodified.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8u_C4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8u_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8u_C4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 8-bit unsigned color twist, not affecting Alpha.
     *
     * An input color twist matrix with floating-point coefficient values is applied with
     * in ROI.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 8-bit unsigned in place color twist, not affecting Alpha.
     *
     * An input color twist matrix with floating-point coefficient values is applied with
     * in ROI.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 8-bit unsigned color twist with 4x4 matrix and constant vector addition.
     *
     * An input 4x4 color twist matrix with floating-point coefficient values with an additional constant vector addition
     * is applied within ROI.  For this particular version of the function the result is generated as shown below.
     *
     *  \code
     *      dst[0] = aTwist[0][0] * src[0] + aTwist[0][1] * src[1] + aTwist[0][2] * src[2] + aTwist[0][3] * src[3] + aConstants[0]
     *      dst[1] = aTwist[1][0] * src[0] + aTwist[1][1] * src[1] + aTwist[1][2] * src[2] + aTwist[1][3] * src[3] + aConstants[1]
     *      dst[2] = aTwist[2][0] * src[0] + aTwist[2][1] * src[1] + aTwist[2][2] * src[2] + aTwist[2][3] * src[3] + aConstants[2]
     *      dst[3] = aTwist[3][0] * src[0] + aTwist[3][1] * src[1] + aTwist[3][2] * src[2] + aTwist[3][3] * src[3] + aConstants[3]
     *  \endcode
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @param aConstants fixed size array of constant values, one per channel..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32fC_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist,
        float[] aConstants)
    {
        return JNpp.checkResult(nppiColorTwist32fC_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist, aConstants));
    }
    private static native int nppiColorTwist32fC_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist,
        float[] aConstants);


    public static int nppiColorTwist32fC_8u_C4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist,
        float[] aConstants)
    {
        return JNpp.checkResult(nppiColorTwist32fC_8u_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist, aConstants));
    }
    private static native int nppiColorTwist32fC_8u_C4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist,
        float[] aConstants);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8u_P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place planar pixel format image pointer array, one pointer per plane.
     * @param nSrcDstStep in place planar pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8u_IP3R(
        TypedPointer<Npp8u>[] pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8u_IP3RNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8u_IP3RNative(
        TypedPointer<Npp8u>[] pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 1 channel 8-bit signed color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 1 channel 8-bit signed in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8s_C1IR(
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8s_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8s_C1IRNative(
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 2 channel 8-bit signed color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8s_C2R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8s_C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8s_C2RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 2 channel 8-bit signed in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8s_C2IR(
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8s_C2IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8s_C2IRNative(
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 3 channel 8-bit signed color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8s_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8s_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 3 channel 8-bit signed in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8s_C3IR(
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8s_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8s_C3IRNative(
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 8-bit signed color twist, with alpha copy.
     *
     * An input color twist matrix with floating-point coefficient values is applied with
     * in ROI.
     * Alpha channel is the last channel and is copied unmodified from the source pixel to the destination pixel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8s_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8s_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 8-bit signed in place color twist, not affecting Alpha.
     *
     * An input color twist matrix with floating-point coefficient values is applied with
     * in ROI.
     * Alpha channel is the last channel and is unmodified.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8s_C4IR(
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8s_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8s_C4IRNative(
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 8-bit signed color twist, not affecting Alpha.
     *
     * An input color twist matrix with floating-point coefficient values is applied with
     * in ROI.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8s_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8s_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 8-bit signed in place color twist, not affecting Alpha.
     *
     * An input color twist matrix with floating-point coefficient values is applied with
     * in ROI.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8s_AC4IR(
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8s_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8s_AC4IRNative(
        TypedPointer<Npp8s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 3 channel 8-bit signed planar color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8s_P3R(
        TypedPointer<Npp8s>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8s>[] pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8s_P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8s_P3RNative(
        TypedPointer<Npp8s>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp8s>[] pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 3 channel 8-bit signed planar in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place planar pixel format image pointer array, one pointer per plane.
     * @param nSrcDstStep in place planar pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_8s_IP3R(
        TypedPointer<Npp8s>[] pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_8s_IP3RNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_8s_IP3RNative(
        TypedPointer<Npp8s>[] pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 1 channel 16-bit unsigned color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 1 channel 16-bit unsigned in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16u_C1IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16u_C1IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    public static int nppiColorTwist32f_16u_C2R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16u_C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16u_C2RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 2 channel 16-bit unsigned in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16u_C2IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16u_C2IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16u_C2IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 3 channel 16-bit unsigned color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 3 channel 16-bit unsigned in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16u_C3IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16u_C3IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    public static int nppiColorTwist32f_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 16-bit unsigned in place color twist, not affecting Alpha.
     *
     * An input color twist matrix with floating-point coefficient values is applied with
     * in ROI.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16u_AC4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 3 channel 16-bit unsigned planar color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16u_P3R(
        TypedPointer<Npp16u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16u_P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16u_P3RNative(
        TypedPointer<Npp16u>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 3 channel 16-bit unsigned planar in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place planar pixel format image pointer array, one pointer per plane.
     * @param nSrcDstStep in place planar pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16u_IP3R(
        TypedPointer<Npp16u>[] pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16u_IP3RNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16u_IP3RNative(
        TypedPointer<Npp16u>[] pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 1 channel 16-bit signed color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 1 channel 16-bit signed in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16s_C1IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16s_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16s_C1IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 2 channel 16-bit signed color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16s_C2R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16s_C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16s_C2RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 2 channel 16-bit signed in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16s_C2IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16s_C2IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16s_C2IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 3 channel 16-bit signed color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 3 channel 16-bit signed in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16s_C3IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16s_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16s_C3IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 16-bit signed color twist, not affecting Alpha.
     *
     * An input color twist matrix with floating-point coefficient values is applied with
     * in ROI.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 16-bit signed in place color twist, not affecting Alpha.
     *
     * An input color twist matrix with floating-point coefficient values is applied with
     * in ROI.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16s_AC4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16s_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16s_AC4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 3 channel 16-bit signed planar color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16s_P3R(
        TypedPointer<Npp16s>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp16s>[] pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16s_P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16s_P3RNative(
        TypedPointer<Npp16s>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp16s>[] pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 3 channel 16-bit signed planar in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place planar pixel format image pointer array, one pointer per plane.
     * @param nSrcDstStep in place planar pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist32f_16s_IP3R(
        TypedPointer<Npp16s>[] pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist32f_16s_IP3RNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist32f_16s_IP3RNative(
        TypedPointer<Npp16s>[] pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 1 channel 32-bit floating point color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 1 channel 32-bit floating point in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist_32f_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 2 channel 32-bit floating point color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist_32f_C2R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist_32f_C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist_32f_C2RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 2 channel 32-bit floating point in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist_32f_C2IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist_32f_C2IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist_32f_C2IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 3 channel 32-bit floating point color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 3 channel 32-bit floating point in place color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 32-bit floating point color twist, with alpha copy.
     *
     * An input color twist matrix with floating-point coefficient values is applied with
     * in ROI.
     * Alpha channel is the last channel and is copied unmodified from the source pixel to the destination pixel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 32-bit floating point in place color twist, not affecting Alpha.
     *
     * An input color twist matrix with floating-point coefficient values is applied with
     * in ROI.
     * Alpha channel is the last channel and is not modified.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist_32f_C4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist_32f_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist_32f_C4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 32-bit floating point color twist, not affecting Alpha.
     *
     * An input color twist matrix with floating-point coefficient values is applied with
     * in ROI.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 32-bit floating point in place color twist, not affecting Alpha.
     *
     * An input color twist matrix with floating-point coefficient values is applied with
     * in ROI.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist_32f_AC4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist_32f_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 4 channel 32-bit floating point color twist with 4x4 matrix and constant vector addition.
     *
     * An input 4x4 color twist matrix with floating-point coefficient values with an additional constant vector addition
     * is applied within ROI.  For this particular version of the function the result is generated as shown below.
     *
     *  \code
     *      dst[0] = aTwist[0][0] * src[0] + aTwist[0][1] * src[1] + aTwist[0][2] * src[2] + aTwist[0][3] * src[3] + aConstants[0]
     *      dst[1] = aTwist[1][0] * src[0] + aTwist[1][1] * src[1] + aTwist[1][2] * src[2] + aTwist[1][3] * src[3] + aConstants[1]
     *      dst[2] = aTwist[2][0] * src[0] + aTwist[2][1] * src[1] + aTwist[2][2] * src[2] + aTwist[2][3] * src[3] + aConstants[2]
     *      dst[3] = aTwist[3][0] * src[0] + aTwist[3][1] * src[1] + aTwist[3][2] * src[2] + aTwist[3][3] * src[3] + aConstants[3]
     *  \endcode
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @param aConstants fixed size array of constant values, one per channel..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist_32fC_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist,
        float[] aConstants)
    {
        return JNpp.checkResult(nppiColorTwist_32fC_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist, aConstants));
    }
    private static native int nppiColorTwist_32fC_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist,
        float[] aConstants);


    /**
     * <pre>
     * 4 channel 32-bit floating point in place color twist with 4x4 matrix and an additional constant vector addition.
     *
     * An input 4x4 color twist matrix with floating-point coefficient values with an additional constant vector addition
     * is applied within ROI.  For this particular version of the function the result is generated as shown below.
     *
     *  \code
     *      dst[0] = aTwist[0][0] * src[0] + aTwist[0][1] * src[1] + aTwist[0][2] * src[2] + aTwist[0][3] * src[3] + aConstants[0]
     *      dst[1] = aTwist[1][0] * src[0] + aTwist[1][1] * src[1] + aTwist[1][2] * src[2] + aTwist[1][3] * src[3] + aConstants[1]
     *      dst[2] = aTwist[2][0] * src[0] + aTwist[2][1] * src[1] + aTwist[2][2] * src[2] + aTwist[2][3] * src[3] + aConstants[2]
     *      dst[3] = aTwist[3][0] * src[0] + aTwist[3][1] * src[1] + aTwist[3][2] * src[2] + aTwist[3][3] * src[3] + aConstants[3]
     *  \endcode
     *
     * @param pSrcDst in place packed pixel format image pointer.
     * @param nSrcDstStep in place packed pixel format image line step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @param aConstants fixed size array of constant values, one per channel..
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist_32fC_C4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist,
        float[] aConstants)
    {
        return JNpp.checkResult(nppiColorTwist_32fC_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist, aConstants));
    }
    private static native int nppiColorTwist_32fC_C4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist,
        float[] aConstants);


    /**
     * <pre>
     * 3 channel 32-bit floating point planar color twist.
     *
     * An input color twist matrix with floating-point coefficient values is applied
     * within ROI.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aTwist The color twist matrix with floating-point coefficient values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiColorTwist_32f_P3R(
        TypedPointer<Npp32f>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist_32f_P3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist_32f_P3RNative(
        TypedPointer<Npp32f>[] pSrc,
        int nSrcStep,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    public static int nppiColorTwist_32f_IP3R(
        TypedPointer<Npp32f>[] pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist)
    {
        return JNpp.checkResult(nppiColorTwist_32f_IP3RNative(pSrcDst, nSrcDstStep, oSizeROI, aTwist));
    }
    private static native int nppiColorTwist_32f_IP3RNative(
        TypedPointer<Npp32f>[] pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[][] aTwist);


    /**
     * <pre>
     * 8-bit unsigned look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points with no interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    /**
     * <pre>
     * 8-bit unsigned look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points with no interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_8u_C1IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_8u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_8u_C1IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    public static int nppiLUT_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    public static int nppiLUT_8u_C3IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_8u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_8u_C3IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    public static int nppiLUT_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 8-bit unsigned look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_8u_C4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_8u_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_8u_C4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    public static int nppiLUT_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 8-bit unsigned look-up-table in place color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_8u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    public static int nppiLUT_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    /**
     * <pre>
     * 16-bit unsigned look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points with no interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_16u_C1IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_16u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_16u_C1IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    /**
     * <pre>
     * 3 channel 16-bit unsigned look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    public static int nppiLUT_16u_C3IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_16u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_16u_C3IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit unsigned look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points with no interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit unsigned look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_16u_C4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_16u_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_16u_C4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit unsigned look-up-table color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit unsigned look-up-table in place color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_16u_AC4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_16u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 16-bit signed look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points with no interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    /**
     * <pre>
     * 16-bit signed look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points with no interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_16s_C1IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_16s_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_16s_C1IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    /**
     * <pre>
     * 3 channel 16-bit signed look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 3 channel 16-bit signed look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points with no interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_16s_C3IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_16s_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_16s_C3IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit signed look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points with no interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit signed look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_16s_C4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_16s_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_16s_C4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit signed look-up-table color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit signed look-up-table in place color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_16s_AC4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_16s_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_16s_AC4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 32-bit floating point look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points with no interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pValues,
        TypedPointer<Npp32f> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pValues,
        TypedPointer<Npp32f> pLevels,
        int nLevels);


    /**
     * <pre>
     * 32-bit floating point look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points with no interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pValues,
        TypedPointer<Npp32f> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_32f_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pValues,
        TypedPointer<Npp32f> pLevels,
        int nLevels);


    /**
     * <pre>
     * 3 channel 32-bit floating point look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 3 channel 32-bit floating point look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points with no interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 32-bit floating point look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points with no interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 32-bit floating point look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_32f_C4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_32f_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_32f_C4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 32-bit floating point look-up-table color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 32-bit floating point look-up-table in place color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_32f_AC4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_32f_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 8-bit unsigned linear interpolated look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through linear interpolation.
     *
     * >>>>>>> ATTENTION ATTENTION <<<<<<<
     *
     * NOTE: As of the 5.0 release of NPP, the pValues and pLevels pointers need to be device memory pointers.
     *
     * >>>>>>>                     <<<<<<<
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is now a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is now a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_Linear_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    public static int nppiLUT_Linear_8u_C1IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_8u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_8u_C1IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    /**
     * <pre>
     * 3 channel 8-bit unsigned linear interpolated look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through linear interpolation.
     *
     * >>>>>>> ATTENTION ATTENTION <<<<<<<
     *
     * NOTE: As of the 5.0 release of NPP, the pValues and pLevels pointers need to be host memory pointers to arrays of device memory pointers.
     *
     * >>>>>>>                     <<<<<<<
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_Linear_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 3 channel 8-bit unsigned linear interpolated look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through linear interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_Linear_8u_C3IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_8u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_8u_C3IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 8-bit unsigned linear interpolated look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through linear interpolation.
     *
     * >>>>>>> ATTENTION ATTENTION <<<<<<<
     *
     * NOTE: As of the 5.0 release of NPP, the pValues and pLevels pointers need to be host memory pointers to arrays of device memory pointers.
     *
     * >>>>>>>                     <<<<<<<
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_Linear_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 8-bit unsigned linear interpolated look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through linear interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_Linear_8u_C4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_8u_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_8u_C4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    public static int nppiLUT_Linear_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    public static int nppiLUT_Linear_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_8u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    public static int nppiLUT_Linear_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    /**
     * <pre>
     * 16-bit unsigned look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using linear interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_16u_C1IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_16u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_16u_C1IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    /**
     * <pre>
     * 3 channel 16-bit unsigned look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 3 channel 16-bit unsigned look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using linear interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_16u_C3IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_16u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_16u_C3IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit unsigned look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using linear interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit unsigned look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_16u_C4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_16u_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_16u_C4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit unsigned look-up-table color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit unsigned look-up-table in place color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_16u_AC4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_16u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 16-bit signed look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using linear interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    public static int nppiLUT_Linear_16s_C1IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_16s_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_16s_C1IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    /**
     * <pre>
     * 3 channel 16-bit signed look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 3 channel 16-bit signed look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using linear interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_16s_C3IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_16s_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_16s_C3IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit signed look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using linear interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    public static int nppiLUT_Linear_16s_C4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_16s_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_16s_C4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit signed look-up-table color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit signed look-up-table in place color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_16s_AC4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_16s_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_16s_AC4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 32-bit floating point look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using linear interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pValues,
        TypedPointer<Npp32f> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pValues,
        TypedPointer<Npp32f> pLevels,
        int nLevels);


    /**
     * <pre>
     * 32-bit floating point look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using linear interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pValues,
        TypedPointer<Npp32f> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_32f_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pValues,
        TypedPointer<Npp32f> pLevels,
        int nLevels);


    /**
     * <pre>
     * 3 channel 32-bit floating point look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 3 channel 32-bit floating point look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using linear interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    public static int nppiLUT_Linear_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 32-bit floating point look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_32f_C4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_32f_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_32f_C4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 32-bit floating point look-up-table color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 32-bit floating point look-up-table in place color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Linear_32f_AC4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Linear_32f_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Linear_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    public static int nppiLUT_Cubic_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    /**
     * <pre>
     * 8-bit unsigned cubic interpolated look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_Cubic_8u_C1IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_8u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_8u_C1IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    /**
     * <pre>
     * 3 channel 8-bit unsigned cubic interpolated look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_Cubic_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 3 channel 8-bit unsigned cubic interpolated look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_Cubic_8u_C3IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_8u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_8u_C3IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 8-bit unsigned cubic interpolated look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_Cubic_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 8-bit unsigned cubic interpolated look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_Cubic_8u_C4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_8u_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_8u_C4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 8-bit unsigned cubic interpolated look-up-table color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_Cubic_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 8-bit unsigned cubic interpolated look-up-table in place color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_Cubic_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_8u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    public static int nppiLUT_Cubic_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    /**
     * <pre>
     * 16-bit unsigned look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_16u_C1IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_16u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_16u_C1IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    /**
     * <pre>
     * 3 channel 16-bit unsigned look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 3 channel 16-bit unsigned look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_16u_C3IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_16u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_16u_C3IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit unsigned look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit unsigned look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_16u_C4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_16u_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_16u_C4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit unsigned look-up-table color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit unsigned look-up-table in place color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_16u_AC4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_16u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 16-bit signed look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    /**
     * <pre>
     * 16-bit signed look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_16s_C1IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_16s_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_16s_C1IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pValues,
        TypedPointer<Npp32s> pLevels,
        int nLevels);


    /**
     * <pre>
     * 3 channel 16-bit signed look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 3 channel 16-bit signed look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_16s_C3IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_16s_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_16s_C3IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    public static int nppiLUT_Cubic_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit signed look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_16s_C4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_16s_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_16s_C4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit signed look-up-table color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 16-bit signed look-up-table in place color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_16s_AC4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_16s_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_16s_AC4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pValues,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 32-bit floating point look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pValues,
        TypedPointer<Npp32f> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pValues,
        TypedPointer<Npp32f> pLevels,
        int nLevels);


    /**
     * <pre>
     * 32-bit floating point look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
     * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
     * @param nLevels Number of user defined number of input/output mapping points (levels)
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pValues,
        TypedPointer<Npp32f> pLevels,
        int nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_32f_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pValues,
        TypedPointer<Npp32f> pLevels,
        int nLevels);


    /**
     * <pre>
     * 3 channel 32-bit floating point look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 3 channel 32-bit floating point look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 32-bit floating point look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points through cubic interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 32-bit floating point look-up-table in place color conversion.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_32f_C4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_32f_C4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_32f_C4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 32-bit floating point look-up-table color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * 4 channel 32-bit floating point look-up-table in place color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points using no interpolation.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
     * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
     * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
     * </pre>
     */
    public static int nppiLUT_Cubic_32f_AC4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels)
    {
        return JNpp.checkResult(nppiLUT_Cubic_32f_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels));
    }
    private static native int nppiLUT_Cubic_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f>[] pValues,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels);


    /**
     * <pre>
     * Four channel 8-bit unsigned 3D trilinear interpolated look-up-table color conversion, with alpha copy.
     * Alpha channel is the last channel and is copied to the destination unmodified.
     *
     * The LUT is derived from a set of user defined mapping points through trilinear interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Device pointer to aLevels[2] number of contiguous 2D x,y planes of 4-byte packed RGBX values
     *        containing the user defined base OUTPUT values at that x,y, and z (R,G,B) level location. Each level must contain x * y 4-byte
     *        packed pixel values (4th byte is used for alignement only and is ignored) in row (x) order.
     * @param pLevels Host pointer to an array of 3 host pointers, one per cube edge, pointing to user defined INPUT level values.
     * @param aLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per 3D cube edge.
     *        aLevels[0] represents the number of x axis levels (Red), aLevels[1] represents the number of y axis levels (Green),
     *        and aLevels[2] represets the number of z axis levels (Blue).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_Trilinear_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32u> pValues,
        TypedPointer<Npp8u>[] pLevels,
        int[] aLevels)
    {
        return JNpp.checkResult(nppiLUT_Trilinear_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, aLevels));
    }
    private static native int nppiLUT_Trilinear_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32u> pValues,
        TypedPointer<Npp8u>[] pLevels,
        int[] aLevels);


    /**
     * <pre>
     * Four channel 8-bit unsigned 3D trilinear interpolated look-up-table color conversion, not affecting alpha.
     * Alpha channel is the last channel and is not processed.
     *
     * The LUT is derived from a set of user defined mapping points through trilinear interpolation.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Device pointer to aLevels[2] number of contiguous 2D x,y planes of 4-byte packed RGBX values
     *        containing the user defined base OUTPUT values at that x,y, and z (R,G,B) level location. Each level must contain x * y 4-byte
     *        packed pixel values (4th byte is used for alignement only and is ignored) in row (x) order.
     * @param pLevels Host pointer to an array of 3 host pointers, one per cube edge, pointing to user defined INPUT level values.
     * @param aLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per 3D cube edge.
     *        aLevels[0] represents the number of x axis levels (Red), aLevels[1] represents the number of y axis levels (Green),
     *        and aLevels[2] represets the number of z axis levels (Blue).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_Trilinear_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32u> pValues,
        TypedPointer<Npp8u>[] pLevels,
        int[] aLevels)
    {
        return JNpp.checkResult(nppiLUT_Trilinear_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, aLevels));
    }
    private static native int nppiLUT_Trilinear_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32u> pValues,
        TypedPointer<Npp8u>[] pLevels,
        int[] aLevels);


    /**
     * <pre>
     * Four channel 8-bit unsigned 3D trilinear interpolated look-up-table in place color conversion, not affecting alpha.
     * Alpha channel is the last channel and is not processed.
     *
     * The LUT is derived from a set of user defined mapping points through trilinear interpolation.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pValues Device pointer aLevels[2] number of contiguous 2D x,y planes of 4-byte packed RGBX values
     *        containing the user defined base OUTPUT values at that x,y, and z (R,G,B) level location. Each level must contain x * y 4-byte
     *        packed pixel values (4th byte is used for alignement only and is ignored) in row (x) order.
     * @param pLevels Host pointer to an array of 3 host pointers, one per cube edge, pointing to user defined INPUT level values.
     * @param aLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per 3D cube edge.
     *        aLevels[0] represents the number of x axis levels (Red), aLevels[1] represents the number of y axis levels (Green),
     *        and aLevels[2] represets the number of z axis levels (Blue).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
     * </pre>
     */
    public static int nppiLUT_Trilinear_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32u> pValues,
        TypedPointer<Npp8u>[] pLevels,
        int[] aLevels)
    {
        return JNpp.checkResult(nppiLUT_Trilinear_8u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, aLevels));
    }
    private static native int nppiLUT_Trilinear_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32u> pValues,
        TypedPointer<Npp8u>[] pLevels,
        int[] aLevels);


    /**
     * <pre>
     * One channel 8-bit unsigned bit range restricted palette look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points in a palette and
     * source pixels are then processed using a restricted bit range when looking up palette values.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pTable Pointer to an array of user defined OUTPUT palette values (this is a device memory pointer)
     * @param nBitSize Number of least significant bits (must be > 0 and <= 8) of each source pixel value to use as index into palette table during conversion.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 8.
     * </pre>
     */
    public static int nppiLUTPalette_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pTable,
        int nBitSize)
    {
        return JNpp.checkResult(nppiLUTPalette_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTable, nBitSize));
    }
    private static native int nppiLUTPalette_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pTable,
        int nBitSize);


    /**
     * <pre>
     * One channel 8-bit unsigned bit range restricted 24-bit palette look-up-table color conversion with 24-bit destination output per pixel.
     *
     * The LUT is derived from a set of user defined mapping points in a palette and
     * source pixels are then processed using a restricted bit range when looking up palette values.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step (3 bytes per pixel).
     * @param oSizeROI \ref roi_specification.
     * @param pTable Pointer to an array of user defined OUTPUT palette values (this is a device memory pointer)
     * @param nBitSize Number of least significant bits (must be > 0 and <= 8) of each source pixel value to use as index into palette table during conversion.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 8.
     * </pre>
     */
    public static int nppiLUTPalette_8u24u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pTable,
        int nBitSize)
    {
        return JNpp.checkResult(nppiLUTPalette_8u24u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTable, nBitSize));
    }
    private static native int nppiLUTPalette_8u24u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pTable,
        int nBitSize);


    /**
     * <pre>
     * One channel 8-bit unsigned bit range restricted 32-bit palette look-up-table color conversion with 32-bit destination output per pixel.
     *
     * The LUT is derived from a set of user defined mapping points in a palette and
     * source pixels are then processed using a restricted bit range when looking up palette values.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step (4 bytes per pixel).
     * @param oSizeROI \ref roi_specification.
     * @param pTable Pointer to an array of user defined OUTPUT palette values (this is a device memory pointer)
     * @param nBitSize Number of least significant bits (must be > 0 and <= 8) of each source pixel value to use as index into palette table during conversion.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 8.
     * </pre>
     */
    public static int nppiLUTPalette_8u32u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32u> pTable,
        int nBitSize)
    {
        return JNpp.checkResult(nppiLUTPalette_8u32u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTable, nBitSize));
    }
    private static native int nppiLUTPalette_8u32u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32u> pTable,
        int nBitSize);


    /**
     * <pre>
     * Three channel 8-bit unsigned bit range restricted palette look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points in a palette and
     * source pixels are then processed using a restricted bit range when looking up palette values.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pTables Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT palette values.
     * @param nBitSize Number of least significant bits (must be > 0 and <= 8) of each source pixel value to use as index into palette table during conversion.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 8.
     * </pre>
     */
    public static int nppiLUTPalette_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u>[] pTables,
        int nBitSize)
    {
        return JNpp.checkResult(nppiLUTPalette_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTables, nBitSize));
    }
    private static native int nppiLUTPalette_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u>[] pTables,
        int nBitSize);


    /**
     * <pre>
     * Four channel 8-bit unsigned bit range restricted palette look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points in a palette and
     * source pixels are then processed using a restricted bit range when looking up palette values.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pTables Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT palette values.
     * @param nBitSize Number of least significant bits (must be > 0 and <= 8) of each source pixel value to use as index into palette table during conversion.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 8.
     * </pre>
     */
    public static int nppiLUTPalette_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u>[] pTables,
        int nBitSize)
    {
        return JNpp.checkResult(nppiLUTPalette_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTables, nBitSize));
    }
    private static native int nppiLUTPalette_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u>[] pTables,
        int nBitSize);


    /**
     * <pre>
     * Four channel 8-bit unsigned bit range restricted palette look-up-table color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points in a palette and
     * source pixels are then processed using a restricted bit range when looking up palette values.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pTables Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT palette values.
     * @param nBitSize Number of least significant bits (must be > 0 and <= 8) of each source pixel value to use as index into palette table during conversion.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 8.
     * </pre>
     */
    public static int nppiLUTPalette_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u>[] pTables,
        int nBitSize)
    {
        return JNpp.checkResult(nppiLUTPalette_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTables, nBitSize));
    }
    private static native int nppiLUTPalette_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u>[] pTables,
        int nBitSize);


    /**
     * <pre>
     * One channel 16-bit unsigned bit range restricted palette look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points in a palette and
     * source pixels are then processed using a restricted bit range when looking up palette values.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pTable Pointer to an array of user defined OUTPUT palette values (this is a device memory pointer)
     * @param nBitSize Number of least significant bits (must be > 0 and <= 16) of each source pixel value to use as index into palette table during conversion.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 16.
     * </pre>
     */
    public static int nppiLUTPalette_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16u> pTable,
        int nBitSize)
    {
        return JNpp.checkResult(nppiLUTPalette_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTable, nBitSize));
    }
    private static native int nppiLUTPalette_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16u> pTable,
        int nBitSize);


    /**
     * <pre>
     * One channel 16-bit unsigned bit range restricted 8-bit unsigned palette look-up-table color conversion with 8-bit unsigned destination output per pixel.
     *
     * The LUT is derived from a set of user defined mapping points in a palette and
     * source pixels are then processed using a restricted bit range when looking up palette values.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step (1 unsigned byte per pixel).
     * @param oSizeROI \ref roi_specification.
     * @param pTable Pointer to an array of user defined OUTPUT palette values (this is a device memory pointer)
     * @param nBitSize Number of least significant bits (must be > 0 and <= 16) of each source pixel value to use as index into palette table during conversion.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 16.
     * </pre>
     */
    public static int nppiLUTPalette_16u8u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pTable,
        int nBitSize)
    {
        return JNpp.checkResult(nppiLUTPalette_16u8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTable, nBitSize));
    }
    private static native int nppiLUTPalette_16u8u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pTable,
        int nBitSize);


    /**
     * <pre>
     * One channel 16-bit unsigned bit range restricted 24-bit unsigned palette look-up-table color conversion with 24-bit unsigned destination output per pixel.
     *
     * The LUT is derived from a set of user defined mapping points in a palette and
     * source pixels are then processed using a restricted bit range when looking up palette values.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step (3 unsigned bytes per pixel).
     * @param oSizeROI \ref roi_specification.
     * @param pTable Pointer to an array of user defined OUTPUT palette values (this is a device memory pointer)
     * @param nBitSize Number of least significant bits (must be > 0 and <= 16) of each source pixel value to use as index into palette table during conversion.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 16.
     * </pre>
     */
    public static int nppiLUTPalette_16u24u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pTable,
        int nBitSize)
    {
        return JNpp.checkResult(nppiLUTPalette_16u24u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTable, nBitSize));
    }
    private static native int nppiLUTPalette_16u24u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pTable,
        int nBitSize);


    /**
     * <pre>
     * One channel 16-bit unsigned bit range restricted 32-bit palette look-up-table color conversion with 32-bit unsigned destination output per pixel.
     *
     * The LUT is derived from a set of user defined mapping points in a palette and
     * source pixels are then processed using a restricted bit range when looking up palette values.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step (4 bytes per pixel).
     * @param oSizeROI \ref roi_specification.
     * @param pTable Pointer to an array of user defined OUTPUT palette values (this is a device memory pointer)
     * @param nBitSize Number of least significant bits (must be > 0 and <= 16) of each source pixel value to use as index into palette table during conversion.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 16.
     * </pre>
     */
    public static int nppiLUTPalette_16u32u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32u> pTable,
        int nBitSize)
    {
        return JNpp.checkResult(nppiLUTPalette_16u32u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTable, nBitSize));
    }
    private static native int nppiLUTPalette_16u32u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32u> pTable,
        int nBitSize);


    /**
     * <pre>
     * Three channel 16-bit unsigned bit range restricted palette look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points in a palette and
     * source pixels are then processed using a restricted bit range when looking up palette values.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pTables Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT palette values.
     * @param nBitSize Number of least significant bits (must be > 0 and <= 16) of each source pixel value to use as index into palette table during conversion.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 16.
     * </pre>
     */
    public static int nppiLUTPalette_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16u>[] pTables,
        int nBitSize)
    {
        return JNpp.checkResult(nppiLUTPalette_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTables, nBitSize));
    }
    private static native int nppiLUTPalette_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16u>[] pTables,
        int nBitSize);


    /**
     * <pre>
     * Four channel 16-bit unsigned bit range restricted palette look-up-table color conversion.
     *
     * The LUT is derived from a set of user defined mapping points in a palette and
     * source pixels are then processed using a restricted bit range when looking up palette values.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pTables Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT palette values.
     * @param nBitSize Number of least significant bits (must be > 0 and <= 16) of each source pixel value to use as index into palette table during conversion.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 16.
     * </pre>
     */
    public static int nppiLUTPalette_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16u>[] pTables,
        int nBitSize)
    {
        return JNpp.checkResult(nppiLUTPalette_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTables, nBitSize));
    }
    private static native int nppiLUTPalette_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16u>[] pTables,
        int nBitSize);


    /**
     * <pre>
     * Four channel 16-bit unsigned bit range restricted palette look-up-table color conversion, not affecting Alpha.
     *
     * The LUT is derived from a set of user defined mapping points in a palette and
     * source pixels are then processed using a restricted bit range when looking up palette values.
     * Alpha channel is the last channel and is not processed.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pTables Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT palette values.
     * @param nBitSize Number of least significant bits (must be > 0 and <= 16) of each source pixel value to use as index into palette table during conversion.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 16.
     * </pre>
     */
    public static int nppiLUTPalette_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16u>[] pTables,
        int nBitSize)
    {
        return JNpp.checkResult(nppiLUTPalette_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTables, nBitSize));
    }
    private static native int nppiLUTPalette_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16u>[] pTables,
        int nBitSize);


    /**
     * <pre>
     * Three channel 8-bit unsigned source bit range restricted palette look-up-table color conversion to four channel 8-bit unsigned destination output with alpha.
     *
     * The LUT is derived from a set of user defined mapping points in a palette and
     * source pixels are then processed using a restricted bit range when looking up palette values.
     * This function also reverses the source pixel channel order in the destination so the Alpha channel is the first channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step (3 bytes per pixel).
     * @param nAlphaValue Signed alpha value that will be used to initialize the pixel alpha channel position in all modified destination pixels.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step (4 bytes per pixel with alpha).
     * @param oSizeROI \ref roi_specification.
     * @param pTables Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT palette values.
     * Alpha values < 0 or > 255 will cause destination pixel alpha channel values to be unmodified.
     * @param nBitSize Number of least significant bits (must be > 0 and <= 8) of each source pixel value to use as index into palette table during conversion.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 8.
     * </pre>
     */
    public static int nppiLUTPaletteSwap_8u_C3A0C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        int nAlphaValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u>[] pTables,
        int nBitSize)
    {
        return JNpp.checkResult(nppiLUTPaletteSwap_8u_C3A0C4RNative(pSrc, nSrcStep, nAlphaValue, pDst, nDstStep, oSizeROI, pTables, nBitSize));
    }
    private static native int nppiLUTPaletteSwap_8u_C3A0C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        int nAlphaValue,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u>[] pTables,
        int nBitSize);


    /**
     * <pre>
     * Three channel 16-bit unsigned source bit range restricted palette look-up-table color conversion to four channel 16-bit unsigned destination output with alpha.
     *
     * The LUT is derived from a set of user defined mapping points in a palette and
     * source pixels are then processed using a restricted bit range when looking up palette values.
     * This function also reverses the source pixel channel order in the destination so the Alpha channel is the first channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step (3 unsigned short integers per pixel).
     * @param nAlphaValue Signed alpha value that will be used to initialize the pixel alpha channel position in all modified destination pixels.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step (4 unsigned short integers per pixel with alpha).
     * @param oSizeROI \ref roi_specification.
     * @param pTables Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT palette values.
     * Alpha values < 0 or > 65535 will cause destination pixel alpha channel values to be unmodified.
     * @param nBitSize Number of least significant bits (must be > 0 and <= 16) of each source pixel value to use as index into palette table during conversion.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 16.
     * </pre>
     */
    public static int nppiLUTPaletteSwap_16u_C3A0C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        int nAlphaValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16u>[] pTables,
        int nBitSize)
    {
        return JNpp.checkResult(nppiLUTPaletteSwap_16u_C3A0C4RNative(pSrc, nSrcStep, nAlphaValue, pDst, nDstStep, oSizeROI, pTables, nBitSize));
    }
    private static native int nppiLUTPaletteSwap_16u_C3A0C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        int nAlphaValue,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16u>[] pTables,
        int nBitSize);


    public static int nppiThreshold_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nThreshold,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, eComparisonOperation));
    }
    private static native int nppiThreshold_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nThreshold,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 8-bit unsigned char in place threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_8u_C1IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte nThreshold,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_8u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold, eComparisonOperation));
    }
    private static native int nppiThreshold_8u_C1IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte nThreshold,
        int eComparisonOperation);


    public static int nppiThreshold_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, eComparisonOperation));
    }
    private static native int nppiThreshold_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 16-bit unsigned short in place threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_16u_C1IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_16u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold, eComparisonOperation));
    }
    private static native int nppiThreshold_16u_C1IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 16-bit signed short threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, eComparisonOperation));
    }
    private static native int nppiThreshold_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 16-bit signed short in place threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_16s_C1IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_16s_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold, eComparisonOperation));
    }
    private static native int nppiThreshold_16s_C1IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        int eComparisonOperation);


    public static int nppiThreshold_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nThreshold,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, eComparisonOperation));
    }
    private static native int nppiThreshold_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nThreshold,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 32-bit floating point in place threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nThreshold,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_32f_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold, eComparisonOperation));
    }
    private static native int nppiThreshold_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nThreshold,
        int eComparisonOperation);


    public static int nppiThreshold_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, eComparisonOperation));
    }
    private static native int nppiThreshold_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 8-bit unsigned char in place threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_8u_C3IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_8u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, eComparisonOperation));
    }
    private static native int nppiThreshold_8u_C3IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 16-bit unsigned short threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, eComparisonOperation));
    }
    private static native int nppiThreshold_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 16-bit unsigned short in place threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_16u_C3IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_16u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, eComparisonOperation));
    }
    private static native int nppiThreshold_16u_C3IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 16-bit signed short threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, eComparisonOperation));
    }
    private static native int nppiThreshold_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 16-bit signed short in place threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_16s_C3IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_16s_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, eComparisonOperation));
    }
    private static native int nppiThreshold_16s_C3IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 32-bit floating point threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, eComparisonOperation));
    }
    private static native int nppiThreshold_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 32-bit floating point in place threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, eComparisonOperation));
    }
    private static native int nppiThreshold_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 8-bit unsigned char image threshold, not affecting Alpha.
     * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
     * value is set to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, eComparisonOperation));
    }
    private static native int nppiThreshold_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 8-bit unsigned char in place image threshold, not affecting Alpha.
     * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
     * value is set to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_8u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, eComparisonOperation));
    }
    private static native int nppiThreshold_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        int eComparisonOperation);


    public static int nppiThreshold_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, eComparisonOperation));
    }
    private static native int nppiThreshold_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 16-bit unsigned short in place image threshold, not affecting Alpha.
     * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
     * value is set to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_16u_AC4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_16u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, eComparisonOperation));
    }
    private static native int nppiThreshold_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 16-bit signed short image threshold, not affecting Alpha.
     * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
     * value is set to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, eComparisonOperation));
    }
    private static native int nppiThreshold_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 16-bit signed short in place image threshold, not affecting Alpha.
     * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
     * value is set to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_16s_AC4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_16s_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, eComparisonOperation));
    }
    private static native int nppiThreshold_16s_AC4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        int eComparisonOperation);


    public static int nppiThreshold_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, eComparisonOperation));
    }
    private static native int nppiThreshold_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        int eComparisonOperation);


    public static int nppiThreshold_32f_AC4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_32f_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, eComparisonOperation));
    }
    private static native int nppiThreshold_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 8-bit unsigned char threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nThreshold)
    {
        return JNpp.checkResult(nppiThreshold_GT_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold));
    }
    private static native int nppiThreshold_GT_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nThreshold);


    /**
     * <pre>
     * 1 channel 8-bit unsigned char in place threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_8u_C1IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte nThreshold)
    {
        return JNpp.checkResult(nppiThreshold_GT_8u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold));
    }
    private static native int nppiThreshold_GT_8u_C1IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte nThreshold);


    /**
     * <pre>
     * 1 channel 16-bit unsigned short threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold)
    {
        return JNpp.checkResult(nppiThreshold_GT_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold));
    }
    private static native int nppiThreshold_GT_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold);


    /**
     * <pre>
     * 1 channel 16-bit unsigned short in place threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_16u_C1IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold)
    {
        return JNpp.checkResult(nppiThreshold_GT_16u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold));
    }
    private static native int nppiThreshold_GT_16u_C1IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold);


    /**
     * <pre>
     * 1 channel 16-bit signed short threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold)
    {
        return JNpp.checkResult(nppiThreshold_GT_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold));
    }
    private static native int nppiThreshold_GT_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold);


    /**
     * <pre>
     * 1 channel 16-bit signed short in place threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_16s_C1IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold)
    {
        return JNpp.checkResult(nppiThreshold_GT_16s_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold));
    }
    private static native int nppiThreshold_GT_16s_C1IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold);


    /**
     * <pre>
     * 1 channel 32-bit floating point threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nThreshold)
    {
        return JNpp.checkResult(nppiThreshold_GT_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold));
    }
    private static native int nppiThreshold_GT_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nThreshold);


    /**
     * <pre>
     * 1 channel 32-bit floating point in place threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nThreshold)
    {
        return JNpp.checkResult(nppiThreshold_GT_32f_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold));
    }
    private static native int nppiThreshold_GT_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nThreshold);


    public static int nppiThreshold_GT_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_GT_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_GT_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds);


    /**
     * <pre>
     * 3 channel 8-bit unsigned char in place threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_8u_C3IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_GT_8u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_GT_8u_C3IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds);


    public static int nppiThreshold_GT_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_GT_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_GT_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds);


    /**
     * <pre>
     * 3 channel 16-bit unsigned short in place threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_16u_C3IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_GT_16u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_GT_16u_C3IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds);


    /**
     * <pre>
     * 3 channel 16-bit signed short threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_GT_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_GT_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds);


    public static int nppiThreshold_GT_16s_C3IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_GT_16s_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_GT_16s_C3IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds);


    /**
     * <pre>
     * 3 channel 32-bit floating point threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_GT_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_GT_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds);


    /**
     * <pre>
     * 3 channel 32-bit floating point in place threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_GT_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_GT_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds);


    /**
     * <pre>
     * 4 channel 8-bit unsigned char image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * value is set to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_GT_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_GT_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds);


    public static int nppiThreshold_GT_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_GT_8u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_GT_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds);


    public static int nppiThreshold_GT_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_GT_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_GT_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds);


    /**
     * <pre>
     * 4 channel 16-bit unsigned short in place image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * value is set to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_16u_AC4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_GT_16u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_GT_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds);


    public static int nppiThreshold_GT_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_GT_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_GT_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds);


    /**
     * <pre>
     * 4 channel 16-bit signed short in place image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * value is set to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_16s_AC4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_GT_16s_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_GT_16s_AC4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds);


    /**
     * <pre>
     * 4 channel 32-bit floating point image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * value is set to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GT_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_GT_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_GT_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds);


    public static int nppiThreshold_GT_32f_AC4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_GT_32f_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_GT_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds);


    /**
     * <pre>
     * 1 channel 8-bit unsigned char threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nThreshold)
    {
        return JNpp.checkResult(nppiThreshold_LT_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold));
    }
    private static native int nppiThreshold_LT_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nThreshold);


    /**
     * <pre>
     * 1 channel 8-bit unsigned char in place threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_8u_C1IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte nThreshold)
    {
        return JNpp.checkResult(nppiThreshold_LT_8u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold));
    }
    private static native int nppiThreshold_LT_8u_C1IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte nThreshold);


    /**
     * <pre>
     * 1 channel 16-bit unsigned short threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold)
    {
        return JNpp.checkResult(nppiThreshold_LT_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold));
    }
    private static native int nppiThreshold_LT_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold);


    /**
     * <pre>
     * 1 channel 16-bit unsigned short in place threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_16u_C1IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold)
    {
        return JNpp.checkResult(nppiThreshold_LT_16u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold));
    }
    private static native int nppiThreshold_LT_16u_C1IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold);


    /**
     * <pre>
     * 1 channel 16-bit signed short threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold)
    {
        return JNpp.checkResult(nppiThreshold_LT_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold));
    }
    private static native int nppiThreshold_LT_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold);


    public static int nppiThreshold_LT_16s_C1IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold)
    {
        return JNpp.checkResult(nppiThreshold_LT_16s_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold));
    }
    private static native int nppiThreshold_LT_16s_C1IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold);


    public static int nppiThreshold_LT_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nThreshold)
    {
        return JNpp.checkResult(nppiThreshold_LT_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold));
    }
    private static native int nppiThreshold_LT_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nThreshold);


    /**
     * <pre>
     * 1 channel 32-bit floating point in place threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nThreshold)
    {
        return JNpp.checkResult(nppiThreshold_LT_32f_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold));
    }
    private static native int nppiThreshold_LT_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nThreshold);


    /**
     * <pre>
     * 3 channel 8-bit unsigned char threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_LT_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_LT_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds);


    /**
     * <pre>
     * 3 channel 8-bit unsigned char in place threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_8u_C3IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_LT_8u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_LT_8u_C3IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds);


    /**
     * <pre>
     * 3 channel 16-bit unsigned short threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_LT_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_LT_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds);


    /**
     * <pre>
     * 3 channel 16-bit unsigned short in place threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_16u_C3IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_LT_16u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_LT_16u_C3IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds);


    /**
     * <pre>
     * 3 channel 16-bit signed short threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_LT_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_LT_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds);


    public static int nppiThreshold_LT_16s_C3IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_LT_16s_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_LT_16s_C3IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds);


    /**
     * <pre>
     * 3 channel 32-bit floating point threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_LT_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_LT_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds);


    /**
     * <pre>
     * 3 channel 32-bit floating point in place threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_LT_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_LT_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds);


    /**
     * <pre>
     * 4 channel 8-bit unsigned char image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * value is set to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_LT_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_LT_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds);


    public static int nppiThreshold_LT_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_LT_8u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_LT_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds);


    /**
     * <pre>
     * 4 channel 16-bit unsigned short image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * value is set to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_LT_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_LT_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds);


    /**
     * <pre>
     * 4 channel 16-bit unsigned short in place image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * value is set to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_16u_AC4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_LT_16u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_LT_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds);


    public static int nppiThreshold_LT_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_LT_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_LT_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds);


    /**
     * <pre>
     * 4 channel 16-bit signed short in place image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * value is set to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_16s_AC4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_LT_16s_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_LT_16s_AC4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds);


    /**
     * <pre>
     * 4 channel 32-bit floating point image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * value is set to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_LT_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_LT_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds);


    /**
     * <pre>
     * 4 channel 32-bit floating point in place image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * value is set to nThreshold, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LT_32f_AC4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds)
    {
        return JNpp.checkResult(nppiThreshold_LT_32f_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds));
    }
    private static native int nppiThreshold_LT_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds);


    /**
     * <pre>
     * 1 channel 8-bit unsigned char threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nThreshold,
        byte nValue,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nThreshold,
        byte nValue,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 8-bit unsigned char in place threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_8u_C1IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte nThreshold,
        byte nValue,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_8u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_8u_C1IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte nThreshold,
        byte nValue,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 16-bit unsigned short threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue,
        int eComparisonOperation);


    public static int nppiThreshold_Val_16u_C1IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_16u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_16u_C1IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 16-bit signed short threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 16-bit signed short in place threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_16s_C1IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_16s_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_16s_C1IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 32-bit floating point threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nThreshold,
        float nValue,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nThreshold,
        float nValue,
        int eComparisonOperation);


    public static int nppiThreshold_Val_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nThreshold,
        float nValue,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_32f_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nThreshold,
        float nValue,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 8-bit unsigned char threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 8-bit unsigned char in place threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_8u_C3IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_8u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_8u_C3IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues,
        int eComparisonOperation);


    public static int nppiThreshold_Val_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 16-bit unsigned short in place threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_16u_C3IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_16u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_16u_C3IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues,
        int eComparisonOperation);


    public static int nppiThreshold_Val_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 16-bit signed short in place threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_16s_C3IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_16s_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_16s_C3IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 32-bit floating point threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 32-bit floating point in place threshold.
     * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues,
        int eComparisonOperation);


    public static int nppiThreshold_Val_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 8-bit unsigned char in place image threshold, not affecting Alpha.
     * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
     * value is set to nValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_8u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 16-bit unsigned short image threshold, not affecting Alpha.
     * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
     * value is set to nValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 16-bit unsigned short in place image threshold, not affecting Alpha.
     * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
     * value is set to nValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_16u_AC4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_16u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 16-bit signed short image threshold, not affecting Alpha.
     * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
     * value is set to nValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 16-bit signed short in place image threshold, not affecting Alpha.
     * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
     * value is set to nValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_16s_AC4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_16s_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_16s_AC4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 32-bit floating point image threshold, not affecting Alpha.
     * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
     * value is set to nValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @param eComparisonOperation The type of comparison operation to be used. The only valid
     *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
     * comparison operation type is specified.
     * </pre>
     */
    public static int nppiThreshold_Val_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues,
        int eComparisonOperation);


    public static int nppiThreshold_Val_32f_AC4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiThreshold_Val_32f_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation));
    }
    private static native int nppiThreshold_Val_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 8-bit unsigned char threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nThreshold,
        byte nValue)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue));
    }
    private static native int nppiThreshold_GTVal_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nThreshold,
        byte nValue);


    /**
     * <pre>
     * 1 channel 8-bit unsigned char in place threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_8u_C1IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte nThreshold,
        byte nValue)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_8u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue));
    }
    private static native int nppiThreshold_GTVal_8u_C1IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte nThreshold,
        byte nValue);


    /**
     * <pre>
     * 1 channel 16-bit unsigned short threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue));
    }
    private static native int nppiThreshold_GTVal_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue);


    /**
     * <pre>
     * 1 channel 16-bit unsigned short in place threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_16u_C1IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_16u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue));
    }
    private static native int nppiThreshold_GTVal_16u_C1IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue);


    /**
     * <pre>
     * 1 channel 16-bit signed short threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue));
    }
    private static native int nppiThreshold_GTVal_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue);


    /**
     * <pre>
     * 1 channel 16-bit signed short in place threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_16s_C1IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_16s_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue));
    }
    private static native int nppiThreshold_GTVal_16s_C1IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue);


    /**
     * <pre>
     * 1 channel 32-bit floating point threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nThreshold,
        float nValue)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue));
    }
    private static native int nppiThreshold_GTVal_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nThreshold,
        float nValue);


    /**
     * <pre>
     * 1 channel 32-bit floating point in place threshold.
     * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nThreshold,
        float nValue)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_32f_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue));
    }
    private static native int nppiThreshold_GTVal_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nThreshold,
        float nValue);


    /**
     * <pre>
     * 3 channel 8-bit unsigned char threshold.
     * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
     * to rValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_GTVal_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues);


    /**
     * <pre>
     * 3 channel 8-bit unsigned char in place threshold.
     * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
     * to rValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_8u_C3IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_8u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_GTVal_8u_C3IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues);


    /**
     * <pre>
     * 3 channel 16-bit unsigned short threshold.
     * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
     * to rValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_GTVal_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues);


    public static int nppiThreshold_GTVal_16u_C3IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_16u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_GTVal_16u_C3IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues);


    public static int nppiThreshold_GTVal_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_GTVal_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues);


    public static int nppiThreshold_GTVal_16s_C3IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_16s_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_GTVal_16s_C3IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues);


    /**
     * <pre>
     * 3 channel 32-bit floating point threshold.
     * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
     * to rValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_GTVal_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues);


    /**
     * <pre>
     * 3 channel 32-bit floating point in place threshold.
     * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
     * to rValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_GTVal_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues);


    /**
     * <pre>
     * 4 channel 8-bit unsigned char image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
     * value is set to rValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_GTVal_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues);


    /**
     * <pre>
     * 4 channel 8-bit unsigned char in place image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
     * value is set to rValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_8u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_GTVal_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues);


    /**
     * <pre>
     * 4 channel 16-bit unsigned short image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
     * value is set to rValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_GTVal_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues);


    /**
     * <pre>
     * 4 channel 16-bit unsigned short in place image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
     * value is set to rValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_16u_AC4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_16u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_GTVal_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues);


    /**
     * <pre>
     * 4 channel 16-bit signed short image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
     * value is set to rValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_GTVal_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues);


    /**
     * <pre>
     * 4 channel 16-bit signed short in place image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
     * value is set to rValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_16s_AC4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_16s_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_GTVal_16s_AC4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues);


    /**
     * <pre>
     * 4 channel 32-bit floating point image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
     * value is set to rValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_GTVal_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues);


    /**
     * <pre>
     * 4 channel 32-bit floating point in place image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
     * value is set to rValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_GTVal_32f_AC4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_GTVal_32f_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_GTVal_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues);


    /**
     * <pre>
     * 1 channel 8-bit unsigned char threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nThreshold,
        byte nValue)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue));
    }
    private static native int nppiThreshold_LTVal_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nThreshold,
        byte nValue);


    /**
     * <pre>
     * 1 channel 8-bit unsigned char in place threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_8u_C1IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte nThreshold,
        byte nValue)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_8u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue));
    }
    private static native int nppiThreshold_LTVal_8u_C1IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte nThreshold,
        byte nValue);


    public static int nppiThreshold_LTVal_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue));
    }
    private static native int nppiThreshold_LTVal_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue);


    /**
     * <pre>
     * 1 channel 16-bit unsigned short in place threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_16u_C1IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_16u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue));
    }
    private static native int nppiThreshold_LTVal_16u_C1IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue);


    /**
     * <pre>
     * 1 channel 16-bit signed short threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue));
    }
    private static native int nppiThreshold_LTVal_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue);


    public static int nppiThreshold_LTVal_16s_C1IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_16s_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue));
    }
    private static native int nppiThreshold_LTVal_16s_C1IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThreshold,
        short nValue);


    /**
     * <pre>
     * 1 channel 32-bit floating point threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nThreshold,
        float nValue)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue));
    }
    private static native int nppiThreshold_LTVal_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nThreshold,
        float nValue);


    /**
     * <pre>
     * 1 channel 32-bit floating point in place threshold.
     * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
     * to nValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThreshold The threshold value.
     * @param nValue The threshold replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nThreshold,
        float nValue)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_32f_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue));
    }
    private static native int nppiThreshold_LTVal_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nThreshold,
        float nValue);


    /**
     * <pre>
     * 3 channel 8-bit unsigned char threshold.
     * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
     * to rValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_LTVal_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues);


    /**
     * <pre>
     * 3 channel 8-bit unsigned char in place threshold.
     * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
     * to rValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_8u_C3IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_8u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_LTVal_8u_C3IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues);


    /**
     * <pre>
     * 3 channel 16-bit unsigned short threshold.
     * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
     * to rValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_LTVal_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues);


    /**
     * <pre>
     * 3 channel 16-bit unsigned short in place threshold.
     * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
     * to rValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_16u_C3IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_16u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_LTVal_16u_C3IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues);


    /**
     * <pre>
     * 3 channel 16-bit signed short threshold.
     * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
     * to rValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_LTVal_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues);


    /**
     * <pre>
     * 3 channel 16-bit signed short in place threshold.
     * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
     * to rValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_16s_C3IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_16s_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_LTVal_16s_C3IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues);


    /**
     * <pre>
     * 3 channel 32-bit floating point threshold.
     * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
     * to rValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_LTVal_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues);


    public static int nppiThreshold_LTVal_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_LTVal_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues);


    /**
     * <pre>
     * 4 channel 8-bit unsigned char image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
     * value is set to rValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_LTVal_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues);


    /**
     * <pre>
     * 4 channel 8-bit unsigned char in place image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
     * value is set to rValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_8u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_LTVal_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholds,
        byte[] rValues);


    /**
     * <pre>
     * 4 channel 16-bit unsigned short image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
     * value is set to rValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_LTVal_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues);


    /**
     * <pre>
     * 4 channel 16-bit unsigned short in place image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
     * value is set to rValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_16u_AC4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_16u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_LTVal_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues);


    /**
     * <pre>
     * 4 channel 16-bit signed short image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
     * value is set to rValue, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_LTVal_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues);


    /**
     * <pre>
     * 4 channel 16-bit signed short in place image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
     * value is set to rValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_16s_AC4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_16s_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_LTVal_16s_AC4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholds,
        short[] rValues);


    public static int nppiThreshold_LTVal_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_LTVal_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues);


    /**
     * <pre>
     * 4 channel 32-bit floating point in place image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
     * value is set to rValue, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholds The threshold values, one per color channel.
     * @param rValues The threshold replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTVal_32f_AC4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues)
    {
        return JNpp.checkResult(nppiThreshold_LTVal_32f_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues));
    }
    private static native int nppiThreshold_LTVal_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholds,
        float[] rValues);


    /**
     * <pre>
     * 1 channel 8-bit unsigned char threshold.
     * If for a comparison operations sourcePixel is less than nThresholdLT is true, the pixel is set
     * to nValueLT, else if sourcePixel is greater than nThresholdGT the pixel is set to nValueGT, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThresholdLT The thresholdLT value.
     * @param nValueLT The thresholdLT replacement value.
     * @param nThresholdGT The thresholdGT value.
     * @param nValueGT The thresholdGT replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nThresholdLT,
        byte nValueLT,
        byte nThresholdGT,
        byte nValueGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThresholdLT, nValueLT, nThresholdGT, nValueGT));
    }
    private static native int nppiThreshold_LTValGTVal_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte nThresholdLT,
        byte nValueLT,
        byte nThresholdGT,
        byte nValueGT);


    /**
     * <pre>
     * 1 channel 8-bit unsigned char in place threshold.
     * If for a comparison operations sourcePixel is less than nThresholdLT is true, the pixel is set
     * to nValueLT, else if sourcePixel is greater than nThresholdGT the pixel is set to nValueGT, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThresholdLT The thresholdLT value.
     * @param nValueLT The thresholdLT replacement value.
     * @param nThresholdGT The thresholdGT value.
     * @param nValueGT The thresholdGT replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_8u_C1IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte nThresholdLT,
        byte nValueLT,
        byte nThresholdGT,
        byte nValueGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_8u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThresholdLT, nValueLT, nThresholdGT, nValueGT));
    }
    private static native int nppiThreshold_LTValGTVal_8u_C1IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte nThresholdLT,
        byte nValueLT,
        byte nThresholdGT,
        byte nValueGT);


    /**
     * <pre>
     * 1 channel 16-bit unsigned short threshold.
     * If for a comparison operations sourcePixel is less than nThresholdLT is true, the pixel is set
     * to nValueLT, else if sourcePixel is greater than nThresholdGT the pixel is set to nValueGT, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThresholdLT The thresholdLT value.
     * @param nValueLT The thresholdLT replacement value.
     * @param nThresholdGT The thresholdGT value.
     * @param nValueGT The thresholdGT replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThresholdLT,
        short nValueLT,
        short nThresholdGT,
        short nValueGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThresholdLT, nValueLT, nThresholdGT, nValueGT));
    }
    private static native int nppiThreshold_LTValGTVal_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThresholdLT,
        short nValueLT,
        short nThresholdGT,
        short nValueGT);


    /**
     * <pre>
     * 1 channel 16-bit unsigned short in place threshold.
     * If for a comparison operations sourcePixel is less than nThresholdLT is true, the pixel is set
     * to nValueLT, else if sourcePixel is greater than nThresholdGT the pixel is set to nValueGT, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThresholdLT The thresholdLT value.
     * @param nValueLT The thresholdLT replacement value.
     * @param nThresholdGT The thresholdGT value.
     * @param nValueGT The thresholdGT replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_16u_C1IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThresholdLT,
        short nValueLT,
        short nThresholdGT,
        short nValueGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_16u_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThresholdLT, nValueLT, nThresholdGT, nValueGT));
    }
    private static native int nppiThreshold_LTValGTVal_16u_C1IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThresholdLT,
        short nValueLT,
        short nThresholdGT,
        short nValueGT);


    /**
     * <pre>
     * 1 channel 16-bit signed short threshold.
     * If for a comparison operations sourcePixel is less than nThresholdLT is true, the pixel is set
     * to nValueLT, else if sourcePixel is greater than nThresholdGT the pixel is set to nValueGT, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThresholdLT The thresholdLT value.
     * @param nValueLT The thresholdLT replacement value.
     * @param nThresholdGT The thresholdGT value.
     * @param nValueGT The thresholdGT replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThresholdLT,
        short nValueLT,
        short nThresholdGT,
        short nValueGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThresholdLT, nValueLT, nThresholdGT, nValueGT));
    }
    private static native int nppiThreshold_LTValGTVal_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short nThresholdLT,
        short nValueLT,
        short nThresholdGT,
        short nValueGT);


    /**
     * <pre>
     * 1 channel 16-bit signed short in place threshold.
     * If for a comparison operations sourcePixel is less than nThresholdLT is true, the pixel is set
     * to nValueLT, else if sourcePixel is greater than nThresholdGT the pixel is set to nValueGT, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThresholdLT The thresholdLT value.
     * @param nValueLT The thresholdLT replacement value.
     * @param nThresholdGT The thresholdGT value.
     * @param nValueGT The thresholdGT replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_16s_C1IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThresholdLT,
        short nValueLT,
        short nThresholdGT,
        short nValueGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_16s_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThresholdLT, nValueLT, nThresholdGT, nValueGT));
    }
    private static native int nppiThreshold_LTValGTVal_16s_C1IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short nThresholdLT,
        short nValueLT,
        short nThresholdGT,
        short nValueGT);


    /**
     * <pre>
     * 1 channel 32-bit floating point threshold.
     * If for a comparison operations sourcePixel is less than nThresholdLT is true, the pixel is set
     * to nValueLT, else if sourcePixel is greater than nThresholdGT the pixel is set to nValueGT, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nThresholdLT The thresholdLT value.
     * @param nValueLT The thresholdLT replacement value.
     * @param nThresholdGT The thresholdGT value.
     * @param nValueGT The thresholdGT replacement value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nThresholdLT,
        float nValueLT,
        float nThresholdGT,
        float nValueGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThresholdLT, nValueLT, nThresholdGT, nValueGT));
    }
    private static native int nppiThreshold_LTValGTVal_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nThresholdLT,
        float nValueLT,
        float nThresholdGT,
        float nValueGT);


    public static int nppiThreshold_LTValGTVal_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nThresholdLT,
        float nValueLT,
        float nThresholdGT,
        float nValueGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_32f_C1IRNative(pSrcDst, nSrcDstStep, oSizeROI, nThresholdLT, nValueLT, nThresholdGT, nValueGT));
    }
    private static native int nppiThreshold_LTValGTVal_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float nThresholdLT,
        float nValueLT,
        float nThresholdGT,
        float nValueGT);


    /**
     * <pre>
     * 3 channel 8-bit unsigned char threshold.
     * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
     * to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholdsLT The thresholdLT values, one per color channel.
     * @param rValuesLT The thresholdLT replacement values, one per color channel.
     * @param rThresholdsGT The thresholdGT values, one per channel.
     * @param rValuesGT The thresholdGT replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholdsLT,
        byte[] rValuesLT,
        byte[] rThresholdsGT,
        byte[] rValuesGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT));
    }
    private static native int nppiThreshold_LTValGTVal_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholdsLT,
        byte[] rValuesLT,
        byte[] rThresholdsGT,
        byte[] rValuesGT);


    /**
     * <pre>
     * 3 channel 8-bit unsigned char in place threshold.
     * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
     * to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref destination_image_pointer.
     * @param nSrcDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholdsLT The thresholdLT values, one per color channel.
     * @param rValuesLT The thresholdLT replacement values, one per color channel.
     * @param rThresholdsGT The thresholdGT values, one per channel.
     * @param rValuesGT The thresholdGT replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_8u_C3IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholdsLT,
        byte[] rValuesLT,
        byte[] rThresholdsGT,
        byte[] rValuesGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_8u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT));
    }
    private static native int nppiThreshold_LTValGTVal_8u_C3IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholdsLT,
        byte[] rValuesLT,
        byte[] rThresholdsGT,
        byte[] rValuesGT);


    /**
     * <pre>
     * 3 channel 16-bit unsigned short threshold.
     * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
     * to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholdsLT The thresholdLT values, one per color channel.
     * @param rValuesLT The thresholdLT replacement values, one per color channel.
     * @param rThresholdsGT The thresholdGT values, one per channel.
     * @param rValuesGT The thresholdGT replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholdsLT,
        short[] rValuesLT,
        short[] rThresholdsGT,
        short[] rValuesGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT));
    }
    private static native int nppiThreshold_LTValGTVal_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholdsLT,
        short[] rValuesLT,
        short[] rThresholdsGT,
        short[] rValuesGT);


    /**
     * <pre>
     * 3 channel 16-bit unsigned short in place threshold.
     * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
     * to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholdsLT The thresholdLT values, one per color channel.
     * @param rValuesLT The thresholdLT replacement values, one per color channel.
     * @param rThresholdsGT The thresholdGT values, one per channel.
     * @param rValuesGT The thresholdGT replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_16u_C3IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholdsLT,
        short[] rValuesLT,
        short[] rThresholdsGT,
        short[] rValuesGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_16u_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT));
    }
    private static native int nppiThreshold_LTValGTVal_16u_C3IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholdsLT,
        short[] rValuesLT,
        short[] rThresholdsGT,
        short[] rValuesGT);


    /**
     * <pre>
     * 3 channel 16-bit signed short threshold.
     * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
     * to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholdsLT The thresholdLT values, one per color channel.
     * @param rValuesLT The thresholdLT replacement values, one per color channel.
     * @param rThresholdsGT The thresholdGT values, one per channel.
     * @param rValuesGT The thresholdGT replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholdsLT,
        short[] rValuesLT,
        short[] rThresholdsGT,
        short[] rValuesGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT));
    }
    private static native int nppiThreshold_LTValGTVal_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholdsLT,
        short[] rValuesLT,
        short[] rThresholdsGT,
        short[] rValuesGT);


    public static int nppiThreshold_LTValGTVal_16s_C3IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholdsLT,
        short[] rValuesLT,
        short[] rThresholdsGT,
        short[] rValuesGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_16s_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT));
    }
    private static native int nppiThreshold_LTValGTVal_16s_C3IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholdsLT,
        short[] rValuesLT,
        short[] rThresholdsGT,
        short[] rValuesGT);


    /**
     * <pre>
     * 3 channel 32-bit floating point threshold.
     * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
     * to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholdsLT The thresholdLT values, one per color channel.
     * @param rValuesLT The thresholdLT replacement values, one per color channel.
     * @param rThresholdsGT The thresholdGT values, one per channel.
     * @param rValuesGT The thresholdGT replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholdsLT,
        float[] rValuesLT,
        float[] rThresholdsGT,
        float[] rValuesGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT));
    }
    private static native int nppiThreshold_LTValGTVal_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholdsLT,
        float[] rValuesLT,
        float[] rThresholdsGT,
        float[] rValuesGT);


    /**
     * <pre>
     * 3 channel 32-bit floating point in place threshold.
     * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
     * to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholdsLT The thresholdLT values, one per color channel.
     * @param rValuesLT The thresholdLT replacement values, one per color channel.
     * @param rThresholdsGT The thresholdGT values, one per channel.
     * @param rValuesGT The thresholdGT replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholdsLT,
        float[] rValuesLT,
        float[] rThresholdsGT,
        float[] rValuesGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_32f_C3IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT));
    }
    private static native int nppiThreshold_LTValGTVal_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholdsLT,
        float[] rValuesLT,
        float[] rThresholdsGT,
        float[] rValuesGT);


    /**
     * <pre>
     * 4 channel 8-bit unsigned char image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
     * value is set to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholdsLT The thresholdLT values, one per color channel.
     * @param rValuesLT The thresholdLT replacement values, one per color channel.
     * @param rThresholdsGT The thresholdGT values, one per channel.
     * @param rValuesGT The thresholdGT replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholdsLT,
        byte[] rValuesLT,
        byte[] rThresholdsGT,
        byte[] rValuesGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT));
    }
    private static native int nppiThreshold_LTValGTVal_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        byte[] rThresholdsLT,
        byte[] rValuesLT,
        byte[] rThresholdsGT,
        byte[] rValuesGT);


    /**
     * <pre>
     * 4 channel 8-bit unsigned char in place image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
     * value is set to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholdsLT The thresholdLT values, one per color channel.
     * @param rValuesLT The thresholdLT replacement values, one per color channel.
     * @param rThresholdsGT The thresholdGT values, one per channel.
     * @param rValuesGT The thresholdGT replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholdsLT,
        byte[] rValuesLT,
        byte[] rThresholdsGT,
        byte[] rValuesGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_8u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT));
    }
    private static native int nppiThreshold_LTValGTVal_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        byte[] rThresholdsLT,
        byte[] rValuesLT,
        byte[] rThresholdsGT,
        byte[] rValuesGT);


    public static int nppiThreshold_LTValGTVal_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholdsLT,
        short[] rValuesLT,
        short[] rThresholdsGT,
        short[] rValuesGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT));
    }
    private static native int nppiThreshold_LTValGTVal_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholdsLT,
        short[] rValuesLT,
        short[] rThresholdsGT,
        short[] rValuesGT);


    public static int nppiThreshold_LTValGTVal_16u_AC4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholdsLT,
        short[] rValuesLT,
        short[] rThresholdsGT,
        short[] rValuesGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_16u_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT));
    }
    private static native int nppiThreshold_LTValGTVal_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholdsLT,
        short[] rValuesLT,
        short[] rThresholdsGT,
        short[] rValuesGT);


    public static int nppiThreshold_LTValGTVal_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholdsLT,
        short[] rValuesLT,
        short[] rThresholdsGT,
        short[] rValuesGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT));
    }
    private static native int nppiThreshold_LTValGTVal_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        short[] rThresholdsLT,
        short[] rValuesLT,
        short[] rThresholdsGT,
        short[] rValuesGT);


    public static int nppiThreshold_LTValGTVal_16s_AC4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholdsLT,
        short[] rValuesLT,
        short[] rThresholdsGT,
        short[] rValuesGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_16s_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT));
    }
    private static native int nppiThreshold_LTValGTVal_16s_AC4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        short[] rThresholdsLT,
        short[] rValuesLT,
        short[] rThresholdsGT,
        short[] rValuesGT);


    /**
     * <pre>
     * 4 channel 32-bit floating point image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
     * value is set to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholdsLT The thresholdLT values, one per color channel.
     * @param rValuesLT The thresholdLT replacement values, one per color channel.
     * @param rThresholdsGT The thresholdGT values, one per channel.
     * @param rValuesGT The thresholdGT replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholdsLT,
        float[] rValuesLT,
        float[] rThresholdsGT,
        float[] rValuesGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT));
    }
    private static native int nppiThreshold_LTValGTVal_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float[] rThresholdsLT,
        float[] rValuesLT,
        float[] rThresholdsGT,
        float[] rValuesGT);


    /**
     * <pre>
     * 4 channel 32-bit floating point in place image threshold, not affecting Alpha.
     * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
     * value is set to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param rThresholdsLT The thresholdLT values, one per color channel.
     * @param rValuesLT The thresholdLT replacement values, one per color channel.
     * @param rThresholdsGT The thresholdGT values, one per channel.
     * @param rValuesGT The thresholdGT replacement values, one per color channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiThreshold_LTValGTVal_32f_AC4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholdsLT,
        float[] rValuesLT,
        float[] rThresholdsGT,
        float[] rValuesGT)
    {
        return JNpp.checkResult(nppiThreshold_LTValGTVal_32f_AC4IRNative(pSrcDst, nSrcDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT));
    }
    private static native int nppiThreshold_LTValGTVal_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI,
        float[] rThresholdsLT,
        float[] rValuesLT,
        float[] rThresholdsGT,
        float[] rValuesGT);


    /**
     * <pre>
     * 1 channel 8-bit unsigned char image compare.
     * Compare pSrc1's pixels with corresponding pixels in pSrc2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompare_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompare_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompare_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    public static int nppiCompare_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompare_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompare_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    public static int nppiCompare_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompare_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompare_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 8-bit unsigned char image compare, not affecting Alpha.
     * Compare pSrc1's pixels with corresponding pixels in pSrc2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompare_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompare_8u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompare_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 16-bit unsigned short image compare.
     * Compare pSrc1's pixels with corresponding pixels in pSrc2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompare_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompare_16u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompare_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 16-bit unsigned short image compare.
     * Compare pSrc1's pixels with corresponding pixels in pSrc2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompare_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompare_16u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompare_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 16-bit unsigned short image compare.
     * Compare pSrc1's pixels with corresponding pixels in pSrc2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompare_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompare_16u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompare_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 16-bit unsigned short image compare, not affecting Alpha.
     * Compare pSrc1's pixels with corresponding pixels in pSrc2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompare_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompare_16u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompare_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 16-bit signed short image compare.
     * Compare pSrc1's pixels with corresponding pixels in pSrc2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompare_16s_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompare_16s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompare_16s_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 16-bit signed short image compare.
     * Compare pSrc1's pixels with corresponding pixels in pSrc2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompare_16s_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompare_16s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompare_16s_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    public static int nppiCompare_16s_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompare_16s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompare_16s_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 16-bit signed short image compare, not affecting Alpha.
     * Compare pSrc1's pixels with corresponding pixels in pSrc2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompare_16s_AC4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompare_16s_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompare_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 32-bit floating point image compare.
     * Compare pSrc1's pixels with corresponding pixels in pSrc2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompare_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompare_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompare_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    public static int nppiCompare_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompare_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompare_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 32-bit floating point image compare.
     * Compare pSrc1's pixels with corresponding pixels in pSrc2.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompare_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompare_32f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompare_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    public static int nppiCompare_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompare_32f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompare_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 8-bit unsigned char image compare with constant value.
     * Compare pSrc's pixels with constant value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param nConstant constant value.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompareC_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompareC_8u_C1RNative(pSrc, nSrcStep, nConstant, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompareC_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        byte nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 8-bit unsigned char image compare with constant value.
     * Compare pSrc's pixels with constant value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pConstants pointer to a list of constant values, one per color channel..
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompareC_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompareC_8u_C3RNative(pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompareC_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    public static int nppiCompareC_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompareC_8u_C4RNative(pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompareC_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    public static int nppiCompareC_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompareC_8u_AC4RNative(pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompareC_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 16-bit unsigned short image compare with constant value.
     * Compare pSrc's pixels with constant value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param nConstant constant value
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompareC_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        short nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompareC_16u_C1RNative(pSrc, nSrcStep, nConstant, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompareC_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        short nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 16-bit unsigned short image compare with constant value.
     * Compare pSrc's pixels with constant value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pConstants pointer to a list of constants, one per color channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompareC_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompareC_16u_C3RNative(pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompareC_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    public static int nppiCompareC_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompareC_16u_C4RNative(pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompareC_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 16-bit unsigned short image compare, not affecting Alpha.
     * Compare pSrc's pixels with constant value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pConstants pointer to a list of constants, one per color channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompareC_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompareC_16u_AC4RNative(pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompareC_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    public static int nppiCompareC_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        short nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompareC_16s_C1RNative(pSrc, nSrcStep, nConstant, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompareC_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        short nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 16-bit signed short image compare with constant value.
     * Compare pSrc's pixels with constant value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pConstants pointer to a list of constants, one per color channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompareC_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompareC_16s_C3RNative(pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompareC_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    public static int nppiCompareC_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompareC_16s_C4RNative(pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompareC_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    public static int nppiCompareC_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompareC_16s_AC4RNative(pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompareC_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    public static int nppiCompareC_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        float nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompareC_32f_C1RNative(pSrc, nSrcStep, nConstant, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompareC_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        float nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 3 channel 32-bit floating point image compare with constant value.
     * Compare pSrc's pixels with constant value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pConstants pointer to a list of constants, one per color channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompareC_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompareC_32f_C3RNative(pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompareC_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 4 channel 32-bit floating point image compare with constant value.
     * Compare pSrc's pixels with constant value.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pConstants pointer to a list of constants, one per color channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompareC_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompareC_32f_C4RNative(pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompareC_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    public static int nppiCompareC_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation)
    {
        return JNpp.checkResult(nppiCompareC_32f_AC4RNative(pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation));
    }
    private static native int nppiCompareC_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eComparisonOperation);


    /**
     * <pre>
     * 1 channel 32-bit floating point image compare whether two images are equal within epsilon.
     * Compare pSrc1's pixels with corresponding pixels in pSrc2 to determine whether they are equal with a difference of epsilon.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nEpsilon epsilon tolerance value to compare to pixel absolute differences
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompareEqualEps_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nEpsilon)
    {
        return JNpp.checkResult(nppiCompareEqualEps_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nEpsilon));
    }
    private static native int nppiCompareEqualEps_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nEpsilon);


    /**
     * <pre>
     * 3 channel 32-bit floating point image compare whether two images are equal within epsilon.
     * Compare pSrc1's pixels with corresponding pixels in pSrc2 to determine whether they are equal with a difference of epsilon.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nEpsilon epsilon tolerance value to compare to per color channel pixel absolute differences
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompareEqualEps_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nEpsilon)
    {
        return JNpp.checkResult(nppiCompareEqualEps_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nEpsilon));
    }
    private static native int nppiCompareEqualEps_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nEpsilon);


    /**
     * <pre>
     * 4 channel 32-bit floating point image compare whether two images are equal within epsilon.
     * Compare pSrc1's pixels with corresponding pixels in pSrc2 to determine whether they are equal with a difference of epsilon.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nEpsilon epsilon tolerance value to compare to per color channel pixel absolute differences
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompareEqualEps_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nEpsilon)
    {
        return JNpp.checkResult(nppiCompareEqualEps_32f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nEpsilon));
    }
    private static native int nppiCompareEqualEps_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nEpsilon);


    /**
     * <pre>
     * 4 channel 32-bit signed floating point compare whether two images are equal within epsilon, not affecting Alpha.
     * Compare pSrc1's pixels with corresponding pixels in pSrc2 to determine whether they are equal with a difference of epsilon.
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nEpsilon epsilon tolerance value to compare to per color channel pixel absolute differences
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompareEqualEps_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nEpsilon)
    {
        return JNpp.checkResult(nppiCompareEqualEps_32f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nEpsilon));
    }
    private static native int nppiCompareEqualEps_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nEpsilon);


    /**
     * <pre>
     * 1 channel 32-bit floating point image compare whether image and constant are equal within epsilon.
     * Compare pSrc's pixels with constant value to determine whether they are equal within a difference of epsilon.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param nConstant constant value
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nEpsilon epsilon tolerance value to compare to pixel absolute differences
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompareEqualEpsC_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        float nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nEpsilon)
    {
        return JNpp.checkResult(nppiCompareEqualEpsC_32f_C1RNative(pSrc, nSrcStep, nConstant, pDst, nDstStep, oSizeROI, nEpsilon));
    }
    private static native int nppiCompareEqualEpsC_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        float nConstant,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nEpsilon);


    /**
     * <pre>
     * 3 channel 32-bit floating point image compare whether image and constant are equal within epsilon.
     * Compare pSrc's pixels with constant value to determine whether they are equal within a difference of epsilon.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pConstants pointer to a list of constants, one per color channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nEpsilon epsilon tolerance value to compare to per color channel pixel absolute differences
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompareEqualEpsC_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nEpsilon)
    {
        return JNpp.checkResult(nppiCompareEqualEpsC_32f_C3RNative(pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, nEpsilon));
    }
    private static native int nppiCompareEqualEpsC_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nEpsilon);


    /**
     * <pre>
     * 4 channel 32-bit floating point image compare whether image and constant are equal within epsilon.
     * Compare pSrc's pixels with constant value to determine whether they are equal within a difference of epsilon.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pConstants pointer to a list of constants, one per color channel.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nEpsilon epsilon tolerance value to compare to per color channel pixel absolute differences
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCompareEqualEpsC_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nEpsilon)
    {
        return JNpp.checkResult(nppiCompareEqualEpsC_32f_C4RNative(pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, nEpsilon));
    }
    private static native int nppiCompareEqualEpsC_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nEpsilon);


    public static int nppiCompareEqualEpsC_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nEpsilon)
    {
        return JNpp.checkResult(nppiCompareEqualEpsC_32f_AC4RNative(pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, nEpsilon));
    }
    private static native int nppiCompareEqualEpsC_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pConstants,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        float nEpsilon);


    /**
     * <pre>
     * Single-channel 8-bit unsigned integer dilation.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiDilate_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiDilate_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    public static int nppiDilate_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiDilate_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiDilate_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer dilation.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiDilate_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiDilate_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    public static int nppiDilate_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiDilate_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiDilate_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single-channel 16-bit unsigned integer dilation.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiDilate_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiDilate_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three-channel 16-bit unsigned integer dilation.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiDilate_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiDilate_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    public static int nppiDilate_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiDilate_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiDilate_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer dilation, ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiDilate_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiDilate_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single-channel 32-bit floating-point dilation.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiDilate_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiDilate_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    public static int nppiDilate_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiDilate_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiDilate_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four-channel 32-bit floating-point dilation.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiDilate_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiDilate_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four-channel 32-bit floating-point dilation, ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiDilate_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiDilate_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    public static int nppiDilateBorder_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilateBorder_8u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiDilateBorder_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    public static int nppiDilateBorder_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilateBorder_8u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiDilateBorder_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    public static int nppiDilateBorder_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilateBorder_8u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiDilateBorder_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer dilation with border control, ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilateBorder_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilateBorder_8u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiDilateBorder_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Single-channel 16-bit unsigned integer dilation with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilateBorder_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilateBorder_16u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiDilateBorder_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    public static int nppiDilateBorder_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilateBorder_16u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiDilateBorder_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer dilation with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilateBorder_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilateBorder_16u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiDilateBorder_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer dilation with border control, ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilateBorder_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilateBorder_16u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiDilateBorder_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    public static int nppiDilateBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilateBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiDilateBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Three-channel 32-bit floating-point dilation with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilateBorder_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilateBorder_32f_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiDilateBorder_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 32-bit floating-point dilation with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilateBorder_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilateBorder_32f_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiDilateBorder_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    public static int nppiDilateBorder_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilateBorder_32f_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiDilateBorder_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    public static int nppiDilate3x3_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDilate3x3_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDilate3x3_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 8-bit unsigned integer 3x3 dilation.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDilate3x3_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDilate3x3_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiDilate3x3_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDilate3x3_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDilate3x3_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer 3x3 dilation, ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDilate3x3_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDilate3x3_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single-channel 16-bit unsigned integer 3x3 dilation.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDilate3x3_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDilate3x3_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 16-bit unsigned integer 3x3 dilation.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDilate3x3_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDilate3x3_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer 3x3 dilation.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDilate3x3_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDilate3x3_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiDilate3x3_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDilate3x3_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDilate3x3_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single-channel 32-bit floating-point 3x3 dilation.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDilate3x3_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDilate3x3_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 32-bit floating-point 3x3 dilation.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDilate3x3_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDilate3x3_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 32-bit floating-point 3x3 dilation.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDilate3x3_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDilate3x3_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 32-bit floating-point 3x3 dilation, ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDilate3x3_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDilate3x3_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiDilate3x3_64f_C1R(
        TypedPointer<Npp64f> pSrc,
        int nSrcStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDilate3x3_64f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiDilate3x3_64f_C1RNative(
        TypedPointer<Npp64f> pSrc,
        int nSrcStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiDilate3x3Border_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilate3x3Border_8u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiDilate3x3Border_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Three-channel 8-bit unsigned integer 3x3 dilation with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3Border_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilate3x3Border_8u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiDilate3x3Border_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer 3x3 dilation with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3Border_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilate3x3Border_8u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiDilate3x3Border_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer 3x3 dilation with border control, ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3Border_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilate3x3Border_8u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiDilate3x3Border_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Single-channel 16-bit unsigned integer 3x3 dilation with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3Border_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilate3x3Border_16u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiDilate3x3Border_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Three-channel 16-bit unsigned integer 3x3 dilation with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3Border_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilate3x3Border_16u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiDilate3x3Border_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer 3x3 dilation with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3Border_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilate3x3Border_16u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiDilate3x3Border_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer 3x3 dilation with border control, ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3Border_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilate3x3Border_16u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiDilate3x3Border_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Single-channel 32-bit floating-point 3x3 dilation with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3Border_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilate3x3Border_32f_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiDilate3x3Border_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    public static int nppiDilate3x3Border_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilate3x3Border_32f_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiDilate3x3Border_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 32-bit floating-point 3x3 dilation with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiDilate3x3Border_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilate3x3Border_32f_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiDilate3x3Border_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    public static int nppiDilate3x3Border_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiDilate3x3Border_32f_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiDilate3x3Border_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Single-channel 8-bit unsigned integer erosion.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiErode_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiErode_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three-channel 8-bit unsigned integer erosion.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiErode_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiErode_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    public static int nppiErode_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiErode_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiErode_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer erosion, ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiErode_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiErode_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single-channel 16-bit unsigned integer erosion.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiErode_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiErode_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three-channel 16-bit unsigned integer erosion.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiErode_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiErode_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer erosion.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiErode_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiErode_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    public static int nppiErode_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiErode_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiErode_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    public static int nppiErode_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiErode_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiErode_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three-channel 32-bit floating-point erosion.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiErode_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiErode_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four-channel 32-bit floating-point erosion.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiErode_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiErode_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four-channel 32-bit floating-point erosion, ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiErode_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor));
    }
    private static native int nppiErode_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single-channel 8-bit unsigned integer erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErodeBorder_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErodeBorder_8u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiErodeBorder_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Three-channel 8-bit unsigned integer erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErodeBorder_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErodeBorder_8u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiErodeBorder_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErodeBorder_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErodeBorder_8u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiErodeBorder_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer erosion with border control, ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErodeBorder_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErodeBorder_8u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiErodeBorder_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Single-channel 16-bit unsigned integer erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErodeBorder_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErodeBorder_16u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiErodeBorder_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Three-channel 16-bit unsigned integer erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErodeBorder_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErodeBorder_16u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiErodeBorder_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErodeBorder_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErodeBorder_16u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiErodeBorder_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer erosion with border control, ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErodeBorder_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErodeBorder_16u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiErodeBorder_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Single-channel 32-bit floating-point erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErodeBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErodeBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiErodeBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Three-channel 32-bit floating-point erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErodeBorder_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErodeBorder_32f_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiErodeBorder_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 32-bit floating-point erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMask Pointer to the start address of the mask array
     * @param oMaskSize Width and Height mask array.
     * @param oAnchor X and Y offsets of the mask origin frame of reference
     *        w.r.t the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErodeBorder_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErodeBorder_32f_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiErodeBorder_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    public static int nppiErodeBorder_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErodeBorder_32f_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType));
    }
    private static native int nppiErodeBorder_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMask,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Single-channel 8-bit unsigned integer 3x3 erosion.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiErode3x3_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiErode3x3_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiErode3x3_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiErode3x3_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiErode3x3_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiErode3x3_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiErode3x3_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiErode3x3_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiErode3x3_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiErode3x3_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiErode3x3_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single-channel 16-bit unsigned integer 3x3 erosion.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiErode3x3_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiErode3x3_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiErode3x3_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiErode3x3_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiErode3x3_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer 3x3 erosion.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiErode3x3_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiErode3x3_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer 3x3 erosion, ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiErode3x3_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiErode3x3_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single-channel 32-bit floating-point 3x3 erosion.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiErode3x3_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiErode3x3_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 32-bit floating-point 3x3 erosion.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiErode3x3_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiErode3x3_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiErode3x3_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiErode3x3_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiErode3x3_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 32-bit floating-point 3x3 erosion, ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiErode3x3_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiErode3x3_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single-channel 64-bit floating-point 3x3 erosion.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3_64f_C1R(
        TypedPointer<Npp64f> pSrc,
        int nSrcStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiErode3x3_64f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiErode3x3_64f_C1RNative(
        TypedPointer<Npp64f> pSrc,
        int nSrcStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single-channel 8-bit unsigned integer 3x3 erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3Border_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErode3x3Border_8u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiErode3x3Border_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Three-channel 8-bit unsigned integer 3x3 erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3Border_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErode3x3Border_8u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiErode3x3Border_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer 3x3 erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3Border_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErode3x3Border_8u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiErode3x3Border_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer 3x3 erosion with border control, ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3Border_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErode3x3Border_8u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiErode3x3Border_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Single-channel 16-bit unsigned integer 3x3 erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3Border_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErode3x3Border_16u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiErode3x3Border_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Three-channel 16-bit unsigned integer 3x3 erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3Border_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErode3x3Border_16u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiErode3x3Border_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer 3x3 erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3Border_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErode3x3Border_16u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiErode3x3Border_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer 3x3 erosion with border control, ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3Border_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErode3x3Border_16u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiErode3x3Border_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Single-channel 32-bit floating-point 3x3 erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3Border_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErode3x3Border_32f_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiErode3x3Border_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    public static int nppiErode3x3Border_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErode3x3Border_32f_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiErode3x3Border_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four-channel 32-bit floating-point 3x3 erosion with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiErode3x3Border_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErode3x3Border_32f_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiErode3x3Border_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    public static int nppiErode3x3Border_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiErode3x3Border_32f_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiErode3x3Border_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * 8-bit unsigned single-channel 1D column convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *                 operation should be divided.  If equal to the sum of
     *                 coefficients, this will keep the maximum result value within
     *                 full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterColumn_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterColumn_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterColumn_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    public static int nppiFilterColumn_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterColumn_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterColumn_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    public static int nppiFilterColumn_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterColumn_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterColumn_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    public static int nppiFilterColumn_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterColumn_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterColumn_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    public static int nppiFilterColumn_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterColumn_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterColumn_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    /**
     * <pre>
     * 16-bit unsigned three-channel 1D column convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *                 operation should be divided.  If equal to the sum of
     *                 coefficients, this will keep the maximum result value within
     *                 full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterColumn_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterColumn_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterColumn_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    public static int nppiFilterColumn_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterColumn_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterColumn_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    /**
     * <pre>
     * 16-bit unsigned four-channel 1D column convolution ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *                 operation should be divided.  If equal to the sum of
     *                 coefficients, this will keep the maximum result value within
     *                 full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterColumn_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterColumn_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterColumn_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    /**
     * <pre>
     * 16-bit single-channel 1D column convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *                 operation should be divided.  If equal to the sum of
     *                 coefficients, this will keep the maximum result value within
     *                 full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterColumn_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterColumn_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterColumn_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    /**
     * <pre>
     * 16-bit three-channel 1D column convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *                 operation should be divided.  If equal to the sum of
     *                 coefficients, this will keep the maximum result value within
     *                 full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterColumn_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterColumn_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterColumn_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    public static int nppiFilterColumn_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterColumn_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterColumn_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    /**
     * <pre>
     * 16-bit four-channel 1D column convolution ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *                 operation should be divided.  If equal to the sum of
     *                 coefficients, this will keep the maximum result value within
     *                 full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterColumn_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterColumn_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterColumn_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    public static int nppiFilterColumn_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 32-bit float three-channel 1D column convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterColumn_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    public static int nppiFilterColumn_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 32-bit float four-channel 1D column convolution ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterColumn_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    public static int nppiFilterColumn_64f_C1R(
        TypedPointer<Npp64f> pSrc,
        int nSrcStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp64f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn_64f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn_64f_C1RNative(
        TypedPointer<Npp64f> pSrc,
        int nSrcStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp64f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 8-bit unsigned single-channel 1D column convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterColumn32f_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn32f_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn32f_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    public static int nppiFilterColumn32f_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn32f_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn32f_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 8-bit unsigned four-channel 1D column convolution.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterColumn32f_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn32f_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn32f_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    public static int nppiFilterColumn32f_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn32f_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn32f_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    public static int nppiFilterColumn32f_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn32f_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn32f_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 16-bit unsigned three-channel 1D column convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterColumn32f_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn32f_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn32f_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 16-bit unsigned four-channel 1D column convolution.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterColumn32f_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn32f_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn32f_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 16-bit unsigned four-channel 1D column convolution ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterColumn32f_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn32f_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn32f_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    public static int nppiFilterColumn32f_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn32f_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn32f_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 16-bit three-channel 1D column convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterColumn32f_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn32f_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn32f_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 16-bit four-channel 1D column convolution.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterColumn32f_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn32f_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn32f_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 16-bit four-channel 1D column convolution ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterColumn32f_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterColumn32f_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterColumn32f_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 8-bit unsigned single-channel 1D row convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *                 operation should be divided.  If equal to the sum of
     *                 coefficients, this will keep the maximum result value within
     *                 full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterRow_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterRow_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    /**
     * <pre>
     * 8-bit unsigned three-channel 1D row convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *                 operation should be divided.  If equal to the sum of
     *                 coefficients, this will keep the maximum result value within
     *                 full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterRow_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterRow_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    /**
     * <pre>
     * 8-bit unsigned four-channel 1D row convolution.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *                 operation should be divided.  If equal to the sum of
     *                 coefficients, this will keep the maximum result value within
     *                 full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterRow_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterRow_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    public static int nppiFilterRow_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterRow_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterRow_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    /**
     * <pre>
     * 16-bit unsigned single-channel 1D row convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *                 operation should be divided.  If equal to the sum of
     *                 coefficients, this will keep the maximum result value within
     *                 full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterRow_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterRow_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    public static int nppiFilterRow_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterRow_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterRow_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    /**
     * <pre>
     * 16-bit unsigned four-channel 1D row convolution.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *                 operation should be divided.  If equal to the sum of
     *                 coefficients, this will keep the maximum result value within
     *                 full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterRow_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterRow_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    /**
     * <pre>
     * 16-bit unsigned four-channel 1D row convolution ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *                 operation should be divided.  If equal to the sum of
     *                 coefficients, this will keep the maximum result value within
     *                 full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterRow_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterRow_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    /**
     * <pre>
     * 16-bit single-channel 1D row convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *                 operation should be divided.  If equal to the sum of
     *                 coefficients, this will keep the maximum result value within
     *                 full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterRow_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterRow_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    /**
     * <pre>
     * 16-bit three-channel 1D row convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *                 operation should be divided.  If equal to the sum of
     *                 coefficients, this will keep the maximum result value within
     *                 full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterRow_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterRow_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    /**
     * <pre>
     * 16-bit four-channel 1D row convolution.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *                 operation should be divided.  If equal to the sum of
     *                 coefficients, this will keep the maximum result value within
     *                 full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterRow_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterRow_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    /**
     * <pre>
     * 16-bit four-channel 1D row convolution ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *                 operation should be divided.  If equal to the sum of
     *                 coefficients, this will keep the maximum result value within
     *                 full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilterRow_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor));
    }
    private static native int nppiFilterRow_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor);


    /**
     * <pre>
     * 32-bit float single-channel 1D row convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 32-bit float three-channel 1D row convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 32-bit float four-channel 1D row convolution.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    public static int nppiFilterRow_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    public static int nppiFilterRow_64f_C1R(
        TypedPointer<Npp64f> pSrc,
        int nSrcStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp64f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow_64f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow_64f_C1RNative(
        TypedPointer<Npp64f> pSrc,
        int nSrcStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp64f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * Single channel 8-bit unsigned 1D row convolution filter with border control.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param nMaskSize Width of the kernel.
     * @param nAnchor X offset of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRowBorder_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterRowBorder_8u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterRowBorder_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType);


    public static int nppiFilterRowBorder_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterRowBorder_8u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterRowBorder_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Four channel channel 8-bit unsigned 1D row convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param nMaskSize Width of the kernel.
     * @param nAnchor X offset of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRowBorder_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterRowBorder_8u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterRowBorder_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 8-bit unsigned convolution 1D row filter with border control, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param nMaskSize Width of the kernel.
     * @param nAnchor X offset of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRowBorder_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterRowBorder_8u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterRowBorder_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType);


    public static int nppiFilterRowBorder_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterRowBorder_16u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterRowBorder_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Three channel 16-bit unsigned 1D row convolution filter with border control.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param nMaskSize Width of the kernel.
     * @param nAnchor X offset of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRowBorder_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterRowBorder_16u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterRowBorder_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Four channel channel 16-bit 1D row unsigned convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param nMaskSize Width of the kernel.
     * @param nAnchor X offset of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRowBorder_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterRowBorder_16u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterRowBorder_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 16-bit unsigned 1D row convolution filter with border control, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param nMaskSize Width of the kernel.
     * @param nAnchor X offset of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRowBorder_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterRowBorder_16u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterRowBorder_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType);


    public static int nppiFilterRowBorder_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterRowBorder_16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterRowBorder_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Three channel 16-bit 1D row convolution filter with border control.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param nMaskSize Width of the kernel.
     * @param nAnchor X offset of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRowBorder_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterRowBorder_16s_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterRowBorder_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Four channel channel 16-bit 1D row convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param nMaskSize Width of the kernel.
     * @param nAnchor X offset of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRowBorder_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterRowBorder_16s_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterRowBorder_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 16-bit 1D row convolution filter with border control, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param nMaskSize Width of the kernel.
     * @param nAnchor X offset of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRowBorder_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterRowBorder_16s_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterRowBorder_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        int nMaskSize,
        int nAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Single channel 32-bit float 1D row convolution filter with border control.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param nMaskSize Width of the kernel.
     * @param nAnchor X offset of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRowBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterRowBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, eBorderType));
    }
    private static native int nppiFilterRowBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor,
        int eBorderType);


    /**
     * <pre>
     * Three channel 32-bit float 1D row convolution filter with border control.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param nMaskSize Width of the kernel.
     * @param nAnchor X offset of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRowBorder_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterRowBorder_32f_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, eBorderType));
    }
    private static native int nppiFilterRowBorder_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor,
        int eBorderType);


    public static int nppiFilterRowBorder_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterRowBorder_32f_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, eBorderType));
    }
    private static native int nppiFilterRowBorder_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 32-bit float 1D row convolution filter with border control, ignoring alpha channel.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param nMaskSize Width of the kernel.
     * @param nAnchor X offset of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRowBorder_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterRowBorder_32f_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, eBorderType));
    }
    private static native int nppiFilterRowBorder_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor,
        int eBorderType);


    /**
     * <pre>
     * 8-bit unsigned single-channel 1D row convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow32f_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow32f_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow32f_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 8-bit unsigned three-channel 1D row convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow32f_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow32f_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow32f_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    public static int nppiFilterRow32f_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow32f_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow32f_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    public static int nppiFilterRow32f_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow32f_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow32f_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    public static int nppiFilterRow32f_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow32f_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow32f_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    public static int nppiFilterRow32f_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow32f_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow32f_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 16-bit unsigned four-channel 1D row convolution.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow32f_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow32f_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow32f_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    public static int nppiFilterRow32f_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow32f_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow32f_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    public static int nppiFilterRow32f_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow32f_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow32f_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 16-bit three-channel 1D row convolution.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow32f_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow32f_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow32f_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 16-bit four-channel 1D row convolution.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow32f_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow32f_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow32f_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * 16-bit four-channel 1D row convolution ignoring alpha-channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *                Coefficients are expected to be stored in reverse order.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *                 source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRow32f_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiFilterRow32f_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor));
    }
    private static native int nppiFilterRow32f_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        TypedPointer<Npp32f> pKernel,
        int nMaskSize,
        int nAnchor);


    public static int nppiSumWindowColumn_8u32f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowColumn_8u32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowColumn_8u32f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * Three channel 8-bit unsigned 1D (column) sum to 32f.
     *
     * Apply Column Window Summation filter over a 1D mask region around each
     * source pixel for 3-channel 8 bit/pixel input images with 32-bit floating point
     * output.
     * Result 32-bit floating point pixel is equal to the sum of the corresponding and
     * neighboring column pixel values in a mask region of the source image defined by
     * nMaskSize and nAnchor.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *        source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSumWindowColumn_8u32f_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowColumn_8u32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowColumn_8u32f_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    public static int nppiSumWindowColumn_8u32f_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowColumn_8u32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowColumn_8u32f_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    public static int nppiSumWindowColumn_16u32f_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowColumn_16u32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowColumn_16u32f_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * Three channel 16-bit unsigned 1D (column) sum to 32f.
     *
     * Apply Column Window Summation filter over a 1D mask region around each
     * source pixel for 3-channel 16 bit/pixel input images with 32-bit floating point
     * output.
     * Result 32-bit floating point pixel is equal to the sum of the corresponding and
     * neighboring column pixel values in a mask region of the source image defined by
     * nMaskSize and nAnchor.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *        source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSumWindowColumn_16u32f_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowColumn_16u32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowColumn_16u32f_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * Four channel 16-bit unsigned 1D (column) sum to 32f.
     *
     * Apply Column Window Summation filter over a 1D mask region around each
     * source pixel for 4-channel 16 bit/pixel input images with 32-bit floating point
     * output.
     * Result 32-bit floating point pixel is equal to the sum of the corresponding and
     * neighboring column pixel values in a mask region of the source image defined by
     * nMaskSize and nAnchor.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *        source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSumWindowColumn_16u32f_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowColumn_16u32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowColumn_16u32f_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    public static int nppiSumWindowColumn_16s32f_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowColumn_16s32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowColumn_16s32f_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * Three channel 16-bit signed 1D (column) sum to 32f.
     *
     * Apply Column Window Summation filter over a 1D mask region around each
     * source pixel for 1-channel 16 bit/pixel input images with 32-bit floating point
     * output.
     * Result 32-bit floating point pixel is equal to the sum of the corresponding and
     * neighboring column pixel values in a mask region of the source image defined by
     * nMaskSize and nAnchor.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *        source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSumWindowColumn_16s32f_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowColumn_16s32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowColumn_16s32f_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * Four channel 16-bit signed 1D (column) sum to 32f.
     *
     * Apply Column Window Summation filter over a 1D mask region around each
     * source pixel for 4-channel 16 bit/pixel input images with 32-bit floating point
     * output.
     * Result 32-bit floating point pixel is equal to the sum of the corresponding and
     * neighboring column pixel values in a mask region of the source image defined by
     * nMaskSize and nAnchor.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor Y offset of the kernel origin frame of reference relative to the
     *        source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSumWindowColumn_16s32f_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowColumn_16s32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowColumn_16s32f_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * One channel 8-bit unsigned 1D (row) sum to 32f.
     *
     * Apply Row Window Summation filter over a 1D mask region around each source
     * pixel for 1-channel 8-bit pixel input images with 32-bit floating point output.
     * Result 32-bit floating point pixel is equal to the sum of the corresponding and
     * neighboring row pixel values in a mask region of the source image defined
     * by iKernelDim and iAnchorX.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *        source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSumWindowRow_8u32f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowRow_8u32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowRow_8u32f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * Three channel 8-bit unsigned 1D (row) sum to 32f.
     *
     * Apply Row Window Summation filter over a 1D mask region around each source
     * pixel for 3-channel 8-bit pixel input images with 32-bit floating point output.
     * Result 32-bit floating point pixel is equal to the sum of the corresponding and
     * neighboring row pixel values in a mask region of the source image defined
     * by iKernelDim and iAnchorX.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *        source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSumWindowRow_8u32f_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowRow_8u32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowRow_8u32f_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * Four channel 8-bit unsigned 1D (row) sum to 32f.
     *
     * Apply Row Window Summation filter over a 1D mask region around each source
     * pixel for 4-channel 8-bit pixel input images with 32-bit floating point output.
     * Result 32-bit floating point pixel is equal to the sum of the corresponding and
     * neighboring row pixel values in a mask region of the source image defined
     * by iKernelDim and iAnchorX.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *        source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSumWindowRow_8u32f_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowRow_8u32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowRow_8u32f_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    public static int nppiSumWindowRow_16u32f_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowRow_16u32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowRow_16u32f_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * Three channel 16-bit unsigned 1D (row) sum to 32f.
     *
     * Apply Row Window Summation filter over a 1D mask region around each source
     * pixel for 3-channel 16-bit pixel input images with 32-bit floating point output.
     * Result 32-bit floating point pixel is equal to the sum of the corresponding and
     * neighboring row pixel values in a mask region of the source image defined
     * by iKernelDim and iAnchorX.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *        source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSumWindowRow_16u32f_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowRow_16u32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowRow_16u32f_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * Four channel 16-bit unsigned 1D (row) sum to 32f.
     *
     * Apply Row Window Summation filter over a 1D mask region around each source
     * pixel for 4-channel 16-bit pixel input images with 32-bit floating point output.
     * Result 32-bit floating point pixel is equal to the sum of the corresponding and
     * neighboring row pixel values in a mask region of the source image defined
     * by iKernelDim and iAnchorX.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *        source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSumWindowRow_16u32f_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowRow_16u32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowRow_16u32f_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * One channel 16-bit signed 1D (row) sum to 32f.
     *
     * Apply Row Window Summation filter over a 1D mask region around each source
     * pixel for 1-channel 16-bit pixel input images with 32-bit floating point output.
     * Result 32-bit floating point pixel is equal to the sum of the corresponding and
     * neighboring row pixel values in a mask region of the source image defined
     * by iKernelDim and iAnchorX.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *        source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSumWindowRow_16s32f_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowRow_16s32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowRow_16s32f_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    /**
     * <pre>
     * Three channel 16-bit signed 1D (row) sum to 32f.
     *
     * Apply Row Window Summation filter over a 1D mask region around each source
     * pixel for 3-channel 16-bit pixel input images with 32-bit floating point output.
     * Result 32-bit floating point pixel is equal to the sum of the corresponding and
     * neighboring row pixel values in a mask region of the source image defined
     * by iKernelDim and iAnchorX.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param nMaskSize Length of the linear kernel array.
     * @param nAnchor X offset of the kernel origin frame of reference relative to the
     *        source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSumWindowRow_16s32f_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowRow_16s32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowRow_16s32f_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    public static int nppiSumWindowRow_16s32f_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor)
    {
        return JNpp.checkResult(nppiSumWindowRow_16s32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor));
    }
    private static native int nppiSumWindowRow_16s32f_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int nMaskSize,
        int nAnchor);


    public static int nppiFilter_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilter_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor));
    }
    private static native int nppiFilter_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor);


    /**
     * <pre>
     * Three channel 8-bit unsigned convolution filter.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilter_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor));
    }
    private static native int nppiFilter_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor);


    public static int nppiFilter_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilter_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor));
    }
    private static native int nppiFilter_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor);


    /**
     * <pre>
     * Four channel 8-bit unsigned convolution filter, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilter_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor));
    }
    private static native int nppiFilter_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor);


    public static int nppiFilter_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilter_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor));
    }
    private static native int nppiFilter_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor);


    /**
     * <pre>
     * Three channel 16-bit unsigned convolution filter.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilter_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor));
    }
    private static native int nppiFilter_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor);


    /**
     * <pre>
     * Four channel channel 16-bit unsigned convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilter_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor));
    }
    private static native int nppiFilter_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor);


    /**
     * <pre>
     * Four channel 16-bit unsigned convolution filter, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilter_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor));
    }
    private static native int nppiFilter_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor);


    /**
     * <pre>
     * Single channel 16-bit convolution filter.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilter_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor));
    }
    private static native int nppiFilter_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor);


    /**
     * <pre>
     * Three channel 16-bit convolution filter.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilter_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor));
    }
    private static native int nppiFilter_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor);


    /**
     * <pre>
     * Four channel channel 16-bit convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilter_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor));
    }
    private static native int nppiFilter_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor);


    /**
     * <pre>
     * Four channel 16-bit convolution filter, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor)
    {
        return JNpp.checkResult(nppiFilter_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor));
    }
    private static native int nppiFilter_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor);


    public static int nppiFilter_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    public static int nppiFilter_32f_C2R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter_32f_C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter_32f_C2RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 32-bit float convolution filter.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    public static int nppiFilter_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 32-bit float convolution filter, ignoring alpha channel.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 64-bit float convolution filter.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter_64f_C1R(
        TypedPointer<Npp64f> pSrc,
        int nSrcStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter_64f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter_64f_C1RNative(
        TypedPointer<Npp64f> pSrc,
        int nSrcStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 8-bit unsigned convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Two channel 8-bit unsigned convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_8u_C2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8u_C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8u_C2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 8-bit unsigned convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    public static int nppiFilter32f_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 8-bit unsigned convolution filter, ignorint alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 8-bit signed convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_8s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    public static int nppiFilter32f_8s_C2R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8s_C2RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8s_C2RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    public static int nppiFilter32f_8s_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8s_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 8-bit signed convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_8s_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8s_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    public static int nppiFilter32f_8s_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8s_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 16-bit unsigned convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 16-bit unsigned convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    public static int nppiFilter32f_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 16-bit unsigned convolution filter, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 16-bit convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 16-bit convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 16-bit convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 16-bit convolution filter, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 32-bit convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_32s_C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_32s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_32s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 32-bit convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_32s_C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_32s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_32s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 32-bit convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_32s_C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_32s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_32s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 32-bit convolution filter, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_32s_AC4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_32s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    public static int nppiFilter32f_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8u16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 8-bit unsigned to 16-bit signed convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_8u16s_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8u16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8u16s_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    public static int nppiFilter32f_8u16s_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8u16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8u16s_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 8-bit unsigned to 16-bit signed convolution filter, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_8u16s_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8u16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8u16s_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 8-bit to 16-bit signed convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8s16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 8-bit to 16-bit signed convolution filter.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_8s16s_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8s16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8s16s_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    public static int nppiFilter32f_8s16s_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8s16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8s16s_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 8-bit to 16-bit signed convolution filter, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilter32f_8s16s_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilter32f_8s16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor));
    }
    private static native int nppiFilter32f_8s16s_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 8-bit unsigned convolution filter with border control.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_8u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterBorder_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Three channel 8-bit unsigned convolution filter with border control.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_8u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterBorder_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Four channel channel 8-bit unsigned convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_8u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterBorder_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 8-bit unsigned convolution filter with border control, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_8u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterBorder_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Single channel 16-bit unsigned convolution filter with border control.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_16u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterBorder_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Three channel 16-bit unsigned convolution filter with border control.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_16u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterBorder_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Four channel channel 16-bit unsigned convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_16u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterBorder_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType);


    public static int nppiFilterBorder_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_16u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterBorder_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Single channel 16-bit convolution filter with border control.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterBorder_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType);


    /**
     * <pre>
     * Three channel 16-bit convolution filter with border control.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param nDivisor The factor by which the convolved summation from the Filter
     *        operation should be divided.  If equal to the sum of coefficients,
     *        this will keep the maximum result value within full scale.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_16s_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterBorder_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType);


    public static int nppiFilterBorder_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_16s_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterBorder_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType);


    public static int nppiFilterBorder_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_16s_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType));
    }
    private static native int nppiFilterBorder_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int nDivisor,
        int eBorderType);


    public static int nppiFilterBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    public static int nppiFilterBorder_32f_C2R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_32f_C2RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder_32f_C2RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Three channel 32-bit float convolution filter with border control.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_32f_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 32-bit float convolution filter with border control.
     *
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_32f_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    public static int nppiFilterBorder_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder_32f_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    public static int nppiFilterBorder32f_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Two channel 8-bit unsigned convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_8u_C2R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8u_C2RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8u_C2RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    public static int nppiFilterBorder32f_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 8-bit unsigned convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    public static int nppiFilterBorder32f_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Single channel 8-bit signed convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_8s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    public static int nppiFilterBorder32f_8s_C2R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8s_C2RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8s_C2RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    public static int nppiFilterBorder32f_8s_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8s_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8s_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 8-bit signed convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_8s_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8s_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8s_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 8-bit signed convolution filter with border control, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_8s_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8s_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8s_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Single channel 16-bit unsigned convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_16u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Three channel 16-bit unsigned convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_16u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 16-bit unsigned convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_16u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 16-bit unsigned convolution filter with border control, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_16u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Single channel 16-bit convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Three channel 16-bit convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_16s_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 16-bit convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_16s_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 16-bit convolution filter with border control, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_16s_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Single channel 32-bit convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_32s_C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_32s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_32s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Three channel 32-bit convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_32s_C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_32s_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_32s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 32-bit convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_32s_C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_32s_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_32s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 32-bit convolution filter with border control, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_32s_AC4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_32s_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    public static int nppiFilterBorder32f_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8u16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Three channel 8-bit unsigned to 16-bit signed convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_8u16s_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8u16s_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8u16s_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 8-bit unsigned to 16-bit signed convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_8u16s_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8u16s_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8u16s_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 8-bit unsigned to 16-bit signed convolution filter with border control, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_8u16s_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8u16s_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8u16s_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Single channel 8-bit to 16-bit signed convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8s16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Three channel 8-bit to 16-bit signed convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_8s16s_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8s16s_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8s16s_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 8-bit to 16-bit signed convolution filter with border control.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_8s16s_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8s16s_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8s16s_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Four channel 8-bit to 16-bit signed convolution filter with border control, ignoring alpha channel.
     *
     * @param pSrc  \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pKernel Pointer to the start address of the kernel coefficient array.
     *        Coeffcients are expected to be stored in reverse order.
     * @param oKernelSize Width and Height of the rectangular kernel.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBorder32f_8s16s_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterBorder32f_8s16s_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType));
    }
    private static native int nppiFilterBorder32f_8s16s_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pKernel,
        NppiSize oKernelSize,
        NppiPoint oAnchor,
        int eBorderType);


    /**
     * <pre>
     * Single channel 8-bit unsigned box filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Avg operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
     *        the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBox_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    public static int nppiFilterBox_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 8-bit unsigned box filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Avg operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
     *        the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBox_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 8-bit unsigned box filter, ignorting alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Avg operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
     *        the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBox_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    public static int nppiFilterBox_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 16-bit unsigned box filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Avg operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
     *        the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBox_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 16-bit unsigned box filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Avg operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
     *        the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBox_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 16-bit unsigned box filter, ignorting alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Avg operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
     *        the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBox_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 16-bit box filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Avg operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
     *        the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBox_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    public static int nppiFilterBox_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 16-bit box filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Avg operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
     *        the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBox_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 16-bit box filter, ignorting alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Avg operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
     *        the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBox_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    public static int nppiFilterBox_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 32-bit floating-point box filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Avg operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
     *        the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBox_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 32-bit floating-point box filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Avg operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
     *        the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBox_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 32-bit floating-point box filter, ignorting alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Avg operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
     *        the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBox_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 64-bit floating-point box filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Avg operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
     *        the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterBox_64f_C1R(
        TypedPointer<Npp64f> pSrc,
        int nSrcStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterBox_64f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterBox_64f_C1RNative(
        TypedPointer<Npp64f> pSrc,
        int nSrcStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 8-bit unsigned maximum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMax_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMax_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMax_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 8-bit unsigned maximum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMax_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMax_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMax_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 8-bit unsigned maximum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMax_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMax_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMax_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 8-bit unsigned maximum filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMax_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMax_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMax_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 16-bit unsigned maximum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMax_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMax_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMax_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 16-bit unsigned maximum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMax_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMax_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMax_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    public static int nppiFilterMax_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMax_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMax_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 16-bit unsigned maximum filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMax_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMax_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMax_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 16-bit signed maximum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMax_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMax_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMax_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 16-bit signed maximum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMax_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMax_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMax_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 16-bit signed maximum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMax_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMax_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMax_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 16-bit signed maximum filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMax_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMax_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMax_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 32-bit floating-point maximum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMax_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMax_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMax_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 32-bit floating-point maximum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMax_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMax_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMax_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 32-bit floating-point maximum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMax_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMax_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMax_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 32-bit floating-point maximum filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMax_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMax_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMax_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 8-bit unsigned minimum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMin_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMin_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMin_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 8-bit unsigned minimum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMin_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMin_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMin_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 8-bit unsigned minimum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMin_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMin_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMin_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 8-bit unsigned minimum filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMin_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMin_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMin_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 16-bit unsigned minimum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMin_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMin_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMin_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 16-bit unsigned minimum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMin_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMin_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMin_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 16-bit unsigned minimum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMin_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMin_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMin_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 16-bit unsigned minimum filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMin_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMin_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMin_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 16-bit signed minimum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMin_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMin_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMin_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 16-bit signed minimum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMin_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMin_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMin_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    public static int nppiFilterMin_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMin_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMin_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 16-bit signed minimum filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMin_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMin_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMin_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 32-bit floating-point minimum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMin_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMin_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMin_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Three channel 32-bit floating-point minimum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMin_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMin_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMin_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Four channel 32-bit floating-point minimum filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Max operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMin_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMin_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMin_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    public static int nppiFilterMin_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor)
    {
        return JNpp.checkResult(nppiFilterMin_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor));
    }
    private static native int nppiFilterMin_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor);


    /**
     * <pre>
     * Single channel 8-bit unsigned median filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Median operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMedian_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiFilterMedian_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer));
    }
    private static native int nppiFilterMedian_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Three channel 8-bit unsigned median filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Median operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMedian_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiFilterMedian_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer));
    }
    private static native int nppiFilterMedian_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer);


    public static int nppiFilterMedian_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiFilterMedian_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer));
    }
    private static native int nppiFilterMedian_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four channel 8-bit unsigned median filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Median operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMedian_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiFilterMedian_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer));
    }
    private static native int nppiFilterMedian_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer);


    public static int nppiFilterMedian_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiFilterMedian_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer));
    }
    private static native int nppiFilterMedian_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Three channel 16-bit unsigned median filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Median operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMedian_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiFilterMedian_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer));
    }
    private static native int nppiFilterMedian_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four channel 16-bit unsigned median filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Median operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMedian_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiFilterMedian_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer));
    }
    private static native int nppiFilterMedian_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four channel 16-bit unsigned median filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Median operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMedian_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiFilterMedian_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer));
    }
    private static native int nppiFilterMedian_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Single channel 16-bit signed median filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Median operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMedian_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiFilterMedian_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer));
    }
    private static native int nppiFilterMedian_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Three channel 16-bit signed median filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Median operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMedian_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiFilterMedian_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer));
    }
    private static native int nppiFilterMedian_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four channel 16-bit signed median filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Median operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMedian_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiFilterMedian_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer));
    }
    private static native int nppiFilterMedian_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer);


    public static int nppiFilterMedian_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiFilterMedian_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer));
    }
    private static native int nppiFilterMedian_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Single channel 32-bit floating-point median filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Median operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMedian_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiFilterMedian_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer));
    }
    private static native int nppiFilterMedian_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Three channel 32-bit floating-point median filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Median operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMedian_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiFilterMedian_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer));
    }
    private static native int nppiFilterMedian_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four channel 32-bit floating-point median filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Median operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMedian_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiFilterMedian_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer));
    }
    private static native int nppiFilterMedian_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four channel 32-bit floating-point median filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local
     *        Median operation.
     * @param oAnchor X and Y offsets of the kernel origin frame of reference
     *        relative to the source pixel.
     * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterMedian_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiFilterMedian_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer));
    }
    private static native int nppiFilterMedian_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        NppiPoint oAnchor,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Single channel 8-bit unsigned median filter scratch memory size.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
     * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes
     * </pre>
     */
    public static int nppiFilterMedianGetBufferSize_8u_C1R(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize)
    {
        return JNpp.checkResult(nppiFilterMedianGetBufferSize_8u_C1RNative(oSizeROI, oMaskSize, nBufferSize));
    }
    private static native int nppiFilterMedianGetBufferSize_8u_C1RNative(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize);


    /**
     * <pre>
     * Three channel 8-bit unsigned median filter scratch memory size.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
     * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes
     * </pre>
     */
    public static int nppiFilterMedianGetBufferSize_8u_C3R(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize)
    {
        return JNpp.checkResult(nppiFilterMedianGetBufferSize_8u_C3RNative(oSizeROI, oMaskSize, nBufferSize));
    }
    private static native int nppiFilterMedianGetBufferSize_8u_C3RNative(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize);


    /**
     * <pre>
     * Four channel 8-bit unsigned median filter scratch memory size.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
     * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes
     * </pre>
     */
    public static int nppiFilterMedianGetBufferSize_8u_C4R(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize)
    {
        return JNpp.checkResult(nppiFilterMedianGetBufferSize_8u_C4RNative(oSizeROI, oMaskSize, nBufferSize));
    }
    private static native int nppiFilterMedianGetBufferSize_8u_C4RNative(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize);


    /**
     * <pre>
     * Four channel 8-bit unsigned median filter, ignoring alpha channel.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
     * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes
     * </pre>
     */
    public static int nppiFilterMedianGetBufferSize_8u_AC4R(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize)
    {
        return JNpp.checkResult(nppiFilterMedianGetBufferSize_8u_AC4RNative(oSizeROI, oMaskSize, nBufferSize));
    }
    private static native int nppiFilterMedianGetBufferSize_8u_AC4RNative(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize);


    /**
     * <pre>
     * Single channel 16-bit unsigned median filter scratch memory size.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
     * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes
     * </pre>
     */
    public static int nppiFilterMedianGetBufferSize_16u_C1R(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize)
    {
        return JNpp.checkResult(nppiFilterMedianGetBufferSize_16u_C1RNative(oSizeROI, oMaskSize, nBufferSize));
    }
    private static native int nppiFilterMedianGetBufferSize_16u_C1RNative(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize);


    /**
     * <pre>
     * Three channel 16-bit unsigned median filter scratch memory size.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
     * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes
     * </pre>
     */
    public static int nppiFilterMedianGetBufferSize_16u_C3R(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize)
    {
        return JNpp.checkResult(nppiFilterMedianGetBufferSize_16u_C3RNative(oSizeROI, oMaskSize, nBufferSize));
    }
    private static native int nppiFilterMedianGetBufferSize_16u_C3RNative(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize);


    /**
     * <pre>
     * Four channel 16-bit unsigned median filter scratch memory size.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
     * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes
     * </pre>
     */
    public static int nppiFilterMedianGetBufferSize_16u_C4R(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize)
    {
        return JNpp.checkResult(nppiFilterMedianGetBufferSize_16u_C4RNative(oSizeROI, oMaskSize, nBufferSize));
    }
    private static native int nppiFilterMedianGetBufferSize_16u_C4RNative(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize);


    /**
     * <pre>
     * Four channel 16-bit unsigned median filter, ignoring alpha channel.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
     * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes
     * </pre>
     */
    public static int nppiFilterMedianGetBufferSize_16u_AC4R(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize)
    {
        return JNpp.checkResult(nppiFilterMedianGetBufferSize_16u_AC4RNative(oSizeROI, oMaskSize, nBufferSize));
    }
    private static native int nppiFilterMedianGetBufferSize_16u_AC4RNative(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize);


    /**
     * <pre>
     * Single channel 16-bit signed median filter scratch memory size.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
     * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes
     * </pre>
     */
    public static int nppiFilterMedianGetBufferSize_16s_C1R(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize)
    {
        return JNpp.checkResult(nppiFilterMedianGetBufferSize_16s_C1RNative(oSizeROI, oMaskSize, nBufferSize));
    }
    private static native int nppiFilterMedianGetBufferSize_16s_C1RNative(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize);


    /**
     * <pre>
     * Three channel 16-bit signed median filter scratch memory size.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
     * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes
     * </pre>
     */
    public static int nppiFilterMedianGetBufferSize_16s_C3R(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize)
    {
        return JNpp.checkResult(nppiFilterMedianGetBufferSize_16s_C3RNative(oSizeROI, oMaskSize, nBufferSize));
    }
    private static native int nppiFilterMedianGetBufferSize_16s_C3RNative(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize);


    /**
     * <pre>
     * Four channel 16-bit signed median filter scratch memory size.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
     * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes
     * </pre>
     */
    public static int nppiFilterMedianGetBufferSize_16s_C4R(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize)
    {
        return JNpp.checkResult(nppiFilterMedianGetBufferSize_16s_C4RNative(oSizeROI, oMaskSize, nBufferSize));
    }
    private static native int nppiFilterMedianGetBufferSize_16s_C4RNative(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize);


    /**
     * <pre>
     * Four channel 16-bit signed median filter, ignoring alpha channel.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
     * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes
     * </pre>
     */
    public static int nppiFilterMedianGetBufferSize_16s_AC4R(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize)
    {
        return JNpp.checkResult(nppiFilterMedianGetBufferSize_16s_AC4RNative(oSizeROI, oMaskSize, nBufferSize));
    }
    private static native int nppiFilterMedianGetBufferSize_16s_AC4RNative(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize);


    public static int nppiFilterMedianGetBufferSize_32f_C1R(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize)
    {
        return JNpp.checkResult(nppiFilterMedianGetBufferSize_32f_C1RNative(oSizeROI, oMaskSize, nBufferSize));
    }
    private static native int nppiFilterMedianGetBufferSize_32f_C1RNative(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize);


    /**
     * <pre>
     * Three channel 32-bit floating-point median filter scratch memory size.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
     * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes
     * </pre>
     */
    public static int nppiFilterMedianGetBufferSize_32f_C3R(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize)
    {
        return JNpp.checkResult(nppiFilterMedianGetBufferSize_32f_C3RNative(oSizeROI, oMaskSize, nBufferSize));
    }
    private static native int nppiFilterMedianGetBufferSize_32f_C3RNative(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize);


    public static int nppiFilterMedianGetBufferSize_32f_C4R(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize)
    {
        return JNpp.checkResult(nppiFilterMedianGetBufferSize_32f_C4RNative(oSizeROI, oMaskSize, nBufferSize));
    }
    private static native int nppiFilterMedianGetBufferSize_32f_C4RNative(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize);


    /**
     * <pre>
     * Four channel 32-bit floating-point median filter, ignoring alpha channel.
     * @param oSizeROI \ref roi_specification.
     * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
     * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
     * @return
     *  \ref image_data_error_codes
     * </pre>
     */
    public static int nppiFilterMedianGetBufferSize_32f_AC4R(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize)
    {
        return JNpp.checkResult(nppiFilterMedianGetBufferSize_32f_AC4RNative(oSizeROI, oMaskSize, nBufferSize));
    }
    private static native int nppiFilterMedianGetBufferSize_32f_AC4RNative(
        NppiSize oSizeROI,
        NppiSize oMaskSize,
        TypedPointer<Npp32u> nBufferSize);


    /**
     * <pre>
     * Single channel 8-bit unsigned horizontal Prewitt filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittHoriz_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittHoriz_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittHoriz_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 8-bit unsigned horizontal Prewitt filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittHoriz_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittHoriz_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittHoriz_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned horizontal Prewitt filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittHoriz_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittHoriz_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittHoriz_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiFilterPrewittHoriz_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittHoriz_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittHoriz_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 16-bit signed horizontal Prewitt filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittHoriz_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittHoriz_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittHoriz_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 16-bit signed horizontal Prewitt filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittHoriz_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittHoriz_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittHoriz_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed horizontal Prewitt filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittHoriz_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittHoriz_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittHoriz_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed horizontal Prewitt filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittHoriz_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittHoriz_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittHoriz_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 32-bit floating-point horizontal Prewitt filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittHoriz_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittHoriz_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittHoriz_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 32-bit floating-point horizontal Prewitt filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittHoriz_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittHoriz_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittHoriz_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit floating-point horizontal Prewitt filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittHoriz_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittHoriz_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittHoriz_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit floating-point horizontal Prewitt filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittHoriz_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittHoriz_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittHoriz_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiFilterPrewittVert_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittVert_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittVert_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 8-bit unsigned vertical Prewitt filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittVert_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittVert_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittVert_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned vertical Prewitt filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittVert_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittVert_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittVert_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned vertical Prewitt filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittVert_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittVert_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittVert_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 16-bit signed vertical Prewitt filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittVert_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittVert_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittVert_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiFilterPrewittVert_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittVert_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittVert_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed vertical Prewitt filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittVert_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittVert_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittVert_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiFilterPrewittVert_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittVert_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittVert_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 32-bit floating-point vertical Prewitt filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittVert_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittVert_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittVert_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiFilterPrewittVert_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittVert_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittVert_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiFilterPrewittVert_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittVert_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittVert_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit floating-point vertical Prewitt filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterPrewittVert_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterPrewittVert_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterPrewittVert_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiFilterScharrHoriz_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterScharrHoriz_8u16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterScharrHoriz_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiFilterScharrHoriz_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterScharrHoriz_8s16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterScharrHoriz_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 32-bit floating-point horizontal Scharr filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterScharrHoriz_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterScharrHoriz_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterScharrHoriz_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 16-bit signed vertical Scharr filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterScharrVert_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterScharrVert_8u16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterScharrVert_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit signed to 16-bit signed vertical Scharr filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterScharrVert_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterScharrVert_8s16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterScharrVert_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 32-bit floating-point vertical Scharr filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterScharrVert_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterScharrVert_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterScharrVert_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 16-bit signed horizontal Scharr filter kernel with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterScharrHorizBorder_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterScharrHorizBorder_8u16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterScharrHorizBorder_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    public static int nppiFilterScharrHorizBorder_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterScharrHorizBorder_8s16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterScharrHorizBorder_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Single channel 32-bit floating-point horizontal Scharr filter kernel with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterScharrHorizBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterScharrHorizBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterScharrHorizBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 16-bit signed vertical Scharr filter kernel with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterScharrVertBorder_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterScharrVertBorder_8u16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterScharrVertBorder_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Single channel 8-bit signed to 16-bit signed vertical Scharr filter kernel with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterScharrVertBorder_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterScharrVertBorder_8s16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterScharrVertBorder_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Single channel 32-bit floating-point vertical Scharr filter kernel with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterScharrVertBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterScharrVertBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterScharrVertBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Single channel 8-bit unsigned horizontal Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHoriz_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelHoriz_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelHoriz_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 8-bit unsigned horizontal Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHoriz_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelHoriz_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelHoriz_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned horizontal Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHoriz_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelHoriz_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelHoriz_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed horizontal Sobel filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHoriz_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelHoriz_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelHoriz_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 16-bit signed horizontal Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHoriz_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelHoriz_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelHoriz_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiFilterSobelHoriz_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelHoriz_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelHoriz_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed horizontal Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHoriz_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelHoriz_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelHoriz_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned horizontal Sobel filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHoriz_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelHoriz_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelHoriz_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 32-bit floating-point horizontal Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHoriz_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelHoriz_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelHoriz_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiFilterSobelHoriz_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelHoriz_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelHoriz_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit floating-point horizontal Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHoriz_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelHoriz_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelHoriz_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit floating-point horizontal Sobel filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHoriz_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelHoriz_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelHoriz_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiFilterSobelHoriz_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterSobelHoriz_8u16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterSobelHoriz_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 8-bit signed to 16-bit signed horizontal Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHoriz_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterSobelHoriz_8s16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterSobelHoriz_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 32-bit floating-point horizontal Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizMask_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterSobelHorizMask_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterSobelHorizMask_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 8-bit unsigned vertical Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVert_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelVert_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelVert_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 8-bit unsigned vertical Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVert_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelVert_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelVert_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned vertical Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVert_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelVert_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelVert_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed vertical Sobel filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVert_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelVert_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelVert_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiFilterSobelVert_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelVert_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelVert_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 16-bit signed vertical Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVert_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelVert_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelVert_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed vertical Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVert_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelVert_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelVert_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned vertical Sobel filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVert_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelVert_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelVert_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 32-bit floating-point vertical Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVert_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelVert_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelVert_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 32-bit floating-point vertical Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVert_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelVert_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelVert_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit floating-point vertical Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVert_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelVert_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelVert_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit floating-point vertical Sobel filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVert_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSobelVert_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSobelVert_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 16-bit signed vertical Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVert_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterSobelVert_8u16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterSobelVert_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    public static int nppiFilterSobelVert_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterSobelVert_8s16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterSobelVert_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 32-bit floating-point vertical Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertMask_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterSobelVertMask_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterSobelVertMask_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 16-bit signed second derivative, horizontal Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizSecond_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterSobelHorizSecond_8u16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterSobelHorizSecond_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 8-bit signed to 16-bit signed second derivative, horizontal Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizSecond_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterSobelHorizSecond_8s16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterSobelHorizSecond_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 32-bit floating-point second derivative, horizontal Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizSecond_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterSobelHorizSecond_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterSobelHorizSecond_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 16-bit signed second derivative, vertical Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertSecond_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterSobelVertSecond_8u16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterSobelVertSecond_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 8-bit signed to 16-bit signed second derivative, vertical Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertSecond_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterSobelVertSecond_8s16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterSobelVertSecond_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 32-bit floating-point second derivative, vertical Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertSecond_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterSobelVertSecond_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterSobelVertSecond_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 16-bit signed second cross derivative Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelCross_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterSobelCross_8u16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterSobelCross_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 8-bit signed to 16-bit signed second cross derivative Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelCross_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterSobelCross_8s16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterSobelCross_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 32-bit floating-point second cross derivative Sobel filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelCross_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterSobelCross_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterSobelCross_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 8-bit unsigned horizontal Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizBorder_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizBorder_8u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelHorizBorder_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Three channel 8-bit unsigned horizontal Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizBorder_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizBorder_8u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelHorizBorder_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four channel 8-bit unsigned horizontal Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizBorder_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizBorder_8u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelHorizBorder_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four channel 16-bit signed horizontal Sobel filter with border control, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizBorder_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizBorder_8u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelHorizBorder_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Single channel 16-bit signed horizontal Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizBorder_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizBorder_16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelHorizBorder_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Three channel 16-bit signed horizontal Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizBorder_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizBorder_16s_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelHorizBorder_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four channel 16-bit signed horizontal Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizBorder_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizBorder_16s_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelHorizBorder_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four channel 8-bit unsigned horizontal Sobel filter with border control, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizBorder_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizBorder_16s_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelHorizBorder_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Single channel 32-bit floating-point horizontal Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelHorizBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    public static int nppiFilterSobelHorizBorder_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizBorder_32f_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelHorizBorder_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four channel 32-bit floating-point horizontal Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizBorder_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizBorder_32f_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelHorizBorder_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four channel 32-bit floating-point horizontal Sobel filter with border control, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizBorder_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizBorder_32f_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelHorizBorder_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    public static int nppiFilterSobelHorizBorder_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizBorder_8u16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterSobelHorizBorder_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Single channel 8-bit signed to 16-bit signed horizontal Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizBorder_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizBorder_8s16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterSobelHorizBorder_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    public static int nppiFilterSobelHorizMaskBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizMaskBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterSobelHorizMaskBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Single channel 8-bit unsigned vertical Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertBorder_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertBorder_8u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelVertBorder_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Three channel 8-bit unsigned vertical Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertBorder_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertBorder_8u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelVertBorder_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four channel 8-bit unsigned vertical Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertBorder_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertBorder_8u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelVertBorder_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four channel 16-bit signed vertical Sobel filter with border control, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertBorder_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertBorder_8u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelVertBorder_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    public static int nppiFilterSobelVertBorder_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertBorder_16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelVertBorder_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    public static int nppiFilterSobelVertBorder_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertBorder_16s_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelVertBorder_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four channel 16-bit signed vertical Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertBorder_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertBorder_16s_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelVertBorder_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four channel 8-bit unsigned vertical Sobel filter with border control, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertBorder_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertBorder_16s_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelVertBorder_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    public static int nppiFilterSobelVertBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelVertBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Three channel 32-bit floating-point vertical Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertBorder_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertBorder_32f_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelVertBorder_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four channel 32-bit floating-point vertical Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertBorder_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertBorder_32f_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelVertBorder_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Four channel 32-bit floating-point vertical Sobel filter with border control, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertBorder_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertBorder_32f_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType));
    }
    private static native int nppiFilterSobelVertBorder_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eBorderType);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 16-bit signed vertical Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertBorder_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertBorder_8u16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterSobelVertBorder_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Single channel 8-bit signed to 16-bit signed vertical Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertBorder_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertBorder_8s16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterSobelVertBorder_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Single channel 32-bit floating-point vertical Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertMaskBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertMaskBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterSobelVertMaskBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 16-bit signed second derivative, horizontal Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizSecondBorder_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizSecondBorder_8u16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterSobelHorizSecondBorder_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    public static int nppiFilterSobelHorizSecondBorder_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizSecondBorder_8s16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterSobelHorizSecondBorder_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Single channel 32-bit floating-point second derivative, horizontal Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelHorizSecondBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelHorizSecondBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterSobelHorizSecondBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 16-bit signed second derivative, vertical Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertSecondBorder_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertSecondBorder_8u16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterSobelVertSecondBorder_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Single channel 8-bit signed to 16-bit signed second derivative, vertical Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertSecondBorder_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertSecondBorder_8s16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterSobelVertSecondBorder_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Single channel 32-bit floating-point second derivative, vertical Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelVertSecondBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelVertSecondBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterSobelVertSecondBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 16-bit signed second cross derivative Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelCrossBorder_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelCrossBorder_8u16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterSobelCrossBorder_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Single channel 8-bit signed to 16-bit signed second cross derivative Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelCrossBorder_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelCrossBorder_8s16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterSobelCrossBorder_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Single channel 32-bit floating-point second cross derivative Sobel filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSobelCrossBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterSobelCrossBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterSobelCrossBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * @name FilterRobertsDown
     *
     * Filters the image using a horizontal Roberts filter kernel:
     *
     * \f[
     *  \left( \begin{array}{rrr}
     *   0 & 0 &  0 \\
     *   0 & 1 &  0 \\
     *   0 & 0 & -1 \\
     *  \end{array} \right)
     * \f]
     *
     * @{
     *
     * </pre>
     */
    /**
     * <pre>
     * Single channel 8-bit unsigned horizontal Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsDown_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsDown_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsDown_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 8-bit unsigned horizontal Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsDown_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsDown_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsDown_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned horizontal Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsDown_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsDown_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsDown_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned horizontal Roberts filter, ignoring alpha-channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsDown_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsDown_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsDown_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 16-bit signed horizontal Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsDown_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsDown_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsDown_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 16-bit signed horizontal Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsDown_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsDown_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsDown_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed horizontal Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsDown_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsDown_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsDown_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed horizontal Roberts filter, ignoring alpha-channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsDown_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsDown_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsDown_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 32-bit floating-point horizontal Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsDown_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsDown_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsDown_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 32-bit floating-point horizontal Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsDown_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsDown_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsDown_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit floating-point horizontal Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsDown_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsDown_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsDown_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit floating-point horizontal Roberts filter, ignoring alpha-channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsDown_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsDown_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsDown_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit unsigned vertical Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsUp_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsUp_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsUp_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 8-bit unsigned vertical Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsUp_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsUp_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsUp_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned vertical Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsUp_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsUp_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsUp_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned vertical Roberts filter, ignoring alpha-channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsUp_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsUp_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsUp_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 16-bit signed vertical Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsUp_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsUp_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsUp_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 16-bit signed vertical Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsUp_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsUp_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsUp_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed vertical Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsUp_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsUp_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsUp_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed vertical Roberts filter, ignoring alpha-channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsUp_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsUp_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsUp_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 32-bit floating-point vertical Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsUp_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsUp_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsUp_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 32-bit floating-point vertical Roberts filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsUp_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsUp_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsUp_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiFilterRobertsUp_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsUp_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsUp_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit floating-point vertical Roberts filter, ignoring alpha-channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterRobertsUp_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterRobertsUp_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterRobertsUp_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 8-bit unsigned Laplace filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLaplace_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLaplace_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLaplace_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Three channel 8-bit unsigned Laplace filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLaplace_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLaplace_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLaplace_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 8-bit unsigned Laplace filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLaplace_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLaplace_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLaplace_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 8-bit unsigned Laplace filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLaplace_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLaplace_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLaplace_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 16-bit signed Laplace filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLaplace_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLaplace_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLaplace_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Three channel 16-bit signed Laplace filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLaplace_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLaplace_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLaplace_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 16-bit signed Laplace filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLaplace_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLaplace_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLaplace_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 16-bit signed Laplace filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLaplace_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLaplace_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLaplace_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 32-bit floating-point Laplace filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLaplace_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLaplace_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLaplace_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Three channel 32-bit floating-point Laplace filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLaplace_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLaplace_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLaplace_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    public static int nppiFilterLaplace_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLaplace_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLaplace_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 32-bit floating-point Laplace filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLaplace_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLaplace_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLaplace_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 8-bit unsigned to 16-bit signed Laplace filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLaplace_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLaplace_8u16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLaplace_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 8-bit signed to 16-bit signed Laplace filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLaplace_8s16s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLaplace_8s16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLaplace_8s16s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 8-bit unsigned Gauss filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGauss_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterGauss_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterGauss_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Three channel 8-bit unsigned Gauss filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGauss_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterGauss_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterGauss_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 8-bit unsigned Gauss filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGauss_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterGauss_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterGauss_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 8-bit unsigned Gauss filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGauss_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterGauss_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterGauss_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 16-bit unsigned Gauss filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGauss_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterGauss_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterGauss_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    public static int nppiFilterGauss_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterGauss_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterGauss_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    public static int nppiFilterGauss_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterGauss_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterGauss_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 16-bit unsigned Gauss filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGauss_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterGauss_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterGauss_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 16-bit signed Gauss filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGauss_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterGauss_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterGauss_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Three channel 16-bit signed Gauss filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGauss_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterGauss_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterGauss_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 16-bit signed Gauss filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGauss_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterGauss_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterGauss_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 16-bit signed Gauss filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGauss_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterGauss_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterGauss_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 32-bit floating-point Gauss filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGauss_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterGauss_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterGauss_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Three channel 32-bit floating-point Gauss filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGauss_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterGauss_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterGauss_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 32-bit floating-point Gauss filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGauss_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterGauss_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterGauss_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 32-bit floating-point Gauss filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGauss_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterGauss_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterGauss_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 8-bit unsigned Gauss filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGaussBorder_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterGaussBorder_8u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterGaussBorder_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Three channel 8-bit unsigned Gauss filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGaussBorder_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterGaussBorder_8u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterGaussBorder_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Four channel 8-bit unsigned Gauss filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGaussBorder_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterGaussBorder_8u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterGaussBorder_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Four channel 8-bit unsigned Gauss filter with border control, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGaussBorder_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterGaussBorder_8u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterGaussBorder_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Single channel 16-bit unsigned Gauss filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGaussBorder_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterGaussBorder_16u_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterGaussBorder_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Three channel 16-bit unsigned Gauss filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGaussBorder_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterGaussBorder_16u_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterGaussBorder_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Four channel 16-bit unsigned Gauss filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGaussBorder_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterGaussBorder_16u_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterGaussBorder_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Four channel 16-bit unsigned Gauss filter with border control, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGaussBorder_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterGaussBorder_16u_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterGaussBorder_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Single channel 16-bit signed Gauss filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGaussBorder_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterGaussBorder_16s_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterGaussBorder_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Three channel 16-bit signed Gauss filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGaussBorder_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterGaussBorder_16s_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterGaussBorder_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Four channel 16-bit signed Gauss filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGaussBorder_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterGaussBorder_16s_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterGaussBorder_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    public static int nppiFilterGaussBorder_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterGaussBorder_16s_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterGaussBorder_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Single channel 32-bit floating-point Gauss filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGaussBorder_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterGaussBorder_32f_C1RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterGaussBorder_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Three channel 32-bit floating-point Gauss filter with border control.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGaussBorder_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterGaussBorder_32f_C3RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterGaussBorder_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    public static int nppiFilterGaussBorder_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterGaussBorder_32f_C4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterGaussBorder_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Four channel 32-bit floating-point Gauss filter with border control, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Source image width and height in pixels relative to pSrc.
     * @param oSrcOffset Source image starting point relative to pSrc.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @param eBorderType The border type operation to be applied at source image border boundaries.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterGaussBorder_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType)
    {
        return JNpp.checkResult(nppiFilterGaussBorder_32f_AC4RNative(pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType));
    }
    private static native int nppiFilterGaussBorder_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcSize,
        NppiPoint oSrcOffset,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize,
        int eBorderType);


    /**
     * <pre>
     * Single channel 8-bit unsigned high-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterHighPass_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterHighPass_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterHighPass_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Three channel 8-bit unsigned high-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterHighPass_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterHighPass_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterHighPass_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 8-bit unsigned high-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterHighPass_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterHighPass_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterHighPass_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 8-bit unsigned high-pass filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterHighPass_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterHighPass_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterHighPass_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 16-bit unsigned high-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterHighPass_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterHighPass_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterHighPass_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    public static int nppiFilterHighPass_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterHighPass_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterHighPass_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    public static int nppiFilterHighPass_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterHighPass_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterHighPass_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 16-bit unsigned high-pass filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterHighPass_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterHighPass_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterHighPass_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 16-bit signed high-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterHighPass_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterHighPass_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterHighPass_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Three channel 16-bit signed high-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterHighPass_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterHighPass_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterHighPass_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 16-bit signed high-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterHighPass_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterHighPass_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterHighPass_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 16-bit signed high-pass filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterHighPass_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterHighPass_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterHighPass_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 32-bit floating-point high-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterHighPass_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterHighPass_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterHighPass_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Three channel 32-bit floating-point high-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterHighPass_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterHighPass_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterHighPass_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 32-bit floating-point high-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterHighPass_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterHighPass_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterHighPass_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 32-bit floating-point high-pass filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterHighPass_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterHighPass_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterHighPass_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 8-bit unsigned low-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLowPass_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLowPass_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLowPass_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Three channel 8-bit unsigned low-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLowPass_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLowPass_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLowPass_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 8-bit unsigned low-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLowPass_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLowPass_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLowPass_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 8-bit unsigned low-pass filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLowPass_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLowPass_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLowPass_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 16-bit unsigned low-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLowPass_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLowPass_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLowPass_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Three channel 16-bit unsigned low-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLowPass_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLowPass_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLowPass_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 16-bit unsigned low-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLowPass_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLowPass_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLowPass_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 16-bit unsigned low-pass filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLowPass_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLowPass_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLowPass_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 16-bit signed low-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLowPass_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLowPass_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLowPass_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Three channel 16-bit signed low-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLowPass_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLowPass_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLowPass_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 16-bit signed low-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLowPass_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLowPass_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLowPass_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 16-bit signed low-pass filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLowPass_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLowPass_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLowPass_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 32-bit floating-point low-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLowPass_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLowPass_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLowPass_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Three channel 32-bit floating-point low-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLowPass_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLowPass_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLowPass_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 32-bit floating-point low-pass filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLowPass_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLowPass_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLowPass_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Four channel 32-bit floating-point high-pass filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param eMaskSize Enumeration value specifying the mask size.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterLowPass_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize)
    {
        return JNpp.checkResult(nppiFilterLowPass_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize));
    }
    private static native int nppiFilterLowPass_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        int eMaskSize);


    /**
     * <pre>
     * Single channel 8-bit unsigned sharpening filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSharpen_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSharpen_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSharpen_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 8-bit unsigned sharpening filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSharpen_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSharpen_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSharpen_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned sharpening filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSharpen_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSharpen_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSharpen_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 8-bit unsigned sharpening filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSharpen_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSharpen_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSharpen_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 16-bit unsigned sharpening filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSharpen_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSharpen_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSharpen_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 16-bit unsigned sharpening filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSharpen_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSharpen_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSharpen_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit unsigned sharpening filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSharpen_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSharpen_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSharpen_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit unsigned sharpening filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSharpen_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSharpen_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSharpen_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    public static int nppiFilterSharpen_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSharpen_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSharpen_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 16-bit signed sharpening filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSharpen_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSharpen_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSharpen_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed sharpening filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSharpen_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSharpen_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSharpen_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 16-bit signed sharpening filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSharpen_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSharpen_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSharpen_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Single channel 32-bit floating-point sharpening filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSharpen_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSharpen_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSharpen_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three channel 32-bit floating-point sharpening filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSharpen_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSharpen_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSharpen_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit floating-point sharpening filter.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSharpen_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSharpen_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSharpen_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four channel 32-bit floating-point sharpening filter, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiFilterSharpen_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiFilterSharpen_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiFilterSharpen_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One-channel 8-bit unsigned image sum.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     *        Use \ref nppiSumGetBufferHostSize_8u_C1R to determine the minium number of bytes required.
     * @param pSum Pointer to the computed sum.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSum_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pSum)
    {
        return JNpp.checkResult(nppiSum_8u_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pSum));
    }
    private static native int nppiSum_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pSum);


    /**
     * <pre>
     * One-channel 8-bit unsigned image sum. The result is 64-bit long long integer.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     *        Use \ref nppiSumGetBufferHostSize_8u64s_C1R to determine the minium number of bytes required.
     * @param pSum Pointer to the computed sum.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSum_8u64s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64s> pSum)
    {
        return JNpp.checkResult(nppiSum_8u64s_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pSum));
    }
    private static native int nppiSum_8u64s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64s> pSum);


    public static int nppiSum_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pSum)
    {
        return JNpp.checkResult(nppiSum_16u_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pSum));
    }
    private static native int nppiSum_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pSum);


    /**
     * <pre>
     * One-channel 16-bit signed image sum.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     *        Use \ref nppiSumGetBufferHostSize_16s_C1R to determine the minium number of bytes required.
     * @param pSum Pointer to the computed sum.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSum_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pSum)
    {
        return JNpp.checkResult(nppiSum_16s_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pSum));
    }
    private static native int nppiSum_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pSum);


    /**
     * <pre>
     * One-channel 32-bit floating point image sum.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiSumGetBufferHostSize_32f_C1R to determine the minium number of bytes required.
     * @param pSum Pointer to the computed sum.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSum_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pSum)
    {
        return JNpp.checkResult(nppiSum_32f_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pSum));
    }
    private static native int nppiSum_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pSum);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image sum.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     *        Use \ref nppiSumGetBufferHostSize_8u_C3R to determine the minium number of bytes required.
     * @param aSum Array that contains computed sum for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSum_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum)
    {
        return JNpp.checkResult(nppiSum_8u_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum));
    }
    private static native int nppiSum_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum);


    public static int nppiSum_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum)
    {
        return JNpp.checkResult(nppiSum_16u_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum));
    }
    private static native int nppiSum_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum);


    /**
     * <pre>
     * Three-channel 16-bit signed image sum.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     *        Use \ref nppiSumGetBufferHostSize_16s_C3R to determine the minium number of bytes required.
     * @param aSum Array that contains computed sum for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSum_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum)
    {
        return JNpp.checkResult(nppiSum_16s_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum));
    }
    private static native int nppiSum_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum);


    /**
     * <pre>
     * Three-channel 32-bit floating point image sum.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     *        Use \ref nppiSumGetBufferHostSize_32f_C3R to determine the minium number of bytes required.
     * @param aSum Array that contains computed sum for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSum_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum)
    {
        return JNpp.checkResult(nppiSum_32f_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum));
    }
    private static native int nppiSum_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum);


    public static int nppiSum_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum)
    {
        return JNpp.checkResult(nppiSum_8u_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum));
    }
    private static native int nppiSum_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image sum ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     *        Use \ref nppiSumGetBufferHostSize_16u_AC4R to determine the minium number of bytes required.
     * @param aSum Array that contains computed sum for each channel (alpha channel is not computed).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSum_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum)
    {
        return JNpp.checkResult(nppiSum_16u_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum));
    }
    private static native int nppiSum_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum);


    /**
     * <pre>
     * Four-channel 16-bit signed image sum ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     *        Use \ref nppiSumGetBufferHostSize_16s_AC4R to determine the minium number of bytes required.
     * @param aSum Array that contains computed sum for each channel (alpha channel is not computed).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSum_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum)
    {
        return JNpp.checkResult(nppiSum_16s_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum));
    }
    private static native int nppiSum_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum);


    public static int nppiSum_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum)
    {
        return JNpp.checkResult(nppiSum_32f_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum));
    }
    private static native int nppiSum_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image sum.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiSumGetBufferHostSize_8u_C4R to determine the minium number of bytes required.
     * @param aSum Array that contains computed sum for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSum_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum)
    {
        return JNpp.checkResult(nppiSum_8u_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum));
    }
    private static native int nppiSum_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image sum. The result is 64-bit long long integer.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiSumGetBufferHostSize_8u64s_C4R to determine the minium number of bytes required.
     * @param aSum Array that contains computed sum for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSum_8u64s_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        long[] aSum)
    {
        return JNpp.checkResult(nppiSum_8u64s_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum));
    }
    private static native int nppiSum_8u64s_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        long[] aSum);


    public static int nppiSum_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum)
    {
        return JNpp.checkResult(nppiSum_16u_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum));
    }
    private static native int nppiSum_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum);


    /**
     * <pre>
     * Four-channel 16-bit signed image sum.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiSumGetBufferHostSize_16s_C4R to determine the minium number of bytes required.
     * @param aSum Array that contains computed sum for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSum_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum)
    {
        return JNpp.checkResult(nppiSum_16s_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum));
    }
    private static native int nppiSum_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum);


    /**
     * <pre>
     * Four-channel 32-bit floating point image sum.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiSumGetBufferHostSize_32f_C4R to determine the minium number of bytes required.
     * @param aSum Array that contains computed sum for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSum_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum)
    {
        return JNpp.checkResult(nppiSum_32f_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum));
    }
    private static native int nppiSum_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aSum);


    /**
     * <pre>
     * Buffer size for \ref nppiSum_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSumGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiSum_8u64s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSumGetBufferHostSize_8u64s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_8u64s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_8u64s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiSumGetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiSum_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSumGetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiSumGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiSum_8u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSumGetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiSum_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSumGetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiSumGetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiSum_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSumGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiSum_8u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSumGetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiSumGetBufferHostSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_16u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiSum_16s_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSumGetBufferHostSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_16s_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiSum_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSumGetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiSum_8u64s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSumGetBufferHostSize_8u64s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_8u64s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_8u64s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiSum_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSumGetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiSum_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSumGetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiSum_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSumGetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiSum_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSumGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiSumGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSumGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * One-channel 8-bit unsigned image min.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinGetBufferHostSize_8u_C1R to determine the minium number of bytes required.
     * @param pMin Pointer to the computed minimum result.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMin_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp8u> pMin)
    {
        return JNpp.checkResult(nppiMin_8u_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMin));
    }
    private static native int nppiMin_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp8u> pMin);


    /**
     * <pre>
     * One-channel 16-bit unsigned image min.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinGetBufferHostSize_16u_C1R to determine the minium number of bytes required.
     * @param pMin Pointer to the computed minimum result.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMin_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp16u> pMin)
    {
        return JNpp.checkResult(nppiMin_16u_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMin));
    }
    private static native int nppiMin_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp16u> pMin);


    /**
     * <pre>
     * One-channel 16-bit signed image min.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinGetBufferHostSize_16s_C1R to determine the minium number of bytes required.
     * @param pMin Pointer to the computed minimum result.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMin_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp16s> pMin)
    {
        return JNpp.checkResult(nppiMin_16s_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMin));
    }
    private static native int nppiMin_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp16s> pMin);


    public static int nppiMin_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp32f> pMin)
    {
        return JNpp.checkResult(nppiMin_32f_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMin));
    }
    private static native int nppiMin_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp32f> pMin);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image min.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinGetBufferHostSize_8u_C3R to determine the minium number of bytes required.
     * @param aMin Array that contains the computed minimum results for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMin_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMin)
    {
        return JNpp.checkResult(nppiMin_8u_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin));
    }
    private static native int nppiMin_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMin);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image min.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinGetBufferHostSize_16u_C3R to determine the minium number of bytes required.
     * @param aMin Array that contains the computed minimum results for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMin_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin)
    {
        return JNpp.checkResult(nppiMin_16u_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin));
    }
    private static native int nppiMin_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin);


    /**
     * <pre>
     * Three-channel 16-bit signed image min.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinGetBufferHostSize_16s_C3R to determine the minium number of bytes required.
     * @param aMin Array that contains the computed minimum results for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMin_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin)
    {
        return JNpp.checkResult(nppiMin_16s_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin));
    }
    private static native int nppiMin_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin);


    public static int nppiMin_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMin)
    {
        return JNpp.checkResult(nppiMin_32f_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin));
    }
    private static native int nppiMin_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMin);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image min.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinGetBufferHostSize_8u_C4R to determine the minium number of bytes required.
     * @param aMin Array that contains the computed minimum results for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMin_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMin)
    {
        return JNpp.checkResult(nppiMin_8u_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin));
    }
    private static native int nppiMin_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMin);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image min.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinGetBufferHostSize_16u_C4R to determine the minium number of bytes required.
     * @param aMin Array that contains the computed minimum results for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMin_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin)
    {
        return JNpp.checkResult(nppiMin_16u_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin));
    }
    private static native int nppiMin_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin);


    public static int nppiMin_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin)
    {
        return JNpp.checkResult(nppiMin_16s_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin));
    }
    private static native int nppiMin_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin);


    /**
     * <pre>
     * Four-channel 32-bit floating point image min.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinGetBufferHostSize_32f_C4R to determine the minium number of bytes required.
     * @param aMin Array that contains the computed minimum results for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMin_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMin)
    {
        return JNpp.checkResult(nppiMin_32f_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin));
    }
    private static native int nppiMin_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMin);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image min ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinGetBufferHostSize_8u_AC4R to determine the minium number of bytes required.
     * @param aMin Array that contains the computed minimum results for each channel (alpha channel is not processed).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMin_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMin)
    {
        return JNpp.checkResult(nppiMin_8u_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin));
    }
    private static native int nppiMin_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMin);


    public static int nppiMin_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin)
    {
        return JNpp.checkResult(nppiMin_16u_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin));
    }
    private static native int nppiMin_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin);


    /**
     * <pre>
     * Four-channel 16-bit signed image min ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinGetBufferHostSize_16s_AC4R to determine the minium number of bytes required.
     * @param aMin Array that contains the computed minimum results for each channel (alpha channel is not processed).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMin_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin)
    {
        return JNpp.checkResult(nppiMin_16s_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin));
    }
    private static native int nppiMin_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin);


    /**
     * <pre>
     * Four-channel 32-bit floating point image min ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinGetBufferHostSize_32f_AC4R to determine the minium number of bytes required.
     * @param aMin Array that contains the computed minimum results for each channel (alpha channel is not processed).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMin_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMin)
    {
        return JNpp.checkResult(nppiMin_32f_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin));
    }
    private static native int nppiMin_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMin);


    public static int nppiMinGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMin_16u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinGetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinGetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinGetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMinGetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinGetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinGetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMin_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMin_8u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinGetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMinGetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinGetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMin_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinGetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinGetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinGetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMin_16s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinGetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMinGetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinGetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMin_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMin_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinGetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinGetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinGetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMin_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinGetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMinGetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinGetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMin_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinGetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinGetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinGetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMin_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMinGetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinGetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinGetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMinGetBufferHostSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMinGetBufferHostSize_16u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinGetBufferHostSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiMinGetBufferHostSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinGetBufferHostSize_16s_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinGetBufferHostSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMin_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinGetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinGetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinGetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * One-channel 8-bit unsigned image MinIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinIndxGetBufferHostSize_8u_C1R to determine the minium number of bytes required.
     * @param pMin Pointer to the computed min result.
     * @param pIndexX Pointer to the X coordinate of the image min value.
     * @param pIndexY Ppointer to the Y coordinate of the image min value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinIndx_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp8u> pMin,
        int[] pIndexX,
        int[] pIndexY)
    {
        return JNpp.checkResult(nppiMinIndx_8u_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMin, pIndexX, pIndexY));
    }
    private static native int nppiMinIndx_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp8u> pMin,
        int[] pIndexX,
        int[] pIndexY);


    public static int nppiMinIndx_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp16u> pMin,
        int[] pIndexX,
        int[] pIndexY)
    {
        return JNpp.checkResult(nppiMinIndx_16u_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMin, pIndexX, pIndexY));
    }
    private static native int nppiMinIndx_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp16u> pMin,
        int[] pIndexX,
        int[] pIndexY);


    public static int nppiMinIndx_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp16s> pMin,
        int[] pIndexX,
        int[] pIndexY)
    {
        return JNpp.checkResult(nppiMinIndx_16s_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMin, pIndexX, pIndexY));
    }
    private static native int nppiMinIndx_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp16s> pMin,
        int[] pIndexX,
        int[] pIndexY);


    /**
     * <pre>
     * One-channel 32-bit floating point image MinIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinIndxGetBufferHostSize_32f_C1R to determine the minium number of bytes required.
     * @param pMin Pointer to the computed min result.
     * @param pIndexX Pointer to the X coordinate of the image min value.
     * @param pIndexY Pointer to the Y coordinate of the image min value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinIndx_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp32f> pMin,
        int[] pIndexX,
        int[] pIndexY)
    {
        return JNpp.checkResult(nppiMinIndx_32f_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMin, pIndexX, pIndexY));
    }
    private static native int nppiMinIndx_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp32f> pMin,
        int[] pIndexX,
        int[] pIndexY);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image MinIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinIndxGetBufferHostSize_8u_C3R to determine the minium number of bytes required.
     * @param aMin Array that contains the min values.
     * @param aIndexX Array that contains the X coordinates of the image min values.
     * @param aIndexY Array that contains the Y coordinates of the image min values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinIndx_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMin,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMinIndx_8u_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY));
    }
    private static native int nppiMinIndx_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMin,
        int[] aIndexX,
        int[] aIndexY);


    public static int nppiMinIndx_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMinIndx_16u_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY));
    }
    private static native int nppiMinIndx_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin,
        int[] aIndexX,
        int[] aIndexY);


    /**
     * <pre>
     * Three-channel 16-bit signed image MinIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinIndxGetBufferHostSize_16s_C3R to determine the minium number of bytes required.
     * @param aMin Array that contains the min values.
     * @param aIndexX Array that contains the X coordinates of the image min values.
     * @param aIndexY Array that contains the Y coordinates of the image min values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinIndx_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMinIndx_16s_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY));
    }
    private static native int nppiMinIndx_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin,
        int[] aIndexX,
        int[] aIndexY);


    public static int nppiMinIndx_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMin,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMinIndx_32f_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY));
    }
    private static native int nppiMinIndx_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMin,
        int[] aIndexX,
        int[] aIndexY);


    public static int nppiMinIndx_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMin,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMinIndx_8u_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY));
    }
    private static native int nppiMinIndx_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMin,
        int[] aIndexX,
        int[] aIndexY);


    public static int nppiMinIndx_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMinIndx_16u_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY));
    }
    private static native int nppiMinIndx_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin,
        int[] aIndexX,
        int[] aIndexY);


    /**
     * <pre>
     * Four-channel 16-bit signed image MinIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinIndxGetBufferHostSize_16s_C4R to determine the minium number of bytes required.
     * @param aMin Array that contains the min values.
     * @param aIndexX Array that contains the X coordinates of the image min values.
     * @param aIndexY Array that contains the Y coordinates of the image min values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinIndx_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMinIndx_16s_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY));
    }
    private static native int nppiMinIndx_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin,
        int[] aIndexX,
        int[] aIndexY);


    public static int nppiMinIndx_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMin,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMinIndx_32f_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY));
    }
    private static native int nppiMinIndx_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMin,
        int[] aIndexX,
        int[] aIndexY);


    public static int nppiMinIndx_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMin,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMinIndx_8u_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY));
    }
    private static native int nppiMinIndx_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMin,
        int[] aIndexX,
        int[] aIndexY);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image MinIndx ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinIndxGetBufferHostSize_16u_AC4R to determine the minium number of bytes required.
     * @param aMin Array that contains the min values.
     * @param aIndexX Array that contains the X coordinates of the image min values.
     * @param aIndexY Array that contains the Y coordinates of the image min values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinIndx_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMinIndx_16u_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY));
    }
    private static native int nppiMinIndx_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin,
        int[] aIndexX,
        int[] aIndexY);


    /**
     * <pre>
     * Four-channel 16-bit signed image MinIndx ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinIndxGetBufferHostSize_16s_AC4R to determine the minium number of bytes required.
     * @param aMin Array that contains the min values.
     * @param aIndexX Array that contains the X coordinates of the image min values.
     * @param aIndexY Array that contains the Y coordinates of the image min values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinIndx_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMinIndx_16s_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY));
    }
    private static native int nppiMinIndx_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMin,
        int[] aIndexX,
        int[] aIndexY);


    /**
     * <pre>
     * Four-channel 32-bit floating point image MinIndx ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMinIndxGetBufferHostSize_32f_AC4R to determine the minium number of bytes required.
     * @param aMin Array that contains the min values.
     * @param aIndexX Array that contains the X coordinates of the image min values.
     * @param aIndexY Array that contains the Y coordinates of the image min values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinIndx_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMin,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMinIndx_32f_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY));
    }
    private static native int nppiMinIndx_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMin,
        int[] aIndexX,
        int[] aIndexY);


    public static int nppiMinIndxGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinIndxGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinIndxGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_16u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinIndxGetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinIndxGetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinIndxGetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMinIndxGetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinIndxGetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinIndxGetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMinIndxGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinIndxGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinIndxGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMinIndxGetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinIndxGetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinIndxGetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinIndxGetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinIndxGetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinIndxGetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_16s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinIndxGetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinIndxGetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinIndxGetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMinIndxGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMinIndxGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinIndxGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiMinIndxGetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMinIndxGetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinIndxGetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinIndxGetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinIndxGetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinIndxGetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinIndxGetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMinIndxGetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinIndxGetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinIndxGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMinIndxGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinIndxGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_8u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinIndxGetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinIndxGetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinIndxGetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_8u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinIndxGetBufferHostSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinIndxGetBufferHostSize_16u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinIndxGetBufferHostSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMinIndxGetBufferHostSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinIndxGetBufferHostSize_16s_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinIndxGetBufferHostSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinIndxGetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinIndxGetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinIndxGetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * One-channel 8-bit unsigned image Max.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxGetBufferHostSize_8u_C1R to determaxe the maxium number of bytes required.
     * @param pMax Pointer to the computed maximum result.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMax_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp8u> pMax)
    {
        return JNpp.checkResult(nppiMax_8u_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMax));
    }
    private static native int nppiMax_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp8u> pMax);


    /**
     * <pre>
     * One-channel 16-bit unsigned image Max.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxGetBufferHostSize_16u_C1R to determaxe the maxium number of bytes required.
     * @param pMax Pointer to the computed maximum result.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMax_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp16u> pMax)
    {
        return JNpp.checkResult(nppiMax_16u_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMax));
    }
    private static native int nppiMax_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp16u> pMax);


    /**
     * <pre>
     * One-channel 16-bit signed image Max.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxGetBufferHostSize_16s_C1R to determaxe the maxium number of bytes required.
     * @param pMax Pointer to the computed maximum result.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMax_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp16s> pMax)
    {
        return JNpp.checkResult(nppiMax_16s_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMax));
    }
    private static native int nppiMax_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp16s> pMax);


    /**
     * <pre>
     * One-channel 32-bit floating point image Max.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxGetBufferHostSize_32f_C1R to determaxe the maxium number of bytes required.
     * @param pMax Pointer to the computed maximum result.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMax_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp32f> pMax)
    {
        return JNpp.checkResult(nppiMax_32f_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMax));
    }
    private static native int nppiMax_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp32f> pMax);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image Max.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxGetBufferHostSize_8u_C3R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the computed maximum results for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMax_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMax)
    {
        return JNpp.checkResult(nppiMax_8u_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax));
    }
    private static native int nppiMax_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMax);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image Max.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxGetBufferHostSize_16u_C3R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the computed maximum results for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMax_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax)
    {
        return JNpp.checkResult(nppiMax_16u_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax));
    }
    private static native int nppiMax_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax);


    /**
     * <pre>
     * Three-channel 16-bit signed image Max.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxGetBufferHostSize_16s_C3R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the computed maximum results for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMax_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax)
    {
        return JNpp.checkResult(nppiMax_16s_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax));
    }
    private static native int nppiMax_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax);


    /**
     * <pre>
     * Three-channel 32-bit floating point image Max.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxGetBufferHostSize_32f_C3R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the computed maximum results for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMax_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMax)
    {
        return JNpp.checkResult(nppiMax_32f_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax));
    }
    private static native int nppiMax_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMax);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image Max.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxGetBufferHostSize_8u_C4R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the computed maximum results for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMax_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMax)
    {
        return JNpp.checkResult(nppiMax_8u_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax));
    }
    private static native int nppiMax_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMax);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image Max.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxGetBufferHostSize_16u_C4R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the computed maximum results for each channel.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMax_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax)
    {
        return JNpp.checkResult(nppiMax_16u_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax));
    }
    private static native int nppiMax_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax);


    public static int nppiMax_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax)
    {
        return JNpp.checkResult(nppiMax_16s_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax));
    }
    private static native int nppiMax_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax);


    public static int nppiMax_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMax)
    {
        return JNpp.checkResult(nppiMax_32f_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax));
    }
    private static native int nppiMax_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMax);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image Max ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxGetBufferHostSize_8u_AC4R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the computed maximum results for each channel (alpha channel is not processed).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMax_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMax)
    {
        return JNpp.checkResult(nppiMax_8u_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax));
    }
    private static native int nppiMax_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMax);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image Max ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxGetBufferHostSize_16u_AC4R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the computed maximum results for each channel (alpha channel is not processed).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMax_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax)
    {
        return JNpp.checkResult(nppiMax_16u_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax));
    }
    private static native int nppiMax_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax);


    /**
     * <pre>
     * Four-channel 16-bit signed image Max ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxGetBufferHostSize_16s_AC4R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the computed maximum results for each channel (alpha channel is not processed).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMax_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax)
    {
        return JNpp.checkResult(nppiMax_16s_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax));
    }
    private static native int nppiMax_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax);


    /**
     * <pre>
     * Four-channel 32-bit floating point image Max ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxGetBufferHostSize_32f_AC4R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the computed maximum results for each channel (alpha channel is not processed).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMax_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMax)
    {
        return JNpp.checkResult(nppiMax_32f_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax));
    }
    private static native int nppiMax_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMax);


    /**
     * <pre>
     * Buffer size for \ref nppiMax_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMaxGetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxGetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxGetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMax_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxGetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaxGetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxGetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMax_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMax_8u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxGetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxGetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxGetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMax_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxGetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaxGetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxGetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiMaxGetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxGetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxGetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMax_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMax_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxGetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaxGetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxGetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMax_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxGetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxGetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxGetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMax_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxGetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxGetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxGetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMax_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMaxGetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxGetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxGetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMax_16u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxGetBufferHostSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxGetBufferHostSize_16u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxGetBufferHostSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMaxGetBufferHostSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxGetBufferHostSize_16s_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxGetBufferHostSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMax_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxGetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaxGetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxGetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * One-channel 8-bit unsigned image MaxIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxIndxGetBufferHostSize_8u_C1R to determaxe the maxium number of bytes required.
     * @param pMax Pointer to the computed max result.
     * @param pIndexX Pointer to the X coordinate of the image max value.
     * @param pIndexY Ppointer to the Y coordinate of the image max value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMaxIndx_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp8u> pMax,
        int[] pIndexX,
        int[] pIndexY)
    {
        return JNpp.checkResult(nppiMaxIndx_8u_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMax, pIndexX, pIndexY));
    }
    private static native int nppiMaxIndx_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp8u> pMax,
        int[] pIndexX,
        int[] pIndexY);


    /**
     * <pre>
     * One-channel 16-bit unsigned image MaxIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxIndxGetBufferHostSize_16u_C1R to determaxe the maxium number of bytes required.
     * @param pMax Pointer to the computed max result.
     * @param pIndexX Pointer to the X coordinate of the image max value.
     * @param pIndexY Pointer to the Y coordinate of the image max value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMaxIndx_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp16u> pMax,
        int[] pIndexX,
        int[] pIndexY)
    {
        return JNpp.checkResult(nppiMaxIndx_16u_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMax, pIndexX, pIndexY));
    }
    private static native int nppiMaxIndx_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp16u> pMax,
        int[] pIndexX,
        int[] pIndexY);


    /**
     * <pre>
     * One-channel 16-bit signed image MaxIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxIndxGetBufferHostSize_16s_C1R to determaxe the maxium number of bytes required.
     * @param pMax Pointer to the computed max result.
     * @param pIndexX Pointer to the X coordinate of the image max value.
     * @param pIndexY Pointer to the Y coordinate of the image max value.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMaxIndx_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp16s> pMax,
        int[] pIndexX,
        int[] pIndexY)
    {
        return JNpp.checkResult(nppiMaxIndx_16s_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMax, pIndexX, pIndexY));
    }
    private static native int nppiMaxIndx_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp16s> pMax,
        int[] pIndexX,
        int[] pIndexY);


    public static int nppiMaxIndx_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp32f> pMax,
        int[] pIndexX,
        int[] pIndexY)
    {
        return JNpp.checkResult(nppiMaxIndx_32f_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMax, pIndexX, pIndexY));
    }
    private static native int nppiMaxIndx_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp32f> pMax,
        int[] pIndexX,
        int[] pIndexY);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image MaxIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxIndxGetBufferHostSize_8u_C3R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the max values.
     * @param aIndexX Array that contains the X coordinates of the image max values.
     * @param aIndexY Array that contains the Y coordinates of the image max values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMaxIndx_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMax,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMaxIndx_8u_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY));
    }
    private static native int nppiMaxIndx_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMax,
        int[] aIndexX,
        int[] aIndexY);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image MaxIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxIndxGetBufferHostSize_16u_C3R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the max values.
     * @param aIndexX Array that contains the X coordinates of the image max values.
     * @param aIndexY Array that contains the Y coordinates of the image max values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMaxIndx_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMaxIndx_16u_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY));
    }
    private static native int nppiMaxIndx_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax,
        int[] aIndexX,
        int[] aIndexY);


    public static int nppiMaxIndx_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMaxIndx_16s_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY));
    }
    private static native int nppiMaxIndx_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax,
        int[] aIndexX,
        int[] aIndexY);


    /**
     * <pre>
     * Three-channel 32-bit floating point image MaxIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxIndxGetBufferHostSize_32f_C3R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the max values.
     * @param aIndexX Array that contains the X coordinates of the image max values.
     * @param aIndexY Array that contains the Y coordinates of the image max values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMaxIndx_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMax,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMaxIndx_32f_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY));
    }
    private static native int nppiMaxIndx_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMax,
        int[] aIndexX,
        int[] aIndexY);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image MaxIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxIndxGetBufferHostSize_8u_C4R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the max values.
     * @param aIndexX Array that contains the X coordinates of the image max values.
     * @param aIndexY Array that contains the Y coordinates of the image max values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMaxIndx_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMax,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMaxIndx_8u_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY));
    }
    private static native int nppiMaxIndx_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMax,
        int[] aIndexX,
        int[] aIndexY);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image MaxIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxIndxGetBufferHostSize_16u_C4R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the max values.
     * @param aIndexX Array that contains the X coordinates of the image max values.
     * @param aIndexY Array that contains the Y coordinates of the image max values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMaxIndx_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMaxIndx_16u_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY));
    }
    private static native int nppiMaxIndx_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax,
        int[] aIndexX,
        int[] aIndexY);


    /**
     * <pre>
     * Four-channel 16-bit signed image MaxIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxIndxGetBufferHostSize_16s_C4R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the max values.
     * @param aIndexX Array that contains the X coordinates of the image max values.
     * @param aIndexY Array that contains the Y coordinates of the image max values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMaxIndx_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMaxIndx_16s_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY));
    }
    private static native int nppiMaxIndx_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax,
        int[] aIndexX,
        int[] aIndexY);


    /**
     * <pre>
     * Four-channel 32-bit floating point image MaxIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxIndxGetBufferHostSize_32f_C4R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the max values.
     * @param aIndexX Array that contains the X coordinates of the image max values.
     * @param aIndexY Array that contains the Y coordinates of the image max values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMaxIndx_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMax,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMaxIndx_32f_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY));
    }
    private static native int nppiMaxIndx_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMax,
        int[] aIndexX,
        int[] aIndexY);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image MaxIndx ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxIndxGetBufferHostSize_8u_AC4R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the max values.
     * @param aIndexX Array that contains the X coordinates of the image max values.
     * @param aIndexY Array that contains the Y coordinates of the image max values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMaxIndx_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMax,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMaxIndx_8u_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY));
    }
    private static native int nppiMaxIndx_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        byte[] aMax,
        int[] aIndexX,
        int[] aIndexY);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image MaxIndx ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxIndxGetBufferHostSize_16u_AC4R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the max values.
     * @param aIndexX Array that contains the X coordinates of the image max values.
     * @param aIndexY Array that contains the Y coordinates of the image max values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMaxIndx_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMaxIndx_16u_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY));
    }
    private static native int nppiMaxIndx_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax,
        int[] aIndexX,
        int[] aIndexY);


    /**
     * <pre>
     * Four-channel 16-bit signed image MaxIndx ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMaxIndxGetBufferHostSize_16s_AC4R to determaxe the maxium number of bytes required.
     * @param aMax Array that contains the max values.
     * @param aIndexX Array that contains the X coordinates of the image max values.
     * @param aIndexY Array that contains the Y coordinates of the image max values.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMaxIndx_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMaxIndx_16s_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY));
    }
    private static native int nppiMaxIndx_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        short[] aMax,
        int[] aIndexX,
        int[] aIndexY);


    public static int nppiMaxIndx_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMax,
        int[] aIndexX,
        int[] aIndexY)
    {
        return JNpp.checkResult(nppiMaxIndx_32f_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY));
    }
    private static native int nppiMaxIndx_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        float[] aMax,
        int[] aIndexX,
        int[] aIndexY);


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxIndxGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxIndxGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxIndxGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMaxIndxGetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaxIndxGetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxIndxGetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxIndxGetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaxIndxGetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxIndxGetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiMaxIndxGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaxIndxGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxIndxGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiMaxIndxGetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxIndxGetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxIndxGetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxIndxGetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaxIndxGetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxIndxGetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiMaxIndxGetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxIndxGetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxIndxGetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxIndxGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxIndxGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxIndxGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxIndxGetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxIndxGetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxIndxGetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxIndxGetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxIndxGetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxIndxGetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxIndxGetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxIndxGetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxIndxGetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxIndxGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxIndxGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxIndxGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_8u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxIndxGetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaxIndxGetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxIndxGetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_8u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxIndxGetBufferHostSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxIndxGetBufferHostSize_16u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxIndxGetBufferHostSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_16u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxIndxGetBufferHostSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaxIndxGetBufferHostSize_16s_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxIndxGetBufferHostSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxIndxGetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMaxIndxGetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaxIndxGetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * One-channel 8-bit unsigned image MinMax.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMin Pointer to the computed minimal result.
     * @param pMax Pointer to the computed maximal result.
     * @param pDeviceBuffer Buffer to a scratch memory.
     *        Use \ref nppiMinMaxGetBufferHostSize_8u_C1R to determine the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinMax_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMin,
        TypedPointer<Npp8u> pMax,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMax_8u_C1RNative(pSrc, nSrcStep, oSizeROI, pMin, pMax, pDeviceBuffer));
    }
    private static native int nppiMinMax_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMin,
        TypedPointer<Npp8u> pMax,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image MinMax.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMin Pointer to the computed minimal result.
     * @param pMax Pointer to the computed maximal result.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_16u_C1R to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinMax_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16u> pMin,
        TypedPointer<Npp16u> pMax,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMax_16u_C1RNative(pSrc, nSrcStep, oSizeROI, pMin, pMax, pDeviceBuffer));
    }
    private static native int nppiMinMax_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16u> pMin,
        TypedPointer<Npp16u> pMax,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed image MinMax.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMin Pointer to the computed minimal result.
     * @param pMax Pointer to the computed maximal result.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_16s_C1R to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinMax_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16s> pMin,
        TypedPointer<Npp16s> pMax,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMax_16s_C1RNative(pSrc, nSrcStep, oSizeROI, pMin, pMax, pDeviceBuffer));
    }
    private static native int nppiMinMax_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16s> pMin,
        TypedPointer<Npp16s> pMax,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image MinMax.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMin Pointer to the computed minimal result.
     * @param pMax Pointer to the computed maximal result.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_32f_C1R to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinMax_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pMin,
        TypedPointer<Npp32f> pMax,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMax_32f_C1RNative(pSrc, nSrcStep, oSizeROI, pMin, pMax, pDeviceBuffer));
    }
    private static native int nppiMinMax_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pMin,
        TypedPointer<Npp32f> pMax,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiMinMax_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        byte[] aMin,
        byte[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMax_8u_C3RNative(pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer));
    }
    private static native int nppiMinMax_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        byte[] aMin,
        byte[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image MinMax.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aMin Array that contains the minima.
     * @param aMax Array that contains the maxima.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_16u_C3R to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinMax_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        short[] aMin,
        short[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMax_16u_C3RNative(pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer));
    }
    private static native int nppiMinMax_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        short[] aMin,
        short[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed image MinMax.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aMin Array that contains the minima.
     * @param aMax Array that contains the maxima.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_16s_C3R to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinMax_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        short[] aMin,
        short[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMax_16s_C3RNative(pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer));
    }
    private static native int nppiMinMax_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        short[] aMin,
        short[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image MinMax.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aMin Array that contains the minima.
     * @param aMax Array that contains the maxima.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_32f_C3R to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinMax_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        float[] aMin,
        float[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMax_32f_C3RNative(pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer));
    }
    private static native int nppiMinMax_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        float[] aMin,
        float[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image MinMax ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aMin Array that contains the minima.
     * @param aMax Array that contains the maxima.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_8u_AC4R to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinMax_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        byte[] aMin,
        byte[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMax_8u_AC4RNative(pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer));
    }
    private static native int nppiMinMax_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        byte[] aMin,
        byte[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image MinMax ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aMin Array that contains the minima.
     * @param aMax Array that contains the maxima.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_16u_AC4R to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinMax_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        short[] aMin,
        short[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMax_16u_AC4RNative(pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer));
    }
    private static native int nppiMinMax_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        short[] aMin,
        short[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image MinMax ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aMin Array that contains the minima.
     * @param aMax Array that contains the maxima.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_16s_AC4R to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinMax_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        short[] aMin,
        short[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMax_16s_AC4RNative(pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer));
    }
    private static native int nppiMinMax_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        short[] aMin,
        short[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image MinMax ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aMin Array that contains the minima.
     * @param aMax Array that contains the maxima.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_32f_AC4R to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinMax_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        float[] aMin,
        float[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMax_32f_AC4RNative(pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer));
    }
    private static native int nppiMinMax_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        float[] aMin,
        float[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image MinMax.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aMin Array that contains the minima.
     * @param aMax Array that contains the maxima.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_8u_C4R to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinMax_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        byte[] aMin,
        byte[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMax_8u_C4RNative(pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer));
    }
    private static native int nppiMinMax_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        byte[] aMin,
        byte[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image MinMax.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aMin Array that contains the minima.
     * @param aMax Array that contains the maxima.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_16u_C4R to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinMax_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        short[] aMin,
        short[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMax_16u_C4RNative(pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer));
    }
    private static native int nppiMinMax_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        short[] aMin,
        short[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image MinMax.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aMin Array that contains the minima.
     * @param aMax Array that contains the maxima.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_16s_C4R to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMinMax_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        short[] aMin,
        short[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMax_16s_C4RNative(pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer));
    }
    private static native int nppiMinMax_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        short[] aMin,
        short[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiMinMax_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        float[] aMin,
        float[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMax_32f_C4RNative(pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer));
    }
    private static native int nppiMinMax_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        float[] aMin,
        float[] aMax,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMax_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMax_16u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxGetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxGetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxGetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMax_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxGetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMinMaxGetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxGetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMinMax_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMax_8u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxGetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxGetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxGetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMax_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxGetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxGetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxGetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMax_16s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxGetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxGetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxGetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMax_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMinMaxGetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxGetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxGetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMinMaxGetBufferHostSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMinMaxGetBufferHostSize_16u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxGetBufferHostSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiMinMaxGetBufferHostSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxGetBufferHostSize_16s_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxGetBufferHostSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMax_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxGetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxGetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxGetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMax_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxGetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxGetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxGetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMax_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxGetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxGetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxGetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMax_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxGetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMinMaxGetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxGetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMinMax_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the minimal and the maximal pixel values with their X and Y coordinates of 1-channel 8-bit unsigned char image.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMinValue Pointer to the minimum value.
     * @param pMaxValue Pointer to the maximum value.
     * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
     * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_8u_C1R to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
     * </pre>
     */
    public static int nppiMinMaxIndx_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMinValue,
        TypedPointer<Npp8u> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMaxIndx_8u_C1RNative(pSrc, nSrcStep, oSizeROI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer));
    }
    private static native int nppiMinMaxIndx_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMinValue,
        TypedPointer<Npp8u> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Computes the minimal and the maximal pixel values with their X and Y coordinates of 1-channel 8-bit signed char image.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMinValue Pointer to the minimum value.
     * @param pMaxValue Pointer to the maximum value.
     * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
     * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_8s_C1R to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
     * </pre>
     */
    public static int nppiMinMaxIndx_8s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8s> pMinValue,
        TypedPointer<Npp8s> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMaxIndx_8s_C1RNative(pSrc, nSrcStep, oSizeROI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer));
    }
    private static native int nppiMinMaxIndx_8s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8s> pMinValue,
        TypedPointer<Npp8s> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Computes the minimal and the maximal pixel values with their X and Y coordinates of 1-channel 16-bit unsigned short image.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMinValue Pointer to the minimum value.
     * @param pMaxValue Pointer to the maximum value.
     * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
     * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_16u_C1R to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
     * </pre>
     */
    public static int nppiMinMaxIndx_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16u> pMinValue,
        TypedPointer<Npp16u> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMaxIndx_16u_C1RNative(pSrc, nSrcStep, oSizeROI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer));
    }
    private static native int nppiMinMaxIndx_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16u> pMinValue,
        TypedPointer<Npp16u> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiMinMaxIndx_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pMinValue,
        TypedPointer<Npp32f> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMaxIndx_32f_C1RNative(pSrc, nSrcStep, oSizeROI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer));
    }
    private static native int nppiMinMaxIndx_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pMinValue,
        TypedPointer<Npp32f> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiMinMaxIndx_8u_C1MR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMinValue,
        TypedPointer<Npp8u> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMaxIndx_8u_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer));
    }
    private static native int nppiMinMaxIndx_8u_C1MRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pMinValue,
        TypedPointer<Npp8u> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiMinMaxIndx_8s_C1MR(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8s> pMinValue,
        TypedPointer<Npp8s> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMaxIndx_8s_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer));
    }
    private static native int nppiMinMaxIndx_8s_C1MRNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8s> pMinValue,
        TypedPointer<Npp8s> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 16-bit unsigned image MinMaxIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMinValue Pointer to the minimum value.
     * @param pMaxValue Pointer to the maximum value.
     * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
     * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_16u_C1MR to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * If the mask is filled with zeros, then all the returned values are zeros, i.e., pMinIndex = {0, 0}, pMaxIndex = {0, 0},
     * pMinValue = 0, pMaxValue = 0.
     * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
     * </pre>
     */
    public static int nppiMinMaxIndx_16u_C1MR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16u> pMinValue,
        TypedPointer<Npp16u> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMaxIndx_16u_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer));
    }
    private static native int nppiMinMaxIndx_16u_C1MRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp16u> pMinValue,
        TypedPointer<Npp16u> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 32-bit floating point image MinMaxIndx.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pMinValue Pointer to the minimum value.
     * @param pMaxValue Pointer to the maximum value.
     * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
     * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_32f_C1MR to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an invalid floating-point image is specified.
     * If the mask is filled with zeros, then all the returned values are zeros, i.e., pMinIndex = {0, 0}, pMaxIndex = {0, 0},
     * pMinValue = 0, pMaxValue = 0.
     * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
     * </pre>
     */
    public static int nppiMinMaxIndx_32f_C1MR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pMinValue,
        TypedPointer<Npp32f> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMaxIndx_32f_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer));
    }
    private static native int nppiMinMaxIndx_32f_C1MRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32f> pMinValue,
        TypedPointer<Npp32f> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image MinMaxIndx affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pMinValue Pointer to the minimum value.
     * @param pMaxValue Pointer to the maximum value.
     * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
     * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_8u_C3CR to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
     * </pre>
     */
    public static int nppiMinMaxIndx_8u_C3CR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pMinValue,
        TypedPointer<Npp8u> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMaxIndx_8u_C3CRNative(pSrc, nSrcStep, oSizeROI, nCOI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer));
    }
    private static native int nppiMinMaxIndx_8u_C3CRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pMinValue,
        TypedPointer<Npp8u> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit signed image MinMaxIndx affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pMinValue Pointer to the minimum value.
     * @param pMaxValue Pointer to the maximum value.
     * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
     * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_8s_C3CR to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
     * </pre>
     */
    public static int nppiMinMaxIndx_8s_C3CR(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8s> pMinValue,
        TypedPointer<Npp8s> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMaxIndx_8s_C3CRNative(pSrc, nSrcStep, oSizeROI, nCOI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer));
    }
    private static native int nppiMinMaxIndx_8s_C3CRNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8s> pMinValue,
        TypedPointer<Npp8s> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image MinMaxIndx affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pMinValue Pointer to the minimum value.
     * @param pMaxValue Pointer to the maximum value.
     * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
     * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_16u_C3CR to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
     * </pre>
     */
    public static int nppiMinMaxIndx_16u_C3CR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp16u> pMinValue,
        TypedPointer<Npp16u> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMaxIndx_16u_C3CRNative(pSrc, nSrcStep, oSizeROI, nCOI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer));
    }
    private static native int nppiMinMaxIndx_16u_C3CRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp16u> pMinValue,
        TypedPointer<Npp16u> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiMinMaxIndx_32f_C3CR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp32f> pMinValue,
        TypedPointer<Npp32f> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMaxIndx_32f_C3CRNative(pSrc, nSrcStep, oSizeROI, nCOI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer));
    }
    private static native int nppiMinMaxIndx_32f_C3CRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp32f> pMinValue,
        TypedPointer<Npp32f> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit unsigned image MinMaxIndx affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pMinValue Pointer to the minimum value.
     * @param pMaxValue Pointer to the maximum value.
     * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
     * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_8u_C3CMR to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * If the mask is filled with zeros, then all the returned values are zeros, i.e., pMinIndex = {0, 0}, pMaxIndex = {0, 0},
     * pMinValue = 0, pMaxValue = 0.
     * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
     * </pre>
     */
    public static int nppiMinMaxIndx_8u_C3CMR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pMinValue,
        TypedPointer<Npp8u> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMaxIndx_8u_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer));
    }
    private static native int nppiMinMaxIndx_8u_C3CMRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pMinValue,
        TypedPointer<Npp8u> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit signed image MinMaxIndx affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pMinValue Pointer to the minimum value.
     * @param pMaxValue Pointer to the maximum value.
     * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
     * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_8s_C3CMR to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * If the mask is filled with zeros, then all the returned values are zeros, i.e., pMinIndex = {0, 0}, pMaxIndex = {0, 0},
     * pMinValue = 0, pMaxValue = 0.
     * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
     * </pre>
     */
    public static int nppiMinMaxIndx_8s_C3CMR(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8s> pMinValue,
        TypedPointer<Npp8s> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMaxIndx_8s_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer));
    }
    private static native int nppiMinMaxIndx_8s_C3CMRNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8s> pMinValue,
        TypedPointer<Npp8s> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 16-bit unsigned image MinMaxIndx affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pMinValue Pointer to the minimum value.
     * @param pMaxValue Pointer to the maximum value.
     * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
     * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_16u_C3CMR to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * If the mask is filled with zeros, then all the returned values are zeros, i.e., pMinIndex = {0, 0}, pMaxIndex = {0, 0},
     * pMinValue = 0, pMaxValue = 0.
     * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
     * </pre>
     */
    public static int nppiMinMaxIndx_16u_C3CMR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp16u> pMinValue,
        TypedPointer<Npp16u> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMaxIndx_16u_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer));
    }
    private static native int nppiMinMaxIndx_16u_C3CMRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp16u> pMinValue,
        TypedPointer<Npp16u> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 32-bit floating point image MinMaxIndx affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pMinValue Pointer to the minimum value.
     * @param pMaxValue Pointer to the maximum value.
     * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
     * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
     * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_32f_C3CMR to determine
     *          the minium number of bytes required.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an invalid floating-point image is specified, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * If the mask is filled with zeros, then all the returned values are zeros, i.e., pMinIndex = {0, 0}, pMaxIndex = {0, 0},
     * pMinValue = 0, pMaxValue = 0.
     * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
     * </pre>
     */
    public static int nppiMinMaxIndx_32f_C3CMR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp32f> pMinValue,
        TypedPointer<Npp32f> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMinMaxIndx_32f_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer));
    }
    private static native int nppiMinMaxIndx_32f_C3CMRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp32f> pMinValue,
        TypedPointer<Npp32f> pMaxValue,
        NppiPoint[] pMinIndex,
        NppiPoint[] pMaxIndex,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMaxIndx_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxIndxGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMinMaxIndxGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxIndxGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMinMaxIndx_8s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxIndxGetBufferHostSize_8s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxIndxGetBufferHostSize_8s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxIndxGetBufferHostSize_8s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMinMaxIndxGetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxIndxGetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxIndxGetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMaxIndx_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxIndxGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxIndxGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxIndxGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMinMaxIndxGetBufferHostSize_8u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMinMaxIndxGetBufferHostSize_8u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxIndxGetBufferHostSize_8u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMinMaxIndx_8s_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxIndxGetBufferHostSize_8s_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxIndxGetBufferHostSize_8s_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxIndxGetBufferHostSize_8s_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMaxIndx_16u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxIndxGetBufferHostSize_16u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxIndxGetBufferHostSize_16u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxIndxGetBufferHostSize_16u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMaxIndx_32f_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxIndxGetBufferHostSize_32f_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxIndxGetBufferHostSize_32f_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxIndxGetBufferHostSize_32f_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMaxIndx_8u_C3CR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxIndxGetBufferHostSize_8u_C3CR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxIndxGetBufferHostSize_8u_C3CRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxIndxGetBufferHostSize_8u_C3CRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMaxIndx_8s_C3CR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxIndxGetBufferHostSize_8s_C3CR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxIndxGetBufferHostSize_8s_C3CRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxIndxGetBufferHostSize_8s_C3CRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMinMaxIndxGetBufferHostSize_16u_C3CR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxIndxGetBufferHostSize_16u_C3CRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxIndxGetBufferHostSize_16u_C3CRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMaxIndx_32f_C3CR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxIndxGetBufferHostSize_32f_C3CR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxIndxGetBufferHostSize_32f_C3CRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxIndxGetBufferHostSize_32f_C3CRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMaxIndx_8u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxIndxGetBufferHostSize_8u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMinMaxIndxGetBufferHostSize_8u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxIndxGetBufferHostSize_8u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMinMaxIndx_8s_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxIndxGetBufferHostSize_8s_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxIndxGetBufferHostSize_8s_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxIndxGetBufferHostSize_8s_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMinMaxIndxGetBufferHostSize_16u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxIndxGetBufferHostSize_16u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxIndxGetBufferHostSize_16u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMinMaxIndx_32f_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinMaxIndxGetBufferHostSize_32f_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMinMaxIndxGetBufferHostSize_32f_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMinMaxIndxGetBufferHostSize_32f_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * One-channel 8-bit unsigned image Mean.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_8u_C1R to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean result.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMean_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean)
    {
        return JNpp.checkResult(nppiMean_8u_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean));
    }
    private static native int nppiMean_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean);


    /**
     * <pre>
     * One-channel 16-bit unsigned image Mean.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_16u_C1R to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean result.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMean_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean)
    {
        return JNpp.checkResult(nppiMean_16u_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean));
    }
    private static native int nppiMean_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean);


    /**
     * <pre>
     * One-channel 16-bit signed image Mean.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_16s_C1R to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean result.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMean_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean)
    {
        return JNpp.checkResult(nppiMean_16s_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean));
    }
    private static native int nppiMean_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean);


    /**
     * <pre>
     * One-channel 32-bit floating point image Mean.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_32f_C1R to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean result.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an invalid
     * floating-point image is specified.
     * </pre>
     */
    public static int nppiMean_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean)
    {
        return JNpp.checkResult(nppiMean_32f_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean));
    }
    private static native int nppiMean_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image Mean.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_8u_C3R to determine the minium number of bytes required.
     * @param aMean Array that contains the computed mean results.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMean_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean)
    {
        return JNpp.checkResult(nppiMean_8u_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean));
    }
    private static native int nppiMean_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image Mean.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_16u_C3R to determine the minium number of bytes required.
     * @param aMean Array that contains the computed mean results.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMean_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean)
    {
        return JNpp.checkResult(nppiMean_16u_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean));
    }
    private static native int nppiMean_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean);


    /**
     * <pre>
     * Three-channel 16-bit signed image Mean.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_16s_C3R to determine the minium number of bytes required.
     * @param aMean Array that contains the computed mean results.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMean_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean)
    {
        return JNpp.checkResult(nppiMean_16s_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean));
    }
    private static native int nppiMean_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean);


    /**
     * <pre>
     * Three-channel 32-bit floating point image Mean.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_32f_C3R to determine the minium number of bytes required.
     * @param aMean Array that contains the computed mean results.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an invalid
     * floating-point image is specified.
     * </pre>
     */
    public static int nppiMean_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean)
    {
        return JNpp.checkResult(nppiMean_32f_C3RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean));
    }
    private static native int nppiMean_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image Mean.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_8u_C4R to determine the minium number of bytes required.
     * @param aMean Array that contains the computed mean results.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMean_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean)
    {
        return JNpp.checkResult(nppiMean_8u_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean));
    }
    private static native int nppiMean_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image Mean.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_16u_C4R to determine the minium number of bytes required.
     * @param aMean Array that contains the computed mean results.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMean_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean)
    {
        return JNpp.checkResult(nppiMean_16u_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean));
    }
    private static native int nppiMean_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean);


    /**
     * <pre>
     * Four-channel 16-bit signed image Mean.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_16s_C4R to determine the minium number of bytes required.
     * @param aMean Array that contains the computed mean results.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMean_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean)
    {
        return JNpp.checkResult(nppiMean_16s_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean));
    }
    private static native int nppiMean_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean);


    public static int nppiMean_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean)
    {
        return JNpp.checkResult(nppiMean_32f_C4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean));
    }
    private static native int nppiMean_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image Mean ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_8u_AC4R to determine the minium number of bytes required.
     * @param aMean Array that contains the computed mean results.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMean_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean)
    {
        return JNpp.checkResult(nppiMean_8u_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean));
    }
    private static native int nppiMean_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean);


    public static int nppiMean_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean)
    {
        return JNpp.checkResult(nppiMean_16u_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean));
    }
    private static native int nppiMean_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean);


    /**
     * <pre>
     * Four-channel 16-bit signed image Mean ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_16s_AC4R to determine the minium number of bytes required.
     * @param aMean Array that contains the computed mean results.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMean_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean)
    {
        return JNpp.checkResult(nppiMean_16s_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean));
    }
    private static native int nppiMean_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean);


    /**
     * <pre>
     * Four-channel 32-bit floating point image Mean ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_32f_AC4R to determine the minium number of bytes required.
     * @param aMean Array that contains the computed mean results.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an invalid
     * floating-point image is specified.
     * </pre>
     */
    public static int nppiMean_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean)
    {
        return JNpp.checkResult(nppiMean_32f_AC4RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean));
    }
    private static native int nppiMean_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        double[] aMean);


    /**
     * <pre>
     * Masked one-channel 8-bit unsigned image Mean.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_8u_C1MR to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean result.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMean_8u_C1MR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean)
    {
        return JNpp.checkResult(nppiMean_8u_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pDeviceBuffer, pMean));
    }
    private static native int nppiMean_8u_C1MRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean);


    /**
     * <pre>
     * Masked one-channel 8-bit signed image Mean.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_8s_C1MR to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean result.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMean_8s_C1MR(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean)
    {
        return JNpp.checkResult(nppiMean_8s_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pDeviceBuffer, pMean));
    }
    private static native int nppiMean_8s_C1MRNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean);


    /**
     * <pre>
     * Masked one-channel 16-bit unsigned image Mean.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_16u_C1MR to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean result.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMean_16u_C1MR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean)
    {
        return JNpp.checkResult(nppiMean_16u_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pDeviceBuffer, pMean));
    }
    private static native int nppiMean_16u_C1MRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean);


    public static int nppiMean_32f_C1MR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean)
    {
        return JNpp.checkResult(nppiMean_32f_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pDeviceBuffer, pMean));
    }
    private static native int nppiMean_32f_C1MRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean);


    /**
     * <pre>
     * Masked three-channel 8-bit unsigned image Mean affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_8u_C3CMR to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean result.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiMean_8u_C3CMR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean)
    {
        return JNpp.checkResult(nppiMean_8u_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pDeviceBuffer, pMean));
    }
    private static native int nppiMean_8u_C3CMRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean);


    /**
     * <pre>
     * Masked three-channel 8-bit signed image Mean affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanGetBufferHostSize_8s_C3CMR to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean result.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiMean_8s_C3CMR(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean)
    {
        return JNpp.checkResult(nppiMean_8s_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pDeviceBuffer, pMean));
    }
    private static native int nppiMean_8s_C3CMRNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean);


    public static int nppiMean_16u_C3CMR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean)
    {
        return JNpp.checkResult(nppiMean_16u_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pDeviceBuffer, pMean));
    }
    private static native int nppiMean_16u_C3CMRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean);


    public static int nppiMean_32f_C3CMR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean)
    {
        return JNpp.checkResult(nppiMean_32f_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pDeviceBuffer, pMean));
    }
    private static native int nppiMean_32f_C3CMRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_16u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMean_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMeanGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_8u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMeanGetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMean_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_8u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_16u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_16u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_16s_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_16s_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiMeanGetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMeanGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_8u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_8u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_8u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_8u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_8s_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_8s_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_8s_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_8s_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_16u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_16u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_16u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_16u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_32f_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_32f_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_32f_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_32f_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_8u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_8u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_8u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_8u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_8s_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_8s_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_8s_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_8s_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_16u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanGetBufferHostSize_16u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_16u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_16u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMeanGetBufferHostSize_32f_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanGetBufferHostSize_32f_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanGetBufferHostSize_32f_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMean_StdDev_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev)
    {
        return JNpp.checkResult(nppiMean_StdDev_8u_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean, pStdDev));
    }
    private static native int nppiMean_StdDev_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev);


    /**
     * <pre>
     * One-channel 8-bit signed image Mean_StdDev.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanStdDevGetBufferHostSize_8s_C1R to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean.
     * @param pStdDev Pointer to the computed standard deviation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMean_StdDev_8s_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev)
    {
        return JNpp.checkResult(nppiMean_StdDev_8s_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean, pStdDev));
    }
    private static native int nppiMean_StdDev_8s_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev);


    public static int nppiMean_StdDev_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev)
    {
        return JNpp.checkResult(nppiMean_StdDev_16u_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean, pStdDev));
    }
    private static native int nppiMean_StdDev_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev);


    public static int nppiMean_StdDev_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev)
    {
        return JNpp.checkResult(nppiMean_StdDev_32f_C1RNative(pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean, pStdDev));
    }
    private static native int nppiMean_StdDev_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev);


    /**
     * <pre>
     * Masked one-channel 8-bit unsigned image Mean_StdDev.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanStdDevGetBufferHostSize_8u_C1MR to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean.
     * @param pStdDev Pointer to the computed standard deviation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMean_StdDev_8u_C1MR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev)
    {
        return JNpp.checkResult(nppiMean_StdDev_8u_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pDeviceBuffer, pMean, pStdDev));
    }
    private static native int nppiMean_StdDev_8u_C1MRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev);


    /**
     * <pre>
     * Masked one-channel 8-bit signed image Mean_StdDev.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanStdDevGetBufferHostSize_8s_C1MR to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean.
     * @param pStdDev Pointer to the computed standard deviation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMean_StdDev_8s_C1MR(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev)
    {
        return JNpp.checkResult(nppiMean_StdDev_8s_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pDeviceBuffer, pMean, pStdDev));
    }
    private static native int nppiMean_StdDev_8s_C1MRNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev);


    /**
     * <pre>
     * Masked one-channel 16-bit unsigned image Mean_StdDev.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanStdDevGetBufferHostSize_16u_C1MR to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean.
     * @param pStdDev Pointer to the computed standard deviation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMean_StdDev_16u_C1MR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev)
    {
        return JNpp.checkResult(nppiMean_StdDev_16u_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pDeviceBuffer, pMean, pStdDev));
    }
    private static native int nppiMean_StdDev_16u_C1MRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev);


    public static int nppiMean_StdDev_32f_C1MR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev)
    {
        return JNpp.checkResult(nppiMean_StdDev_32f_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pDeviceBuffer, pMean, pStdDev));
    }
    private static native int nppiMean_StdDev_32f_C1MRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image Mean_StdDev affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanStdDevGetBufferHostSize_8u_C3CR to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean.
     * @param pStdDev Pointer to the computed standard deviation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiMean_StdDev_8u_C3CR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev)
    {
        return JNpp.checkResult(nppiMean_StdDev_8u_C3CRNative(pSrc, nSrcStep, oSizeROI, nCOI, pDeviceBuffer, pMean, pStdDev));
    }
    private static native int nppiMean_StdDev_8u_C3CRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev);


    /**
     * <pre>
     * Three-channel 8-bit signed image Mean_StdDev affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanStdDevGetBufferHostSize_8s_C3CR to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean.
     * @param pStdDev Pointer to the computed standard deviation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiMean_StdDev_8s_C3CR(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev)
    {
        return JNpp.checkResult(nppiMean_StdDev_8s_C3CRNative(pSrc, nSrcStep, oSizeROI, nCOI, pDeviceBuffer, pMean, pStdDev));
    }
    private static native int nppiMean_StdDev_8s_C3CRNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image Mean_StdDev affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanStdDevGetBufferHostSize_16u_C3CR to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean.
     * @param pStdDev Pointer to the computed standard deviation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiMean_StdDev_16u_C3CR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev)
    {
        return JNpp.checkResult(nppiMean_StdDev_16u_C3CRNative(pSrc, nSrcStep, oSizeROI, nCOI, pDeviceBuffer, pMean, pStdDev));
    }
    private static native int nppiMean_StdDev_16u_C3CRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev);


    /**
     * <pre>
     * Three-channel 32-bit floating point image Mean_StdDev affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanStdDevGetBufferHostSize_32f_C3CR to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean.
     * @param pStdDev Pointer to the computed standard deviation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an invalid
     * floating-point image is specified, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiMean_StdDev_32f_C3CR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev)
    {
        return JNpp.checkResult(nppiMean_StdDev_32f_C3CRNative(pSrc, nSrcStep, oSizeROI, nCOI, pDeviceBuffer, pMean, pStdDev));
    }
    private static native int nppiMean_StdDev_32f_C3CRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev);


    /**
     * <pre>
     * Masked three-channel 8-bit unsigned image Mean_StdDev.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanStdDevGetBufferHostSize_8u_C3CMR to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean.
     * @param pStdDev Pointer to the computed standard deviation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiMean_StdDev_8u_C3CMR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev)
    {
        return JNpp.checkResult(nppiMean_StdDev_8u_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pDeviceBuffer, pMean, pStdDev));
    }
    private static native int nppiMean_StdDev_8u_C3CMRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev);


    /**
     * <pre>
     * Masked three-channel 8-bit signed image Mean_StdDev.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanStdDevGetBufferHostSize_8s_C3CMR to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean.
     * @param pStdDev Pointer to the computed standard deviation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiMean_StdDev_8s_C3CMR(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev)
    {
        return JNpp.checkResult(nppiMean_StdDev_8s_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pDeviceBuffer, pMean, pStdDev));
    }
    private static native int nppiMean_StdDev_8s_C3CMRNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev);


    /**
     * <pre>
     * Masked three-channel 16-bit unsigned image Mean_StdDev.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanStdDevGetBufferHostSize_16u_C3CMR to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean.
     * @param pStdDev Pointer to the computed standard deviation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiMean_StdDev_16u_C3CMR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev)
    {
        return JNpp.checkResult(nppiMean_StdDev_16u_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pDeviceBuffer, pMean, pStdDev));
    }
    private static native int nppiMean_StdDev_16u_C3CMRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev);


    /**
     * <pre>
     * Masked three-channel 32-bit floating point image Mean_StdDev.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
     *        Use \ref nppiMeanStdDevGetBufferHostSize_32f_C3CMR to determine the minium number of bytes required.
     * @param pMean Pointer to the computed mean.
     * @param pStdDev Pointer to the computed standard deviation.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an invalid
     * floating-point image is specified, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiMean_StdDev_32f_C3CMR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev)
    {
        return JNpp.checkResult(nppiMean_StdDev_32f_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pDeviceBuffer, pMean, pStdDev));
    }
    private static native int nppiMean_StdDev_32f_C3CMRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp8u> pDeviceBuffer,
        TypedPointer<Npp64f> pMean,
        TypedPointer<Npp64f> pStdDev);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_StdDev_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanStdDevGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanStdDevGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanStdDevGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMeanStdDevGetBufferHostSize_8s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanStdDevGetBufferHostSize_8s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanStdDevGetBufferHostSize_8s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMeanStdDevGetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanStdDevGetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanStdDevGetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_StdDev_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanStdDevGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMeanStdDevGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanStdDevGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMean_StdDev_8u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanStdDevGetBufferHostSize_8u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanStdDevGetBufferHostSize_8u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanStdDevGetBufferHostSize_8u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_StdDev_8s_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanStdDevGetBufferHostSize_8s_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMeanStdDevGetBufferHostSize_8s_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanStdDevGetBufferHostSize_8s_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiMeanStdDevGetBufferHostSize_16u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanStdDevGetBufferHostSize_16u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanStdDevGetBufferHostSize_16u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_StdDev_32f_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanStdDevGetBufferHostSize_32f_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMeanStdDevGetBufferHostSize_32f_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanStdDevGetBufferHostSize_32f_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMean_StdDev_8u_C3CR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanStdDevGetBufferHostSize_8u_C3CR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanStdDevGetBufferHostSize_8u_C3CRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanStdDevGetBufferHostSize_8u_C3CRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiMeanStdDevGetBufferHostSize_8s_C3CR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanStdDevGetBufferHostSize_8s_C3CRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanStdDevGetBufferHostSize_8s_C3CRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_StdDev_16u_C3CR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanStdDevGetBufferHostSize_16u_C3CR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanStdDevGetBufferHostSize_16u_C3CRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanStdDevGetBufferHostSize_16u_C3CRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_StdDev_32f_C3CR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanStdDevGetBufferHostSize_32f_C3CR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanStdDevGetBufferHostSize_32f_C3CRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanStdDevGetBufferHostSize_32f_C3CRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_StdDev_8u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanStdDevGetBufferHostSize_8u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanStdDevGetBufferHostSize_8u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanStdDevGetBufferHostSize_8u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_StdDev_8s_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanStdDevGetBufferHostSize_8s_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMeanStdDevGetBufferHostSize_8s_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanStdDevGetBufferHostSize_8s_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiMeanStdDevGetBufferHostSize_16u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanStdDevGetBufferHostSize_16u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanStdDevGetBufferHostSize_16u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiMean_StdDev_32f_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMeanStdDevGetBufferHostSize_32f_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiMeanStdDevGetBufferHostSize_32f_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMeanStdDevGetBufferHostSize_32f_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * One-channel 8-bit unsigned image Norm_Inf.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_8u_C1RNative(pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image Norm_Inf.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_16u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_16u_C1RNative(pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed image Norm_Inf.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_16s_C1RNative(pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit signed image Norm_Inf.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_32s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_32s_C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_32s_C1RNative(pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_32s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image Norm_Inf.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_32f_C1RNative(pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image Norm_Inf.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_8u_C3RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image Norm_Inf.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_16u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_16u_C3RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNorm_Inf_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_16s_C3RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image Norm_Inf.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_32f_C3RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image Norm_Inf ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_8u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_8u_AC4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image Norm_Inf ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_16u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_16u_AC4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNorm_Inf_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_16s_AC4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNorm_Inf_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_32f_AC4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image Norm_Inf.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Four-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_8u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_8u_C4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image Norm_Inf.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Four-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_16u_C4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image Norm_Inf.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Four-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_16s_C4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image Norm_Inf.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Four-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_32f_C4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 8-bit unsigned image Norm_Inf.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_8u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_8u_C1MR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_8u_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_8u_C1MRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 8-bit signed image Norm_Inf.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_8s_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_8s_C1MR(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_8s_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_8s_C1MRNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 16-bit unsigned image Norm_Inf.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_16u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_Inf_16u_C1MR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_16u_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_16u_C1MRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNorm_Inf_32f_C1MR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_32f_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_32f_C1MRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit unsigned image Norm_Inf affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNorm_Inf_8u_C3CMR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_8u_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_8u_C3CMRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit signed image Norm_Inf affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNorm_Inf_8s_C3CMR(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_8s_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_8s_C3CMRNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 16-bit unsigned image Norm_Inf affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormInfGetBufferHostSize_16u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNorm_Inf_16u_C3CMR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_16u_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_16u_C3CMRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNorm_Inf_32f_C3CMR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_Inf_32f_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_Inf_32f_C3CMRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNormInfGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_16u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_32s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_32s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_32s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_32s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormInfGetBufferHostSize_8u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_8u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_8u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_8s_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_8s_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_8s_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_8s_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_16u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_16u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_16u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_16u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_32f_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_32f_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_32f_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_32f_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_8u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_16s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_8u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_16u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_16u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiNormInfGetBufferHostSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_16s_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormInfGetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_8u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_8u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_8u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_8u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_8s_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_8s_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_8s_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_8s_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormInfGetBufferHostSize_16u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_16u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_16u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_Inf_32f_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormInfGetBufferHostSize_32f_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormInfGetBufferHostSize_32f_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormInfGetBufferHostSize_32f_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * One-channel 8-bit unsigned image Norm_L1.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_8u_C1RNative(pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image Norm_L1.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_16u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_16u_C1RNative(pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed image Norm_L1.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_16s_C1RNative(pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image Norm_L1.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_32f_C1RNative(pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image Norm_L1.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_8u_C3RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image Norm_L1.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_16u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_16u_C3RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed image Norm_L1.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_16s_C3RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNorm_L1_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_32f_C3RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image Norm_L1 ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_8u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_8u_AC4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image Norm_L1 ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_16u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_16u_AC4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image Norm_L1 ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_16s_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_16s_AC4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image Norm_L1 ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_32f_AC4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image Norm_L1.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Four-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_8u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_8u_C4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image Norm_L1.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Four-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_16u_C4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image Norm_L1.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Four-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_16s_C4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image Norm_L1.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Four-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_32f_C4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 8-bit unsigned image Norm_L1.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_8u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_8u_C1MR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_8u_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_8u_C1MRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 8-bit signed image Norm_L1.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_8s_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_8s_C1MR(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_8s_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_8s_C1MRNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 16-bit unsigned image Norm_L1.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_16u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L1_16u_C1MR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_16u_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_16u_C1MRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNorm_L1_32f_C1MR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_32f_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_32f_C1MRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit unsigned image Norm_L1 affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNorm_L1_8u_C3CMR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_8u_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_8u_C3CMRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit signed image Norm_L1 affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNorm_L1_8s_C3CMR(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_8s_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_8s_C3CMRNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 16-bit unsigned image Norm_L1 affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_16u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNorm_L1_16u_C3CMR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_16u_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_16u_C3CMRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 32-bit floating point image Norm_L1 affecting only single channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL1GetBufferHostSize_32f_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if the step of
     * the source image cannot be divided by 4, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNorm_L1_32f_C3CMR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L1_32f_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L1_32f_C3CMRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNormL1GetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L1_16u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL1GetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L1_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL1GetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L1_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL1GetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiNormL1GetBufferHostSize_8u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_8u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_8u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormL1GetBufferHostSize_8s_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_8s_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_8s_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L1_16u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL1GetBufferHostSize_16u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_16u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_16u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L1_32f_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL1GetBufferHostSize_32f_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_32f_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_32f_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiNormL1GetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L1_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL1GetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiNormL1GetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiNormL1GetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L1_8u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL1GetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L1_16u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL1GetBufferHostSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_16u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L1_16s_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL1GetBufferHostSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_16s_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L1_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL1GetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L1_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL1GetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L1_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL1GetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L1_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL1GetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L1_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL1GetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormL1GetBufferHostSize_8u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_8u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_8u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L1_8s_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL1GetBufferHostSize_8s_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_8s_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_8s_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L1_16u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL1GetBufferHostSize_16u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_16u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_16u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormL1GetBufferHostSize_32f_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL1GetBufferHostSize_32f_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL1GetBufferHostSize_32f_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * One-channel 8-bit unsigned image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_8u_C1RNative(pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_16u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_16u_C1RNative(pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_16s_C1RNative(pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_32f_C1RNative(pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_8u_C3RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNorm_L2_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_16u_C3RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_16s_C3RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_32f_C3RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNorm_L2_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_8u_AC4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image Norm_L2 ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_16u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_16u_AC4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image Norm_L2 ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_16s_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_16s_AC4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image Norm_L2 ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Three-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_32f_AC4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Four-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_8u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_8u_C4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Four-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_16u_C4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Four-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_16s_C4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNorm Array that contains the norm values of Four-channels.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     *        Use \ref nppiNormL2GetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_32f_C4RNative(pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        double[] aNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 8-bit unsigned image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_8u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_8u_C1MR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_8u_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_8u_C1MRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 8-bit signed image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_8s_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_8s_C1MR(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_8s_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_8s_C1MRNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 16-bit unsigned image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_16u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNorm_L2_16u_C1MR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_16u_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_16u_C1MRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 32-bit floating point image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormL2GetBufferHostSize_32f_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if the step
     * of the source image cannot be divided by 4.
     * </pre>
     */
    public static int nppiNorm_L2_32f_C1MR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_32f_C1MRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_32f_C1MRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit unsigned image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     *        Use \ref nppiNormL2GetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNorm_L2_8u_C3CMR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_8u_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_8u_C3CMRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit signed image Norm_L2.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNorm Pointer to the norm value.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     *        Use \ref nppiNormL2GetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNorm_L2_8s_C3CMR(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_8s_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_8s_C3CMRNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNorm_L2_16u_C3CMR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_16u_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_16u_C3CMRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNorm_L2_32f_C3CMR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNorm_L2_32f_C3CMRNative(pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer));
    }
    private static native int nppiNorm_L2_32f_C3CMRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNorm,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_16u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiNormL2GetBufferHostSize_8u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_8u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_8u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_8s_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_8s_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_8s_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_8s_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_16u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_16u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_16u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_16u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiNormL2GetBufferHostSize_32f_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_32f_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_32f_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormL2GetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiNormL2GetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_16s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_8u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_16u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_16u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormL2GetBufferHostSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_16s_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_8u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_8u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_8u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_8u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormL2GetBufferHostSize_8s_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_8s_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_8s_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_16u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_16u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_16u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_16u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNorm_L2_32f_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormL2GetBufferHostSize_32f_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormL2GetBufferHostSize_32f_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormL2GetBufferHostSize_32f_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * One-channel 8-bit unsigned image NormDiff_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_Inf_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNormDiff_Inf_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_16u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed image NormDiff_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_Inf_16s_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_16s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_16s_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image NormDiff_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiNormDiff_Inf_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image NormDiff_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_Inf_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNormDiff_Inf_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_16u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed image NormDiff_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_Inf_16s_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_16s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_16s_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNormDiff_Inf_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image NormDiff_Inf ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_8u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_Inf_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_8u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image NormDiff_Inf ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_16u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_Inf_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_16u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image NormDiff_Inf ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_16s_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_Inf_16s_AC4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_16s_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image NormDiff_Inf ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiNormDiff_Inf_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_32f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image NormDiff_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_8u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_Inf_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image NormDiff_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_Inf_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_16u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNormDiff_Inf_16s_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_16s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_16s_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image NormDiff_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiNormDiff_Inf_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_32f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 8-bit unsigned images NormDiff_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_8u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_Inf_8u_C1MR(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_8u_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_8u_C1MRNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNormDiff_Inf_8s_C1MR(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_8s_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_8s_C1MRNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 16-bit unsigned images NormDiff_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_16u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_Inf_16u_C1MR(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_16u_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_16u_C1MRNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 32-bit floating point images NormDiff_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_32f_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiNormDiff_Inf_32f_C1MR(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_32f_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_32f_C1MRNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit unsigned image NormDiff_Inf affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNormDiff_Inf_8u_C3CMR(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_8u_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_8u_C3CMRNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit signed image NormDiff_Inf affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNormDiff_Inf_8s_C3CMR(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_8s_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_8s_C3CMRNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 16-bit unsigned image NormDiff_Inf affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_16u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNormDiff_Inf_16u_C3CMR(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_16u_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_16u_C3CMRNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 32-bit floating point image NormDiff_Inf affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffInfGetBufferHostSize_32f_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel
     * of interest is specified, or NPP_NOT_EVEN_STEP_ERROR if an invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiNormDiff_Inf_32f_C3CMR(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_Inf_32f_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_Inf_32f_C3CMRNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_16u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormDiffInfGetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_8u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_8u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_8u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_8u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_8s_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_8s_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_8s_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_8s_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiNormDiffInfGetBufferHostSize_16u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_16u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_16u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_32f_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_32f_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_32f_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_32f_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_8u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_16s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /** host pointer */
    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_8u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_16u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_16u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_16s_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_16s_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_8u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_8u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_8u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_8u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_8s_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_8s_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_8s_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_8s_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_16u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_16u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_16u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_16u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiNormDiff_Inf_32f_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffInfGetBufferHostSize_32f_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffInfGetBufferHostSize_32f_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffInfGetBufferHostSize_32f_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * One-channel 8-bit unsigned image NormDiff_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L1_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image NormDiff_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_16u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L1_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_16u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed image NormDiff_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L1_16s_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_16s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_16s_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNormDiff_L1_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image NormDiff_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L1_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image NormDiff_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_16u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L1_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_16u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed image NormDiff_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L1_16s_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_16s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_16s_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image NormDiff_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiNormDiff_L1_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image NormDiff_L1 ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_8u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L1_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_8u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image NormDiff_L1 ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_16u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L1_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_16u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image NormDiff_L1 ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_16s_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L1_16s_AC4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_16s_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image NormDiff_L1 ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiNormDiff_L1_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_32f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image NormDiff_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_8u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L1_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image NormDiff_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L1_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_16u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image NormDiff_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L1_16s_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_16s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_16s_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image NormDiff_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiNormDiff_L1_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_32f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 8-bit unsigned image NormDiff_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_8u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L1_8u_C1MR(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_8u_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_8u_C1MRNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 8-bit signed image NormDiff_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_8s_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L1_8s_C1MR(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_8s_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_8s_C1MRNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 16-bit unsigned image NormDiff_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_16u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L1_16u_C1MR(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_16u_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_16u_C1MRNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 32-bit floating point image NormDiff_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_32f_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiNormDiff_L1_32f_C1MR(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_32f_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_32f_C1MRNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit unsigned image NormDiff_L1 affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNormDiff_L1_8u_C3CMR(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_8u_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_8u_C3CMRNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit signed image NormDiff_L1 affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNormDiff_L1_8s_C3CMR(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_8s_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_8s_C3CMRNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 16-bit unsigned image NormDiff_L1 affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_16u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNormDiff_L1_16u_C3CMR(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_16u_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_16u_C3CMRNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 32-bit floating point image NormDiff_L1 affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL1GetBufferHostSize_32f_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel
     * of interest is specified, or NPP_NOT_EVEN_STEP_ERROR if an invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiNormDiff_L1_32f_C3CMR(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L1_32f_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L1_32f_C3CMRNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /** @} */
    /**
     * <pre>
     * @name NormDiffL1GetBufferHostSize
     * Companion primitives for computing the device buffer size (in bytes) required by the NormDiff_L1 primitives.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiNormDiffL1GetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_8u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_8u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_8u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_8u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_8s_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_8s_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_8s_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_8s_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_16u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_16u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_16u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_32f_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_32f_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_32f_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_32f_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormDiffL1GetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /** host pointer */
    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_8u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_16u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16s_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_16s_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_8u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_8u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_8u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_8u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_8s_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_8s_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_8s_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_8s_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_16u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_16u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_16u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_32f_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL1GetBufferHostSize_32f_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL1GetBufferHostSize_32f_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL1GetBufferHostSize_32f_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * One-channel 8-bit unsigned image NormDiff_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L2_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image NormDiff_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_16u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L2_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_16u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed image NormDiff_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L2_16s_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_16s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_16s_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image NormDiff_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiNormDiff_L2_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image NormDiff_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L2_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image NormDiff_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_16u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L2_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_16u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNormDiff_L2_16s_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_16s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_16s_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image NormDiff_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiNormDiff_L2_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image NormDiff_L2 ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_8u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L2_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_8u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image NormDiff_L2 ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_16u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L2_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_16u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image NormDiff_L2 ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_16s_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L2_16s_AC4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_16s_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image NormDiff_L2 ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiNormDiff_L2_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_32f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image NormDiff_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_8u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L2_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image NormDiff_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L2_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_16u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image NormDiff_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L2_16s_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_16s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_16s_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image NormDiff_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormDiff Array that contains computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiNormDiff_L2_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_32f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 8-bit unsigned image NormDiff_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_8u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L2_8u_C1MR(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_8u_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_8u_C1MRNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 8-bit signed image NormDiff_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_8s_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L2_8s_C1MR(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_8s_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_8s_C1MRNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 16-bit unsigned image NormDiff_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_16u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiff_L2_16u_C1MR(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_16u_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_16u_C1MRNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 32-bit floating point image NormDiff_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_32f_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiNormDiff_L2_32f_C1MR(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_32f_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_32f_C1MRNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit unsigned image NormDiff_L2 affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNormDiff_L2_8u_C3CMR(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_8u_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_8u_C3CMRNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit signed image NormDiff_L2 affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNormDiff_L2_8s_C3CMR(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_8s_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_8s_C3CMRNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 16-bit unsigned image NormDiff_L2 affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_16u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
     * </pre>
     */
    public static int nppiNormDiff_L2_16u_C3CMR(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_16u_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_16u_C3CMRNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 32-bit floating point image NormDiff_L2 affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormDiff Pointer to the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormDiffL2GetBufferHostSize_32f_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel
     * of interest is specified, or NPP_NOT_EVEN_STEP_ERROR if an invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiNormDiff_L2_32f_C3CMR(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormDiff_L2_32f_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer));
    }
    private static native int nppiNormDiff_L2_32f_C3CMRNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormDiff,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_8u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_8u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_8u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_8u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_8s_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_8s_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_8s_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_8s_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_16u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_16u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_16u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_32f_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_32f_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_32f_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_32f_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_8u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_8u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_16u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiNormDiffL2GetBufferHostSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_16s_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormDiffL2GetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_8u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_8u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_8u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_8u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_8s_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_8s_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_8s_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_8s_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_16u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_16u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_16u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_32f_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormDiffL2GetBufferHostSize_32f_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormDiffL2GetBufferHostSize_32f_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormDiffL2GetBufferHostSize_32f_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormRel_Inf_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image NormRel_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_16u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_16u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed image NormRel_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_16s_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_16s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_16s_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image NormRel_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image NormRel_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image NormRel_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_16u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_16u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed image NormRel_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_16s_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_16s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_16s_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image NormRel_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image NormRel_Inf ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_8u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_8u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image NormRel_Inf ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_16u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_16u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image NormRel_Inf ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_16s_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_16s_AC4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_16s_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image NormRel_Inf ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_32f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image NormRel_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_8u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image NormRel_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_16u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image NormRel_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_16s_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_16s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_16s_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image NormRel_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_32f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 8-bit unsigned image NormRel_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_8u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_8u_C1MR(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_8u_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_8u_C1MRNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 8-bit signed image NormRel_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_8s_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_8s_C1MR(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_8s_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_8s_C1MRNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 16-bit unsigned image NormRel_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_16u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_16u_C1MR(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_16u_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_16u_C1MRNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 32-bit floating point image NormRel_Inf.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_32f_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_32f_C1MR(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_32f_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_32f_C1MRNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit unsigned image NormRel_Inf affecting only signle channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormRel Pointer to the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_COI_ERROR if an
     * invalid channel of interest is specified, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_8u_C3CMR(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_8u_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_8u_C3CMRNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit signed image NormRel_Inf affecting only signle channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormRel Pointer to the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_COI_ERROR if an invalid
     * channel of interest is specified, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_8s_C3CMR(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_8s_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_8s_C3CMRNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 16-bit unsigned image NormRel_Inf affecting only signle channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormRel Pointer to the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_16u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_COI_ERROR if an invalid
     * channel of interest is specified, or NPP_DIVISOR_ERROR if the infinity
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_16u_C3CMR(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_16u_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_16u_C3CMRNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 32-bit floating point image NormRel_Inf affecting only signle channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormRel Pointer to the computed relative error for the infinity norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelInfGetBufferHostSize_32f_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_COI_ERROR if an invalid channel
     * of interest is specified, or NPP_NOT_EVEN_STEP_ERROR if an invalid floating-point
     * image is specified, or NPP_DIVISOR_ERROR if the infinity norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_Inf_32f_C3CMR(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_Inf_32f_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_Inf_32f_C3CMRNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_32s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_32s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_32s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_32s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_8u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_8u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_8u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_8u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_8s_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_8s_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_8s_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_8s_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_16u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_16u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_16u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_32f_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_32f_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_32f_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_32f_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_8u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_8u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_16u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16s_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_16s_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormRelInfGetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_8u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_8u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_8u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_8u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    public static int nppiNormRelInfGetBufferHostSize_8s_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_8s_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_8s_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_16u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_16u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_16u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_32f_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelInfGetBufferHostSize_32f_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelInfGetBufferHostSize_32f_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelInfGetBufferHostSize_32f_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * One-channel 8-bit unsigned image NormRel_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNormRel_L1_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_16u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed image NormRel_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_16s_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_16s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_16s_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image NormRel_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image NormRel_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image NormRel_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_16u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_16u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed image NormRel_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_16s_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_16s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_16s_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image NormRel_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit signed image NormRel_L1 ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_8u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_8u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image NormRel_L1 ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_16u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_16u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image NormRel_L1 ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_16s_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_16s_AC4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_16s_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image NormRel_L1 ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_32f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNormRel_L1_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image NormRel_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_16u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image NormRel_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_16s_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_16s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_16s_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image NormRel_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_32f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 8-bit unsigned image NormRel_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_8u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_8u_C1MR(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_8u_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_8u_C1MRNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 8-bit signed image NormRel_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_8s_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_8s_C1MR(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_8s_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_8s_C1MRNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image NormRel_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_16u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_16u_C1MR(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_16u_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_16u_C1MRNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image NormRel_L1.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_32f_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L1
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_32f_C1MR(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_32f_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_32f_C1MRNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit unsigned image NormRel_L1 affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormRel Pointer to the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid
     * channel of interest is specified, or NPP_DIVISOR_ERROR if the L1 norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_8u_C3CMR(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_8u_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_8u_C3CMRNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit signed image NormRel_L1 affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormRel Pointer to the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid
     * channel of interest is specified, or NPP_DIVISOR_ERROR if the L1 norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_8s_C3CMR(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_8s_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_8s_C3CMRNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 16-bit unsigned image NormRel_L1 affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormRel Pointer to the computed relative error for the L1 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL1GetBufferHostSize_16u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid
     * channel of interest is specified, or NPP_DIVISOR_ERROR if the L1 norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L1_16u_C3CMR(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_16u_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_16u_C3CMRNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNormRel_L1_32f_C3CMR(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L1_32f_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L1_32f_C3CMRNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_16u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_8u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_8u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_8u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_8u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_8s_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_8s_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_8s_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_8s_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_16u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_16u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_16u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_16u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_32f_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_32f_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_32f_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_32f_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_8u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormRelL1GetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /** host pointer */
    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_16s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /** host pointer */
    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormRelL1GetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_8u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_16u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_16u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_16s_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_16s_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_8u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_8u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_8u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_8u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_8s_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_8s_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_8s_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_8s_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_16u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_16u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_16u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_16u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_32f_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL1GetBufferHostSize_32f_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL1GetBufferHostSize_32f_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL1GetBufferHostSize_32f_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * One-channel 8-bit unsigned image NormRel_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image NormRel_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_16u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_16u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed image NormRel_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_16s_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_16s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_16s_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image NormRel_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image NormRel_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image NormRel_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_16u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_16u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed image NormRel_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_16s_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_16s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_16s_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image NormRel_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image NormRel_L2 ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_8u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_8u_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_8u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image NormRel_L2 ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_16u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_16u_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_16u_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image NormRel_L2 ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_16s_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_16s_AC4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_16s_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image NormRel_L2 ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_32f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image NormRel_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_8u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image NormRel_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_16u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image NormRel_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_16s_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_16s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_16s_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image NormRel_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aNormRel Array that contains the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_32f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 8-bit unsigned image NormRel_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_8u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_8u_C1MR(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_8u_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_8u_C1MRNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 8-bit signed image NormRel_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_8s_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_8s_C1MR(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_8s_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_8s_C1MRNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 16-bit unsigned image NormRel_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_16u_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_16u_C1MR(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_16u_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_16u_C1MRNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked one-channel 32-bit floating point image NormRel_L2.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pNormRel Pointer to the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_32f_C1MR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if
     * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_32f_C1MR(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_32f_C1MRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_32f_C1MRNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit unsigned image NormRel_L2 affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormRel Pointer to the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid
     * channel of interest is specified, or NPP_DIVISOR_ERROR if the L2 norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_8u_C3CMR(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_8u_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_8u_C3CMRNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 8-bit signed image NormRel_L2 affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormRel Pointer to the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid
     * channel of interest is specified, or NPP_DIVISOR_ERROR if the L2 norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_8s_C3CMR(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_8s_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_8s_C3CMRNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Masked three-channel 16-bit unsigned image NormRel_L2 affecting only single channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param pMask \ref mask_image_pointer.
     * @param nMaskStep \ref mask_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param nCOI \ref channel_of_interest_number.
     * @param pNormRel Pointer to the computed relative error for the L2 norm of two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiNormRelL2GetBufferHostSize_16u_C3CMR to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, NPP_COI_ERROR if an
     * invalid channel of interest is specified, or NPP_DIVISOR_ERROR if the L2
     * norm of the second image is zero.
     * </pre>
     */
    public static int nppiNormRel_L2_16u_C3CMR(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_16u_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_16u_C3CMRNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiNormRel_L2_32f_C3CMR(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiNormRel_L2_32f_C3CMRNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer));
    }
    private static native int nppiNormRel_L2_32f_C3CMRNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        TypedPointer<Npp8u> pMask,
        int nMaskStep,
        NppiSize oSizeROI,
        int nCOI,
        TypedPointer<Npp64f> pNormRel,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_8u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_8u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_8u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_8u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_8s_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_8s_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_8s_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_8s_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16u_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_16u_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_16u_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_16u_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /** host pointer */
    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_32f_C1MR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_32f_C1MR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_32f_C1MRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_32f_C1MRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_8u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiNormRelL2GetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_8u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_16u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16s_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_16s_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_8u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_8u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_8u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_8u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_8s_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_8s_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_8s_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_8s_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16u_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_16u_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_16u_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_16u_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_32f_C3CMR.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiNormRelL2GetBufferHostSize_32f_C3CMR(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiNormRelL2GetBufferHostSize_32f_C3CMRNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiNormRelL2GetBufferHostSize_32f_C3CMRNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * One-channel 8-bit unsigned image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDp Pointer to the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_8u64f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_8u64f_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_8u64f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_8u64f_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 8-bit signed image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDp Pointer to the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_8s64f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_8s64f_C1R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_8s64f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_8s64f_C1RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDp Pointer to the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_16u64f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_16u64f_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_16u64f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_16u64f_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDp Pointer to the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_16s64f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_16s64f_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_16s64f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_16s64f_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit unsigned image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDp Pointer to the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_32u64f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_32u64f_C1R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_32u64f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_32u64f_C1RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit signed image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDp Pointer to the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_32s64f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_32s64f_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_32s64f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_32s64f_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pDp Pointer to the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_32f64f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_32f64f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_32f64f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_32f64f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiDotProd_8u64f_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_8u64f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_8u64f_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit signed image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_8s64f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_8s64f_C3R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_8s64f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_8s64f_C3RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_16u64f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_16u64f_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_16u64f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_16u64f_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_16s64f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_16s64f_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_16s64f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_16s64f_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit unsigned image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_32u64f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_32u64f_C3R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_32u64f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_32u64f_C3RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit signed image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_32s64f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_32s64f_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_32s64f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_32s64f_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_32f64f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_32f64f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_32f64f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_32f64f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_8u64f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_8u64f_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_8u64f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_8u64f_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit signed image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_8s64f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_8s64f_C4R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_8s64f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_8s64f_C4RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_16u64f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_16u64f_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_16u64f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_16u64f_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_16s64f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_16s64f_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_16s64f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_16s64f_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiDotProd_32u64f_C4R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_32u64f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_32u64f_C4RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit signed image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_32s64f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_32s64f_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_32s64f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_32s64f_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image DotProd.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_32f64f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_32f64f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_32f64f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_32f64f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image DotProd ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_8u64f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_8u64f_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_8u64f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_8u64f_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit signed image DotProd ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_8s64f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_8s64f_AC4R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_8s64f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_8s64f_AC4RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image DotProd ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed Inf-norm of differences.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_16u64f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_16u64f_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_16u64f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_16u64f_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image DotProd ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_16s64f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_16s64f_AC4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_16s64f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_16s64f_AC4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit unsigned image DotProd ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_32u64f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_32u64f_AC4R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_32u64f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_32u64f_AC4RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit signed image DotProd ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_32s64f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_32s64f_AC4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_32s64f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_32s64f_AC4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image DotProd ignoring alpha channel.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aDp Array that contains the computed dot product of the two images.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiDotProdGetBufferHostSize_32f64f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProd_32f64f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiDotProd_32f64f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer));
    }
    private static native int nppiDotProd_32f64f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        double[] aDp,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * @name DotProdGetBufferHostSize
     * Companion primitives for computing the device buffer size (in bytes) required by the Mean_StdDev primitives.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_8u64f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_8u64f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_8u64f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_8u64f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_8s64f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_8s64f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_8s64f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_8s64f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_16u64f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_16u64f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_16u64f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_16u64f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiDotProdGetBufferHostSize_16s64f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_16s64f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_16s64f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiDotProdGetBufferHostSize_32u64f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_32u64f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_32u64f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /** host pointer */
    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_32s64f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_32s64f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_32s64f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_32s64f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_32f64f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_32f64f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_32f64f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_32f64f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_8u64f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_8u64f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_8u64f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_8u64f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiDotProdGetBufferHostSize_8s64f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_8s64f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_8s64f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /** host pointer */
    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_16u64f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_16u64f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_16u64f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_16u64f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_16s64f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_16s64f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_16s64f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_16s64f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_32u64f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_32u64f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_32u64f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_32u64f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_32s64f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_32s64f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_32s64f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_32s64f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_32f64f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_32f64f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_32f64f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_32f64f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_8u64f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_8u64f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_8u64f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_8u64f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_8s64f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_8s64f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_8s64f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_8s64f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_16u64f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_16u64f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_16u64f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_16u64f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_16s64f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_16s64f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_16s64f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_16s64f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_32u64f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_32u64f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_32u64f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_32u64f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_32s64f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_32s64f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_32s64f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_32s64f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_32f64f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_32f64f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_32f64f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_32f64f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_8u64f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_8u64f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_8u64f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_8u64f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /** host pointer */
    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_8s64f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_8s64f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_8s64f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_8s64f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_16u64f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_16u64f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_16u64f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_16u64f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_16s64f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_16s64f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_16s64f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_16s64f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_32u64f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_32u64f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_32u64f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_32u64f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_32s64f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_32s64f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_32s64f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_32s64f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiDotProd_32f64f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiDotProdGetBufferHostSize_32f64f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiDotProdGetBufferHostSize_32f64f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiDotProdGetBufferHostSize_32f64f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * One-channel 8-bit unsigned image CountInRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pCounts Pointer to the number of pixels that fall into the specified range.
     * @param nLowerBound Lower bound of the specified range.
     * @param nUpperBound Upper bound of the specified range.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiCountInRangeGetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_RANGE_ERROR if the lower bound
     * is larger than the upper bound.
     * </pre>
     */
    public static int nppiCountInRange_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int[] pCounts,
        byte nLowerBound,
        byte nUpperBound,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCountInRange_8u_C1RNative(pSrc, nSrcStep, oSizeROI, pCounts, nLowerBound, nUpperBound, pDeviceBuffer));
    }
    private static native int nppiCountInRange_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int[] pCounts,
        byte nLowerBound,
        byte nUpperBound,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image CountInRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pCounts Pointer to the number of pixels that fall into the specified range.
     * @param nLowerBound Lower bound of the specified range.
     * @param nUpperBound Upper bound of the specified range.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiCountInRangeGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_RANGE_ERROR if the lower bound
     * is larger than the upper bound.
     * </pre>
     */
    public static int nppiCountInRange_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int[] pCounts,
        float nLowerBound,
        float nUpperBound,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCountInRange_32f_C1RNative(pSrc, nSrcStep, oSizeROI, pCounts, nLowerBound, nUpperBound, pDeviceBuffer));
    }
    private static native int nppiCountInRange_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int[] pCounts,
        float nLowerBound,
        float nUpperBound,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image CountInRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aCounts Array that contains the number of pixels that fall into the specified range for Three-channels.
     * @param aLowerBound Fixed size array of the lower bound of the specified range, one per channel.
     * @param aUpperBound Fixed size array of the upper bound of the specified range, one per channel.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiCountInRangeGetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_RANGE_ERROR if the lower bound
     * is larger than the upper bound.
     * </pre>
     */
    public static int nppiCountInRange_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int[] aCounts,
        byte[] aLowerBound,
        byte[] aUpperBound,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCountInRange_8u_C3RNative(pSrc, nSrcStep, oSizeROI, aCounts, aLowerBound, aUpperBound, pDeviceBuffer));
    }
    private static native int nppiCountInRange_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int[] aCounts,
        byte[] aLowerBound,
        byte[] aUpperBound,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image CountInRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aCounts Array that contains the number of pixels that fall into the specified range for Three-channels.
     * @param aLowerBound Fixed size array of the lower bound of the specified range, one per channel.
     * @param aUpperBound Fixed size array of the upper bound of the specified range, one per channel.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiCountInRangeGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_RANGE_ERROR if the lower bound
     * is larger than the upper bound.
     * </pre>
     */
    public static int nppiCountInRange_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int[] aCounts,
        float[] aLowerBound,
        float[] aUpperBound,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCountInRange_32f_C3RNative(pSrc, nSrcStep, oSizeROI, aCounts, aLowerBound, aUpperBound, pDeviceBuffer));
    }
    private static native int nppiCountInRange_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int[] aCounts,
        float[] aLowerBound,
        float[] aUpperBound,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CountInRange ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aCounts Array that contains the number of pixels that fall into the specified range for Three-channels.
     * @param aLowerBound Fixed size array of the lower bound of the specified range, one per channel.
     * @param aUpperBound Fixed size array of the upper bound of the specified range, one per channel.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiCountInRangeGetBufferHostSize_8u_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_RANGE_ERROR if the lower bound
     * is larger than the upper bound.
     * </pre>
     */
    public static int nppiCountInRange_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int[] aCounts,
        byte[] aLowerBound,
        byte[] aUpperBound,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCountInRange_8u_AC4RNative(pSrc, nSrcStep, oSizeROI, aCounts, aLowerBound, aUpperBound, pDeviceBuffer));
    }
    private static native int nppiCountInRange_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int[] aCounts,
        byte[] aLowerBound,
        byte[] aUpperBound,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image CountInRange ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param aCounts Array that contains the number of pixels that fall into the specified range for Three-channels.
     * @param aLowerBound Fixed size array of the lower bound of the specified range, one per channel.
     * @param aUpperBound Fixed size array of the upper bound of the specified range, one per channel.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiCountInRangeGetBufferHostSize_32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_RANGE_ERROR if the lower bound
     * is larger than the upper bound.
     * </pre>
     */
    public static int nppiCountInRange_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int[] aCounts,
        float[] aLowerBound,
        float[] aUpperBound,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCountInRange_32f_AC4RNative(pSrc, nSrcStep, oSizeROI, aCounts, aLowerBound, aUpperBound, pDeviceBuffer));
    }
    private static native int nppiCountInRange_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        int[] aCounts,
        float[] aLowerBound,
        float[] aUpperBound,
        TypedPointer<Npp8u> pDeviceBuffer);


    public static int nppiCountInRangeGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiCountInRangeGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiCountInRangeGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiCountInRange_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiCountInRangeGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiCountInRangeGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiCountInRangeGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiCountInRange_8u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiCountInRangeGetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiCountInRangeGetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiCountInRangeGetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiCountInRange_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiCountInRangeGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiCountInRangeGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiCountInRangeGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiCountInRange_8u_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiCountInRangeGetBufferHostSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiCountInRangeGetBufferHostSize_8u_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiCountInRangeGetBufferHostSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);


    /**
     * <pre>
     * Device scratch buffer size (in bytes) for nppiCountInRange_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiCountInRangeGetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiCountInRangeGetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiCountInRangeGetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * One-channel 8-bit unsigned image MaxEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxEvery_8u_C1IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMaxEvery_8u_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMaxEvery_8u_C1IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One-channel 16-bit unsigned image MaxEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxEvery_16u_C1IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMaxEvery_16u_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMaxEvery_16u_C1IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One-channel 16-bit signed image MaxEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxEvery_16s_C1IR(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMaxEvery_16s_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMaxEvery_16s_C1IRNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One-channel 32-bit floating point image MaxEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxEvery_32f_C1IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMaxEvery_32f_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMaxEvery_32f_C1IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image MaxEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxEvery_8u_C3IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMaxEvery_8u_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMaxEvery_8u_C3IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image MaxEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxEvery_16u_C3IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMaxEvery_16u_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMaxEvery_16u_C3IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 16-bit signed image MaxEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxEvery_16s_C3IR(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMaxEvery_16s_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMaxEvery_16s_C3IRNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 32-bit floating point image MaxEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxEvery_32f_C3IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMaxEvery_32f_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMaxEvery_32f_C3IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image MaxEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxEvery_8u_C4IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMaxEvery_8u_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMaxEvery_8u_C4IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image MaxEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxEvery_16u_C4IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMaxEvery_16u_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMaxEvery_16u_C4IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 16-bit signed image MaxEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxEvery_16s_C4IR(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMaxEvery_16s_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMaxEvery_16s_C4IRNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 32-bit floating point image MaxEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxEvery_32f_C4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMaxEvery_32f_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMaxEvery_32f_C4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image MaxEvery ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxEvery_8u_AC4IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMaxEvery_8u_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMaxEvery_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image MaxEvery ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxEvery_16u_AC4IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMaxEvery_16u_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMaxEvery_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 16-bit signed image MaxEvery ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxEvery_16s_AC4IR(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMaxEvery_16s_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMaxEvery_16s_AC4IRNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 32-bit floating point image MaxEvery ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaxEvery_32f_AC4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMaxEvery_32f_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMaxEvery_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One-channel 8-bit unsigned image MinEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinEvery_8u_C1IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMinEvery_8u_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMinEvery_8u_C1IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One-channel 16-bit unsigned image MinEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinEvery_16u_C1IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMinEvery_16u_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMinEvery_16u_C1IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One-channel 16-bit signed image MinEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinEvery_16s_C1IR(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMinEvery_16s_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMinEvery_16s_C1IRNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One-channel 32-bit floating point image MinEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinEvery_32f_C1IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMinEvery_32f_C1IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMinEvery_32f_C1IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image MinEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinEvery_8u_C3IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMinEvery_8u_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMinEvery_8u_C3IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image MinEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinEvery_16u_C3IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMinEvery_16u_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMinEvery_16u_C3IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 16-bit signed image MinEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinEvery_16s_C3IR(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMinEvery_16s_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMinEvery_16s_C3IRNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Three-channel 32-bit floating point image MinEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinEvery_32f_C3IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMinEvery_32f_C3IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMinEvery_32f_C3IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image MinEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinEvery_8u_C4IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMinEvery_8u_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMinEvery_8u_C4IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image MinEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinEvery_16u_C4IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMinEvery_16u_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMinEvery_16u_C4IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    public static int nppiMinEvery_16s_C4IR(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMinEvery_16s_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMinEvery_16s_C4IRNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 32-bit floating point image MinEvery.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinEvery_32f_C4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMinEvery_32f_C4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMinEvery_32f_C4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image MinEvery ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinEvery_8u_AC4IR(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMinEvery_8u_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMinEvery_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image MinEvery ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinEvery_16u_AC4IR(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMinEvery_16u_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMinEvery_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 16-bit signed image MinEvery ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinEvery_16s_AC4IR(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMinEvery_16s_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMinEvery_16s_AC4IRNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Four-channel 32-bit floating point image MinEvery ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMinEvery_32f_AC4IR(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMinEvery_32f_AC4IRNative(pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI));
    }
    private static native int nppiMinEvery_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * One-channel 8-bit unsigned image Integral with 32-bit signed output.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param nVal The value to add to pDst image pixels
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiIntegral_8u32s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oROI,
        int nVal)
    {
        return JNpp.checkResult(nppiIntegral_8u32s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nVal));
    }
    private static native int nppiIntegral_8u32s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oROI,
        int nVal);


    /**
     * <pre>
     * One-channel 8-bit unsigned image Integral with 32-bit floating point output.
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param nVal The value to add to pDst image pixels
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiIntegral_8u32f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        float nVal)
    {
        return JNpp.checkResult(nppiIntegral_8u32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, nVal));
    }
    private static native int nppiIntegral_8u32f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        float nVal);


    /**
     * <pre>
     * One-channel 8-bit unsigned image SqrIntegral.
     * Destination integral image and square integral image are 32-bit signed int.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pSqr \ref destination_image_pointer.
     * @param nSqrStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     * @param nVal The value to add to pDst image pixels
     * @param nValSqr The value to add to pSqr image pixels
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrIntegral_8u32s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        TypedPointer<Npp32s> pSqr,
        int nSqrStep,
        NppiSize oSrcROI,
        int nVal,
        int nValSqr)
    {
        return JNpp.checkResult(nppiSqrIntegral_8u32s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, pSqr, nSqrStep, oSrcROI, nVal, nValSqr));
    }
    private static native int nppiSqrIntegral_8u32s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        TypedPointer<Npp32s> pSqr,
        int nSqrStep,
        NppiSize oSrcROI,
        int nVal,
        int nValSqr);


    /**
     * <pre>
     * One-channel 8-bit unsigned image SqrIntegral.
     * Destination integral image is 32-bit signed int.
     * Destination square integral image is 64-bit double floating point.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pSqr \ref destination_image_pointer.
     * @param nSqrStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     * @param nVal The value to add to pDst image pixels
     * @param nValSqr The value to add to pSqr image pixels
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrIntegral_8u32s64f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        TypedPointer<Npp64f> pSqr,
        int nSqrStep,
        NppiSize oSrcROI,
        int nVal,
        double nValSqr)
    {
        return JNpp.checkResult(nppiSqrIntegral_8u32s64f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, pSqr, nSqrStep, oSrcROI, nVal, nValSqr));
    }
    private static native int nppiSqrIntegral_8u32s64f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        TypedPointer<Npp64f> pSqr,
        int nSqrStep,
        NppiSize oSrcROI,
        int nVal,
        double nValSqr);


    /**
     * <pre>
     * One-channel 8-bit unsigned image SqrIntegral.
     * Destination integral image is 32-bit floating point.
     * Destination square integral image is 64-bit double floating point.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pSqr \ref destination_image_pointer.
     * @param nSqrStep \ref destination_image_line_step.
     * @param oSrcROI \ref roi_specification.
     * @param nVal The value to add to pDst image pixels
     * @param nValSqr The value to add to pSqr image pixels
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrIntegral_8u32f64f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp64f> pSqr,
        int nSqrStep,
        NppiSize oSrcROI,
        float nVal,
        double nValSqr)
    {
        return JNpp.checkResult(nppiSqrIntegral_8u32f64f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, pSqr, nSqrStep, oSrcROI, nVal, nValSqr));
    }
    private static native int nppiSqrIntegral_8u32f64f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp64f> pSqr,
        int nSqrStep,
        NppiSize oSrcROI,
        float nVal,
        double nValSqr);


    /**
     * <pre>
     * One-channel 32-bit floating point image RectStdDev.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSqr \ref destination_image_pointer.
     * @param nSqrStep \ref destination_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oRect rectangular window
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRectStdDev_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp64f> pSqr,
        int nSqrStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiRect oRect)
    {
        return JNpp.checkResult(nppiRectStdDev_32f_C1RNative(pSrc, nSrcStep, pSqr, nSqrStep, pDst, nDstStep, oSizeROI, oRect));
    }
    private static native int nppiRectStdDev_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp64f> pSqr,
        int nSqrStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiRect oRect);


    /**
     * <pre>
     * One-channel 32-bit signed image RectStdDev, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSqr \ref destination_image_pointer.
     * @param nSqrStep \ref destination_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oRect rectangular window
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRectStdDev_32s_C1RSfs(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSqr,
        int nSqrStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiRect oRect,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiRectStdDev_32s_C1RSfsNative(pSrc, nSrcStep, pSqr, nSqrStep, pDst, nDstStep, oSizeROI, oRect, nScaleFactor));
    }
    private static native int nppiRectStdDev_32s_C1RSfsNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pSqr,
        int nSqrStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiRect oRect,
        int nScaleFactor);


    /**
     * <pre>
     * One-channel 32-bit signed image RectStdDev.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pSqr \ref destination_image_pointer.
     * @param nSqrStep \ref destination_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param oRect rectangular window
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiRectStdDev_32s32f_C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp64f> pSqr,
        int nSqrStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiRect oRect)
    {
        return JNpp.checkResult(nppiRectStdDev_32s32f_C1RNative(pSrc, nSrcStep, pSqr, nSqrStep, pDst, nDstStep, oSizeROI, oRect));
    }
    private static native int nppiRectStdDev_32s32f_C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp64f> pSqr,
        int nSqrStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI,
        NppiRect oRect);


    /**
     * <pre>
     * @name HistogramEven
     * The \f$nLowerLevel\f$ (inclusive) and \f$nUpperLevel\f$ (exclusive) define the boundaries of the range,
     * which are evenly segmented into \f$nLevel - 1\f$ bins. The computed histogram is stored in \f$pHist\f$.
     * The levels are calculated by another primitive \ref nppiEvenLevelsHost_32s and are stored
     * in a host pointer \f$hpLevels\f$. The number of levels is also \f$nLevel - 1\f$.
     * The histogram \f$pHist[k]\f$ is defined as the total number of pixels that fall into the range:
     * \f$hpLevels[k] <= pSrc(j, i) < hpLevels[k+1]\f$.
     * The functions require additional scratch buffer for computations.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * Compute levels with even distribution.
     *
     * @param hpLevels A host pointer to array which receives the levels being
     *        computed. The array needs to be of size nLevels.
     * @param nLevels The number of levels being computed. nLevels must be at least 2.
     * @param nLowerLevel Lower boundary value of the lowest level.
     * @param nUpperLevel Upper boundary value of the greatest level.
     * @return
     *  image_data_error_codes, or NPP_HISTO_NUMBER_OF_LEVELS_ERROR if an invalid nLevels is specified.
     * </pre>
     */
    public static int nppiEvenLevelsHost_32s(
        TypedPointer<Npp32s> hpLevels,
        int nLevels,
        int nLowerLevel,
        int nUpperLevel)
    {
        return JNpp.checkResult(nppiEvenLevelsHost_32sNative(hpLevels, nLevels, nLowerLevel, nUpperLevel));
    }
    private static native int nppiEvenLevelsHost_32sNative(
        TypedPointer<Npp32s> hpLevels,
        int nLevels,
        int nLowerLevel,
        int nUpperLevel);


    /**
     * <pre>
     * One-channel 8-bit unsigned HistogramEven.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Pointer to array that receives the computed histogram.
     *      The array must be of size nLevels-1.
     * @param nLevels Number of levels.
     * @param nLowerLevel Lower boundary of lowest level bin.
     * @param nUpperLevel Upper boundary of highest level bin.
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_8u_C1R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiHistogramEven_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pHist,
        int nLevels,
        int nLowerLevel,
        int nUpperLevel,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramEven_8u_C1RNative(pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer));
    }
    private static native int nppiHistogramEven_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pHist,
        int nLevels,
        int nLowerLevel,
        int nUpperLevel,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned HistogramEven.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving computed histograms per color channel.
     *      Array pointed by pHist[i] be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
     * @param nUpperLevel Array containing upper-level of highest bin per color channel.
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_8u_C3R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiHistogramEven_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramEven_8u_C3RNative(pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer));
    }
    private static native int nppiHistogramEven_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned HistogramEven.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving computed histograms per color channel.
     *      Array pointed by pHist[i] be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
     * @param nUpperLevel Array containing upper-level of highest bin per color channel.
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_8u_C4R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiHistogramEven_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramEven_8u_C4RNative(pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer));
    }
    private static native int nppiHistogramEven_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned HistogramEven ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving computed histograms per color channel.
     *      Array pointed by pHist[i] be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
     * @param nUpperLevel Array containing upper-level of highest bin per color channel.
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_8u_AC4R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramEven_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramEven_8u_AC4RNative(pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer));
    }
    private static native int nppiHistogramEven_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned HistogramEven.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Pointer to array that receives the computed histogram.
     *      The array must be of size nLevels-1.
     * @param nLevels Number of levels.
     * @param nLowerLevel Lower boundary of lowest level bin.
     * @param nUpperLevel Upper boundary of highest level bin.
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_16u_C1R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramEven_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pHist,
        int nLevels,
        int nLowerLevel,
        int nUpperLevel,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramEven_16u_C1RNative(pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer));
    }
    private static native int nppiHistogramEven_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pHist,
        int nLevels,
        int nLowerLevel,
        int nUpperLevel,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned HistogramEven.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving computed histograms per color channel.
     *      Array pointed by pHist[i] be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
     * @param nUpperLevel Array containing upper-level of highest bin per color channel.
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_16u_C3R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramEven_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramEven_16u_C3RNative(pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer));
    }
    private static native int nppiHistogramEven_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned HistogramEven.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving computed histograms per color channel.
     *      Array pointed by pHist[i] be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
     * @param nUpperLevel Array containing upper-level of highest bin per color channel.
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_16u_C4R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramEven_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramEven_16u_C4RNative(pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer));
    }
    private static native int nppiHistogramEven_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned HistogramEven ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving computed histograms per color channel.
     *      Array pointed by pHist[i] be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
     * @param nUpperLevel Array containing upper-level of highest bin per color channel.
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_16u_AC4R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramEven_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramEven_16u_AC4RNative(pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer));
    }
    private static native int nppiHistogramEven_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed HistogramEven.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Pointer to array that receives the computed histogram.
     *      The array must be of size nLevels-1.
     * @param nLevels Number of levels.
     * @param nLowerLevel Lower boundary of lowest level bin.
     * @param nUpperLevel Upper boundary of highest level bin.
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_16s_C1R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramEven_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pHist,
        int nLevels,
        int nLowerLevel,
        int nUpperLevel,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramEven_16s_C1RNative(pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer));
    }
    private static native int nppiHistogramEven_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pHist,
        int nLevels,
        int nLowerLevel,
        int nUpperLevel,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed HistogramEven.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving computed histograms per color channel.
     *      Array pointed by pHist[i] be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
     * @param nUpperLevel Array containing upper-level of highest bin per color channel.
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_16s_C3R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramEven_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramEven_16s_C3RNative(pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer));
    }
    private static native int nppiHistogramEven_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed HistogramEven.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving computed histograms per color channel.
     *      Array pointed by pHist[i] be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
     * @param nUpperLevel Array containing upper-level of highest bin per color channel.
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_16s_C4R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramEven_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramEven_16s_C4RNative(pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer));
    }
    private static native int nppiHistogramEven_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed HistogramEven ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving computed histograms per color channel.
     *      Array pointed by pHist[i] be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
     * @param nUpperLevel Array containing upper-level of highest bin per color channel.
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_16s_AC4R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramEven_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramEven_16s_AC4RNative(pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer));
    }
    private static native int nppiHistogramEven_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        int[] nLevels,
        int[] nLowerLevel,
        int[] nUpperLevel,
        TypedPointer<Npp8u> pBuffer);


    /** @} */
    /**
     * <pre>
     * @name HistogramEvenGetBufferSize
     * Companion primitives for computing the device buffer size (in bytes) required by the HistogramEven primitives.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * Buffer size for \ref nppiHistogramEven_8u_C1R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Number of levels in the histogram.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramEvenGetBufferSize_8u_C1R(
        NppiSize oSizeROI,
        int nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramEvenGetBufferSize_8u_C1RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramEvenGetBufferSize_8u_C1RNative(
        NppiSize oSizeROI,
        int nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiHistogramEven_8u_C3R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Number of levels in the histogram.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramEvenGetBufferSize_8u_C3R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramEvenGetBufferSize_8u_C3RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramEvenGetBufferSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiHistogramEven_8u_C4R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramEvenGetBufferSize_8u_C4R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramEvenGetBufferSize_8u_C4RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramEvenGetBufferSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiHistogramEven_8u_AC4R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramEvenGetBufferSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramEvenGetBufferSize_8u_AC4RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramEvenGetBufferSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiHistogramEven_16u_C1R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Number of levels in the histogram.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramEvenGetBufferSize_16u_C1R(
        NppiSize oSizeROI,
        int nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramEvenGetBufferSize_16u_C1RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramEvenGetBufferSize_16u_C1RNative(
        NppiSize oSizeROI,
        int nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiHistogramEven_16u_C3R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramEvenGetBufferSize_16u_C3R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramEvenGetBufferSize_16u_C3RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramEvenGetBufferSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiHistogramEven_16u_C4R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramEvenGetBufferSize_16u_C4R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramEvenGetBufferSize_16u_C4RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramEvenGetBufferSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiHistogramEven_16u_AC4R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramEvenGetBufferSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramEvenGetBufferSize_16u_AC4RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramEvenGetBufferSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiHistogramEven_16s_C1R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Number of levels in the histogram.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramEvenGetBufferSize_16s_C1R(
        NppiSize oSizeROI,
        int nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramEvenGetBufferSize_16s_C1RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramEvenGetBufferSize_16s_C1RNative(
        NppiSize oSizeROI,
        int nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiHistogramEven_16s_C3R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramEvenGetBufferSize_16s_C3R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramEvenGetBufferSize_16s_C3RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramEvenGetBufferSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiHistogramEven_16s_C4R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramEvenGetBufferSize_16s_C4R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramEvenGetBufferSize_16s_C4RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramEvenGetBufferSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Buffer size for \ref nppiHistogramEven_16s_AC4R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramEvenGetBufferSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramEvenGetBufferSize_16s_AC4RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramEvenGetBufferSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /** @} */
    /** @} image_histogrameven */
    /**
     * <pre>
     * @defgroup image_histogramrange HistogramRange
     * Primitives for computing the histogram of an image within specified ranges.
     * @{
     *
     * </pre>
     */
    /**
     * <pre>
     * @name HistogramEven
     * The histogram is computed according to the ranges provided in \f$pLevels\f$.
     * The histogram \f$pHist[k]\f$ is defined as the total number of pixels that fall into the range:
     * \f$pLevels[k] <= pSrc(j, i) < pLevels[k+1]\f$. The number of the histogram bins is \f$nLevel - 1\f$.
     * The functions require additional scratch buffer for computations.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One-channel 8-bit unsigned HistogramRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Pointer to array that receives the computed histogram.
     *      The array must be of size nLevels-1.
     * @param pLevels Pointer to array containing the level sizes of the bins.
            The array must be of size nLevels.
     * @param nLevels Number of levels in histogram.
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_8u_C1R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramRange_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pHist,
        TypedPointer<Npp32s> pLevels,
        int nLevels,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramRange_8u_C1RNative(pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer));
    }
    private static native int nppiHistogramRange_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pHist,
        TypedPointer<Npp32s> pLevels,
        int nLevels,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned HistogramRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving the computed histograms per color channel.
     *      Array pointed by pHist[i] must be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param pLevels Array containing pointers to level-arrays per color channel.
            Array pointed by pLevel[i] must be of size nLevels[i].
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_8u_C3R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramRange_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramRange_8u_C3RNative(pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer));
    }
    private static native int nppiHistogramRange_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned HistogramRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving the computed histograms per color channel.
     *      Array pointed by pHist[i] must be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param pLevels Array containing pointers to level-arrays per color channel.
            Array pointed by pLevel[i] must be of size nLevels[i].
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_8u_C4R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramRange_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramRange_8u_C4RNative(pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer));
    }
    private static native int nppiHistogramRange_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned HistogramRange ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving the computed histograms per color channel.
     *      Array pointed by pHist[i] must be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param pLevels Array containing pointers to level-arrays per color channel.
            Array pointed by pLevel[i] must be of size nLevels[i].
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_8u_AC4R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramRange_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramRange_8u_AC4RNative(pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer));
    }
    private static native int nppiHistogramRange_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned HistogramRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Pointer to array that receives the computed histogram.
     *      The array must be of size nLevels-1.
     * @param pLevels Pointer to array containing the level sizes of the bins.
            The array must be of size nLevels.
     * @param nLevels Number of levels in histogram.
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_16u_C1R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramRange_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pHist,
        TypedPointer<Npp32s> pLevels,
        int nLevels,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramRange_16u_C1RNative(pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer));
    }
    private static native int nppiHistogramRange_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pHist,
        TypedPointer<Npp32s> pLevels,
        int nLevels,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned HistogramRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving the computed histograms per color channel.
     *      Array pointed by pHist[i] must be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param pLevels Array containing pointers to level-arrays per color channel.
            Array pointed by pLevel[i] must be of size nLevels[i].
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_16u_C3R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramRange_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramRange_16u_C3RNative(pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer));
    }
    private static native int nppiHistogramRange_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned HistogramRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving the computed histograms per color channel.
     *      Array pointed by pHist[i] must be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param pLevels Array containing pointers to level-arrays per color channel.
            Array pointed by pLevel[i] must be of size nLevels[i].
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_16u_C4R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramRange_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramRange_16u_C4RNative(pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer));
    }
    private static native int nppiHistogramRange_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned HistogramRange ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving the computed histograms per color channel.
     *      Array pointed by pHist[i] must be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param pLevels Array containing pointers to level-arrays per color channel.
            Array pointed by pLevel[i] must be of size nLevels[i].
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_16u_AC4R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramRange_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramRange_16u_AC4RNative(pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer));
    }
    private static native int nppiHistogramRange_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed HistogramRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Pointer to array that receives the computed histogram.
     *      The array must be of size nLevels-1.
     * @param pLevels Pointer to array containing the level sizes of the bins.
            The array must be of size nLevels.
     * @param nLevels Number of levels in histogram.
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_16s_C1R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramRange_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pHist,
        TypedPointer<Npp32s> pLevels,
        int nLevels,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramRange_16s_C1RNative(pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer));
    }
    private static native int nppiHistogramRange_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pHist,
        TypedPointer<Npp32s> pLevels,
        int nLevels,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed HistogramRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving the computed histograms per color channel.
     *      Array pointed by pHist[i] must be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param pLevels Array containing pointers to level-arrays per color channel.
            Array pointed by pLevel[i] must be of size nLevels[i].
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_16s_C3R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramRange_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramRange_16s_C3RNative(pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer));
    }
    private static native int nppiHistogramRange_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed HistogramRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving the computed histograms per color channel.
     *      Array pointed by pHist[i] must be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param pLevels Array containing pointers to level-arrays per color channel.
            Array pointed by pLevel[i] must be of size nLevels[i].
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_16s_C4R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramRange_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramRange_16s_C4RNative(pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer));
    }
    private static native int nppiHistogramRange_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed HistogramRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving the computed histograms per color channel.
     *      Array pointed by pHist[i] must be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param pLevels Array containing pointers to level-arrays per color channel.
            Array pointed by pLevel[i] must be of size nLevels[i].
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_16s_AC4R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramRange_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramRange_16s_AC4RNative(pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer));
    }
    private static native int nppiHistogramRange_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32s>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point HistogramRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Pointer to array that receives the computed histogram.
     *      The array must be of size nLevels-1.
     * @param pLevels Pointer to array containing the level sizes of the bins.
            The array must be of size nLevels.
     * @param nLevels Number of levels in histogram.
     * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_32f_C1R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramRange_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pHist,
        TypedPointer<Npp32f> pLevels,
        int nLevels,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramRange_32f_C1RNative(pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer));
    }
    private static native int nppiHistogramRange_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s> pHist,
        TypedPointer<Npp32f> pLevels,
        int nLevels,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point HistogramRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving the computed histograms per color channel.
     *      Array pointed by pHist[i] must be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param pLevels Array containing pointers to level-arrays per color channel.
            Array pointed by pLevel[i] must be of size nLevels[i].
     * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_32f_C3R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramRange_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramRange_32f_C3RNative(pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer));
    }
    private static native int nppiHistogramRange_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point HistogramRange.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving the computed histograms per color channel.
     *      Array pointed by pHist[i] must be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param pLevels Array containing pointers to level-arrays per color channel.
            Array pointed by pLevel[i] must be of size nLevels[i].
     * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_32f_C4R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramRange_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramRange_32f_C4RNative(pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer));
    }
    private static native int nppiHistogramRange_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point HistogramRange ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pHist Array of pointers which are receiving the computed histograms per color channel.
     *      Array pointed by pHist[i] must be of size nLevels[i]-1.
     * @param nLevels Array containing number of levels per color channel.
     * @param pLevels Array containing pointers to level-arrays per color channel.
            Array pointed by pLevel[i] must be of size nLevels[i].
     * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_32f_AC4R)
     *      scratch buffer.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiHistogramRange_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer)
    {
        return JNpp.checkResult(nppiHistogramRange_32f_AC4RNative(pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer));
    }
    private static native int nppiHistogramRange_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSizeROI,
        TypedPointer<Npp32s>[] pHist,
        TypedPointer<Npp32f>[] pLevels,
        int[] nLevels,
        TypedPointer<Npp8u> pBuffer);


    /** @} */
    /**
     * <pre>
     * @name HistogramRangeGetBufferSize
     * Companion primitives for computing the device buffer size (in bytes) required by the HistogramRange primitives.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * Scratch-buffer size for nppiHistogramRange_8u_C1R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Number of levels in the histogram.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramRangeGetBufferSize_8u_C1R(
        NppiSize oSizeROI,
        int nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramRangeGetBufferSize_8u_C1RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramRangeGetBufferSize_8u_C1RNative(
        NppiSize oSizeROI,
        int nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Scratch-buffer size for nppiHistogramRange_8u_C3R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramRangeGetBufferSize_8u_C3R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramRangeGetBufferSize_8u_C3RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramRangeGetBufferSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Scratch-buffer size for nppiHistogramRange_8u_C4R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramRangeGetBufferSize_8u_C4R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramRangeGetBufferSize_8u_C4RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramRangeGetBufferSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Scratch-buffer size for nppiHistogramRange_8u_AC4R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramRangeGetBufferSize_8u_AC4R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramRangeGetBufferSize_8u_AC4RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramRangeGetBufferSize_8u_AC4RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Scratch-buffer size for nppiHistogramRange_16u_C1R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Number of levels in the histogram.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramRangeGetBufferSize_16u_C1R(
        NppiSize oSizeROI,
        int nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramRangeGetBufferSize_16u_C1RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramRangeGetBufferSize_16u_C1RNative(
        NppiSize oSizeROI,
        int nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Scratch-buffer size for nppiHistogramRange_16u_C3R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramRangeGetBufferSize_16u_C3R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramRangeGetBufferSize_16u_C3RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramRangeGetBufferSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Scratch-buffer size for nppiHistogramRange_16u_C4R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramRangeGetBufferSize_16u_C4R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramRangeGetBufferSize_16u_C4RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramRangeGetBufferSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Scratch-buffer size for nppiHistogramRange_16u_AC4R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramRangeGetBufferSize_16u_AC4R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramRangeGetBufferSize_16u_AC4RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramRangeGetBufferSize_16u_AC4RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Scratch-buffer size for nppiHistogramRange_16s_C1R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Number of levels in the histogram.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramRangeGetBufferSize_16s_C1R(
        NppiSize oSizeROI,
        int nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramRangeGetBufferSize_16s_C1RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramRangeGetBufferSize_16s_C1RNative(
        NppiSize oSizeROI,
        int nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Scratch-buffer size for nppiHistogramRange_16s_C3R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramRangeGetBufferSize_16s_C3R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramRangeGetBufferSize_16s_C3RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramRangeGetBufferSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Scratch-buffer size for nppiHistogramRange_16s_C4R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramRangeGetBufferSize_16s_C4R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramRangeGetBufferSize_16s_C4RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramRangeGetBufferSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Scratch-buffer size for nppiHistogramRange_16s_AC4R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramRangeGetBufferSize_16s_AC4R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramRangeGetBufferSize_16s_AC4RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramRangeGetBufferSize_16s_AC4RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Scratch-buffer size for nppiHistogramRange_32f_C1R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Number of levels in the histogram.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramRangeGetBufferSize_32f_C1R(
        NppiSize oSizeROI,
        int nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramRangeGetBufferSize_32f_C1RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramRangeGetBufferSize_32f_C1RNative(
        NppiSize oSizeROI,
        int nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Scratch-buffer size for nppiHistogramRange_32f_C3R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramRangeGetBufferSize_32f_C3R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramRangeGetBufferSize_32f_C3RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramRangeGetBufferSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Scratch-buffer size for nppiHistogramRange_32f_C4R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramRangeGetBufferSize_32f_C4R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramRangeGetBufferSize_32f_C4RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramRangeGetBufferSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /**
     * <pre>
     * Scratch-buffer size for nppiHistogramRange_32f_AC4R.
     *
     * @param oSizeROI \ref roi_specification.
     * @param nLevels Array containing number of levels per color channel.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
     * </pre>
     */
    public static int nppiHistogramRangeGetBufferSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize)
    {
        return JNpp.checkResult(nppiHistogramRangeGetBufferSize_32f_AC4RNative(oSizeROI, nLevels, hpBufferSize));
    }
    private static native int nppiHistogramRangeGetBufferSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] nLevels,
        int[] hpBufferSize);


    /** @} */
    /** @} image_histogramrange */
    /**
     * <pre>
     * @defgroup image_proximity Image Proximity
     * Primitives for computing the proximity measure between a source image and a template image.
     *
     * \section general_introduction General Introduction
     * There are basically two approaches to compute the proximity measure for template matching,
     * Euclidean distance and the cross correlation.
     * -# Euclidean distance computes the sum of the squared distance (SSD) between the corresponding
     * pixels of the source image and the template image. The smaller the distance is, the more
     * similar the source image and the template image is around the pixel. The anchor of the template
     * image is used during the computations, which always lies in the gemotric center of the image.
     * Given a source image \f$pSrc\f$ (\f$W_s \times H_s\f$) and a template image \f$pTpl\f$ (\f$W_t \times H_t\f$),
     * the Euclidean distance \f$D_{st}(c,r)\f$ between two images at pixel in row \f$r\f$ and column
     * \f$c\f$ is computed as (\f$s\f$ stands for source image and \f$t\f$ for template image for short):
     * \f[D_{st}(c,r)=\sum_{j=0}^{H_t-1}\sum_{i=0}^{W_t-1}[pTpl(j,i)-pSrc(j+c-\frac{H_t}{2}, i+r-\frac{W_t}{2})]^2 \f]
     * -# Cross correlation computes the sum of the product between the corresponding pixels of the source
     * image and the template image. The cross correlation \f$R_{st}(c,r)\f$ is calculated as:
     * \f[R_{st}(c,r)=\sum_{j=0}^{H_t-1}\sum_{i=0}^{W_t-1}[pTpl(j,i)\cdot pSrc(j+c-\frac{H_t}{2}, i+r-\frac{W_t}{2})] \f]
     * The larger the cross correlation value is, the more similar the source image and the template image is around the pixel.
     * -# The cross correlation \f$R_{st}(c,r)\f$ is affected by the brightness of the images which may vary due to the lighting
     * and exposure conditions. Therefore, NPP computes the cross correlation coefficient to circumvent this dependence.
     * This is typically done at every step by subtracting the mean from every pixel value, i.e.,
     * \f[\tilde{R}_{st}(c,r)=\sum_{j=0}^{H_t-1}\sum_{i=0}^{W_t-1}[pTpl(j,i)-Mean_t]\cdot [pSrc(j+c-\frac{H_t}{2}, i+r-\frac{W_t}{2})-Mean_s] \f]
     *
     * NPP computes the normalized values of Euclidean distance, cross correlation and the cross correlation coefficient.
     * -# The normalized Euclidean distance \f$\sigma_{st}(c,r)\f$ is defined as:
     * \f[\sigma_{st}(c,r) = \frac{D_{st}(c,r)}{\sqrt{R_{ss}(c,r)\cdot R_{tt}(\frac{H_t}{2},\frac{W_t}{2})}} \f]
     * -# The normalized cross correlation \f$\rho_{st}(c,r)\f$ is defined as:
     * \f[\rho_{st}(c,r) = \frac{R_{st}(c,r)}{\sqrt{R_{ss}(c,r)\cdot R_{tt}(\frac{H_t}{2},\frac{W_t}{2})}} \f]
     * The \f$R_{ss}(c,r)\f$ and \f$R_{tt}(\frac{H_t}{2}, \frac{W_t}{2}\f$ denote the auto correlation of the source
     * image and the template image individually. They are defined as:
     * \f[R_{ss}(c,r)=\sum_{j=c-\frac{H_t}{2}}^{c+\frac{H_t}{2}}\sum_{i=r-\frac{W_t}{2}}^{r+\frac{W_t}{2}}pSrc(j, i) \f]
     * \f[R_{tt}(\frac{H_t}{2},\frac{W_t}{2})=\sum_{j=0}^{H_t-1}\sum_{i=0}^{W_t-1}pTpl(j,i) \f]
     * -# Similarly, the normalized cross correlation coefficient \f$\gamma_{st}(c,r)\f$ is calculated as:
     * \f[\gamma_{st}(c,r) = \frac{\tilde{R}_{st}(c,r)}{\sqrt{\tilde{R}_{ss}(c,r)\cdot \tilde{R}_{tt}(\frac{H_t}{2},\frac{W_t}{2})}} \f]
     * The \f$\tilde{R}_{ss}(c,r)\f$ and \f$\tilde{R}_{tt}(\frac{H_t}{2}, \frac{W_t}{2}\f$ are defined as:
     * \f[\tilde{R}_{ss}(c,r)=\sum_{j=c-\frac{H_t}{2}}^{c+\frac{H_t}{2}}\sum_{i=r-\frac{W_t}{2}}^{r+\frac{W_t}{2}}[pSrc(j, i)-Mean_s] \f]
     * \f[\tilde{R}_{tt}(\frac{H_t}{2},\frac{W_t}{2})=\sum_{j=0}^{H_t-1}\sum_{i=0}^{W_t-1}[pTpl(j,i)-Mean_t] \f]
     *
     * \section category Categorizations
     * The Euclidean distance and the cross correlation are categorized into three types, full, same, and valid.
     * -# Full mode indicates that the anchor of the template image starts from the outside of the source image,
     * assuming the out-of-boundary pixels are zeor-padded. The size of the destination
     * image is \f$(W_s + W_t - 1) \times (H_s + H_t - 1)\f$.
     * -# Same mode means that the anchor of the template image starts from the top left pixel
     * of the source image. All the out-of-boundary pixels are also zero-padded.
     * The size of the destination image is the same as the source one, i.e., \f$W_s \times H_s\f$.
     * -# Valid mode indicates that there are no out-of-boudnary readings from the source image.
     * The anchor of the template image starts from the inside of the source image. The
     * size of the destination image is \f$(W_s - W_t + 1) \times (H_s - H_t + 1)\f$.
     *
     * @{
     * </pre>
     */
    /**
     * <pre>
     * @defgroup sqrdistancefullnorm SqrDistanceFull_Norm
     * Primitives for computing the normalized Euclidean distance between two images with full mode.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * @name SqrDistanceFull_Norm
     * The functions compute the \f$\sigma_{st}(c,r)\f$ in \ref general_introduction
     * with full mode (see \ref category).
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One-channel 8-bit unsigned image SqrDistanceFull_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_8u_C1RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiSqrDistanceFull_Norm_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image SqrDistanceFull_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_8u_C3RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiSqrDistanceFull_Norm_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image SqrDistanceFull_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_8u_C4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiSqrDistanceFull_Norm_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image SqrDistanceFull_Norm ignoring alpha channel, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_8u_AC4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiSqrDistanceFull_Norm_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * One-channel 32-bit floating point image SqrDistanceFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceFull_Norm_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 32-bit floating point image SqrDistanceFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceFull_Norm_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 32-bit floating point image SqrDistanceFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceFull_Norm_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 32-bit floating point image SqrDistanceFull_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceFull_Norm_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 8-bit unsigned image SqrDistanceFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_8u32f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_8u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceFull_Norm_8u32f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image SqrDistanceFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_8u32f_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_8u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceFull_Norm_8u32f_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image SqrDistanceFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_8u32f_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_8u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceFull_Norm_8u32f_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image SqrDistanceFull_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_8u32f_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_8u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceFull_Norm_8u32f_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 8-bit signed image SqrDistanceFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_8s32f_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_8s32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceFull_Norm_8s32f_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 8-bit signed image SqrDistanceFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_8s32f_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_8s32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceFull_Norm_8s32f_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit signed image SqrDistanceFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_8s32f_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_8s32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceFull_Norm_8s32f_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit signed image SqrDistanceFull_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_8s32f_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_8s32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceFull_Norm_8s32f_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 16-bit unsigned image SqrDistanceFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_16u32f_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_16u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceFull_Norm_16u32f_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image SqrDistanceFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_16u32f_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_16u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceFull_Norm_16u32f_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image SqrDistanceFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_16u32f_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_16u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceFull_Norm_16u32f_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image SqrDistanceFull_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceFull_Norm_16u32f_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceFull_Norm_16u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceFull_Norm_16u32f_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /** @} */
    /** @} SqrDistanceFull_Norm */
    /**
     * <pre>
     * @defgroup sqrdistancesamenorm SqrDistanceSame_Norm
     * Primitives for computing the normalized Euclidean distance between two images with same mode.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * @name SqrDistanceSame_Norm
     * The functions compute the \f$\sigma_{st}(c,r)\f$ in \ref general_introduction
     * with same mode (see \ref category).
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One-channel 8-bit unsigned image SqrDistanceSame_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_8u_C1RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiSqrDistanceSame_Norm_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image SqrDistanceSame_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_8u_C3RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiSqrDistanceSame_Norm_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image SqrDistanceSame_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_8u_C4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiSqrDistanceSame_Norm_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image SqrDistanceSame_Norm ignoring alpha channel, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_8u_AC4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiSqrDistanceSame_Norm_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * One-channel 32-bit floating point image SqrDistanceSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceSame_Norm_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 32-bit floating point image SqrDistanceSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceSame_Norm_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 32-bit floating point image SqrDistanceSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceSame_Norm_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 32-bit floating point image SqrDistanceSame_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceSame_Norm_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 8-bit unsigned image SqrDistanceSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_8u32f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_8u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceSame_Norm_8u32f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image SqrDistanceSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_8u32f_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_8u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceSame_Norm_8u32f_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image SqrDistanceSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_8u32f_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_8u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceSame_Norm_8u32f_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image SqrDistanceSame_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_8u32f_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_8u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceSame_Norm_8u32f_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 8-bit signed image SqrDistanceSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_8s32f_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_8s32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceSame_Norm_8s32f_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 8-bit signed image SqrDistanceSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_8s32f_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_8s32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceSame_Norm_8s32f_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit signed image SqrDistanceSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_8s32f_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_8s32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceSame_Norm_8s32f_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit signed image SqrDistanceSame_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_8s32f_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_8s32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceSame_Norm_8s32f_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 16-bit unsigned image SqrDistanceSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_16u32f_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_16u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceSame_Norm_16u32f_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image SqrDistanceSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_16u32f_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_16u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceSame_Norm_16u32f_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image SqrDistanceSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_16u32f_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_16u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceSame_Norm_16u32f_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image SqrDistanceSame_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceSame_Norm_16u32f_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceSame_Norm_16u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceSame_Norm_16u32f_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /** @} */
    /** @} SqrDistanceFull_Norm */
    /**
     * <pre>
     * @defgroup sqrdistancevalidnorm SqrDistanceValid_Norm
     * Primitives for computing the normalized Euclidean distance between two images with valid mode.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * @name SqrDistanceValid_Norm
     * The functions compute the \f$\sigma_{st}(c,r)\f$ in \ref general_introduction
     * with valid mode (see \ref category).
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One-channel 8-bit unsigned image SqrDistanceValid_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_8u_C1RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiSqrDistanceValid_Norm_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image SqrDistanceValid_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_8u_C3RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiSqrDistanceValid_Norm_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image SqrDistanceValid_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_8u_C4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiSqrDistanceValid_Norm_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image SqrDistanceValid_Norm ignoring alpha channel, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_8u_AC4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiSqrDistanceValid_Norm_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * One-channel 32-bit floating point image SqrDistanceValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceValid_Norm_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 32-bit floating point image SqrDistanceValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceValid_Norm_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 32-bit floating point image SqrDistanceValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceValid_Norm_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 32-bit floating point image SqrDistanceValid_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceValid_Norm_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 8-bit unsigned image SqrDistanceValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_8u32f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_8u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceValid_Norm_8u32f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image SqrDistanceValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_8u32f_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_8u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceValid_Norm_8u32f_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image SqrDistanceValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_8u32f_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_8u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceValid_Norm_8u32f_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image SqrDistanceValid_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_8u32f_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_8u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceValid_Norm_8u32f_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 8-bit signed image SqrDistanceValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_8s32f_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_8s32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceValid_Norm_8s32f_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 8-bit signed image SqrDistanceValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_8s32f_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_8s32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceValid_Norm_8s32f_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit signed image SqrDistanceValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_8s32f_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_8s32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceValid_Norm_8s32f_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit signed image SqrDistanceValid_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_8s32f_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_8s32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceValid_Norm_8s32f_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 16-bit unsigned image SqrDistanceValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_16u32f_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_16u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceValid_Norm_16u32f_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image SqrDistanceValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_16u32f_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_16u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceValid_Norm_16u32f_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image SqrDistanceValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_16u32f_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_16u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceValid_Norm_16u32f_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image SqrDistanceValid_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiSqrDistanceValid_Norm_16u32f_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiSqrDistanceValid_Norm_16u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiSqrDistanceValid_Norm_16u32f_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /** @} */
    /** @} SqrDistanceValid_Norm*/
    /**
     * <pre>
     * @defgroup crosscorrfullnorm CrossCorrFull_Norm
     * Primitives for computing the normalized cross correlation between two images with full mode.
     *
     * @{
     * </pre>
     */
    /**
     * <pre>
     * @name CrossCorrFull_Norm
     * The functions compute the \f$\rho_{st}(c,r)\f$ in \ref general_introduction
     * with full mode (see \ref category).
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One-channel 8-bit unsigned image CrossCorrFull_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_8u_C1RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiCrossCorrFull_Norm_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image CrossCorrFull_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_8u_C3RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiCrossCorrFull_Norm_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrFull_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_8u_C4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiCrossCorrFull_Norm_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrFull_Norm ignoring alpha channel, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_8u_AC4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiCrossCorrFull_Norm_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * One-channel 32-bit floating point image CrossCorrFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrFull_Norm_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 32-bit floating point image CrossCorrFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrFull_Norm_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 32-bit floating point image CrossCorrFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrFull_Norm_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 32-bit floating point image CrossCorrFull_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrFull_Norm_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 8-bit unsigned image CrossCorrFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_8u32f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_8u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrFull_Norm_8u32f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image CrossCorrFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_8u32f_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_8u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrFull_Norm_8u32f_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_8u32f_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_8u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrFull_Norm_8u32f_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrFull_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_8u32f_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_8u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrFull_Norm_8u32f_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 8-bit signed image CrossCorrFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_8s32f_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_8s32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrFull_Norm_8s32f_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 8-bit signed image CrossCorrFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_8s32f_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_8s32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrFull_Norm_8s32f_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit signed image CrossCorrFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_8s32f_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_8s32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrFull_Norm_8s32f_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit signed image CrossCorrFull_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_8s32f_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_8s32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrFull_Norm_8s32f_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 16-bit unsigned image CrossCorrFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_16u32f_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_16u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrFull_Norm_16u32f_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image CrossCorrFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_16u32f_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_16u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrFull_Norm_16u32f_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image CrossCorrFull_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_16u32f_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_16u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrFull_Norm_16u32f_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image CrossCorrFull_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_Norm_16u32f_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrFull_Norm_16u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrFull_Norm_16u32f_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /** @} */
    /** @} CrossCorrFull_Norm */
    /**
     * <pre>
     * @defgroup crosscorrsamenorm CrossCorrSame_Norm
     * Primitives for computing the normalized cross correlation between two images with same mode.
     *
     * @{
     * </pre>
     */
    /**
     * <pre>
     * @name CrossCorrSame_Norm
     * The functions compute the \f$\rho_{st}(c,r)\f$ in \ref general_introduction
     * with same mode (see \ref category).
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One-channel 8-bit unsigned image CrossCorrSame_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_8u_C1RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiCrossCorrSame_Norm_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image CrossCorrSame_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_8u_C3RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiCrossCorrSame_Norm_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrSame_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_8u_C4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiCrossCorrSame_Norm_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrSame_Norm ignoring alpha channel, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_8u_AC4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiCrossCorrSame_Norm_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * One-channel 32-bit floating point image CrossCorrSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrSame_Norm_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 32-bit floating point image CrossCorrSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrSame_Norm_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 32-bit floating point image CrossCorrSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrSame_Norm_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 32-bit floating point image CrossCorrSame_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrSame_Norm_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 8-bit unsigned image CrossCorrSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_8u32f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_8u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrSame_Norm_8u32f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image CrossCorrSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_8u32f_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_8u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrSame_Norm_8u32f_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_8u32f_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_8u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrSame_Norm_8u32f_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrSame_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_8u32f_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_8u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrSame_Norm_8u32f_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 8-bit signed image CrossCorrSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_8s32f_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_8s32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrSame_Norm_8s32f_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 8-bit signed image CrossCorrSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_8s32f_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_8s32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrSame_Norm_8s32f_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit signed image CrossCorrSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_8s32f_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_8s32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrSame_Norm_8s32f_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit signed image CrossCorrSame_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_8s32f_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_8s32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrSame_Norm_8s32f_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 16-bit unsigned image CrossCorrSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_16u32f_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_16u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrSame_Norm_16u32f_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image CrossCorrSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_16u32f_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_16u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrSame_Norm_16u32f_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image CrossCorrSame_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_16u32f_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_16u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrSame_Norm_16u32f_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image CrossCorrSame_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_Norm_16u32f_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrSame_Norm_16u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrSame_Norm_16u32f_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /** @} */
    /** @} CrossCorrSame_Norm*/
    /**
     * <pre>
     * @defgroup crosscorrvalidnorm CrossCorrValid_Norm
     * Primitives for computing the normalized cross correlation between two images with valid mode.
     *
     * @{
     * </pre>
     */
    /**
     * <pre>
     * @name CrossCorrValid_Norm
     * The functions compute the \f$\rho_{st}(c,r)\f$ in \ref general_introduction
     * with valid mode (see \ref category).
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One-channel 8-bit unsigned image CrossCorrValid_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_8u_C1RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiCrossCorrValid_Norm_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image CrossCorrValid_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_8u_C3RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiCrossCorrValid_Norm_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrValid_Norm, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_8u_C4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiCrossCorrValid_Norm_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrValid_Norm ignoring alpha channel, scaled by \f$2^(-nScaleFactor)\f$.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_8u_AC4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor));
    }
    private static native int nppiCrossCorrValid_Norm_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor);


    /**
     * <pre>
     * One-channel 32-bit floating point image CrossCorrValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_Norm_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 32-bit floating point image CrossCorrValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_Norm_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 32-bit floating point image CrossCorrValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_Norm_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 32-bit floating point image CrossCorrValid_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_Norm_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 8-bit unsigned image CrossCorrValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_8u32f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_8u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_Norm_8u32f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image CrossCorrValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_8u32f_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_8u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_Norm_8u32f_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_8u32f_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_8u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_Norm_8u32f_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrValid_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_8u32f_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_8u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_Norm_8u32f_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 8-bit signed image CrossCorrValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_8s32f_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_8s32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_Norm_8s32f_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 8-bit signed image CrossCorrValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_8s32f_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_8s32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_Norm_8s32f_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit signed image CrossCorrValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_8s32f_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_8s32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_Norm_8s32f_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 8-bit signed image CrossCorrValid_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_8s32f_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_8s32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_Norm_8s32f_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 16-bit unsigned image CrossCorrValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_16u32f_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_16u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_Norm_16u32f_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image CrossCorrValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_16u32f_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_16u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_Norm_16u32f_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image CrossCorrValid_Norm.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_16u32f_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_16u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_Norm_16u32f_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image CrossCorrValid_Norm ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_Norm_16u32f_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_Norm_16u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_Norm_16u32f_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /** @} */
    /** @} CrossCorrValid_Norm*/
    /**
     * <pre>
     * @defgroup crosscorrvalid CrossCorrValid
     * Primitives for computing the cross correlation between two images with valid mode.
     *
     * @{
     * </pre>
     */
    /**
     * <pre>
     * @name CrossCorrValid
     * The functions compute the \f$R_{st}(c,r)\f$ in \ref general_introduction
     * with valid mode (see \ref category).
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One-channel 32-bit floating point images CrossCorrValid.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 8-bit unsigned images CrossCorrValid.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_8u32f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_8u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_8u32f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 8-bit signed images CrossCorrValid.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_8s32f_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_8s32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_8s32f_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /**
     * <pre>
     * One-channel 16-bit unsigned images CrossCorrValid.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_16u32f_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep)
    {
        return JNpp.checkResult(nppiCrossCorrValid_16u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep));
    }
    private static native int nppiCrossCorrValid_16u32f_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep);


    /** @} */
    /** @} CrossCorrValid*/
    /**
     * <pre>
     * @defgroup crosscorrfullnormlevel CrossCorrFull_NormLevel
     * Primitives for computing the normalized cross correlation coefficient between two images with full mode.
     *
     * @{
     * </pre>
     */
    /**
     * <pre>
     * @name CrossCorrFull_NormLevel
     * The functions compute the \f$\gamma_{st}(c,r)\f$ in \ref general_introduction
     * with full mode (see \ref category). The functions require additional scratch buffer for computations.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One-channel 8-bit unsigned image CrossCorrFull_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_8u_C1RSfs to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_8u_C1RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image CrossCorrFull_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_8u_C3RSfs to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_8u_C3RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrFull_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_8u_C4RSfs to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_8u_C4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrFull_NormLevel ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_8u_AC4RSfs to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_8u_AC4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image CrossCorrFull_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image CrossCorrFull_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image CrossCorrFull_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image CrossCorrFull_NormLevel ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 8-bit unsigned image CrossCorrFull_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_8u32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_8u32f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_8u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_8u32f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image CrossCorrFull_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_8u32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_8u32f_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_8u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_8u32f_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrFull_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_8u32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_8u32f_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_8u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_8u32f_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrFull_NormLevel ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_8u32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_8u32f_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_8u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_8u32f_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 8-bit signed image CrossCorrFull_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_8s32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_8s32f_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_8s32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_8s32f_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit signed image CrossCorrFull_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_8s32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_8s32f_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_8s32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_8s32f_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit signed image CrossCorrFull_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_8s32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_8s32f_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_8s32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_8s32f_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit signed image CrossCorrFull_NormLevel ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_8s32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_8s32f_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_8s32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_8s32f_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image CrossCorrFull_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_16u32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_16u32f_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_16u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_16u32f_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image CrossCorrFull_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_16u32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_16u32f_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_16u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_16u32f_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image CrossCorrFull_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_16u32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_16u32f_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_16u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_16u32f_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image CrossCorrFull_NormLevel ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiFullNormLevelGetBufferHostSize_16u32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrFull_NormLevel_16u32f_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrFull_NormLevel_16u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrFull_NormLevel_16u32f_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /** @} */
    /**
     * <pre>
     * @name FullNormLevelGetBufferHostSize
     * Companion primitives for computing the device buffer size (in bytes) required by the CrossCorrFull_NormLevel primitives.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * Buffer size (in bytes) for nppiCrossCorrFull_NormLevel_8u_C1RSfs.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_8u_C1RSfs(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_8u_C1RSfsNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_8u_C1RSfsNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8u_C3RSfs.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_8u_C3RSfs(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_8u_C3RSfsNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_8u_C3RSfsNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8u_C4RSfs.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_8u_C4RSfs(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_8u_C4RSfsNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_8u_C4RSfsNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8u_AC4RSfs.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_8u_AC4RSfs(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_8u_AC4RSfsNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_8u_AC4RSfsNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8u32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_8u32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_8u32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_8u32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8u32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_8u32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_8u32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_8u32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8u32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_8u32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_8u32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_8u32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8u32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_8u32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_8u32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_8u32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8s32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_8s32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_8s32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_8s32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8s32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_8s32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_8s32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_8s32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8s32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_8s32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_8s32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_8s32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8s32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_8s32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_8s32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_8s32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_16u32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_16u32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_16u32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_16u32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_16u32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_16u32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_16u32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_16u32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_16u32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_16u32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_16u32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_16u32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_16u32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiFullNormLevelGetBufferHostSize_16u32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiFullNormLevelGetBufferHostSize_16u32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiFullNormLevelGetBufferHostSize_16u32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /** @} */
    /** @} CrossCorrFull_NormLevel*/
    /**
     * <pre>
     * @defgroup crosscorrsamenormlevel CrossCorrSame_NormLevel
     * Primitives for computing the normalized cross correlation coefficient between two images with same mode.
     *
     * @{
     * </pre>
     */
    /**
     * <pre>
     * @name CrossCorrSame_NormLevel
     * The functions compute the \f$\gamma_{st}(c,r)\f$ in \ref general_introduction
     * with same mode (see \ref category). The functions require additional scratch buffer for computations.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One-channel 8-bit unsigned image CrossCorrSame_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_8u_C1RSfs to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_8u_C1RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image CrossCorrSame_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_8u_C3RSfs to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_8u_C3RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrSame_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_8u_C4RSfs to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_8u_C4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrSame_NormLevel ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_8u_AC4RSfs to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_8u_AC4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image CrossCorrSame_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image CrossCorrSame_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image CrossCorrSame_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image CrossCorrSame_NormLevel ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 8-bit unsigned image CrossCorrSame_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_8u32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_8u32f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_8u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_8u32f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image CrossCorrSame_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_8u32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_8u32f_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_8u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_8u32f_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrSame_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_8u32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_8u32f_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_8u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_8u32f_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrSame_NormLevel ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_8u32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_8u32f_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_8u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_8u32f_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 8-bit signed image CrossCorrSame_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_8s32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_8s32f_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_8s32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_8s32f_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit signed image CrossCorrSame_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_8s32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_8s32f_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_8s32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_8s32f_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit signed image CrossCorrSame_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_8s32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_8s32f_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_8s32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_8s32f_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit signed image CrossCorrSame_NormLevel ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_8s32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_8s32f_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_8s32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_8s32f_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image CrossCorrSame_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_16u32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_16u32f_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_16u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_16u32f_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image CrossCorrSame_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_16u32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_16u32f_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_16u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_16u32f_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image CrossCorrSame_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_16u32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_16u32f_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_16u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_16u32f_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image CrossCorrSame_NormLevel ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiSameNormLevelGetBufferHostSize_16u32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrSame_NormLevel_16u32f_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrSame_NormLevel_16u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrSame_NormLevel_16u32f_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /** @} */
    /**
     * <pre>
     * @name SameNormLevelGetBufferHostSize
     * Companion primitives for computing the device buffer size (in bytes) required by the CrossCorrSame_NormLevel primitives.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * Buffer size (in bytes) for nppiCrossCorrSame_NormLevel_8u_C1RSfs.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_8u_C1RSfs(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_8u_C1RSfsNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_8u_C1RSfsNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8u_C3RSfs.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_8u_C3RSfs(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_8u_C3RSfsNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_8u_C3RSfsNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8u_C4RSfs.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_8u_C4RSfs(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_8u_C4RSfsNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_8u_C4RSfsNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8u_AC4RSfs.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_8u_AC4RSfs(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_8u_AC4RSfsNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_8u_AC4RSfsNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8u32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_8u32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_8u32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_8u32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8u32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_8u32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_8u32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_8u32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8u32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_8u32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_8u32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_8u32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8u32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_8u32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_8u32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_8u32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8s32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_8s32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_8s32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_8s32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8s32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_8s32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_8s32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_8s32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8s32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_8s32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_8s32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_8s32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8s32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_8s32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_8s32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_8s32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_16u32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_16u32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_16u32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_16u32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_16u32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_16u32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_16u32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_16u32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_16u32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_16u32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_16u32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_16u32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_16u32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiSameNormLevelGetBufferHostSize_16u32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiSameNormLevelGetBufferHostSize_16u32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiSameNormLevelGetBufferHostSize_16u32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /** @} */
    /** @} CrossCorrSame_NormLevel*/
    /**
     * <pre>
     * @defgroup crosscorrvalidnormlevel CrossCorrValid_NormLevel
     * Primitives for computing the normalized cross correlation coefficient between two images with valid mode.
     *
     * @{
     * </pre>
     */
    /**
     * <pre>
     * @name CrossCorrValid_NormLevel
     * The functions compute the \f$\gamma_{st}(c,r)\f$ in \ref general_introduction
     * with valid mode (see \ref category). The functions require additional scratch buffer for computations.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One-channel 8-bit unsigned image CrossCorrValid_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_8u_C1RSfs to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_8u_C1RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_8u_C1RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_8u_C1RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image CrossCorrValid_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_8u_C3RSfs to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_8u_C3RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_8u_C3RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_8u_C3RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrValid_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_8u_C4RSfs to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_8u_C4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_8u_C4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_8u_C4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrValid_NormLevel ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param nScaleFactor \ref integer_result_scaling.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_8u_AC4RSfs to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_8u_AC4RSfs(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_8u_AC4RSfsNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_8u_AC4RSfsNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        int nScaleFactor,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image CrossCorrValid_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image CrossCorrValid_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image CrossCorrValid_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image CrossCorrValid_NormLevel ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp32f> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 8-bit unsigned image CrossCorrValid_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_8u32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_8u32f_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_8u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_8u32f_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image CrossCorrValid_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_8u32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_8u32f_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_8u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_8u32f_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrValid_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_8u32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_8u32f_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_8u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_8u32f_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image CrossCorrValid_NormLevel ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_8u32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_8u32f_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_8u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_8u32f_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 8-bit signed image CrossCorrValid_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_8s32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_8s32f_C1R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_8s32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_8s32f_C1RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit signed image CrossCorrValid_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_8s32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_8s32f_C3R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_8s32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_8s32f_C3RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit signed image CrossCorrValid_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_8s32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_8s32f_C4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_8s32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_8s32f_C4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit signed image CrossCorrValid_NormLevel ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_8s32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_8s32f_AC4R(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_8s32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_8s32f_AC4RNative(
        TypedPointer<Npp8s> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp8s> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image CrossCorrValid_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_16u32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_16u32f_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_16u32f_C1RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_16u32f_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image CrossCorrValid_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_16u32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_16u32f_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_16u32f_C3RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_16u32f_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image CrossCorrValid_NormLevel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_16u32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_16u32f_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_16u32f_C4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_16u32f_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image CrossCorrValid_NormLevel ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcRoiSize \ref roi_specification.
     * @param pTpl Pointer to the template image.
     * @param nTplStep Number of bytes between successive rows in the template image.
     * @param oTplRoiSize \ref roi_specification.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiValidNormLevelGetBufferHostSize_16u32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiCrossCorrValid_NormLevel_16u32f_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiCrossCorrValid_NormLevel_16u32f_AC4RNative(pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer));
    }
    private static native int nppiCrossCorrValid_NormLevel_16u32f_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        NppiSize oSrcRoiSize,
        TypedPointer<Npp16u> pTpl,
        int nTplStep,
        NppiSize oTplRoiSize,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pDeviceBuffer);


    /** @} */
    /**
     * <pre>
     * @name ValidNormLevelGetBufferHostSize
     * Companion primitives for computing the device buffer size (in bytes) required by the CrossCorrValid_NormLevel primitives.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * Buffer size (in bytes) for nppiCrossCorrValid_NormLevel_8u_C1RSfs.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_8u_C1RSfs(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_8u_C1RSfsNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_8u_C1RSfsNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8u_C3RSfs.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_8u_C3RSfs(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_8u_C3RSfsNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_8u_C3RSfsNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8u_C4RSfs.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_8u_C4RSfs(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_8u_C4RSfsNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_8u_C4RSfsNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8u_AC4RSfs.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_8u_AC4RSfs(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_8u_AC4RSfsNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_8u_AC4RSfsNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8u32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_8u32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_8u32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_8u32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8u32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_8u32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_8u32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_8u32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8u32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_8u32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_8u32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_8u32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8u32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_8u32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_8u32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_8u32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8s32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_8s32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_8s32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_8s32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8s32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_8s32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_8s32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_8s32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8s32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_8s32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_8s32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_8s32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8s32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_8s32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_8s32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_8s32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_16u32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_16u32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_16u32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_16u32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_16u32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_16u32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_16u32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_16u32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_16u32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_16u32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_16u32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_16u32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_16u32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiValidNormLevelGetBufferHostSize_16u32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiValidNormLevelGetBufferHostSize_16u32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiValidNormLevelGetBufferHostSize_16u32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /** @} */
    /** @} CrossCorrValid_NormLevel*/
    /** @} image_proximity */
    /**
     * <pre>
     * @defgroup image_quality_index Image Quality Index
     * Primitives for computing the image quality index of two images.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * @name QualityIndex
     * Given two images \f$M\f$ and \f$N\f$ (both \f$W \times H\f$), the
     * mathematical formula to calculate the image quality index \f$Q\f$ between them is expressed as:
     * \f[Q = \frac{4\sigma_{MN}\tilde{M}\tilde{N}}{[(\tilde{M}^2)+(\tilde{N}^2)][(\sigma_M)^2+(\sigma_N)^2]} \f]
     * where
     * \f[\tilde{M} = \frac{1}{W\cdot H}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}M(j,i)\f]
     * \f[\tilde{N} = \frac{1}{W\cdot H}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}N(j,i)\f]
     * \f[\sigma_{M} = \sqrt{\frac{1}{W\cdot H-1}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}[M(j,i)-\tilde{M}]^2}\f]
     * \f[\sigma_{N} = \sqrt{\frac{1}{W\cdot H-1}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}[N(j,i)-\tilde{N}]^2}\f]
     * \f[\sigma_{MN} = \frac{1}{W\cdot H-1}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}[M(j,i)-\tilde{M}][N(j,i)-\tilde{N}]\f]
     * The functions require additional scratch buffer for computations.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One-channel 8-bit unsigned image QualityIndex.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oRoiSize \ref roi_specification.
     * @param pDst Pointer to the quality index.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiQualityIndexGetBufferHostSize_8u32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of
     * either image are constant numberse.
     * </pre>
     */
    public static int nppiQualityIndex_8u32f_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiQualityIndex_8u32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer));
    }
    private static native int nppiQualityIndex_8u32f_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image QualityIndex.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oRoiSize \ref roi_specification.
     * @param pDst Pointer to the quality index.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiQualityIndexGetBufferHostSize_16u32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of
     * either image are constant numberse.
     * </pre>
     */
    public static int nppiQualityIndex_16u32f_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiQualityIndex_16u32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer));
    }
    private static native int nppiQualityIndex_16u32f_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image QualityIndex.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oRoiSize \ref roi_specification.
     * @param pDst Pointer to the quality index.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiQualityIndexGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of
     * either image are constant numberse.
     * </pre>
     */
    public static int nppiQualityIndex_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiQualityIndex_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer));
    }
    private static native int nppiQualityIndex_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image QualityIndex.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oRoiSize \ref roi_specification.
     * @param pDst Pointer to the quality index.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiQualityIndexGetBufferHostSize_8u32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of
     * either image are constant numberse.
     * </pre>
     */
    public static int nppiQualityIndex_8u32f_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiQualityIndex_8u32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer));
    }
    private static native int nppiQualityIndex_8u32f_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image QualityIndex.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oRoiSize \ref roi_specification.
     * @param pDst Pointer to the quality index.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiQualityIndexGetBufferHostSize_16u32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of
     * either image are constant numberse.
     * </pre>
     */
    public static int nppiQualityIndex_16u32f_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiQualityIndex_16u32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer));
    }
    private static native int nppiQualityIndex_16u32f_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image QualityIndex.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oRoiSize \ref roi_specification.
     * @param pDst Pointer to the quality index.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiQualityIndexGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of
     * either image are constant numberse.
     * </pre>
     */
    public static int nppiQualityIndex_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiQualityIndex_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer));
    }
    private static native int nppiQualityIndex_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image QualityIndex.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oRoiSize \ref roi_specification.
     * @param pDst Pointer to the quality index.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiQualityIndexGetBufferHostSize_8u32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of
     * either image are constant numberse.
     * </pre>
     */
    public static int nppiQualityIndex_8u32f_AC4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiQualityIndex_8u32f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer));
    }
    private static native int nppiQualityIndex_8u32f_AC4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image QualityIndex.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oRoiSize \ref roi_specification.
     * @param pDst Pointer to the quality index.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiQualityIndexGetBufferHostSize_16u32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of
     * either image are constant numberse.
     * </pre>
     */
    public static int nppiQualityIndex_16u32f_AC4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiQualityIndex_16u32f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer));
    }
    private static native int nppiQualityIndex_16u32f_AC4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image QualityIndex.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oRoiSize \ref roi_specification.
     * @param pDst Pointer to the quality index.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiQualityIndexGetBufferHostSize_32f_AC4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of
     * either image are constant numberse.
     * </pre>
     */
    public static int nppiQualityIndex_32f_AC4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiQualityIndex_32f_AC4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer));
    }
    private static native int nppiQualityIndex_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oRoiSize,
        TypedPointer<Npp32f> pDst,
        TypedPointer<Npp8u> pDeviceBuffer);


    /** @} */
    /**
     * <pre>
     * @name QualityIndexGetBufferHostSize
     * Companion primitives for computing the device buffer size (in bytes) required by the QualityIndex primitives.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiQualityIndex_8u32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiQualityIndexGetBufferHostSize_8u32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiQualityIndexGetBufferHostSize_8u32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiQualityIndexGetBufferHostSize_8u32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiQualityIndex_16u32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiQualityIndexGetBufferHostSize_16u32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiQualityIndexGetBufferHostSize_16u32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiQualityIndexGetBufferHostSize_16u32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiQualityIndex_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiQualityIndexGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiQualityIndexGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiQualityIndexGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiQualityIndex_8u32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiQualityIndexGetBufferHostSize_8u32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiQualityIndexGetBufferHostSize_8u32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiQualityIndexGetBufferHostSize_8u32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiQualityIndex_16u32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiQualityIndexGetBufferHostSize_16u32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiQualityIndexGetBufferHostSize_16u32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiQualityIndexGetBufferHostSize_16u32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiQualityIndex_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiQualityIndexGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiQualityIndexGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiQualityIndexGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiQualityIndex_8u32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiQualityIndexGetBufferHostSize_8u32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiQualityIndexGetBufferHostSize_8u32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiQualityIndexGetBufferHostSize_8u32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiQualityIndex_16u32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiQualityIndexGetBufferHostSize_16u32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiQualityIndexGetBufferHostSize_16u32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiQualityIndexGetBufferHostSize_16u32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size (in bytes) for \ref nppiQualityIndex_32f_AC4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiQualityIndexGetBufferHostSize_32f_AC4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiQualityIndexGetBufferHostSize_32f_AC4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiQualityIndexGetBufferHostSize_32f_AC4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /** @}  */
    /** @} image_quality_index */
    /**
     * <pre>
     * @defgroup image_maximum_error MaximumError
     * Primitives for computing the maximum error between two images.
     * Given two images \f$pSrc1\f$ and \f$pSrc2\f$ both with width \f$W\f$ and height \f$H\f$,
     * the maximum error is defined as the largest absolute difference between pixels of two images.
     * If the image is in complex format, the absolute value of the complex number is provided.
     *
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One-channel 8-bit unsigned image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 8-bit signed image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_8s_C1R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_8s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_8s_C1RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_16u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_16s_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_16s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_16s_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed complex image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_16sc_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_16sc_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_16sc_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit unsigned image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_32u_C1R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32u_C1RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit signed image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit signed complex image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_32sc_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32sc_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32sc_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumError_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point complex image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumError_32fc_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32fc_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32fc_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 64-bit floating point image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumError_64f_C1R(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_64f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_64f_C1RNative(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 8-bit unsigned image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_8u_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_8u_C2R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_8u_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_8u_C2RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 8-bit signed image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_8u_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_8s_C2R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_8s_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_8s_C2RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 16-bit unsigned image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16u_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_16u_C2R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_16u_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_16u_C2RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 16-bit signed image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_16s_C2R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_16s_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_16s_C2RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 16-bit signed complex image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_16sc_C2R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_16sc_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_16sc_C2RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit unsigned image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16u_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_32u_C2R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32u_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32u_C2RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit signed image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_32s_C2R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32s_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32s_C2RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit signed complex image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_32sc_C2R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32sc_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32sc_C2RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit floating point image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumError_32f_C2R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32f_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32f_C2RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit floating point complex image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumError_32fc_C2R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32fc_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32fc_C2RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 64-bit floating point image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumError_64f_C2R(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_64f_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_64f_C2RNative(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit signed image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_8s_C3R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_8s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_8s_C3RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_16u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_16s_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_16s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_16s_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed complex image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_16sc_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_16sc_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_16sc_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit unsigned image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_32u_C3R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32u_C3RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit signed image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_32s_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32s_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit signed complex image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_32sc_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32sc_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32sc_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumError_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point complex image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumError_32fc_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32fc_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32fc_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 64-bit floating point image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumError_64f_C3R(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_64f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_64f_C3RNative(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_8u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit signed image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_8u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_8s_C4R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_8s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_8s_C4RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_16u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_16s_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_16s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_16s_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed complex image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_16sc_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_16sc_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_16sc_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit unsigned image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_32u_C4R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32u_C4RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit signed image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_32s_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32s_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit signed complex image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumError_32sc_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32sc_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32sc_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumError_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point complex image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumError_32fc_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_32fc_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_32fc_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 64-bit floating point image Maximum_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumError_64f_C4R(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumError_64f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumError_64f_C4RNative(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /** @} */
    /**
     * <pre>
     * @name NormDiffInfGetBufferHostSize
     * Companion primitives for computing the device buffer size (in bytes) required by the NormDiff_Inf primitives.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_8s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_8s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_8s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_8s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_16u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_16sc_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_16sc_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_16sc_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_16sc_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32sc_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32sc_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32sc_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32sc_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32fc_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32fc_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32fc_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32fc_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_64f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_64f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_64f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_64f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_8u_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_8u_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_8u_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_8u_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_8s_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_8s_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_8s_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_8s_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_16u_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_16u_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_16u_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_16u_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_16s_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_16s_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_16s_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_16s_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_16sc_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_16sc_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_16sc_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_16sc_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32u_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32u_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32u_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32u_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32s_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32s_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32s_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32s_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32sc_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32sc_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32sc_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32sc_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32f_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32f_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32f_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32f_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32fc_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32fc_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32fc_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32fc_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_64f_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_64f_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_64f_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_64f_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_8u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_8s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_8s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_8s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_8s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_16s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_16sc_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_16sc_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_16sc_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_16sc_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32sc_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32sc_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32sc_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32sc_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32fc_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32fc_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32fc_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32fc_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_64f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_64f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_64f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_64f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_8s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_8s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_8s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_8s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_16sc_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_16sc_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_16sc_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_16sc_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32sc_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32sc_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32sc_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32sc_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_32fc_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_32fc_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_32fc_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_32fc_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumError_64f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumErrorGetBufferHostSize_64f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumErrorGetBufferHostSize_64f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumErrorGetBufferHostSize_64f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /** @} */
    /**
     * <pre>
     * @defgroup image_average_error AverageError
     * Primitives for computing the average error between two images.
     * Given two images \f$pSrc1\f$ and \f$pSrc2\f$ both with width \f$W\f$ and height \f$H\f$,
     * the average error is defined as:
     * \f[Average Error = \frac{1}{W\cdot H\cdot N}\sum_{n=0}^{N-1}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}\left|pSrc1(j,i) - pSrc2(j,i)\right|\f]
     * where N stands for the number of channels. If the image is in complex format, the absolute value is used for computation.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One-channel 8-bit unsigned image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 8-bit signed image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_8s_C1R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_8s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_8s_C1RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_16u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_16s_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_16s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_16s_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed complex image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_16sc_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_16sc_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_16sc_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit unsigned image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_32u_C1R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32u_C1RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit signed image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit signed complex image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_32sc_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32sc_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32sc_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageError_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point complex image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageError_32fc_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32fc_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32fc_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 64-bit floating point image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageError_64f_C1R(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_64f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_64f_C1RNative(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 8-bit unsigned image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_8u_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_8u_C2R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_8u_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_8u_C2RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 8-bit signed image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_8u_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_8s_C2R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_8s_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_8s_C2RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 16-bit unsigned image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16u_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_16u_C2R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_16u_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_16u_C2RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 16-bit signed image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_16s_C2R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_16s_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_16s_C2RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 16-bit signed complex image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_16sc_C2R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_16sc_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_16sc_C2RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit unsigned image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16u_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_32u_C2R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32u_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32u_C2RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit signed image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_32s_C2R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32s_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32s_C2RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit signed complex image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_32sc_C2R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32sc_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32sc_C2RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit floating point image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageError_32f_C2R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32f_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32f_C2RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit floating point complex image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageError_32fc_C2R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32fc_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32fc_C2RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 64-bit floating point image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageError_64f_C2R(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_64f_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_64f_C2RNative(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit signed image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_8s_C3R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_8s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_8s_C3RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_16u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_16s_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_16s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_16s_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed complex image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_16sc_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_16sc_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_16sc_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit unsigned image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_32u_C3R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32u_C3RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit signed image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_32s_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32s_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit signed complex image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_32sc_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32sc_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32sc_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageError_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point complex image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageError_32fc_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32fc_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32fc_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 64-bit floating point image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageError_64f_C3R(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_64f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_64f_C3RNative(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_8u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit signed image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_8u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_8s_C4R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_8s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_8s_C4RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_16u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_16s_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_16s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_16s_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed complex image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_16sc_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_16sc_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_16sc_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit unsigned image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_32u_C4R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32u_C4RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit signed image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_32s_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32s_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit signed complex image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageError_32sc_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32sc_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32sc_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageError_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point complex image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageError_32fc_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_32fc_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_32fc_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 64-bit floating point image Average_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageError_64f_C4R(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageError_64f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageError_64f_C4RNative(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /** @} */
    /**
     * <pre>
     * @name NormDiffInfGetBufferHostSize
     * Companion primitives for computing the device buffer size (in bytes) required by the NormDiff_Inf primitives.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_8s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_8s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_8s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_8s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_16u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_16sc_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_16sc_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_16sc_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_16sc_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32sc_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32sc_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32sc_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32sc_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32fc_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32fc_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32fc_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32fc_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_64f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_64f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_64f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_64f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_8u_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_8u_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_8u_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_8u_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_8s_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_8s_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_8s_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_8s_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_16u_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_16u_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_16u_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_16u_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_16s_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_16s_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_16s_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_16s_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_16sc_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_16sc_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_16sc_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_16sc_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32u_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32u_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32u_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32u_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32s_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32s_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32s_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32s_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32sc_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32sc_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32sc_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32sc_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32f_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32f_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32f_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32f_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32fc_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32fc_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32fc_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32fc_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_64f_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_64f_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_64f_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_64f_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_8u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_8s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_8s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_8s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_8s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_16s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_16sc_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_16sc_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_16sc_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_16sc_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32sc_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32sc_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32sc_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32sc_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32fc_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32fc_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32fc_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32fc_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_64f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_64f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_64f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_64f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_8s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_8s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_8s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_8s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_16sc_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_16sc_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_16sc_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_16sc_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32sc_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32sc_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32sc_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32sc_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_32fc_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_32fc_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_32fc_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_32fc_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageError_64f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageErrorGetBufferHostSize_64f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageErrorGetBufferHostSize_64f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageErrorGetBufferHostSize_64f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /** @} */
    /**
     * <pre>
     * @defgroup image_maximum_relative_error MaximumRelativeError
     * Primitives for computing the maximum relative error between two images.
     * Given two images \f$pSrc1\f$ and \f$pSrc2\f$ both with width \f$W\f$ and height \f$H\f$,
     * the maximum relative error is defined as:
     * \f[MaximumRelativeError = max{\frac{\left|pSrc1(j,i) - pSrc2(j,i)\right|}{max(\left|pSrc1(j,i)\right|, \left|pSrc2(j,i)\right|)}}\f]
     * If the image is in complex format, the absolute value is used for computation.
     * For multiple channles, the maximum relative error of all the channles is returned.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One-channel 8-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 8-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_8s_C1R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_8s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_8s_C1RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_16u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_16s_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_16s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_16s_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_16sc_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_16sc_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_16sc_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32u_C1R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32u_C1RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit signed complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32sc_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32sc_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32sc_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32fc_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32fc_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32fc_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 64-bit floating point image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumRelativeError_64f_C1R(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_64f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_64f_C1RNative(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 8-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_8u_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_8u_C2R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_8u_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_8u_C2RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 8-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_8u_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_8s_C2R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_8s_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_8s_C2RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 16-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16u_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_16u_C2R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_16u_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_16u_C2RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 16-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_16s_C2R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_16s_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_16s_C2RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 16-bit signed complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_16sc_C2R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_16sc_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_16sc_C2RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16u_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32u_C2R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32u_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32u_C2RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32s_C2R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32s_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32s_C2RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit signed complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32sc_C2R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32sc_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32sc_C2RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit floating point image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32f_C2R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32f_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32f_C2RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit floating point complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32fc_C2R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32fc_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32fc_C2RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 64-bit floating point image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumRelativeError_64f_C2R(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_64f_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_64f_C2RNative(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_8s_C3R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_8s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_8s_C3RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_16u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_16s_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_16s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_16s_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_16sc_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_16sc_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_16sc_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32u_C3R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32u_C3RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32s_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32s_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit signed complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32sc_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32sc_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32sc_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32fc_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32fc_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32fc_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 64-bit floating point image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumRelativeError_64f_C3R(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_64f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_64f_C3RNative(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_8u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_8u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_8s_C4R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_8s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_8s_C4RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_16u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_16s_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_16s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_16s_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_16sc_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_16sc_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_16sc_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32u_C4R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32u_C4RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32s_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32s_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit signed complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32sc_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32sc_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32sc_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumRelativeError_32fc_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_32fc_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_32fc_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 64-bit floating point image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiMaximumRelativeError_64f_C4R(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiMaximumRelativeError_64f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiMaximumRelativeError_64f_C4RNative(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /** @} */
    /**
     * <pre>
     * @name NormDiffInfGetBufferHostSize
     * Companion primitives for computing the device buffer size (in bytes) required by the NormDiff_Inf primitives.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_8s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_8s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_8s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_8s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_16u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_16sc_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_16sc_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_16sc_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_16sc_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32sc_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32sc_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32sc_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32sc_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32fc_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32fc_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32fc_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32fc_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_64f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_64f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_64f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_64f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_8u_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_8u_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_8u_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_8u_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_8s_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_8s_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_8s_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_8s_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_16u_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_16u_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_16u_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_16u_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_16s_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_16s_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_16s_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_16s_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_16sc_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_16sc_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_16sc_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_16sc_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32u_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32u_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32u_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32u_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32s_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32s_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32s_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32s_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32sc_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32sc_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32sc_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32sc_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32f_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32f_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32f_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32f_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32fc_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32fc_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32fc_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32fc_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_64f_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_64f_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_64f_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_64f_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_8u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_8s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_8s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_8s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_8s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_16s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_16sc_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_16sc_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_16sc_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_16sc_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32sc_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32sc_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32sc_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32sc_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32fc_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32fc_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32fc_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32fc_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_64f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_64f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_64f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_64f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_8s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_8s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_8s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_8s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_16sc_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_16sc_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_16sc_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_16sc_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32sc_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32sc_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32sc_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32sc_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_32fc_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_32fc_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_32fc_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_32fc_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiMaximumRelativeError_64f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiMaximumRelativeErrorGetBufferHostSize_64f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiMaximumRelativeErrorGetBufferHostSize_64f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiMaximumRelativeErrorGetBufferHostSize_64f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /** @} */
    /**
     * <pre>
     * @defgroup image_average_relative_error AverageRelativeError
     * Primitives for computing the average relative error between two images.
     * Given two images \f$pSrc1\f$ and \f$pSrc2\f$ both with width \f$W\f$ and height \f$H\f$,
     * the maximum relative error is defined as:
     * \f[AverageRelativeError = \frac{1}{W\cdot H\cdot N}\sum_{n=0}^{N-1}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}\frac{\left|pSrc1(j,i) - pSrc2(j,i)\right|}{max(\left|pSrc1(j,i)\right|, \left|pSrc2(j,i)\right|)}\f]
     * where N is the number of channels.
     * If the image is in complex format, the absolute value is used for computation.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * One-channel 8-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_8u_C1R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_8u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_8u_C1RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 8-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_8u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_8s_C1R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_8s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_8s_C1RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_16u_C1R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_16u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_16u_C1RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_16s_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_16s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_16s_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 16-bit signed complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_16sc_C1R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_16sc_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_16sc_C1RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16u_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_32u_C1R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32u_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32u_C1RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_32s_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32s_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32s_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit signed complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_32sc_C1R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32sc_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32sc_C1RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageRelativeError_32f_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32f_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 32-bit floating point complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageRelativeError_32fc_C1R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32fc_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32fc_C1RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * One-channel 64-bit floating point image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageRelativeError_64f_C1R(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_64f_C1RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_64f_C1RNative(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 8-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_8u_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_8u_C2R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_8u_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_8u_C2RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 8-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_8u_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_8s_C2R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_8s_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_8s_C2RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 16-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16u_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_16u_C2R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_16u_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_16u_C2RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 16-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_16s_C2R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_16s_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_16s_C2RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 16-bit signed complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_16sc_C2R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_16sc_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_16sc_C2RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16u_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_32u_C2R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32u_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32u_C2RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_32s_C2R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32s_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32s_C2RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit signed complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_32sc_C2R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32sc_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32sc_C2RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit floating point image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageRelativeError_32f_C2R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32f_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32f_C2RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 32-bit floating point complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageRelativeError_32fc_C2R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32fc_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32fc_C2RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Two-channel 64-bit floating point image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageRelativeError_64f_C2R(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_64f_C2RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_64f_C2RNative(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_8u_C3R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_8u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_8u_C3RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 8-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_8u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_8s_C3R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_8s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_8s_C3RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_16u_C3R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_16u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_16u_C3RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_16s_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_16s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_16s_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 16-bit signed complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_16sc_C3R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_16sc_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_16sc_C3RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16u_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_32u_C3R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32u_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32u_C3RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_32s_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32s_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32s_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit signed complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_32sc_C3R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32sc_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32sc_C3RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageRelativeError_32f_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32f_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 32-bit floating point complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageRelativeError_32fc_C3R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32fc_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32fc_C3RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Three-channel 64-bit floating point image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageRelativeError_64f_C3R(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_64f_C3RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_64f_C3RNative(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_8u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_8u_C4R(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_8u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_8u_C4RNative(
        TypedPointer<Npp8u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 8-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_8u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_8s_C4R(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_8s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_8s_C4RNative(
        TypedPointer<Npp8s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp8s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_16u_C4R(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_16u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_16u_C4RNative(
        TypedPointer<Npp16u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_16s_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_16s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_16s_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 16-bit signed complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_16sc_C4R(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_16sc_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_16sc_C4RNative(
        TypedPointer<Npp16s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp16s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit unsigned image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16u_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_32u_C4R(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32u_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32u_C4RNative(
        TypedPointer<Npp32u> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32u> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit signed image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_32s_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32s_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32s_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit signed complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeError_32sc_C4R(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32sc_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32sc_C4RNative(
        TypedPointer<Npp32s> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32s> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageRelativeError_32f_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32f_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 32-bit floating point complex image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error (absolute value).
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageRelativeError_32fc_C4R(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_32fc_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_32fc_C4RNative(
        TypedPointer<Npp32f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp32f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /**
     * <pre>
     * Four-channel 64-bit floating point image MaximumRelative_Error.
     *
     * @param pSrc1 \ref source_image_pointer.
     * @param nSrc1Step \ref source_image_line_step.
     * @param pSrc2 \ref source_image_pointer.
     * @param nSrc2Step \ref source_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @param pError Pointer to the computed error.
     * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
     * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an
     * invalid floating-point image is specified.
     * </pre>
     */
    public static int nppiAverageRelativeError_64f_C4R(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer)
    {
        return JNpp.checkResult(nppiAverageRelativeError_64f_C4RNative(pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer));
    }
    private static native int nppiAverageRelativeError_64f_C4RNative(
        TypedPointer<Npp64f> pSrc1,
        int nSrc1Step,
        TypedPointer<Npp64f> pSrc2,
        int nSrc2Step,
        NppiSize oSizeROI,
        TypedPointer<Npp64f> pError,
        TypedPointer<Npp8u> pDeviceBuffer);


    /** @} */
    /**
     * <pre>
     * @name NormDiffInfGetBufferHostSize
     * Companion primitives for computing the device buffer size (in bytes) required by the NormDiff_Inf primitives.
     * @{
     * </pre>
     */
    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_8u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_8u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_8u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_8u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_8s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_8s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_8s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_8s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_16u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_16u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_16u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_16u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_16s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_16s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_16s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_16s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_16sc_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_16sc_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_16sc_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_16sc_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32u_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32u_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32u_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32u_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32s_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32s_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32s_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32s_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32sc_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32sc_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32sc_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32sc_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32fc_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32fc_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32fc_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32fc_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_64f_C1R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_64f_C1R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_64f_C1RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_64f_C1RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_8u_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_8u_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_8u_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_8u_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_8s_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_8s_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_8s_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_8s_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_16u_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_16u_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_16u_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_16u_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_16s_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_16s_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_16s_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_16s_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_16sc_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_16sc_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_16sc_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_16sc_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32u_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32u_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32u_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32u_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32s_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32s_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32s_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32s_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32sc_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32sc_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32sc_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32sc_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32f_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32f_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32f_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32f_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32fc_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32fc_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32fc_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32fc_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_64f_C2R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_64f_C2R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_64f_C2RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_64f_C2RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_8u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_8u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_8u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_8u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_8s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_8s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_8s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_8s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_16u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_16u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_16u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_16u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_16s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_16s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_16s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_16s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_16sc_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_16sc_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_16sc_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_16sc_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32u_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32u_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32u_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32u_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32s_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32s_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32s_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32s_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32sc_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32sc_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32sc_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32sc_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32fc_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32fc_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32fc_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32fc_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_64f_C3R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_64f_C3R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_64f_C3RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_64f_C3RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_8u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_8u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_8u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_8u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_8s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_8s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_8s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_8s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_16u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_16u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_16u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_16u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_16s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_16s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_16s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_16s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_16sc_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_16sc_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_16sc_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_16sc_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32u_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32u_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32u_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32u_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32s_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32s_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32s_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32s_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32sc_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32sc_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32sc_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32sc_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_32fc_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_32fc_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_32fc_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_32fc_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    /**
     * <pre>
     * Buffer size for \ref nppiAverageRelativeError_64f_C4R.
     * @param oSizeROI \ref roi_specification.
     * @param hpBufferSize Required buffer size. Important: hpBufferSize is a
     *        <em>host pointer.</em> \ref general_scratch_buffer.
     * @return
     *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
     * </pre>
     */
    public static int nppiAverageRelativeErrorGetBufferHostSize_64f_C4R(
        NppiSize oSizeROI,
        int[] hpBufferSize)/** host pointer */
    {
        return JNpp.checkResult(nppiAverageRelativeErrorGetBufferHostSize_64f_C4RNative(oSizeROI, hpBufferSize));
    }
    private static native int nppiAverageRelativeErrorGetBufferHostSize_64f_C4RNative(
        NppiSize oSizeROI,
        int[] hpBufferSize);/** host pointer */


    public static int nppiMagnitude_32fc32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMagnitude_32fc32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMagnitude_32fc32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * 32-bit floating point complex to 32-bit floating point squared magnitude.
     *
     * Converts complex-number pixel image to single channel image computing
     * the result pixels as the squared magnitude of the complex values.
     *
     * The squared magnitude is an itermediate result of magnitude computation and
     * can thus be computed faster than actual magnitude. If magnitudes are required
     * for sorting/comparing only, using this function instead of nppiMagnitude_32fc32f_C1R
     * can be a worthwhile performance optimization.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiMagnitudeSqr_32fc32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiMagnitudeSqr_32fc32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oSizeROI));
    }
    private static native int nppiMagnitudeSqr_32fc32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oSizeROI);


    /**
     * <pre>
     * @param oSrcROI Region of interest in the source image.
     * @param pDstRect User supplied host memory pointer to an NppiRect structure that will be filled in by this function with the region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiGetResizeRect(
        NppiRect oSrcROI,
        NppiRect[] pDstRect,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiGetResizeRectNative(oSrcROI, pDstRect, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiGetResizeRectNative(
        NppiRect oSrcROI,
        NppiRect[] pDstRect,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 1 channel 8-bit unsigned image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_8u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 8-bit unsigned image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_8u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 8-bit unsigned image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_8u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 8-bit unsigned image resize not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of interpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_8u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar image resize.
     *
     * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_8u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 8-bit unsigned planar image resize.
     *
     * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_8u_P4R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_8u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_8u_P4RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 1 channel 16-bit unsigned image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_16u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 16-bit unsigned image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_16u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 16-bit unsigned image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_16u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 16-bit unsigned image resize not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of interpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_16u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 16-bit unsigned planar image resize.
     *
     * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_16u_P3R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_16u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_16u_P3RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    public static int nppiResizeSqrPixel_16u_P4R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_16u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_16u_P4RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    public static int nppiResizeSqrPixel_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_16s_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 16-bit signed image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_16s_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 16-bit signed image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_16s_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 16-bit signed image resize not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of interpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_16s_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    public static int nppiResizeSqrPixel_16s_P3R(
        TypedPointer<Npp16s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_16s_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_16s_P3RNative(
        TypedPointer<Npp16s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 16-bit signed planar image resize.
     *
     * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_16s_P4R(
        TypedPointer<Npp16s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_16s_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_16s_P4RNative(
        TypedPointer<Npp16s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 1 channel 32-bit floating point image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_32f_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 32-bit floating point image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_32f_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 32-bit floating point image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_32f_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 32-bit floating point image resize not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of interpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_32f_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 32-bit floating point planar image resize.
     *
     * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_32f_P3R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_32f_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_32f_P3RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    public static int nppiResizeSqrPixel_32f_P4R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_32f_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_32f_P4RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 1 channel 64-bit floating point image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_64f_C1R(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_64f_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_64f_C1RNative(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    public static int nppiResizeSqrPixel_64f_C3R(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_64f_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_64f_C3RNative(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 64-bit floating point image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_64f_C4R(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_64f_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_64f_C4RNative(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 64-bit floating point image resize not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of interpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_64f_AC4R(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_64f_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_64f_AC4RNative(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 64-bit floating point planar image resize.
     *
     * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_64f_P3R(
        TypedPointer<Npp64f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_64f_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_64f_P3RNative(
        TypedPointer<Npp64f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 64-bit floating point planar image resize.
     *
     * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param nXShift Source pixel shift in x-direction.
     * @param nYShift Source pixel shift in y-direction.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResizeSqrPixel_64f_P4R(
        TypedPointer<Npp64f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResizeSqrPixel_64f_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation));
    }
    private static native int nppiResizeSqrPixel_64f_P4RNative(
        TypedPointer<Npp64f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nXFactor,
        double nYFactor,
        double nXShift,
        double nYShift,
        int eInterpolation);


    public static int nppiResize_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_8u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 8-bit unsigned image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param dstROISize Size in pixels of the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResize_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_8u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    public static int nppiResize_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_8u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 8-bit unsigned image resize not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param dstROISize Size in pixels of the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param eInterpolation The type of interpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResize_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_8u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar image resize.
     *
     * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nDstStep \ref destination_image_line_step.
     * @param dstROISize Size in pixels of the destination image
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResize_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_8u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 8-bit unsigned planar image resize.
     *
     * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nDstStep \ref destination_image_line_step.
     * @param dstROISize Size in pixels of the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResize_8u_P4R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_8u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_8u_P4RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 1 channel 16-bit unsigned image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param dstROISize Size in pixels of the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResize_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_16u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 16-bit unsigned image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param dstROISize Size in pixels of the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResize_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_16u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 16-bit unsigned image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param dstROISize Size in pixels of the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResize_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_16u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 16-bit unsigned image resize not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param dstROISize Size in pixels of the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param eInterpolation The type of interpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResize_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_16u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 16-bit unsigned planar image resize.
     *
     * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nDstStep \ref destination_image_line_step.
     * @param dstROISize Size in pixels of the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResize_16u_P3R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_16u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_16u_P3RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 16-bit unsigned planar image resize.
     *
     * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nDstStep \ref destination_image_line_step.
     * @param dstROISize Size in pixels of the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResize_16u_P4R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_16u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_16u_P4RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 1 channel 32-bit floating point image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param dstROISize Size in pixels of the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResize_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_32f_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 32-bit floating point image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param dstROISize Size in pixels of the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResize_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_32f_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 32-bit floating point image resize.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param dstROISize Size in pixels of the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResize_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_32f_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 32-bit floating point image resize not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param dstROISize Size in pixels of the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param eInterpolation The type of interpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResize_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_32f_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 32-bit floating point planar image resize.
     *
     * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nDstStep \ref destination_image_line_step.
     * @param dstROISize Size in pixels of the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResize_32f_P3R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_32f_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_32f_P3RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 32-bit floating point planar image resize.
     *
     * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
     * @param nDstStep \ref destination_image_line_step.
     * @param dstROISize Size in pixels of the destination image.
     * @param nXFactor Factor by which x dimension is changed.
     * @param nYFactor Factor by which y dimension is changed.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
     * </pre>
     */
    public static int nppiResize_32f_P4R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiResize_32f_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation));
    }
    private static native int nppiResize_32f_P4RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiSize dstROISize,
        double nXFactor,
        double nYFactor,
        int eInterpolation);


    /**
     * <pre>
     * 1 channel 8-bit unsigned image remap.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_8u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    public static int nppiRemap_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_8u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 8-bit unsigned image remap.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_8u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 8-bit unsigned image remap not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of interpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_8u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 8-bit unsigned planar image remap.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_8u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 8-bit unsigned planar image remap.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_8u_P4R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_8u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_8u_P4RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 1 channel 16-bit unsigned image remap.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_16u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    public static int nppiRemap_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_16u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 16-bit unsigned image remap.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_16u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    public static int nppiRemap_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_16u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 16-bit unsigned planar image remap.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_16u_P3R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_16u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_16u_P3RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 16-bit unsigned planar image remap.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_16u_P4R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_16u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_16u_P4RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 1 channel 16-bit signed image remap.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_16s_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 16-bit signed image remap.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_16s_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 16-bit signed image remap.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_16s_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 16-bit signed image remap not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of interpolation to perform resampling
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_16s_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 16-bit signed planar image remap.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_16s_P3R(
        TypedPointer<Npp16s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16s>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_16s_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_16s_P3RNative(
        TypedPointer<Npp16s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16s>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 16-bit signed planar image remap.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_16s_P4R(
        TypedPointer<Npp16s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16s>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_16s_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_16s_P4RNative(
        TypedPointer<Npp16s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp16s>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 1 channel 32-bit floating point image remap.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_32f_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    public static int nppiRemap_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_32f_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 32-bit floating point image remap.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_32f_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 32-bit floating point image remap not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of interpolation to perform resampling
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_32f_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 32-bit floating point planar image remap.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_32f_P3R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_32f_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_32f_P3RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    public static int nppiRemap_32f_P4R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_32f_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_32f_P4RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pXMap,
        int nXMapStep,
        TypedPointer<Npp32f> pYMap,
        int nYMapStep,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 1 channel 64-bit floating point image remap.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_64f_C1R(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pXMap,
        int nXMapStep,
        TypedPointer<Npp64f> pYMap,
        int nYMapStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_64f_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_64f_C1RNative(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pXMap,
        int nXMapStep,
        TypedPointer<Npp64f> pYMap,
        int nYMapStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    public static int nppiRemap_64f_C3R(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pXMap,
        int nXMapStep,
        TypedPointer<Npp64f> pYMap,
        int nYMapStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_64f_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_64f_C3RNative(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pXMap,
        int nXMapStep,
        TypedPointer<Npp64f> pYMap,
        int nYMapStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 64-bit floating point image remap.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_64f_C4R(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pXMap,
        int nXMapStep,
        TypedPointer<Npp64f> pYMap,
        int nYMapStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_64f_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_64f_C4RNative(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pXMap,
        int nXMapStep,
        TypedPointer<Npp64f> pYMap,
        int nYMapStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 64-bit floating point image remap not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of interpolation to perform resampling
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_64f_AC4R(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pXMap,
        int nXMapStep,
        TypedPointer<Npp64f> pYMap,
        int nYMapStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_64f_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_64f_AC4RNative(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pXMap,
        int nXMapStep,
        TypedPointer<Npp64f> pYMap,
        int nYMapStep,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 64-bit floating point planar image remap.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_64f_P3R(
        TypedPointer<Npp64f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pXMap,
        int nXMapStep,
        TypedPointer<Npp64f> pYMap,
        int nYMapStep,
        TypedPointer<Npp64f>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_64f_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_64f_P3RNative(
        TypedPointer<Npp64f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pXMap,
        int nXMapStep,
        TypedPointer<Npp64f> pYMap,
        int nYMapStep,
        TypedPointer<Npp64f>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 64-bit floating point planar image remap.
     *
     * @param pSrc \ref source_planar_image_pointer_array.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image.
     * @param oSrcROI Region of interest in the source image.
     * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image.
     * @param nXMapStep pXMap image array line step in bytes.
     * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image.
     * @param nYMapStep pYMap image array line step in bytes.
     * @param pDst \ref destination_planar_image_pointer_array.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstSizeROI Region of interest size in the destination image.
     * @param eInterpolation The type of eInterpolation to perform resampling.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
     * </pre>
     */
    public static int nppiRemap_64f_P4R(
        TypedPointer<Npp64f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pXMap,
        int nXMapStep,
        TypedPointer<Npp64f> pYMap,
        int nYMapStep,
        TypedPointer<Npp64f>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRemap_64f_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation));
    }
    private static native int nppiRemap_64f_P4RNative(
        TypedPointer<Npp64f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pXMap,
        int nXMapStep,
        TypedPointer<Npp64f> pYMap,
        int nYMapStep,
        TypedPointer<Npp64f>[] pDst,
        int nDstStep,
        NppiSize oDstSizeROI,
        int eInterpolation);


    public static int nppiGetRotateQuad(
        NppiRect oSrcROI,
        double[][] aQuad,
        double nAngle,
        double nShiftX,
        double nShiftY)
    {
        return JNpp.checkResult(nppiGetRotateQuadNative(oSrcROI, aQuad, nAngle, nShiftX, nShiftY));
    }
    private static native int nppiGetRotateQuadNative(
        NppiRect oSrcROI,
        double[][] aQuad,
        double nAngle,
        double nShiftX,
        double nShiftY);


    public static int nppiGetRotateBound(
        NppiRect oSrcROI,
        double[][] aBoundingBox,
        double nAngle,
        double nShiftX,
        double nShiftY)
    {
        return JNpp.checkResult(nppiGetRotateBoundNative(oSrcROI, aBoundingBox, nAngle, nShiftX, nShiftY));
    }
    private static native int nppiGetRotateBoundNative(
        NppiRect oSrcROI,
        double[][] aBoundingBox,
        double nAngle,
        double nShiftX,
        double nShiftY);


    public static int nppiRotate_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRotate_8u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation));
    }
    private static native int nppiRotate_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation);


    public static int nppiRotate_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRotate_8u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation));
    }
    private static native int nppiRotate_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation);


    public static int nppiRotate_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRotate_8u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation));
    }
    private static native int nppiRotate_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 8-bit unsigned image rotate ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nAngle The angle of rotation in degrees.
     * @param nShiftX Shift along horizontal axis
     * @param nShiftY Shift along vertical axis
     * @param eInterpolation The type of interpolation to perform resampling
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
     * </pre>
     */
    public static int nppiRotate_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRotate_8u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation));
    }
    private static native int nppiRotate_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation);


    /**
     * <pre>
     * 16-bit unsigned image rotate.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nAngle The angle of rotation in degrees.
     * @param nShiftX Shift along horizontal axis
     * @param nShiftY Shift along vertical axis
     * @param eInterpolation The type of interpolation to perform resampling
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
     * </pre>
     */
    public static int nppiRotate_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRotate_16u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation));
    }
    private static native int nppiRotate_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation);


    public static int nppiRotate_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRotate_16u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation));
    }
    private static native int nppiRotate_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 16-bit unsigned image rotate.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nAngle The angle of rotation in degrees.
     * @param nShiftX Shift along horizontal axis
     * @param nShiftY Shift along vertical axis
     * @param eInterpolation The type of interpolation to perform resampling
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
     * </pre>
     */
    public static int nppiRotate_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRotate_16u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation));
    }
    private static native int nppiRotate_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 16-bit unsigned image rotate ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nAngle The angle of rotation in degrees.
     * @param nShiftX Shift along horizontal axis
     * @param nShiftY Shift along vertical axis
     * @param eInterpolation The type of interpolation to perform resampling
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
     * </pre>
     */
    public static int nppiRotate_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRotate_16u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation));
    }
    private static native int nppiRotate_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation);


    /**
     * <pre>
     * 32-bit float image rotate.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nAngle The angle of rotation in degrees.
     * @param nShiftX Shift along horizontal axis
     * @param nShiftY Shift along vertical axis
     * @param eInterpolation The type of interpolation to perform resampling
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
     * </pre>
     */
    public static int nppiRotate_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRotate_32f_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation));
    }
    private static native int nppiRotate_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation);


    /**
     * <pre>
     * 3 channel 32-bit float image rotate.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nAngle The angle of rotation in degrees.
     * @param nShiftX Shift along horizontal axis
     * @param nShiftY Shift along vertical axis
     * @param eInterpolation The type of interpolation to perform resampling
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
     * </pre>
     */
    public static int nppiRotate_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRotate_32f_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation));
    }
    private static native int nppiRotate_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation);


    /**
     * <pre>
     * 4 channel 32-bit float image rotate.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcSize Size in pixels of the source image
     * @param oSrcROI Region of interest in the source image.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Region of interest in the destination image.
     * @param nAngle The angle of rotation in degrees.
     * @param nShiftX Shift along horizontal axis
     * @param nShiftY Shift along vertical axis
     * @param eInterpolation The type of interpolation to perform resampling
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
     * </pre>
     */
    public static int nppiRotate_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRotate_32f_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation));
    }
    private static native int nppiRotate_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation);


    public static int nppiRotate_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiRotate_32f_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation));
    }
    private static native int nppiRotate_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double nAngle,
        double nShiftX,
        double nShiftY,
        int eInterpolation);


    public static int nppiMirror_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    public static int nppiMirror_8u_C1IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_8u_C1IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_8u_C1IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 3 channel 8-bit unsigned image mirror.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_8u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 3 channel 8-bit unsigned in place image mirror.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_8u_C3IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_8u_C3IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_8u_C3IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    public static int nppiMirror_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_8u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 4 channel 8-bit unsigned in place image mirror.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_8u_C4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_8u_C4IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_8u_C4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 4 channel 8-bit unsigned image mirror not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep Distance in bytes between starts of consecutive lines of the
     *        destination image.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_8u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    public static int nppiMirror_8u_AC4IR(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_8u_AC4IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_8u_AC4IRNative(
        TypedPointer<Npp8u> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 1 channel 16-bit unsigned image mirror.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_16u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 1 channel 16-bit unsigned in place image mirror.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_16u_C1IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_16u_C1IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_16u_C1IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 3 channel 16-bit unsigned image mirror.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_16u_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    public static int nppiMirror_16u_C3IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_16u_C3IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_16u_C3IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 4 channel 16-bit unsigned image mirror.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep Distance in bytes between starts of consecutive lines of the
     *        destination image.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_16u_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    public static int nppiMirror_16u_C4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_16u_C4IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_16u_C4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    public static int nppiMirror_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_16u_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 4 channel 16-bit unsigned in place image mirror not affecting alpha.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_16u_AC4IR(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_16u_AC4IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_16u_AC4IRNative(
        TypedPointer<Npp16u> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 1 channel 16-bit signed image mirror.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_16s_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_16s_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 1 channel 16-bit signed in place image mirror.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_16s_C1IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_16s_C1IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_16s_C1IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 3 channel 16-bit signed image mirror.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_16s_C3R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_16s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_16s_C3RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    public static int nppiMirror_16s_C3IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_16s_C3IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_16s_C3IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    public static int nppiMirror_16s_C4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_16s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_16s_C4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    public static int nppiMirror_16s_C4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_16s_C4IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_16s_C4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 4 channel 16-bit signed image mirror not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep Distance in bytes between starts of consecutive lines of the
     *        destination image.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_16s_AC4R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_16s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_16s_AC4RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 4 channel 16-bit signed in place image mirror not affecting alpha.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_16s_AC4IR(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_16s_AC4IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_16s_AC4IRNative(
        TypedPointer<Npp16s> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    public static int nppiMirror_32s_C1R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_32s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_32s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 1 channel 32-bit signed in place image mirror.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_32s_C1IR(
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_32s_C1IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_32s_C1IRNative(
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    public static int nppiMirror_32s_C3R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_32s_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_32s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 3 channel 32-bit signed in place image mirror.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_32s_C3IR(
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_32s_C3IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_32s_C3IRNative(
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 4 channel 32-bit image mirror.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep Distance in bytes between starts of consecutive lines of the
     *        destination image.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_32s_C4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_32s_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_32s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 4 channel 32-bit signed in place image mirror.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_32s_C4IR(
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_32s_C4IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_32s_C4IRNative(
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 4 channel 32-bit image mirror not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep Distance in bytes between starts of consecutive lines of the
     *        destination image.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_32s_AC4R(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_32s_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        int nSrcStep,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    public static int nppiMirror_32s_AC4IR(
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_32s_AC4IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_32s_AC4IRNative(
        TypedPointer<Npp32s> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    public static int nppiMirror_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_32f_C1RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 1 channel 32-bit float in place image mirror.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_32f_C1IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_32f_C1IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_32f_C1IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 3 channel 32-bit float image mirror.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_32f_C3RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    public static int nppiMirror_32f_C3IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_32f_C3IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_32f_C3IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    public static int nppiMirror_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_32f_C4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    public static int nppiMirror_32f_C4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_32f_C4IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_32f_C4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 4 channel 32-bit float image mirror not affecting alpha.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep \ref source_image_line_step.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep Distance in bytes between starts of consecutive lines of the
     *        destination image.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_32f_AC4RNative(pSrc, nSrcStep, pDst, nDstStep, oROI, flip));
    }
    private static native int nppiMirror_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        int nSrcStep,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * 4 channel 32-bit float in place image mirror not affecting alpha.
     *
     * @param pSrcDst \ref in_place_image_pointer.
     * @param nSrcDstStep \ref in_place_image_line_step.
     * @param oROI \ref roi_specification.
     * @param flip Specifies the axis about which the image is to be mirrored.
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
     * </pre>
     */
    public static int nppiMirror_32f_AC4IR(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip)
    {
        return JNpp.checkResult(nppiMirror_32f_AC4IRNative(pSrcDst, nSrcDstStep, oROI, flip));
    }
    private static native int nppiMirror_32f_AC4IRNative(
        TypedPointer<Npp32f> pSrcDst,
        int nSrcDstStep,
        NppiSize oROI,
        int flip);


    /**
     * <pre>
     * Computes affine transform coefficients based on source ROI and destination quadrilateral.
     *
     * The function computes the coefficients of an affine transformation that maps the
     * given source ROI (axis aligned rectangle with integer coordinates) to a quadrilateral
     * in the destination image.
     *
     * An affine transform in 2D is fully determined by the mapping of just three vertices.
     * This function's API allows for passing a complete quadrilateral effectively making the
     * prolem overdetermined. What this means in practice is, that for certain quadrilaterals it is
     * not possible to find an affine transform that would map all four corners of the source
     * ROI to the four vertices of that quadrilateral.
     *
     * The function circumvents this problem by only looking at the first three vertices of
     * the destination image quadrilateral to determine the affine transformation's coefficients.
     * If the destination quadrilateral is indeed one that cannot be mapped using an affine
     * transformation the functions informs the user of this situation by returning a
     * ::NPP_AFFINE_QUAD_INCORRECT_WARNING.
     *
     * @param oSrcROI The source ROI. This rectangle needs to be at least one pixel wide and
     *          high. If either width or hight are less than one an ::NPP_RECT_ERROR is returned.
     * @param aQuad The destination quadrilateral.
     * @param aCoeffs The resulting affine transform coefficients.
     * @return
     *  Error codes:
     *         - ::NPP_SIZE_ERROR Indicates an error condition if any image dimension
     *           has zero or negative value
     *         - ::NPP_RECT_ERROR Indicates an error condition if width or height of
     *           the intersection of the oSrcROI and source image is less than or
     *           equal to 1
     *         - ::NPP_COEFF_ERROR Indicates an error condition if coefficient values
     *           are invalid
     *         - ::NPP_AFFINE_QUAD_INCORRECT_WARNING Indicates a warning when quad
     *           does not conform to the transform properties. Fourth vertex is
     *           ignored, internally computed coordinates are used instead
     * </pre>
     */
    public static int nppiGetAffineTransform(
        NppiRect oSrcROI,
        double[][] aQuad,
        double[][] aCoeffs)
    {
        return JNpp.checkResult(nppiGetAffineTransformNative(oSrcROI, aQuad, aCoeffs));
    }
    private static native int nppiGetAffineTransformNative(
        NppiRect oSrcROI,
        double[][] aQuad,
        double[][] aCoeffs);


    public static int nppiGetAffineQuad(
        NppiRect oSrcROI,
        double[][] aQuad,
        double[][] aCoeffs)
    {
        return JNpp.checkResult(nppiGetAffineQuadNative(oSrcROI, aQuad, aCoeffs));
    }
    private static native int nppiGetAffineQuadNative(
        NppiRect oSrcROI,
        double[][] aQuad,
        double[][] aCoeffs);


    public static int nppiGetAffineBound(
        NppiRect oSrcROI,
        double[][] aBound,
        double[][] aCoeffs)
    {
        return JNpp.checkResult(nppiGetAffineBoundNative(oSrcROI, aBound, aCoeffs));
    }
    private static native int nppiGetAffineBoundNative(
        NppiRect oSrcROI,
        double[][] aBound,
        double[][] aCoeffs);


    /**
     * <pre>
     * Single-channel 8-bit unsigned affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_8u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 8-bit unsigned affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_8u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 8-bit unsigned affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_8u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpAffine_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_8u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 8-bit unsigned affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_8u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpAffine_8u_P4R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_8u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_8u_P4RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 16-bit unsigned affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_16u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 16-bit unsigned affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_16u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 16-bit unsigned affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_16u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 16-bit unsigned affine warp, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_16u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 16-bit unsigned affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_16u_P3R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_16u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_16u_P3RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpAffine_16u_P4R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_16u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_16u_P4RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 32-bit signed affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_32s_C1R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_32s_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_32s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 32-bit signed affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_32s_C3R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_32s_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_32s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit signed affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_32s_C4R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_32s_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_32s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit signed affine warp, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_32s_AC4R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_32s_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 32-bit signed affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_32s_P3R(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_32s_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_32s_P3RNative(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 32-bit signed affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_32s_P4R(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_32s_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_32s_P4RNative(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 32-bit floating-point affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_32f_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 32-bit floating-point affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_32f_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpAffine_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_32f_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit floating-point affine warp, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_32f_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 32-bit floating-point affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_32f_P3R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_32f_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_32f_P3RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 32-bit floating-point affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_32f_P4R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_32f_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_32f_P4RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 64-bit floating-point affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_64f_C1R(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_64f_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_64f_C1RNative(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 64-bit floating-point affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_64f_C3R(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_64f_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_64f_C3RNative(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 64-bit floating-point affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_64f_C4R(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_64f_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_64f_C4RNative(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 64-bit floating-point affine warp, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_64f_AC4R(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_64f_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_64f_AC4RNative(
        TypedPointer<Npp64f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 64-bit floating-point affine warp.
     *
     * @param aSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_64f_P3R(
        TypedPointer<Npp64f>[] aSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f>[] aDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_64f_P3RNative(aSrc, oSrcSize, nSrcStep, oSrcROI, aDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_64f_P3RNative(
        TypedPointer<Npp64f>[] aSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f>[] aDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 64-bit floating-point affine warp.
     *
     * @param aSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffine_64f_P4R(
        TypedPointer<Npp64f>[] aSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f>[] aDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffine_64f_P4RNative(aSrc, oSrcSize, nSrcStep, oSrcROI, aDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffine_64f_P4RNative(
        TypedPointer<Npp64f>[] aSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp64f>[] aDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpAffineBack_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_8u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 8-bit unsigned integer backwards affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineBack_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_8u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer backwards affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineBack_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_8u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer backwards affine warp, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineBack_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_8u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 8-bit unsigned integer backwards affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineBack_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_8u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpAffineBack_8u_P4R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_8u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_8u_P4RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 16-bit unsigned integer backwards affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineBack_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_16u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 16-bit unsigned integer backwards affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineBack_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_16u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpAffineBack_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_16u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer backwards affine warp, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineBack_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_16u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpAffineBack_16u_P3R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_16u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_16u_P3RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpAffineBack_16u_P4R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_16u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_16u_P4RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 32-bit signed integer backwards affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineBack_32s_C1R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_32s_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_32s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 32-bit signed integer backwards affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineBack_32s_C3R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_32s_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_32s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit signed integer backwards affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineBack_32s_C4R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_32s_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_32s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpAffineBack_32s_AC4R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_32s_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpAffineBack_32s_P3R(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_32s_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_32s_P3RNative(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 32-bit signed integer backwards affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineBack_32s_P4R(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_32s_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_32s_P4RNative(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 32-bit floating-point backwards affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineBack_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_32f_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 32-bit floating-point backwards affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineBack_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_32f_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit floating-point backwards affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineBack_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_32f_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpAffineBack_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_32f_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 32-bit floating-point backwards affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineBack_32f_P3R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_32f_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_32f_P3RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 32-bit floating-point backwards affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Affine transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineBack_32f_P4R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineBack_32f_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpAffineBack_32f_P4RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 32-bit floating-point quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_8u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 8-bit unsigned integer quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_8u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_8u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    public static int nppiWarpAffineQuad_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_8u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 8-bit unsigned integer quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_8u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 8-bit unsigned integer quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_8u_P4R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_8u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_8u_P4RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 16-bit unsigned integer quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_16u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    public static int nppiWarpAffineQuad_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_16u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    public static int nppiWarpAffineQuad_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_16u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer quad-based affine warp, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_16u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 16-bit unsigned integer quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_16u_P3R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_16u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_16u_P3RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 16-bit unsigned integer quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_16u_P4R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_16u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_16u_P4RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    public static int nppiWarpAffineQuad_32s_C1R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_32s_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_32s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 32-bit signed integer quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_32s_C3R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_32s_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_32s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit signed integer quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_32s_C4R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_32s_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_32s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit signed integer quad-based affine warp, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_32s_AC4R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_32s_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 32-bit signed integer quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_32s_P3R(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_32s_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_32s_P3RNative(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 32-bit signed integer quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_32s_P4R(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_32s_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_32s_P4RNative(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 32-bit floating-point quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_32f_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 32-bit floating-point quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_32f_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit floating-point quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_32f_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit floating-point quad-based affine warp, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_32f_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 32-bit floating-point quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_32f_P3R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_32f_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_32f_P3RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 32-bit floating-point quad-based affine warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
     * </pre>
     */
    public static int nppiWarpAffineQuad_32f_P4R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpAffineQuad_32f_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpAffineQuad_32f_P4RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Calculates perspective transform coefficients given source rectangular ROI
     * and its destination quadrangle projection
     *
     * @param oSrcROI Source ROI
     * @param quad Destination quadrangle
     * @param aCoeffs Perspective transform coefficients
     * @return
     *  Error codes:
     *         - ::NPP_SIZE_ERROR Indicates an error condition if any image dimension
     *           has zero or negative value
     *         - ::NPP_RECT_ERROR Indicates an error condition if width or height of
     *           the intersection of the oSrcROI and source image is less than or
     *           equal to 1
     *         - ::NPP_COEFF_ERROR Indicates an error condition if coefficient values
     *           are invalid
     * </pre>
     */
    public static int nppiGetPerspectiveTransform(
        NppiRect oSrcROI,
        double[][] quad,
        double[][] aCoeffs)
    {
        return JNpp.checkResult(nppiGetPerspectiveTransformNative(oSrcROI, quad, aCoeffs));
    }
    private static native int nppiGetPerspectiveTransformNative(
        NppiRect oSrcROI,
        double[][] quad,
        double[][] aCoeffs);


    /**
     * <pre>
     * Calculates perspective transform projection of given source rectangular
     * ROI
     *
     * @param oSrcROI Source ROI
     * @param quad Destination quadrangle
     * @param aCoeffs Perspective transform coefficients
     * @return
     *  Error codes:
     *         - ::NPP_SIZE_ERROR Indicates an error condition if any image dimension
     *           has zero or negative value
     *         - ::NPP_RECT_ERROR Indicates an error condition if width or height of
     *           the intersection of the oSrcROI and source image is less than or
     *           equal to 1
     *         - ::NPP_COEFF_ERROR Indicates an error condition if coefficient values
     *           are invalid
     * </pre>
     */
    public static int nppiGetPerspectiveQuad(
        NppiRect oSrcROI,
        double[][] quad,
        double[][] aCoeffs)
    {
        return JNpp.checkResult(nppiGetPerspectiveQuadNative(oSrcROI, quad, aCoeffs));
    }
    private static native int nppiGetPerspectiveQuadNative(
        NppiRect oSrcROI,
        double[][] quad,
        double[][] aCoeffs);


    /**
     * <pre>
     * Calculates bounding box of the perspective transform projection of the
     * given source rectangular ROI
     *
     * @param oSrcROI Source ROI
     * @param bound Bounding box of the transformed source ROI
     * @param aCoeffs Perspective transform coefficients
     * @return
     *  Error codes:
     *         - ::NPP_SIZE_ERROR Indicates an error condition if any image dimension
     *           has zero or negative value
     *         - ::NPP_RECT_ERROR Indicates an error condition if width or height of
     *           the intersection of the oSrcROI and source image is less than or
     *           equal to 1
     *         - ::NPP_COEFF_ERROR Indicates an error condition if coefficient values
     *           are invalid
     * </pre>
     */
    public static int nppiGetPerspectiveBound(
        NppiRect oSrcROI,
        double[][] bound,
        double[][] aCoeffs)
    {
        return JNpp.checkResult(nppiGetPerspectiveBoundNative(oSrcROI, bound, aCoeffs));
    }
    private static native int nppiGetPerspectiveBoundNative(
        NppiRect oSrcROI,
        double[][] bound,
        double[][] aCoeffs);


    /**
     * <pre>
     * Single-channel 8-bit unsigned integer perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_8u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 8-bit unsigned integer perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_8u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_8u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer perspective warp, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_8u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 8-bit unsigned integer perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_8u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 8-bit unsigned integer perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_8u_P4R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_8u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_8u_P4RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 16-bit unsigned integer perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_16u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 16-bit unsigned integer perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_16u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_16u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpPerspective_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_16u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 16-bit unsigned integer perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_16u_P3R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_16u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_16u_P3RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpPerspective_16u_P4R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_16u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_16u_P4RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 32-bit signed integer perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_32s_C1R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_32s_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_32s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 32-bit signed integer perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_32s_C3R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_32s_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_32s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit signed integer perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_32s_C4R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_32s_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_32s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit signed integer perspective warp, igoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_32s_AC4R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_32s_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 32-bit signed integer perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_32s_P3R(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_32s_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_32s_P3RNative(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 32-bit signed integer perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_32s_P4R(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_32s_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_32s_P4RNative(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 32-bit floating-point perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_32f_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 32-bit floating-point perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_32f_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit floating-point perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_32f_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit floating-point perspective warp, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_32f_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 32-bit floating-point perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_32f_P3R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_32f_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_32f_P3RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 32-bit floating-point perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspective_32f_P4R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspective_32f_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspective_32f_P4RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 8-bit unsigned integer backwards perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_8u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 8-bit unsigned integer backwards perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_8u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer backwards perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_8u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpPerspectiveBack_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_8u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 8-bit unsigned integer backwards perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_8u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 8-bit unsigned integer backwards perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_8u_P4R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_8u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_8u_P4RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 16-bit unsigned integer backwards perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_16u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 16-bit unsigned integer backwards perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_16u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpPerspectiveBack_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_16u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpPerspectiveBack_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_16u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 16-bit unsigned integer backwards perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_16u_P3R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_16u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_16u_P3RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 16-bit unsigned integer backwards perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_16u_P4R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_16u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_16u_P4RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 32-bit signed integer backwards perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_32s_C1R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_32s_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_32s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 32-bit signed integer backwards perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_32s_C3R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_32s_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_32s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpPerspectiveBack_32s_C4R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_32s_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_32s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit signed integer backwards perspective warp, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_32s_AC4R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_32s_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 32-bit signed integer backwards perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_32s_P3R(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_32s_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_32s_P3RNative(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 32-bit signed integer backwards perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_32s_P4R(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_32s_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_32s_P4RNative(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpPerspectiveBack_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_32f_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    public static int nppiWarpPerspectiveBack_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_32f_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit floating-point backwards perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_32f_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit floating-point backwards perspective warp, ignorning alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_32f_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 32-bit floating-point backwards perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_32f_P3R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_32f_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_32f_P3RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 32-bit floating-point backwards perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aCoeffs Perspective transform coefficients
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveBack_32f_P4R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveBack_32f_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation));
    }
    private static native int nppiWarpPerspectiveBack_32f_P4RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aCoeffs,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 8-bit unsigned integer quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_8u_C1R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_8u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_8u_C1RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 8-bit unsigned integer quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_8u_C3R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_8u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_8u_C3RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_8u_C4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_8u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_8u_C4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 8-bit unsigned integer quad-based perspective warp, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_8u_AC4R(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_8u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_8u_AC4RNative(
        TypedPointer<Npp8u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 8-bit unsigned integer quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_8u_P3R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_8u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_8u_P3RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 8-bit unsigned integer quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_8u_P4R(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_8u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_8u_P4RNative(
        TypedPointer<Npp8u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp8u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 16-bit unsigned integer quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_16u_C1R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_16u_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_16u_C1RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 16-bit unsigned integer quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_16u_C3R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_16u_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_16u_C3RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_16u_C4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_16u_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_16u_C4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 16-bit unsigned integer quad-based perspective warp, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_16u_AC4R(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_16u_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_16u_AC4RNative(
        TypedPointer<Npp16u> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 16-bit unsigned integer quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_16u_P3R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_16u_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_16u_P3RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 16-bit unsigned integer quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_16u_P4R(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_16u_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_16u_P4RNative(
        TypedPointer<Npp16u>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp16u>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 32-bit signed integer quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_32s_C1R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_32s_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_32s_C1RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 32-bit signed integer quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_32s_C3R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_32s_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_32s_C3RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit signed integer quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_32s_C4R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_32s_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_32s_C4RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit signed integer quad-based perspective warp, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_32s_AC4R(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_32s_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_32s_AC4RNative(
        TypedPointer<Npp32s> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    public static int nppiWarpPerspectiveQuad_32s_P3R(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_32s_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_32s_P3RNative(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 32-bit signed integer quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_32s_P4R(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_32s_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_32s_P4RNative(
        TypedPointer<Npp32s>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32s>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Single-channel 32-bit floating-point quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_32f_C1R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_32f_C1RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_32f_C1RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel 32-bit floating-point quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_32f_C3R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_32f_C3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_32f_C3RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit floating-point quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_32f_C4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_32f_C4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_32f_C4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel 32-bit floating-point quad-based perspective warp, ignoring alpha channel.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_32f_AC4R(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_32f_AC4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_32f_AC4RNative(
        TypedPointer<Npp32f> pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f> pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Three-channel planar 32-bit floating-point quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_32f_P3R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_32f_P3RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_32f_P3RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Four-channel planar 32-bit floating-point quad-based perspective warp.
     *
     * @param pSrc \ref source_image_pointer.
     * @param oSrcSize Size of source image in pixels
     * @param nSrcStep \ref source_image_line_step.
     * @param oSrcROI Source ROI
     * @param aSrcQuad Source quad.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param oDstROI Destination ROI
     * @param aDstQuad Destination quad.
     * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
     *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
     * </pre>
     */
    public static int nppiWarpPerspectiveQuad_32f_P4R(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation)
    {
        return JNpp.checkResult(nppiWarpPerspectiveQuad_32f_P4RNative(pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation));
    }
    private static native int nppiWarpPerspectiveQuad_32f_P4RNative(
        TypedPointer<Npp32f>[] pSrc,
        NppiSize oSrcSize,
        int nSrcStep,
        NppiRect oSrcROI,
        double[][] aSrcQuad,
        TypedPointer<Npp32f>[] pDst,
        int nDstStep,
        NppiRect oDstROI,
        double[][] aDstQuad,
        int eInterpolation);


    /**
     * <pre>
     * Apply quality factor to raw 8-bit quantization table.
     *
     * This is effectively and in-place method that modifies a given raw
     * quantization table based on a quality factor.
     * Note that this method is a host method and that the pointer to the
     * raw quantization table is a host pointer.
     *
     * @param hpQuantRawTable Raw quantization table.
     * @param nQualityFactor Quality factor for the table. Range is [1:100].
     * @return
     *  Error code:
     *      ::NPP_NULL_POINTER_ERROR is returned if hpQuantRawTable is 0.
     * </pre>
     */
    public static int nppiQuantFwdRawTableInit_JPEG_8u(
        TypedPointer<Npp8u> hpQuantRawTable,
        int nQualityFactor)
    {
        return JNpp.checkResult(nppiQuantFwdRawTableInit_JPEG_8uNative(hpQuantRawTable, nQualityFactor));
    }
    private static native int nppiQuantFwdRawTableInit_JPEG_8uNative(
        TypedPointer<Npp8u> hpQuantRawTable,
        int nQualityFactor);


    public static int nppiQuantFwdTableInit_JPEG_8u16u(
        TypedPointer<Npp8u> hpQuantRawTable,
        TypedPointer<Npp16u> hpQuantFwdRawTable)
    {
        return JNpp.checkResult(nppiQuantFwdTableInit_JPEG_8u16uNative(hpQuantRawTable, hpQuantFwdRawTable));
    }
    private static native int nppiQuantFwdTableInit_JPEG_8u16uNative(
        TypedPointer<Npp8u> hpQuantRawTable,
        TypedPointer<Npp16u> hpQuantFwdRawTable);


    /**
     * <pre>
     * Initializes a quantization table for nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R().
     *      The nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R() method uses a quantization table
     * in a 16-bit format allowing for faster processing. In addition it converts the
     * data order from zigzag layout to original row-order layout. Typically raw
     * quantization tables are stored in zigzag format.
     *
     * This method is a host method and consumes and produces host data. I.e. the pointers
     * passed to this function must be host pointers. The resulting table needs to be
     * transferred to device memory in order to be used with nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R()
     * function.
     *
     * @param hpQuantRawTable Raw quantization table.
     * @param hpQuantFwdRawTable Inverse quantization table.
     * @return
     *  ::NPP_NULL_POINTER_ERROR pQuantRawTable or pQuantFwdRawTable is0.
     * </pre>
     */
    public static int nppiQuantInvTableInit_JPEG_8u16u(
        TypedPointer<Npp8u> hpQuantRawTable,
        TypedPointer<Npp16u> hpQuantFwdRawTable)
    {
        return JNpp.checkResult(nppiQuantInvTableInit_JPEG_8u16uNative(hpQuantRawTable, hpQuantFwdRawTable));
    }
    private static native int nppiQuantInvTableInit_JPEG_8u16uNative(
        TypedPointer<Npp8u> hpQuantRawTable,
        TypedPointer<Npp16u> hpQuantFwdRawTable);


    public static int nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        TypedPointer<Npp16u> pQuantFwdTable,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1RNative(pSrc, nSrcStep, pDst, nDstStep, pQuantFwdTable, oSizeROI));
    }
    private static native int nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1RNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        TypedPointer<Npp16u> pQuantFwdTable,
        NppiSize oSizeROI);


    public static int nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        TypedPointer<Npp16u> pQuantInvTable,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDCTQuantInv8x8LS_JPEG_16s8u_C1RNative(pSrc, nSrcStep, pDst, nDstStep, pQuantInvTable, oSizeROI));
    }
    private static native int nppiDCTQuantInv8x8LS_JPEG_16s8u_C1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        TypedPointer<Npp16u> pQuantInvTable,
        NppiSize oSizeROI);


    /**
     * <pre>
     * Initializes DCT state structure and allocates additional resources.
     *
     * @see nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R_NEW(), nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R_NEW.
     *
     * @param ppState Pointer to pointer to DCT state structure.
     *
     * @return
     *  NPP_SUCCESS Indicates no error. Any other value indicates an error
     *         or a warning
     *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
     *         has zero or negative value
     *  NPP_NULL_POINTER_ERROR Indicates an error condition if pBufSize
     *         pointer is NULL
     * </pre>
     */
    public static int nppiDCTInitAlloc(
        NppiDCTState ppState)
    {
        return JNpp.checkResult(nppiDCTInitAllocNative(ppState));
    }
    private static native int nppiDCTInitAllocNative(
        NppiDCTState ppState);


    /**
     * <pre>
     * Frees the additional resources of the DCT state structure.
     *
     * @see nppiDCTInitAlloc
     *
     * @param pState Pointer to DCT state structure.
     *
     * @return
     *  NPP_SUCCESS Indicates no error. Any other value indicates an error
     *         or a warning
     *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
     *         has zero or negative value
     *  NPP_NULL_POINTER_ERROR Indicates an error condition if pState
     *         pointer is NULL
     * </pre>
     */
    public static int nppiDCTFree(
        NppiDCTState pState)
    {
        return JNpp.checkResult(nppiDCTFreeNative(pState));
    }
    private static native int nppiDCTFreeNative(
        NppiDCTState pState);


    public static int nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R_NEW(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pQuantizationTable,
        NppiSize oSizeROI,
        NppiDCTState pState)
    {
        return JNpp.checkResult(nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R_NEWNative(pSrc, nSrcStep, pDst, nDstStep, pQuantizationTable, oSizeROI, pState));
    }
    private static native int nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R_NEWNative(
        TypedPointer<Npp8u> pSrc,
        int nSrcStep,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pQuantizationTable,
        NppiSize oSizeROI,
        NppiDCTState pState);


    /**
     * <pre>
     * Inverse DCT, de-quantization and level shift part of the JPEG decoding.
     * Input is expected in 64x1 macro blocks and output is expected to be in 8x8
     * macro blocks. The new version of the primitive takes the ROI in image pixel size and
     * works with DCT coefficients that are in zig-zag order.
     *
     * @param pSrc \ref source_image_pointer.
     * @param nSrcStep Image width in pixels x 8 x sizeof(Npp16s).
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pQuantizationTable Quantization Table in zig-zag order.
     * @param oSizeROI \ref roi_specification.
     * @param pState Pointer to DCT state structure. This structure must be
     *          initialized allocated and initialized using nppiDCTInitAlloc().
     * @return
     *  Error codes:
     *         - ::NPP_SIZE_ERROR For negative input height/width or not a multiple of
     *           8 width/height.
     *         - ::NPP_STEP_ERROR If input image width is not multiple of 8 or does not
     *           match ROI.
     *         - ::NPP_NULL_POINTER_ERROR If the destination pointer is 0.
     * </pre>
     */
    public static int nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R_NEW(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pQuantizationTable,
        NppiSize oSizeROI,
        NppiDCTState pState)
    {
        return JNpp.checkResult(nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R_NEWNative(pSrc, nSrcStep, pDst, nDstStep, pQuantizationTable, oSizeROI, pState));
    }
    private static native int nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R_NEWNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        TypedPointer<Npp8u> pDst,
        int nDstStep,
        TypedPointer<Npp8u> pQuantizationTable,
        NppiSize oSizeROI,
        NppiDCTState pState);


    /**
     * <pre>
     * Returns the length of the NppiDecodeHuffmanSpec structure.
     * @param pSize Pointer to a variable that will receive the length of the NppiDecodeHuffmanSpec structure.
     * @return
     *  Error codes:
     *         - ::NPP_NULL_POINTER_ERROR If one of the pointers is 0.
     * *
     * </pre>
     */
    public static int nppiDecodeHuffmanSpecGetBufSize_JPEG(
        int[] pSize)
    {
        return JNpp.checkResult(nppiDecodeHuffmanSpecGetBufSize_JPEGNative(pSize));
    }
    private static native int nppiDecodeHuffmanSpecGetBufSize_JPEGNative(
        int[] pSize);


    /**
     * <pre>
     * Creates a Huffman table in a format that is suitable for the decoder on the host.
     * @param pRawHuffmanTable Huffman table formated as specified in the JPEG standard.
     * @param eTableType Enum specifying type of table (nppiDCTable or nppiACTable).
     * @param pHuffmanSpec Pointer to the Huffman table for the decoder
     * @return
     *  Error codes:
     *         - ::NPP_NULL_POINTER_ERROR If one of the pointers is 0.
     * *
     * </pre>
     */
    public static int nppiDecodeHuffmanSpecInitHost_JPEG(
        TypedPointer<Npp8u> pRawHuffmanTable,
        int eTableType,
        NppiDecodeHuffmanSpec pHuffmanSpec)
    {
        return JNpp.checkResult(nppiDecodeHuffmanSpecInitHost_JPEGNative(pRawHuffmanTable, eTableType, pHuffmanSpec));
    }
    private static native int nppiDecodeHuffmanSpecInitHost_JPEGNative(
        TypedPointer<Npp8u> pRawHuffmanTable,
        int eTableType,
        NppiDecodeHuffmanSpec pHuffmanSpec);


    /**
     * <pre>
     * Allocates memory and creates a Huffman table in a format that is suitable for the decoder on the host.
     * @param pRawHuffmanTable Huffman table formated as specified in the JPEG standard.
     * @param eTableType Enum specifying type of table (nppiDCTable or nppiACTable).
     * @param ppHuffmanSpec Pointer to returned pointer to the Huffman table for the decoder
     * @return
     *  Error codes:
     *         - ::NPP_NULL_POINTER_ERROR If one of the pointers is 0.
     * *
     * </pre>
     */
    public static int nppiDecodeHuffmanSpecInitAllocHost_JPEG(
        TypedPointer<Npp8u> pRawHuffmanTable,
        int eTableType,
        NppiDecodeHuffmanSpec[] ppHuffmanSpec)
    {
        return JNpp.checkResult(nppiDecodeHuffmanSpecInitAllocHost_JPEGNative(pRawHuffmanTable, eTableType, ppHuffmanSpec));
    }
    private static native int nppiDecodeHuffmanSpecInitAllocHost_JPEGNative(
        TypedPointer<Npp8u> pRawHuffmanTable,
        int eTableType,
        NppiDecodeHuffmanSpec[] ppHuffmanSpec);


    public static int nppiDecodeHuffmanSpecFreeHost_JPEG(
        NppiDecodeHuffmanSpec pHuffmanSpec)
    {
        return JNpp.checkResult(nppiDecodeHuffmanSpecFreeHost_JPEGNative(pHuffmanSpec));
    }
    private static native int nppiDecodeHuffmanSpecFreeHost_JPEGNative(
        NppiDecodeHuffmanSpec pHuffmanSpec);


    /**
     * <pre>
     * Huffman Decoding of the JPEG decoding on the host.
     * Input is expected in byte stuffed huffman encoded JPEG scan and output is expected to be 64x1 macro blocks.
     *
     * @param pSrc Byte-stuffed huffman encoded JPEG scan.
     * @param nLength Byte length of the input.
     * @param restartInterval Restart Interval, see JPEG standard.
     * @param Ss Start Coefficient, see JPEG standard.
     * @param Se End Coefficient, see JPEG standard.
     * @param Ah Bit Approximation High, see JPEG standard.
     * @param Al Bit Approximation Low, see JPEG standard.
     * @param pDst \ref destination_image_pointer.
     * @param nDstStep \ref destination_image_line_step.
     * @param pHuffmanTableDC DC Huffman table.
     * @param pHuffmanTableAC AC Huffman table.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  Error codes:
     *         - ::NPP_SIZE_ERROR For negative input height/width or not a multiple of
     *           8 width/height.
     *         - ::NPP_STEP_ERROR If input image width is not multiple of 8 or does not
     *           match ROI.
     *         - ::NPP_NULL_POINTER_ERROR If the destination pointer is 0.
     * </pre>
     */
    public static int nppiDecodeHuffmanScanHost_JPEG_8u16s_P1R(
        TypedPointer<Npp8u> pSrc,
        int nLength,
        int restartInterval,
        int Ss,
        int Se,
        int Ah,
        int Al,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiDecodeHuffmanSpec pHuffmanTableDC,
        NppiDecodeHuffmanSpec pHuffmanTableAC,
        NppiSize oSizeROI)
    {
        return JNpp.checkResult(nppiDecodeHuffmanScanHost_JPEG_8u16s_P1RNative(pSrc, nLength, restartInterval, Ss, Se, Ah, Al, pDst, nDstStep, pHuffmanTableDC, pHuffmanTableAC, oSizeROI));
    }
    private static native int nppiDecodeHuffmanScanHost_JPEG_8u16s_P1RNative(
        TypedPointer<Npp8u> pSrc,
        int nLength,
        int restartInterval,
        int Ss,
        int Se,
        int Ah,
        int Al,
        TypedPointer<Npp16s> pDst,
        int nDstStep,
        NppiDecodeHuffmanSpec pHuffmanTableDC,
        NppiDecodeHuffmanSpec pHuffmanTableAC,
        NppiSize oSizeROI);


    public static int nppiDecodeHuffmanScanHost_JPEG_8u16s_P3R(
        TypedPointer<Npp8u> pSrc,
        int nLength,
        int nRestartInterval,
        int nSs,
        int nSe,
        int nAh,
        int nAl,
        TypedPointer<Npp16s>[] apDst,
        int[] aDstStep,
        NppiDecodeHuffmanSpec[] apHuffmanDCTable,
        NppiDecodeHuffmanSpec[] apHuffmanACTable,
        NppiSize[] aSizeROI)
    {
        return JNpp.checkResult(nppiDecodeHuffmanScanHost_JPEG_8u16s_P3RNative(pSrc, nLength, nRestartInterval, nSs, nSe, nAh, nAl, apDst, aDstStep, apHuffmanDCTable, apHuffmanACTable, aSizeROI));
    }
    private static native int nppiDecodeHuffmanScanHost_JPEG_8u16s_P3RNative(
        TypedPointer<Npp8u> pSrc,
        int nLength,
        int nRestartInterval,
        int nSs,
        int nSe,
        int nAh,
        int nAl,
        TypedPointer<Npp16s>[] apDst,
        int[] aDstStep,
        NppiDecodeHuffmanSpec[] apHuffmanDCTable,
        NppiDecodeHuffmanSpec[] apHuffmanACTable,
        NppiSize[] aSizeROI);


    /**
     * <pre>
     * Returns the length of the NppiEncodeHuffmanSpec structure.
     * @param pSize Pointer to a variable that will receive the length of the NppiEncodeHuffmanSpec structure.
     * @return
     *  Error codes:
     *         - ::NPP_NULL_POINTER_ERROR If one of the pointers is 0.
     * *
     * </pre>
     */
    public static int nppiEncodeHuffmanSpecGetBufSize_JPEG(
        int[] pSize)
    {
        return JNpp.checkResult(nppiEncodeHuffmanSpecGetBufSize_JPEGNative(pSize));
    }
    private static native int nppiEncodeHuffmanSpecGetBufSize_JPEGNative(
        int[] pSize);


    /**
     * <pre>
     * Creates a Huffman table in a format that is suitable for the encoder.
     * @param pRawHuffmanTable Huffman table formated as specified in the JPEG standard.
     * @param eTableType Enum specifying type of table (nppiDCTable or nppiACTable).
     * @param pHuffmanSpec Pointer to the Huffman table for the decoder
     * @return
     *  Error codes:
     *         - ::NPP_NULL_POINTER_ERROR If one of the pointers is 0.
     * *
     * </pre>
     */
    public static int nppiEncodeHuffmanSpecInit_JPEG(
        TypedPointer<Npp8u> pRawHuffmanTable,
        int eTableType,
        NppiEncodeHuffmanSpec pHuffmanSpec)
    {
        return JNpp.checkResult(nppiEncodeHuffmanSpecInit_JPEGNative(pRawHuffmanTable, eTableType, pHuffmanSpec));
    }
    private static native int nppiEncodeHuffmanSpecInit_JPEGNative(
        TypedPointer<Npp8u> pRawHuffmanTable,
        int eTableType,
        NppiEncodeHuffmanSpec pHuffmanSpec);


    /**
     * <pre>
     * Allocates memory and creates a Huffman table in a format that is suitable for the encoder.
     * @param pRawHuffmanTable Huffman table formated as specified in the JPEG standard.
     * @param eTableType Enum specifying type of table (nppiDCTable or nppiACTable).
     * @param ppHuffmanSpec Pointer to returned pointer to the Huffman table for the encoder
     * @return
     *  Error codes:
     *         - ::NPP_NULL_POINTER_ERROR If one of the pointers is 0.
     * *
     * </pre>
     */
    public static int nppiEncodeHuffmanSpecInitAlloc_JPEG(
        TypedPointer<Npp8u> pRawHuffmanTable,
        int eTableType,
        NppiEncodeHuffmanSpec[] ppHuffmanSpec)
    {
        return JNpp.checkResult(nppiEncodeHuffmanSpecInitAlloc_JPEGNative(pRawHuffmanTable, eTableType, ppHuffmanSpec));
    }
    private static native int nppiEncodeHuffmanSpecInitAlloc_JPEGNative(
        TypedPointer<Npp8u> pRawHuffmanTable,
        int eTableType,
        NppiEncodeHuffmanSpec[] ppHuffmanSpec);


    public static int nppiEncodeHuffmanSpecFree_JPEG(
        NppiEncodeHuffmanSpec pHuffmanSpec)
    {
        return JNpp.checkResult(nppiEncodeHuffmanSpecFree_JPEGNative(pHuffmanSpec));
    }
    private static native int nppiEncodeHuffmanSpecFree_JPEGNative(
        NppiEncodeHuffmanSpec pHuffmanSpec);


    /**
     * <pre>
     * Huffman Encoding of the JPEG Encoding.
     * Input is expected to be 64x1 macro blocks and output is expected as byte stuffed huffman encoded JPEG scan.
     *
     * @param pSrc \ref destination_image_pointer.
     * @param nSrcStep \ref destination_image_line_step.
     * @param nRestartInterval Restart Interval, see JPEG standard. Currently only values <=0 are supported.
     * @param nSs Start Coefficient, see JPEG standard.
     * @param nSe End Coefficient, see JPEG standard.
     * @param nAh Bit Approximation High, see JPEG standard.
     * @param nAl Bit Approximation Low, see JPEG standard.
     * @param pDst Byte-stuffed huffman encoded JPEG scan.
     * @param nLength Byte length of the huffman encoded JPEG scan.
     * @param pHuffmanTableDC DC Huffman table.
     * @param pHuffmanTableAC AC Huffman table.
     * @param oSizeROI \ref roi_specification.
     * @return
     *  Error codes:
     *         - ::NPP_SIZE_ERROR For negative input height/width or not a multiple of
     *           8 width/height.
     *         - ::NPP_STEP_ERROR If input image width is not multiple of 8 or does not
     *           match ROI.
     *         - ::NPP_NULL_POINTER_ERROR If the destination pointer is 0.
     *         - ::NPP_NOT_SUFFICIENT_COMPUTE_CAPABILITY If the device has compute capability < 2.0.
     * </pre>
     */
    public static int nppiEncodeHuffmanScan_JPEG_8u16s_P1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        int restartInterval,
        int Ss,
        int Se,
        int Ah,
        int Al,
        TypedPointer<Npp8u> pDst,
        TypedPointer<Npp32s> nLength,
        NppiEncodeHuffmanSpec pHuffmanTableDC,
        NppiEncodeHuffmanSpec pHuffmanTableAC,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pTempStorage)
    {
        return JNpp.checkResult(nppiEncodeHuffmanScan_JPEG_8u16s_P1RNative(pSrc, nSrcStep, restartInterval, Ss, Se, Ah, Al, pDst, nLength, pHuffmanTableDC, pHuffmanTableAC, oSizeROI, pTempStorage));
    }
    private static native int nppiEncodeHuffmanScan_JPEG_8u16s_P1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        int restartInterval,
        int Ss,
        int Se,
        int Ah,
        int Al,
        TypedPointer<Npp8u> pDst,
        TypedPointer<Npp32s> nLength,
        NppiEncodeHuffmanSpec pHuffmanTableDC,
        NppiEncodeHuffmanSpec pHuffmanTableAC,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pTempStorage);


    /**
     * <pre>
     * Huffman Encoding of the JPEG Encoding.
     * Input is expected to be 64x1 macro blocks and output is expected as byte stuffed huffman encoded JPEG scan.
     *
     * @param apSrc \ref destination_image_pointer.
     * @param aSrcStep \ref destination_image_line_step.
     * @param nRestartInterval Restart Interval, see JPEG standard. Currently only values <=0 are supported.
     * @param nSs Start Coefficient, see JPEG standard.
     * @param nSe End Coefficient, see JPEG standard.
     * @param nAh Bit Approximation High, see JPEG standard.
     * @param nAl Bit Approximation Low, see JPEG standard.
     * @param pDst Byte-stuffed huffman encoded JPEG scan.
     * @param nLength Byte length of the huffman encoded JPEG scan.
     * @param apHuffmanTableDC DC Huffman tables.
     * @param apHuffmanTableAC AC Huffman tables.
     * @param aSizeROI \ref roi_specification.
     * @return
     *  Error codes:
     *         - ::NPP_SIZE_ERROR For negative input height/width or not a multiple of
     *           8 width/height.
     *         - ::NPP_STEP_ERROR If input image width is not multiple of 8 or does not
     *           match ROI.
     *         - ::NPP_NULL_POINTER_ERROR If the destination pointer is 0.
     *         - ::NPP_NOT_SUFFICIENT_COMPUTE_CAPABILITY If the device has compute capability < 2.0.
     * </pre>
     */
    public static int nppiEncodeHuffmanScan_JPEG_8u16s_P3R(
        TypedPointer<Npp16s>[] apSrc,
        int[] aSrcStep,
        int nRestartInterval,
        int nSs,
        int nSe,
        int nAh,
        int nAl,
        TypedPointer<Npp8u> pDst,
        TypedPointer<Npp32s> nLength,
        NppiEncodeHuffmanSpec[] apHuffmanDCTable,
        NppiEncodeHuffmanSpec[] apHuffmanACTable,
        NppiSize[] aSizeROI,
        TypedPointer<Npp8u> pTempStorage)
    {
        return JNpp.checkResult(nppiEncodeHuffmanScan_JPEG_8u16s_P3RNative(apSrc, aSrcStep, nRestartInterval, nSs, nSe, nAh, nAl, pDst, nLength, apHuffmanDCTable, apHuffmanACTable, aSizeROI, pTempStorage));
    }
    private static native int nppiEncodeHuffmanScan_JPEG_8u16s_P3RNative(
        TypedPointer<Npp16s>[] apSrc,
        int[] aSrcStep,
        int nRestartInterval,
        int nSs,
        int nSe,
        int nAh,
        int nAl,
        TypedPointer<Npp8u> pDst,
        TypedPointer<Npp32s> nLength,
        NppiEncodeHuffmanSpec[] apHuffmanDCTable,
        NppiEncodeHuffmanSpec[] apHuffmanACTable,
        NppiSize[] aSizeROI,
        TypedPointer<Npp8u> pTempStorage);


    public static int nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P1R(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        int nRestartInterval,
        int nSs,
        int nSe,
        int nAh,
        int nAl,
        TypedPointer<Npp8u> pDst,
        TypedPointer<Npp32s> pLength,
        TypedPointer<Npp8u> hpCodesDC,
        TypedPointer<Npp8u> hpTableDC,
        TypedPointer<Npp8u> hpCodesAC,
        TypedPointer<Npp8u> hpTableAC,
        NppiEncodeHuffmanSpec aHuffmanDCTable,
        NppiEncodeHuffmanSpec aHuffmanACTable,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pTempStorage)
    {
        return JNpp.checkResult(nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P1RNative(pSrc, nSrcStep, nRestartInterval, nSs, nSe, nAh, nAl, pDst, pLength, hpCodesDC, hpTableDC, hpCodesAC, hpTableAC, aHuffmanDCTable, aHuffmanACTable, oSizeROI, pTempStorage));
    }
    private static native int nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P1RNative(
        TypedPointer<Npp16s> pSrc,
        int nSrcStep,
        int nRestartInterval,
        int nSs,
        int nSe,
        int nAh,
        int nAl,
        TypedPointer<Npp8u> pDst,
        TypedPointer<Npp32s> pLength,
        TypedPointer<Npp8u> hpCodesDC,
        TypedPointer<Npp8u> hpTableDC,
        TypedPointer<Npp8u> hpCodesAC,
        TypedPointer<Npp8u> hpTableAC,
        NppiEncodeHuffmanSpec aHuffmanDCTable,
        NppiEncodeHuffmanSpec aHuffmanACTable,
        NppiSize oSizeROI,
        TypedPointer<Npp8u> pTempStorage);


    /**
     * <pre>
     * Optimize Huffman Encoding of the JPEG Encoding.
     * Input is expected to be 64x1 macro blocks and output is expected as byte stuffed huffman encoded JPEG scan.
     *
     * @param apSrc \ref destination_image_pointer.
     * @param aSrcStep \ref destination_image_line_step.
     * @param nRestartInterval Restart Interval, see JPEG standard. Currently only values <=0 are supported.
     * @param nSs Start Coefficient, see JPEG standard.
     * @param nSe End Coefficient, see JPEG standard.
     * @param nAh Bit Approximation High, see JPEG standard.
     * @param nAl Bit Approximation Low, see JPEG standard.
     * @param pDst Byte-stuffed huffman encoded JPEG scan.
     * @param pLength Pointer to the byte length of the huffman encoded JPEG scan.
     * @param hpCodesDC Host pointer to the code of the huffman tree for DC component.
     * @param hpTableDC Host pointer to the table of the huffman tree for DC component.
     * @param hpCodesAC Host pointer to the code of the huffman tree for AC component.
     * @param hpTableAC Host pointer to the table of the huffman tree for AC component.
     * @param apHuffmanTableDC DC Huffman tables.
     * @param apHuffmanTableAC AC Huffman tables.
     * @param aSizeROI \ref roi_specification.
     * @return
     *  Error codes:
     *         - ::NPP_SIZE_ERROR For negative input height/width or not a multiple of
     *           8 width/height.
     *         - ::NPP_STEP_ERROR If input image width is not multiple of 8 or does not
     *           match ROI.
     *         - ::NPP_NULL_POINTER_ERROR If the destination pointer is 0.
     *         - ::NPP_NOT_SUFFICIENT_COMPUTE_CAPABILITY If the device has compute capability < 2.0.
     * </pre>
     */
    public static int nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3R(
        TypedPointer<Npp16s>[] pSrc,
        int[] aSrcStep,
        int nRestartInterval,
        int nSs,
        int nSe,
        int nAh,
        int nAl,
        TypedPointer<Npp8u> pDst,
        TypedPointer<Npp32s> pLength,
        TypedPointer<Npp8u>[] hpCodesDC,
        TypedPointer<Npp8u>[] hpTableDC,
        TypedPointer<Npp8u>[] hpCodesAC,
        TypedPointer<Npp8u>[] hpTableAC,
        NppiEncodeHuffmanSpec[] aHuffmanDCTable,
        NppiEncodeHuffmanSpec[] aHuffmanACTable,
        NppiSize[] oSizeROI,
        TypedPointer<Npp8u> pTempStorage)
    {
        return JNpp.checkResult(nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3RNative(pSrc, aSrcStep, nRestartInterval, nSs, nSe, nAh, nAl, pDst, pLength, hpCodesDC, hpTableDC, hpCodesAC, hpTableAC, aHuffmanDCTable, aHuffmanACTable, oSizeROI, pTempStorage));
    }
    private static native int nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3RNative(
        TypedPointer<Npp16s>[] pSrc,
        int[] aSrcStep,
        int nRestartInterval,
        int nSs,
        int nSe,
        int nAh,
        int nAl,
        TypedPointer<Npp8u> pDst,
        TypedPointer<Npp32s> pLength,
        TypedPointer<Npp8u>[] hpCodesDC,
        TypedPointer<Npp8u>[] hpTableDC,
        TypedPointer<Npp8u>[] hpCodesAC,
        TypedPointer<Npp8u>[] hpTableAC,
        NppiEncodeHuffmanSpec[] aHuffmanDCTable,
        NppiEncodeHuffmanSpec[] aHuffmanACTable,
        NppiSize[] oSizeROI,
        TypedPointer<Npp8u> pTempStorage);


    /**
     * <pre>
     * Calculates the size of the temporary buffer for baseline Huffman encoding.
     *
     * @see nppiEncodeHuffmanScan_JPEG_8u16s_P1R(), nppiEncodeHuffmanScan_JPEG_8u16s_P3R().
     *
     * @param oSize Image Dimension.
     * @param pBufSize Pointer to variable that returns the size of the
     *        temporary buffer.
     *
     * @return
     *  NPP_SUCCESS Indicates no error. Any other value indicates an error
     *         or a warning
     *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
     *         has zero or negative value
     *  NPP_NULL_POINTER_ERROR Indicates an error condition if pBufSize
     *         pointer is NULL
     * </pre>
     */
    public static int nppiEncodeHuffmanGetSize(
        NppiSize oSize,
        int nChannels,
        int[] pBufSize)
    {
        return JNpp.checkResult(nppiEncodeHuffmanGetSizeNative(oSize, nChannels, pBufSize));
    }
    private static native int nppiEncodeHuffmanGetSizeNative(
        NppiSize oSize,
        int nChannels,
        int[] pBufSize);


    public static int nppiEncodeOptimizeHuffmanGetSize(
        NppiSize oSize,
        int nChannels,
        int[] pBufSize)
    {
        return JNpp.checkResult(nppiEncodeOptimizeHuffmanGetSizeNative(oSize, nChannels, pBufSize));
    }
    private static native int nppiEncodeOptimizeHuffmanGetSizeNative(
        NppiSize oSize,
        int nChannels,
        int[] pBufSize);


    /**
     * <pre>
     * Calculates the size of the temporary buffer for graph-cut with 4 neighborhood labeling.
     *
     * @see nppiGraphcutInitAlloc(), nppiGraphcut_32s8u().
     *
     * @param oSize Graph size.
     * @param pBufSize Pointer to variable that returns the size of the
     *        temporary buffer.
     *
     * @return
     *  NPP_SUCCESS Indicates no error. Any other value indicates an error
     *         or a warning
     *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
     *         has zero or negative value
     *  NPP_NULL_POINTER_ERROR Indicates an error condition if pBufSize
     *         pointer is NULL
     * </pre>
     */
    public static int nppiGraphcutGetSize(
        NppiSize oSize,
        int[] pBufSize)
    {
        return JNpp.checkResult(nppiGraphcutGetSizeNative(oSize, pBufSize));
    }
    private static native int nppiGraphcutGetSizeNative(
        NppiSize oSize,
        int[] pBufSize);


    /**
     * <pre>
     * Calculates the size of the temporary buffer for graph-cut with 8 neighborhood labeling.
     *
     * @see nppiGraphcut8InitAlloc(), nppiGraphcut8_32s8u().
     *
     * @param oSize Graph size.
     * @param pBufSize Pointer to variable that returns the size of the
     *        temporary buffer.
     *
     * @return
     *  NPP_SUCCESS Indicates no error. Any other value indicates an error
     *         or a warning
     *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
     *         has zero or negative value
     *  NPP_NULL_POINTER_ERROR Indicates an error condition if pBufSize
     *         pointer is NULL
     * </pre>
     */
    public static int nppiGraphcut8GetSize(
        NppiSize oSize,
        int[] pBufSize)
    {
        return JNpp.checkResult(nppiGraphcut8GetSizeNative(oSize, pBufSize));
    }
    private static native int nppiGraphcut8GetSizeNative(
        NppiSize oSize,
        int[] pBufSize);


    /**
     * <pre>
     * Initializes graph-cut state structure and allocates additional resources for graph-cut with 8 neighborhood labeling.
     *
     * @see nppiGraphcut_32s8u(), nppiGraphcutGetSize().
     *
     * @param oSize Graph size
     * @param ppState Pointer to pointer to graph-cut state structure.
     * @param pDeviceMem pDeviceMem to the sufficient amount of device memory. The CUDA runtime or NPP memory allocators
     *      must be used to allocate this memory. The minimum amount of device memory required
     *      to run graph-cut on a for a specific image size is computed by nppiGraphcutGetSize().
     *
     * @return
     *  NPP_SUCCESS Indicates no error. Any other value indicates an error
     *         or a warning
     *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
     *         has zero or negative value
     *  NPP_NULL_POINTER_ERROR Indicates an error condition if pBufSize
     *         pointer is NULL
     * </pre>
     */
    public static int nppiGraphcutInitAlloc(
        NppiSize oSize,
        NppiGraphcutState[] ppState,
        TypedPointer<Npp8u> pDeviceMem)
    {
        return JNpp.checkResult(nppiGraphcutInitAllocNative(oSize, ppState, pDeviceMem));
    }
    private static native int nppiGraphcutInitAllocNative(
        NppiSize oSize,
        NppiGraphcutState[] ppState,
        TypedPointer<Npp8u> pDeviceMem);


    public static int nppiGraphcut8InitAlloc(
        NppiSize oSize,
        NppiGraphcutState[] ppState,
        TypedPointer<Npp8u> pDeviceMem)
    {
        return JNpp.checkResult(nppiGraphcut8InitAllocNative(oSize, ppState, pDeviceMem));
    }
    private static native int nppiGraphcut8InitAllocNative(
        NppiSize oSize,
        NppiGraphcutState[] ppState,
        TypedPointer<Npp8u> pDeviceMem);


    /**
     * <pre>
     * Frees the additional resources of the graph-cut state structure.
     *
     * @see nppiGraphcutInitAlloc
     * @see nppiGraphcut8InitAlloc
     *
     * @param pState Pointer to graph-cut state structure.
     *
     * @return
     *  NPP_SUCCESS Indicates no error. Any other value indicates an error
     *         or a warning
     *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
     *         has zero or negative value
     *  NPP_NULL_POINTER_ERROR Indicates an error condition if pState
     *         pointer is NULL
     * </pre>
     */
    public static int nppiGraphcutFree(
        NppiGraphcutState pState)
    {
        return JNpp.checkResult(nppiGraphcutFreeNative(pState));
    }
    private static native int nppiGraphcutFreeNative(
        NppiGraphcutState pState);


    /**
     * <pre>
     * Graphcut of a flow network (32bit signed integer edge capacities). The
     * function computes the minimal cut (graphcut) of a 2D regular 4-connected
     * graph.
     * The inputs are the capacities of the horizontal (in transposed form),
     * vertical and terminal (source and sink) edges. The capacities to source and
     * sink
     * are stored as capacity differences in the terminals array
     * ( terminals(x) = source(x) - sink(x) ). The implementation assumes that the
     * edge capacities
     * for boundary edges that would connect to nodes outside the specified domain
     * are set to 0 (for example left(0,*) == 0). If this is not fulfilled the
     * computed labeling may be wrong!
     * The computed binary labeling is encoded as unsigned 8bit values (0 and >0).
     *
     * @see nppiGraphcutInitAlloc(), nppiGraphcutFree(), nppiGraphcutGetSize().
     *
     * @param pTerminals Pointer to differences of terminal edge capacities
     *        (terminal(x) = source(x) - sink(x))
     * @param pLeftTransposed Pointer to transposed left edge capacities
     *        (left(0,*) must be 0)
     * @param pRightTransposed Pointer to transposed right edge capacities
     *        (right(width-1,*) must be 0)
     * @param pTop Pointer to top edge capacities (top(*,0) must be 0)
     * @param pBottom Pointer to bottom edge capacities (bottom(*,height-1)
     *        must be 0)
     * @param nStep Step in bytes between any pair of sequential rows of edge
     *        capacities
     * @param nTransposedStep Step in bytes between any pair of sequential
     *        rows of tranposed edge capacities
     * @param size Graph size
     * @param pLabel Pointer to destination label image
     * @param nLabelStep Step in bytes between any pair of sequential rows of
     *        label image
     * @param pState Pointer to graph-cut state structure. This structure must be
     *          initialized allocated and initialized using nppiGraphcutInitAlloc().
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiGraphcut_32s8u(
        TypedPointer<Npp32s> pTerminals,
        TypedPointer<Npp32s> pLeftTransposed,
        TypedPointer<Npp32s> pRightTransposed,
        TypedPointer<Npp32s> pTop,
        TypedPointer<Npp32s> pBottom,
        int nStep,
        int nTransposedStep,
        NppiSize size,
        TypedPointer<Npp8u> pLabel,
        int nLabelStep,
        NppiGraphcutState pState)
    {
        return JNpp.checkResult(nppiGraphcut_32s8uNative(pTerminals, pLeftTransposed, pRightTransposed, pTop, pBottom, nStep, nTransposedStep, size, pLabel, nLabelStep, pState));
    }
    private static native int nppiGraphcut_32s8uNative(
        TypedPointer<Npp32s> pTerminals,
        TypedPointer<Npp32s> pLeftTransposed,
        TypedPointer<Npp32s> pRightTransposed,
        TypedPointer<Npp32s> pTop,
        TypedPointer<Npp32s> pBottom,
        int nStep,
        int nTransposedStep,
        NppiSize size,
        TypedPointer<Npp8u> pLabel,
        int nLabelStep,
        NppiGraphcutState pState);


    /**
     * <pre>
     * Graphcut of a flow network (32bit signed integer edge capacities). The
     * function computes the minimal cut (graphcut) of a 2D regular 8-connected
     * graph.
     * The inputs are the capacities of the horizontal (in transposed form),
     * vertical and terminal (source and sink) edges. The capacities to source and
     * sink
     * are stored as capacity differences in the terminals array
     * ( terminals(x) = source(x) - sink(x) ). The implementation assumes that the
     * edge capacities
     * for boundary edges that would connect to nodes outside the specified domain
     * are set to 0 (for example left(0,*) == 0). If this is not fulfilled the
     * computed labeling may be wrong!
     * The computed binary labeling is encoded as unsigned 8bit values (0 and >0).
     *
     * @see nppiGraphcut8InitAlloc(), nppiGraphcutFree(), nppiGraphcut8GetSize().
     *
     * @param pTerminals Pointer to differences of terminal edge capacities
     *        (terminal(x) = source(x) - sink(x))
     * @param pLeftTransposed Pointer to transposed left edge capacities
     *        (left(0,*) must be 0)
     * @param pRightTransposed Pointer to transposed right edge capacities
     *        (right(width-1,*) must be 0)
     * @param pTop Pointer to top edge capacities (top(*,0) must be 0)
     * @param pTopLeft Pointer to top left edge capacities (topleft(*,0)
     *        & topleft(0,*) must be 0)
     * @param pTopRight Pointer to top right edge capacities (topright(*,0)
     *        & topright(width-1,*) must be 0)
     * @param pBottom Pointer to bottom edge capacities (bottom(*,height-1)
     *        must be 0)
     * @param pBottomLeft Pointer to bottom left edge capacities
     *        (bottomleft(*,height-1) && bottomleft(0,*) must be 0)
     * @param pBottomRight Pointer to bottom right edge capacities
     *        (bottomright(*,height-1) && bottomright(width-1,*) must be 0)
     * @param nStep Step in bytes between any pair of sequential rows of edge
     *        capacities
     * @param nTransposedStep Step in bytes between any pair of sequential
     *        rows of tranposed edge capacities
     * @param size Graph size
     * @param pLabel Pointer to destination label image
     * @param nLabelStep Step in bytes between any pair of sequential rows of
     *        label image
     * @param pState Pointer to graph-cut state structure. This structure must be
     *          initialized allocated and initialized using nppiGraphcut8InitAlloc().
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiGraphcut8_32s8u(
        TypedPointer<Npp32s> pTerminals,
        TypedPointer<Npp32s> pLeftTransposed,
        TypedPointer<Npp32s> pRightTransposed,
        TypedPointer<Npp32s> pTop,
        TypedPointer<Npp32s> pTopLeft,
        TypedPointer<Npp32s> pTopRight,
        TypedPointer<Npp32s> pBottom,
        TypedPointer<Npp32s> pBottomLeft,
        TypedPointer<Npp32s> pBottomRight,
        int nStep,
        int nTransposedStep,
        NppiSize size,
        TypedPointer<Npp8u> pLabel,
        int nLabelStep,
        NppiGraphcutState pState)
    {
        return JNpp.checkResult(nppiGraphcut8_32s8uNative(pTerminals, pLeftTransposed, pRightTransposed, pTop, pTopLeft, pTopRight, pBottom, pBottomLeft, pBottomRight, nStep, nTransposedStep, size, pLabel, nLabelStep, pState));
    }
    private static native int nppiGraphcut8_32s8uNative(
        TypedPointer<Npp32s> pTerminals,
        TypedPointer<Npp32s> pLeftTransposed,
        TypedPointer<Npp32s> pRightTransposed,
        TypedPointer<Npp32s> pTop,
        TypedPointer<Npp32s> pTopLeft,
        TypedPointer<Npp32s> pTopRight,
        TypedPointer<Npp32s> pBottom,
        TypedPointer<Npp32s> pBottomLeft,
        TypedPointer<Npp32s> pBottomRight,
        int nStep,
        int nTransposedStep,
        NppiSize size,
        TypedPointer<Npp8u> pLabel,
        int nLabelStep,
        NppiGraphcutState pState);


    public static int nppiGraphcut_32f8u(
        TypedPointer<Npp32f> pTerminals,
        TypedPointer<Npp32f> pLeftTransposed,
        TypedPointer<Npp32f> pRightTransposed,
        TypedPointer<Npp32f> pTop,
        TypedPointer<Npp32f> pBottom,
        int nStep,
        int nTransposedStep,
        NppiSize size,
        TypedPointer<Npp8u> pLabel,
        int nLabelStep,
        NppiGraphcutState pState)
    {
        return JNpp.checkResult(nppiGraphcut_32f8uNative(pTerminals, pLeftTransposed, pRightTransposed, pTop, pBottom, nStep, nTransposedStep, size, pLabel, nLabelStep, pState));
    }
    private static native int nppiGraphcut_32f8uNative(
        TypedPointer<Npp32f> pTerminals,
        TypedPointer<Npp32f> pLeftTransposed,
        TypedPointer<Npp32f> pRightTransposed,
        TypedPointer<Npp32f> pTop,
        TypedPointer<Npp32f> pBottom,
        int nStep,
        int nTransposedStep,
        NppiSize size,
        TypedPointer<Npp8u> pLabel,
        int nLabelStep,
        NppiGraphcutState pState);


    /**
     * <pre>
     * Graphcut of a flow network (32bit float edge capacities). The
     * function computes the minimal cut (graphcut) of a 2D regular 8-connected
     * graph.
     * The inputs are the capacities of the horizontal (in transposed form),
     * vertical and terminal (source and sink) edges. The capacities to source and
     * sink
     * are stored as capacity differences in the terminals array
     * ( terminals(x) = source(x) - sink(x) ). The implementation assumes that the
     * edge capacities
     * for boundary edges that would connect to nodes outside the specified domain
     * are set to 0 (for example left(0,*) == 0). If this is not fulfilled the
     * computed labeling may be wrong!
     * The computed binary labeling is encoded as unsigned 8bit values (0 and >0).
     *
     * @see nppiGraphcut8InitAlloc(), nppiGraphcutFree(), nppiGraphcut8GetSize().
     *
     * @param pTerminals Pointer to differences of terminal edge capacities
     *        (terminal(x) = source(x) - sink(x))
     * @param pLeftTransposed Pointer to transposed left edge capacities
     *        (left(0,*) must be 0)
     * @param pRightTransposed Pointer to transposed right edge capacities
     *        (right(width-1,*) must be 0)
     * @param pTop Pointer to top edge capacities (top(*,0) must be 0)
     * @param pTopLeft Pointer to top left edge capacities (topleft(*,0)
     *        & topleft(0,*) must be 0)
     * @param pTopRight Pointer to top right edge capacities (topright(*,0)
     *        & topright(width-1,*) must be 0)
     * @param pBottom Pointer to bottom edge capacities (bottom(*,height-1)
     *        must be 0)
     * @param pBottomLeft Pointer to bottom left edge capacities
     *        (bottomleft(*,height-1) && bottomleft(0,*) must be 0)
     * @param pBottomRight Pointer to bottom right edge capacities
     *        (bottomright(*,height-1) && bottomright(width-1,*) must be 0)
     * @param nStep Step in bytes between any pair of sequential rows of edge
     *        capacities
     * @param nTransposedStep Step in bytes between any pair of sequential
     *        rows of tranposed edge capacities
     * @param size Graph size
     * @param pLabel Pointer to destination label image
     * @param nLabelStep Step in bytes between any pair of sequential rows of
     *        label image
     * @param pState Pointer to graph-cut state structure. This structure must be
     *          initialized allocated and initialized using nppiGraphcut8InitAlloc().
     * @return
     *  \ref image_data_error_codes, \ref roi_error_codes
     * </pre>
     */
    public static int nppiGraphcut8_32f8u(
        TypedPointer<Npp32f> pTerminals,
        TypedPointer<Npp32f> pLeftTransposed,
        TypedPointer<Npp32f> pRightTransposed,
        TypedPointer<Npp32f> pTop,
        TypedPointer<Npp32f> pTopLeft,
        TypedPointer<Npp32f> pTopRight,
        TypedPointer<Npp32f> pBottom,
        TypedPointer<Npp32f> pBottomLeft,
        TypedPointer<Npp32f> pBottomRight,
        int nStep,
        int nTransposedStep,
        NppiSize size,
        TypedPointer<Npp8u> pLabel,
        int nLabelStep,
        NppiGraphcutState pState)
    {
        return JNpp.checkResult(nppiGraphcut8_32f8uNative(pTerminals, pLeftTransposed, pRightTransposed, pTop, pTopLeft, pTopRight, pBottom, pBottomLeft, pBottomRight, nStep, nTransposedStep, size, pLabel, nLabelStep, pState));
    }
    private static native int nppiGraphcut8_32f8uNative(
        TypedPointer<Npp32f> pTerminals,
        TypedPointer<Npp32f> pLeftTransposed,
        TypedPointer<Npp32f> pRightTransposed,
        TypedPointer<Npp32f> pTop,
        TypedPointer<Npp32f> pTopLeft,
        TypedPointer<Npp32f> pTopRight,
        TypedPointer<Npp32f> pBottom,
        TypedPointer<Npp32f> pBottomLeft,
        TypedPointer<Npp32f> pBottomRight,
        int nStep,
        int nTransposedStep,
        NppiSize size,
        TypedPointer<Npp8u> pLabel,
        int nLabelStep,
        NppiGraphcutState pState);

}

